 /*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
 


#ifdef XFree86Server
#include "xf86_ansic.h"     /* for Bool */
#else
#include "via_types.h"      /* for Bool */
#endif
#include "VidInit.h"

#include "VidStruc.h"       /* VIDDATA, video top level structure */
#include "debug.h"


#include "via.h"            /* for VIDInD & VIDOutD */
#include "vdefCME.h"
#include "ddmpeg.h"
#include "via_memmgr.h"	    /* V4L memory management */


/* Extern Variable for Video function*/
extern VIDEOMEMORY     VidMemory;          /* video memory struct */

/* Extern function for Video function*/
extern void vfHM(PVIDDATA pVidData,unsigned long dwFunSel);
extern int vfInitialAGPControl(PVIDDATA pVidData);
extern int vfReleaseAGPControl(PVIDDATA pVidData);
extern void vfInitVidREGRec_CLE(void );

/* Defination for Video function*/
void vfInitVidFlag_CME(PVIDDATA pVidData);
void vfEnableVideoVirtualQueue(unsigned long dwVideoFlag,
        unsigned char* pjVideo,unsigned long dwVideoVQ_Offset,
        unsigned long dwVQ_Size);
void vfDisableVideoVirtualQueue(unsigned char* pjVideo);


/* function implement for Video Driver */
/* Descrption:This function is only for CLE266 */
unsigned long DispatchVGARevisionID(PVIDDATA pVidData)
{
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;

    if ( lpVIAGraphicInfo->dwRevisionID >= VIA_REVISION_CLECX )
    {
        /* revisionID: Cx chip */
        return  VIA_REVISION_CLECX;
    }
    else if ( lpVIAGraphicInfo->dwRevisionID < VIA_REVISION_CLECX )
    {
        /* revisionID: Ax chip */
        return   VIA_REVISION_CLEAX;
    }
    else
    {
        /* revisionID: other chip */
        return   lpVIAGraphicInfo->dwRevisionID;
    }
}

Bool vfMapVideoFun(PVIDDATA pVidData)
{
    unsigned long dwDeviceID;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;

    dwDeviceID = lpVIAGraphicInfo->dwDeviceID;


    switch( dwDeviceID )
    {
        case VIA_DEVICE_CLE1:
        case VIA_DEVICE_CLE2:
            DBG_DD(("Map CLE Video Function.\n"));
        #ifdef XFree86Server
            fpUpd_Video = Upd_Video;		
            fpCreateSurface = CreateSurface;
            fpDestroySurface = DestroySurface;
            fpFlip = Flip_CLE;
        #endif			
            break;

        case VIA_DEVICE_VT3205:
            DBG_DD(("Map KM Video Function.\n"));
        #ifdef XFree86Server
            fpUpd_Video = Upd_Video;
		    fpCreateSurface = CreateSurface;
            fpDestroySurface = DestroySurface;
            fpFlip = Flip_CLE;
    	#endif				
            break;

        case VIA_DEVICE_VT3204:
        case VIA_DEVICE_VT3314:
            DBG_DD(("Map K8M Video Function.\n"));
        #ifdef XFree86Server
            fpUpd_Video = Upd_Video;
	        fpCreateSurface = CreateSurface;
            fpDestroySurface = DestroySurface;
            fpFlip = Flip_CLE;
	    #endif	
            break;

        case VIA_DEVICE_VT3259:
            DBG_DD(("Map CME Video Function.\n"));
        #ifdef XFree86Server
            fpUpd_Video = Upd_Video_CME;
            fpCreateSurface = CreateSurface_CME;
            fpDestroySurface = DestroySurface_CME;
            fpFlip = Flip_CME;
    	#endif	
            break;
            
        case VIA_DEVICE_VT3324:        
		case VIA_DEVICE_VT3353:
            DBG_DD(("Map MSD Video Function.\n"));
        #ifdef XFree86Server
            fpUpd_Video = Upd_Video_CME;
            fpCreateSurface = CreateSurface_CME;
            fpDestroySurface = DestroySurface_CME;
            fpFlip = Flip_CME;
        #endif	
            break;
            
        case VIA_DEVICE_VT3293:        
            DBG_DD(("Map MSD Video Function.\n"));
        #ifdef XFree86Server
            fpUpd_Video = Upd_Video_CME;
            fpCreateSurface = CreateSurface_CME;
            fpDestroySurface = DestroySurface_CME;
            fpFlip = Flip_CME;
        #endif	
            break;    
            
        case VIA_DEVICE_VT3336: 
        case VIA_DEVICE_VT3327:
        case VIA_DEVICE_VT3364: 
            DBG_DD(("Map K8M Video Function.\n"));
        #ifdef XFree86Server
            fpUpd_Video = Upd_Video_CME;
            fpCreateSurface = CreateSurface_CME;
            fpDestroySurface = DestroySurface_CME;
            fpFlip = Flip_CME;
	    #endif	
            break;

        default:
            DBG_DD(("vfMapVideoFun : Unknow Device ID\n"));
            DBG_DD(("Initial video function pointer with CME Video Function.\n"));
        #ifdef XFree86Server
            fpUpd_Video = Upd_Video_CME;
            fpCreateSurface = CreateSurface_CME;
            fpDestroySurface = DestroySurface_CME;
            fpFlip = Flip_CME;
	    #endif	
            return FALSE;
    }
    return TRUE;
}

void vfInitHWDiff(PVIDDATA pVidData)
{
    LPVIDHWDIFFERENCE lpVideoHWDifference = &pVidData->VideoHWDifference;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;

    switch( lpVIAGraphicInfo->dwProjectIDRevision )
    {
        case UNICHROME_VT3123AX:
            /* HW Difference Flag */
            lpVideoHWDifference->dwThreeHQVBuffer = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwV3SrcHeightSetting = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportExtendFIFO = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVFetchByteUnit = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwHQVInitPatch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSupportV3Gamma=VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwUpdFlip = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwHQVDisablePatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSUBFlip = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV1Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV3Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV4Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwUseSystemMemory = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwExpandVerPatch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwExpandVerHorPatch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3ExpireNumTune = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwV3FIFOThresholdTune = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwMCRacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSLICERacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwCheckHQVFIFOEmpty = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwUseMPEGAGP = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwV3FIFOPatch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSupportTwoColorKey = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dw205BFIFO64= VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportVideoVQ = VID_HWDIFF_FALSE;
    	    lpVideoHWDifference->dwOnlyOneVideoOverlay = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwAGPSupportHeader7 = VID_HWDIFF_FALSE;				
            break;

        case UNICHROME_VT3123CX:
            /* HW Difference Flag */
            lpVideoHWDifference->dwThreeHQVBuffer = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3SrcHeightSetting = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSupportExtendFIFO = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwHQVFetchByteUnit = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVInitPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportV3Gamma=VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwUpdFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVDisablePatch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSUBFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwNeedV1Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV3Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV4Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwUseSystemMemory = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwExpandVerPatch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwExpandVerHorPatch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3ExpireNumTune = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwV3FIFOThresholdTune = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwMCRacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSLICERacingPatch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwCheckHQVFIFOEmpty = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUseMPEGAGP = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwV3FIFOPatch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSupportTwoColorKey = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dw205BFIFO64= VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportVideoVQ = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwOnlyOneVideoOverlay = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwAGPSupportHeader7 = VID_HWDIFF_FALSE;						
            break;

        case UNICHROME_VT3205:
            /* HW Difference Flag */
            lpVideoHWDifference->dwThreeHQVBuffer = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3SrcHeightSetting = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSupportExtendFIFO = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwHQVFetchByteUnit = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVInitPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportV3Gamma=VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwUpdFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVDisablePatch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSUBFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwNeedV1Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV3Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV4Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwUseSystemMemory = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwExpandVerPatch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwExpandVerHorPatch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3ExpireNumTune = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3FIFOThresholdTune = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwMCRacingPatch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSLICERacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwCheckHQVFIFOEmpty = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwUseMPEGAGP = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3FIFOPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportTwoColorKey = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dw205BFIFO64= VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportVideoVQ = VID_HWDIFF_FALSE;
	        lpVideoHWDifference->dwOnlyOneVideoOverlay = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwAGPSupportHeader7 = VID_HWDIFF_FALSE;				 
            break;

        case UNICHROME_VT3204:
            /* HW Difference Flag */
            lpVideoHWDifference->dwThreeHQVBuffer = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3SrcHeightSetting = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSupportExtendFIFO = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwHQVFetchByteUnit = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVInitPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportV3Gamma=VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUpdFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVDisablePatch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSUBFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwNeedV1Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV3Prefetch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwNeedV4Prefetch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUseSystemMemory = VID_HWDIFF_TRUE;/* VID_HWDIFF_FALSE; for performance tuning */
            lpVideoHWDifference->dwExpandVerPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwExpandVerHorPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwV3ExpireNumTune = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3FIFOThresholdTune = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwMCRacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSLICERacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwCheckHQVFIFOEmpty = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUseMPEGAGP = VID_HWDIFF_FALSE;/* VID_HWDIFF_TRUE; for FPGA PCI test */
            lpVideoHWDifference->dwV3FIFOPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportTwoColorKey = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dw205BFIFO64= VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportVideoVQ = VID_HWDIFF_FALSE;
	        lpVideoHWDifference->dwOnlyOneVideoOverlay = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwAGPSupportHeader7 = VID_HWDIFF_FALSE;						
            break;
            
        case UNICHROME_VT3336:
            /* HW Difference Flag */
            lpVideoHWDifference->dwThreeHQVBuffer = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3SrcHeightSetting = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSupportExtendFIFO = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwHQVFetchByteUnit = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVInitPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportV3Gamma=VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUpdFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVDisablePatch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSUBFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwNeedV1Prefetch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwNeedV3Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV4Prefetch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUseSystemMemory = VID_HWDIFF_TRUE;/* VID_HWDIFF_FALSE; for performance tuning */
            lpVideoHWDifference->dwExpandVerPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwExpandVerHorPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwBypassHorExpandMagnify = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwBypassVerExpandMagnify = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3ExpireNumTune = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3FIFOThresholdTune = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwMCRacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSLICERacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwCheckHQVFIFOEmpty = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUseMPEGAGP = VID_HWDIFF_FALSE;/* VID_HWDIFF_TRUE; for FPGA PCI test */
            lpVideoHWDifference->dwV3FIFOPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportTwoColorKey = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dw205BFIFO64= VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportVideoVQ = VID_HWDIFF_FALSE;
	        lpVideoHWDifference->dwOnlyOneVideoOverlay = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwAGPSupportHeader7 = VID_HWDIFF_FALSE;						
            break;
            
        case UNICHROME_VT3327:		/*P4M890*/
        case UNICHROME_VT3364:       /*P4M900*/    
            lpVideoHWDifference->dwThreeHQVBuffer = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3SrcHeightSetting = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSupportExtendFIFO = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwHQVFetchByteUnit = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVInitPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportV3Gamma=VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUpdFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVDisablePatch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSUBFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwNeedV1Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV3Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV4Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwUseSystemMemory = VID_HWDIFF_TRUE;/* VID_HWDIFF_FALSE; for performance tuning */
            lpVideoHWDifference->dwExpandVerPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwExpandVerHorPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwBypassVerExpandMagnify = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3ExpireNumTune = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3FIFOThresholdTune = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwMCRacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSLICERacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwCheckHQVFIFOEmpty = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUseMPEGAGP = VID_HWDIFF_FALSE;/* VID_HWDIFF_TRUE; for FPGA PCI test */
            lpVideoHWDifference->dwV3FIFOPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportTwoColorKey = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dw205BFIFO64= VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportVideoVQ = VID_HWDIFF_FALSE;
	        lpVideoHWDifference->dwOnlyOneVideoOverlay = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwAGPSupportHeader7 = VID_HWDIFF_FALSE;	
            break;

        case UNICHROME_VT3314:		/*CN700*/
            lpVideoHWDifference->dwThreeHQVBuffer = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3SrcHeightSetting = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSupportExtendFIFO = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwHQVFetchByteUnit = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVInitPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportV3Gamma=VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUpdFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVDisablePatch = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSUBFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwNeedV1Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV3Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV4Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwUseSystemMemory = VID_HWDIFF_TRUE;/* VID_HWDIFF_FALSE; for performance tuning */
            lpVideoHWDifference->dwExpandVerPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwExpandVerHorPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwV3ExpireNumTune = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3FIFOThresholdTune = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwMCRacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSLICERacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwCheckHQVFIFOEmpty = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUseMPEGAGP = VID_HWDIFF_FALSE;/* VID_HWDIFF_TRUE; for FPGA PCI test */
            lpVideoHWDifference->dwV3FIFOPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportTwoColorKey = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dw205BFIFO64= VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportVideoVQ = VID_HWDIFF_FALSE;
    	    lpVideoHWDifference->dwOnlyOneVideoOverlay = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwAGPSupportHeader7 = VID_HWDIFF_FALSE;						
            break;

        case UNICHROME_VT3259:
            /* HW Difference Flag */
            lpVideoHWDifference->dwThreeHQVBuffer = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3SrcHeightSetting = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSupportExtendFIFO = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwHQVFetchByteUnit = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVInitPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportV3Gamma=VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUpdFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVDisablePatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSUBFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwNeedV1Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV3Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV4Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwUseSystemMemory = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwExpandVerPatch =  VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwExpandVerHorPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwExpandSAMMHorPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwBypassHorExpandMagnify = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwBypassVerExpandMagnify = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3ExpireNumTune = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwV3FIFOThresholdTune = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwMCRacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSLICERacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwCheckHQVFIFOEmpty = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwUseMPEGAGP = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwV3FIFOPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportTwoColorKey = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwEnableErrorConcealment = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUseMMX = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dw205BFIFO64= VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupport512MAddr = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwHQVOutputFIFO32Level = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwVideoFlipOddEvenBit31 = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportVideoVQ = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwOnlyOneVideoOverlay = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwAGPSupportHeader7 = VID_HWDIFF_TRUE;				
            break;
        case UNICHROME_VT3259Lite:
            /* HW Difference Flag */
            lpVideoHWDifference->dwThreeHQVBuffer = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3SrcHeightSetting = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSupportExtendFIFO = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwHQVFetchByteUnit = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVInitPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportV3Gamma=VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUpdFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVDisablePatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSUBFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwNeedV1Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV3Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV4Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwUseSystemMemory = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwExpandVerPatch =  VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwExpandVerHorPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwExpandSAMMHorPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwBypassHorExpandMagnify = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwBypassVerExpandMagnify = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3ExpireNumTune = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwV3FIFOThresholdTune = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwMCRacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSLICERacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwCheckHQVFIFOEmpty = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwUseMPEGAGP = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwV3FIFOPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportTwoColorKey = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwEnableErrorConcealment = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUseMMX = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dw205BFIFO64= VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupport512MAddr = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwHQVOutputFIFO32Level = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwVideoFlipOddEvenBit31 = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportVideoVQ = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwOnlyOneVideoOverlay = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwAGPSupportHeader7 = VID_HWDIFF_TRUE;					
            break;

        case UNICHROME_VT3324:
        case UNICHROME_VT3324M:
        case UNICHROME_VT3324M2:
        case UNICHROME_VT3293:    
            /* HW Difference Flag */
            lpVideoHWDifference->dwThreeHQVBuffer = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3SrcHeightSetting = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSupportExtendFIFO = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwHQVFetchByteUnit = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVInitPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportV3Gamma=VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUpdFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVDisablePatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSUBFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwNeedV1Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV3Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV4Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwUseSystemMemory = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwExpandVerPatch =  VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwExpandVerHorPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwExpandSAMMHorPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwBypassHorExpandMagnify = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwBypassVerExpandMagnify = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3ExpireNumTune = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwV3FIFOThresholdTune = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwMCRacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSLICERacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwCheckHQVFIFOEmpty = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwUseMPEGAGP = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwV3FIFOPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportTwoColorKey = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwEnableErrorConcealment = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUseMMX = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dw205BFIFO64= VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupport512MAddr = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwHQVOutputFIFO32Level = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwVideoFlipOddEvenBit31 = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportVideoVQ = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwOnlyOneVideoOverlay = VID_HWDIFF_FALSE;            
            lpVideoHWDifference->dwAGPSupportHeader7 = VID_HWDIFF_FALSE;
            break;
		case UNICHROME_VT3353:
            /* HW Difference Flag */
            lpVideoHWDifference->dwThreeHQVBuffer = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3SrcHeightSetting = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwSupportExtendFIFO = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwHQVFetchByteUnit = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVInitPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportV3Gamma=VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUpdFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwHQVDisablePatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSUBFlip = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwNeedV1Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV3Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwNeedV4Prefetch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwUseSystemMemory = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwExpandVerPatch =  VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwExpandVerHorPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwExpandSAMMHorPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwBypassHorExpandMagnify = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwBypassVerExpandMagnify = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwV3ExpireNumTune = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwV3FIFOThresholdTune = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwMCRacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSLICERacingPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwCheckHQVFIFOEmpty = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwUseMPEGAGP = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwV3FIFOPatch = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportTwoColorKey = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwEnableErrorConcealment = VID_HWDIFF_TRUE;
            lpVideoHWDifference->dwUseMMX = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dw205BFIFO64= VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupport512MAddr = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwHQVOutputFIFO32Level = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwVideoFlipOddEvenBit31 = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwSupportVideoVQ = VID_HWDIFF_FALSE;
            lpVideoHWDifference->dwOnlyOneVideoOverlay = VID_HWDIFF_FALSE;            
            lpVideoHWDifference->dwAGPSupportHeader7 = VID_HWDIFF_FALSE;
            break;
        default:
            /*Unkown DeviceID*/
            DBG_DD(("VidInit.c: vfInitHWDiff: Unknown dwProjectIDRevision!!\n"));
            break;
    }/* End of dwProjectIDRevision */
}

/* vfInitVidFlag_CME */
void vfInitVidFlag_CME(PVIDDATA pVidData)
{
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;
    unsigned long*   lpdwCtlHQVflag   = &pVidData->VidCtl.dwCtlHQVflag;
    unsigned long*   lpdwCtlMPEGFlag;
    int i;

    *lpdwCtlHQVflag = 0;

    /* For Deblocking */
    if (lpVIAGraphicInfo->DeBlockingEnable)
    	*lpdwCtlHQVflag |= CTL_HQV_DEBLOCKING_ENABLE;

    /* For 3:2 pull down option */
    if (!(lpVIAGraphicInfo->dw3to2PullDownDisable))
    	*lpdwCtlHQVflag |= CTL_HQV_3to2PULLDOWN_ENABLE_CME;

    /* HQV De-interlaced control */
    *lpdwCtlHQVflag |= CTL_HQV_DEINTERLACE_ENABLE_CME;

    for (i=0;i<2;i++)
    {
        /* Set MPEG tile address mode */
        lpdwCtlMPEGFlag = &pVidData->VidCtl.dwCtlMPEGFlag[i];
        *lpdwCtlMPEGFlag = 0;

        *lpdwCtlMPEGFlag |= CTL_MPG_RESET_ENABLE_CME;
    }
}

void vfEnable2DVQ(PVIDDATA pVidData)
{
    LPVIDEOHWINFO lphwinfo  = &pVidData->Vhwinfo;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;
    unsigned long   dwStartAddr=0, dwSize=0, dwCursorSize;
    unsigned long   dwLen, dwstL, dwendL, dwst_endH;
    int ScrnIndex = 0;
    unsigned char* pjVideo = pVidData->Vhwinfo.pjVideo;

    lphwinfo->dw2DVQUseCount++;
    
    if ( lphwinfo->dw2DVQUseCount == 1 )
    {
        dwSize   = 0x00040000;   /* 256K */
        
        /* Alloc more 8K for Hardware Cursor Use, for viafb driver and accel enable case. */
        dwCursorSize = 0x00002000;  /* 8K */
        
        if((lpVIAGraphicInfo->Screen[ScrnIndex].VideoHeapEnd-lpVIAGraphicInfo->Screen[ScrnIndex].VideoHeapBase)>=(dwSize+dwCursorSize))
        {
            dwStartAddr = lpVIAGraphicInfo->Screen[ScrnIndex].VideoHeapEnd - dwSize- dwCursorSize; 
            lpVIAGraphicInfo->Screen[ScrnIndex].VideoHeapEnd -= (dwSize + dwCursorSize) ;
        }    

        if (!dwStartAddr)
        {
            DBG_DD(("EnableVQ: Create Virtual Queue buffer fail!\n"));
        }

        DBG_DD(("EnableVirtualQueue()\n"));

        switch( lpVIAGraphicInfo->dwDeviceID)
        {
            case VIA_DEVICE_VT3204:
                break;

            case VIA_DEVICE_VT3336:
            case VIA_DEVICE_VT3364:                
                /* Set Video VQ start address Lower 3 bytes [23:0]. Reg:0x70 */
                dwstL  = 0x70000000 |(dwStartAddr & 0x00FFFFFF) ;
                /* Set Video VQ end address Lower 3 bytes [23:0]. Reg:0x71 */
                dwendL = 0x71000000|((dwStartAddr + dwSize -1) & 0x00FFFFFF) ;
                /* Set Video VQ start and end addresses Highest byte [31:24]. Reg:0x72 */
                dwst_endH = 0x72000000 | (dwStartAddr & 0xFF000000)>>24|
                            (((dwStartAddr + dwSize - 1) & 0xFF000000) >>16) ;

                /* Set Video VQ size. Reg:0x73 */
                dwLen = 0x73000000 | ((unsigned long) (dwSize>>3) & 0x00FFFFFF) ;

                dMMWR(pjVideo,0x41c, 0x00100000);
                dMMWR(pjVideo,0x420, dwst_endH);
                dMMWR(pjVideo,0x420, dwstL);
                dMMWR(pjVideo,0x420, dwendL);
                dMMWR(pjVideo,0x420, dwLen);
                dMMWR(pjVideo,0x420, 0x74301001);
                dMMWR(pjVideo,0x420, 0x00000000);
                break;
                
            default:
                /* Set Video VQ start address Lower 3 bytes [23:0]. Reg:0x50 */
                dwstL  = 0x50000000 |(dwStartAddr & 0x00FFFFFF) ;
                /* Set Video VQ end address Lower 3 bytes [23:0]. Reg:0x51 */
                dwendL = 0x51000000|((dwStartAddr + dwSize - 1 ) & 0x00FFFFFF) ;
                /* Set Video VQ start and end addresses Highest byte [31:24]. Reg:0x52 */
                dwst_endH = 0x52000000 | (dwStartAddr & 0xFF000000)>>24|
                            (((dwStartAddr + dwSize - 1) & 0xFF000000) >>16) ;

                /* Set Video VQ size. Reg:0x53 */
                dwLen = 0x53000000 | ((unsigned long) (dwSize>>3) & 0x00FFFFFF) ;

                /* Set I/O address to "FE" 0x00fe0000; */
                dMMWR(pjVideo,GE_TRANS_SET, 0x00fe0000);
                dMMWR(pjVideo,GE_Hpara0, 0x00000006);
                dMMWR(pjVideo,GE_Hpara0, 0x40008C0F);
                dMMWR(pjVideo,GE_Hpara0, 0x44000000);
                dMMWR(pjVideo,GE_Hpara0, 0x45080C04);
                dMMWR(pjVideo,GE_Hpara0, 0x46800408);
                
                dMMWR(pjVideo,GE_Hpara0, dwendL);   /* Reg:0x51 */
                DBG_DD(("%08x %08lx\n",GE_Hpara0,dwendL));
                dMMWR(pjVideo,GE_Hpara0, dwst_endH);/* Reg:0x52 */
                DBG_DD(("%08x %08lx\n",GE_Hpara0,dwst_endH));
                dMMWR(pjVideo,GE_Hpara0, dwstL );   /* Reg:0x50 */
                DBG_DD(("%08x %08lx\n",GE_Hpara0,dwstL));
                dMMWR(pjVideo,GE_Hpara0, dwLen);    /* Reg:0x53 */
                DBG_DD(("%08x %08lx\n",GE_Hpara0,dwLen));
                break;
        }
    }
    else
    {
        DBG_DD(("EnableVQ: Virtual Queue buffer has been enabled\n"));
    }
}

void vfDisable2DVQ(PVIDDATA pVidData)
{
    LPVIDEOHWINFO lphwinfo  = &pVidData->Vhwinfo;
    unsigned char* pjVideo = pVidData->Vhwinfo.pjVideo;

    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;

    lphwinfo->dw2DVQUseCount--;

    if ( lphwinfo->dw2DVQUseCount == 0 )
    {
        vfHM(pVidData, HM_WAIT_VQ_EMPTY);

        /* Diable VQ */
        switch( lpVIAGraphicInfo->dwDeviceID)
        {
            case VIA_DEVICE_VT3204:
                break;
                
            case VIA_DEVICE_VT3336: 
            case VIA_DEVICE_VT3364:                 
                dMMWR(pjVideo,0x41c, 0x00100000);
                dMMWR(pjVideo,0x420, 0x74301000);
                break;
                
            default:
                dMMWR(pjVideo,GE_TRANS_SET, 0x00fe0000);
                dMMWR(pjVideo,GE_Hpara0, 0x00000004);
                dMMWR(pjVideo,GE_Hpara0, 0x40008c0f);
                dMMWR(pjVideo,GE_Hpara0, 0x44000000);
                dMMWR(pjVideo,GE_Hpara0, 0x45080c04);
                dMMWR(pjVideo,GE_Hpara0, 0x46800408);
                break;
        }
    }
}

void vfEnableVideoVirtualQueue(unsigned long dwVideoFlag,unsigned char* pjVideo,unsigned long dwVideoVQ_Offset,unsigned long dwVQ_Size)
{
    unsigned long   dwLen, dwstL, dwendL, dwst_endH;
    unsigned long   dwSetting;

    DBG_DD(("vfEnableVideoVirtualQueue()\n"));

    /* Set I/O address to "FE" 0x00fe0000; */
    dMMWR(pjVideo, GE_TRANS_SET, 0x00fe0000);

    /* Set Video VQ start address Lower 3 bytes [23:0]. Reg:0x60 */
    dwstL  = HVCMDQstL_Header |(dwVideoVQ_Offset & 0x00FFFFFF) ;
    /* Set Video VQ end address Lower 3 bytes [23:0]. Reg:0x61 */
    dwendL = HVCMDQendL_Header|((dwVideoVQ_Offset + dwVQ_Size - 8 ) & 0x00FFFFFF) ;
    /* Set Video VQ start and end addresses Highest byte [31:24]. Reg:0x62 */
    dwst_endH = HVCMDQstendH_Header | (dwVideoVQ_Offset & 0xFF000000)>>24|
                (((dwVideoVQ_Offset + dwVQ_Size - 8) & 0xFF000000) >>16) ;

    /* Set Video VQ size. Reg:0x63 */
    dwLen = HVCMDQLen_Header | ((unsigned long) (dwVQ_Size>>3) & 0x00FFFFFF) ;
    /* Set Video VQ enable setting. Reg:0x64 */
    if (dwVideoFlag&VIDEO_AGP_INUSE)
    {
        dwSetting = HVCMDQSetting_Header|0x001010C4; /* AGP */
    }
    else
    {
        dwSetting = HVCMDQSetting_Header|0x001010C0|HW_ENABLE_VIDEO_VQ; /* PCI */
    }

    dMMWR(pjVideo,GE_Hpara0, dwendL);   /* Reg:0x61 */
    DBG_DD(("%08x %08lx\n",GE_Hpara0,dwendL));
    dMMWR(pjVideo,GE_Hpara0, dwst_endH);/* Reg:0x62 */
    DBG_DD(("%08x %08lx\n",GE_Hpara0,dwst_endH));
    dMMWR(pjVideo,GE_Hpara0, dwstL );   /* Reg:0x60 */
    DBG_DD(("%08x %08lx\n",GE_Hpara0,dwstL));
    dMMWR(pjVideo,GE_Hpara0, dwLen);    /* Reg:0x63 */
    DBG_DD(("%08x %08lx\n",GE_Hpara0,dwLen));

    dMMWR(pjVideo, GE_TRANS_SET, 0x00fe0000);
    DBG_DD(("%08x %08x\n",GE_TRANS_SET,0x00fe0000));
    /* 0x64 */
    dMMWR(pjVideo,GE_Hpara0, dwSetting);
    DBG_DD(("%08x %08lx\n",GE_Hpara0,dwSetting));
}

void vfDisableVideoVirtualQueue(unsigned char* pjVideo)
{
    unsigned long   dwSetting=0;

    DBG_DD(("vfDisableVideoVirtualQueue()\n"));
    dwSetting = HVCMDQSetting_Header|0x00001000;

    /* 0x43c */
    dMMWR(pjVideo, GE_TRANS_SET, 0x00fe0000);
    /* 0x64 */
    dMMWR(pjVideo,GE_Hpara0, dwSetting);

}

void vfEnableVQ_CME(PVIDDATA pVidData)
{
    unsigned long handle=pVidData->VMI_Info.handle;
    unsigned long dwVideoFlag       = pVidData->VidCtl.dwVideoFlag[handle];
    LPVIDEOHWINFO lphwinfo  = &pVidData->Vhwinfo;
    LPVIDEOMEMORY   lpVidMemory = &VidMemory;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;

    lphwinfo->dwVQUseCount++;

    /* Double check VQ Enable/Disable status */
    if ( (!lphwinfo->dwVideoVQ_Offset) )
    {
        lphwinfo->dwVideoVQ_Size   = 0x00080000;   /* 512K */

        if (!lpVIAGraphicInfo->DRMEnabled)
        {
            lpVidMemory->VQMemRequest.checkID = VIA_MID;
            lpVidMemory->VQMemRequest.size    = lphwinfo->dwVideoVQ_Size;
            lpVidMemory->VQMemRequest.type    = VIDMGR_TYPE_VIDEO;
            if ( MEM_OK != viaMgr_vid_ioctl(pVidData, VIAMGR_ALLOC_VIDEO, (char *)(&lpVidMemory->VQMemRequest) ) )
            {
                DBG_DD((" Memory Allocation fail for VQ buffer.\n"));
            }
            lphwinfo->dwVideoVQ_Offset = ALIGN_TO_32_BYTE(lpVidMemory->VQMemRequest.offset);
        }

        if (!lphwinfo->dwVideoVQ_Offset)
        {
            DBG_DD(("vfHQVinit_CME: Create Virtual Queue buffer fail!\n"));
        }

        vfEnableVideoVirtualQueue(dwVideoFlag,
                                  lphwinfo->pjVideo,
                                  lphwinfo->dwVideoVQ_Offset,
                                  lphwinfo->dwVideoVQ_Size);
    }
    else
    {
        DBG_DD(("vfHQVinit_CME: Virtual Queue buffer has been enabled\n"));
        /* copy VQ memory info*/
        lphwinfo->dwVideoVQ_Size   = 0x00080000;   /* 512K */
        if (!lpVIAGraphicInfo->DRMEnabled)
        {
            lpVidMemory->VQMemRequest.checkID = VIA_MID;
            lpVidMemory->VQMemRequest.size    = lphwinfo->dwVideoVQ_Size;
            lpVidMemory->VQMemRequest.type    = VIDMGR_TYPE_VIDEO;
            lpVidMemory->VQMemRequest.offset  = lphwinfo->dwVideoVQ_Offset;
        }
    }
}

void vfDisableVQ_CME(PVIDDATA pVidData)
{
    LPVIDEOHWINFO lphwinfo  = &pVidData->Vhwinfo;
    unsigned char* pjVideo = pVidData->Vhwinfo.pjVideo;

    LPVIDEOMEMORY   lpVidMemory = &VidMemory;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;

    lphwinfo->dwVQUseCount--;

    if ( lphwinfo->dwVQUseCount == 0 )
    {
        if (lphwinfo->dwVideoVQ_Offset)
        {
            vfHM(pVidData,HM_WAIT_VQ_EMPTY);

            if (!lpVIAGraphicInfo->DRMEnabled)
            {
                if (lpVidMemory->VQMemRequest.offset != 0)
                {
                    if ( MEM_OK != viaMgr_vid_ioctl(pVidData, VIAMGR_FREE_VIDEO, (char *)(&lpVidMemory->VQMemRequest) )  )
                    {
                        DBG_DD((" Memory Free fail for VQ buffer \n"));
                    }
                    lpVidMemory->VQMemRequest.offset = 0;
                }
                DBG_DD((" Memory Free for VQ buffer\n"));
            }

            pVidData->Vhwinfo.dwVideoVQ_Offset = 0;
            vfDisableVideoVirtualQueue(pjVideo);
        }
    }
    else
    {
        DBG_DD(("VQ buffer was in used\n"));
    }
}


void vfInitVideoInfo(PVIDDATA pVidData)
{
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;

    pVidData->VDevInfo.dwDeviceID = lpVIAGraphicInfo->dwDeviceID;
    pVidData->VDevInfo.dwRevisionID = lpVIAGraphicInfo->dwRevisionID;

    if (lpVIAGraphicInfo->Screen[lpVIAGraphicInfo->scrnIndex].IsDownScaling)
    {
        lpVIAGraphicInfo->VideoColorKey = 0x0821;
    }

    switch ( lpVIAGraphicInfo->dwDeviceID )
    {
        case VIA_DEVICE_CLE1:
        case VIA_DEVICE_CLE2:
            vfInitVidREGRec_CLE();
            break;
        case VIA_DEVICE_VT3204:
        case VIA_DEVICE_VT3205:
        case VIA_DEVICE_VT3314:
        case VIA_DEVICE_VT3336:
        case VIA_DEVICE_VT3327:
        case VIA_DEVICE_VT3364:    
            break;

        case VIA_DEVICE_VT3259:
        case VIA_DEVICE_VT3324:
        case VIA_DEVICE_VT3293:    
		case VIA_DEVICE_VT3353:
        default:
            vfInitVidFlag_CME(pVidData);
            break;
    }

    /* Init VidHWDifference structure
     *  for chip specific function.
     */
    vfInitHWDiff(pVidData);

    /* Map Video function */
    if (!vfMapVideoFun(pVidData))
    {
        DBG_DD(("vfInitVideoInfo: MAP MPEG Function Fail"));
    }
}

/*  MPEGFlipCurrentFrame()
 *  purpose : if MPEG engine not in flip, do MPEG HW flip once
 */
void MPEGFlipCurrentFrame(PVIDDATA pVidData)
{
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;
    unsigned long handle=pVidData->VMI_Info.handle;
    LPMPGDEVICE lpMPGDevice = &(pVidData->MPGDevice[handle]);
    unsigned long dwMpeg;
    unsigned char* pjVideo = pVidData->Vhwinfo.pjVideo;

    DBG_DD((" FB index: %ld\n", lpMPGDevice->dwMPEGDisplayIndex));
    dwMpeg = VIDInD(0xc00);

    switch ( lpVIAGraphicInfo->dwDeviceID )
    {
        case VIA_DEVICE_CLE1:
        case VIA_DEVICE_CLE2:
            switch ( DispatchVGARevisionID(pVidData) )
            {
                case VIA_REVISION_CLEAX : /* For CLE_Ax */
                    if ((lpMPGDevice->VIAMPGSurface.dwDisplayPictStruct == VIA_PICT_STRUCT_FRAME)&&
                        (lpMPGDevice->VIAMPGSurface.dwDeinterlaceMode == VIA_DEINTERLACE_WEAVE))
                    {
                        VIDOutD( 0xc00, (0x8c|lpMPGDevice->dwMPEGDisplayIndex));
                    }
                    else
                    {
                        VIDOutD( 0xc00, (0x84|lpMPGDevice->dwMPEGDisplayIndex));
                    }
                    break;

                case VIA_REVISION_CLECX : /* For CLE_Cx */
                default : /* For future chips */
                    if ((lpMPGDevice->VIAMPGSurface.dwDisplayPictStruct == VIA_PICT_STRUCT_FRAME)&&
                        (lpMPGDevice->VIAMPGSurface.dwDeinterlaceMode == VIA_DEINTERLACE_WEAVE))
                    {
                        VIDOutD( 0xc00, (0x80 | lpMPGDevice->VIAMPGSurface.dwDisplayPictStruct << 16
                                         | lpMPGDevice->dwMPEGDisplayIndex << 12));
                    }
                    else
                    {
                        VIDOutD( 0xc00, (0x80 | lpMPGDevice->VIAMPGSurface.dwDisplayPictStruct << 16
                                         | lpMPGDevice->dwMPEGDisplayIndex << 12));
                    }
                    break;
            }
            break;

        default:
            if ((lpMPGDevice->VIAMPGSurface.dwDisplayPictStruct == VIA_PICT_STRUCT_FRAME)&&
                (lpMPGDevice->VIAMPGSurface.dwDeinterlaceMode == VIA_DEINTERLACE_WEAVE))
            {
                VIDOutD( 0xc00, (0x80 | lpMPGDevice->VIAMPGSurface.dwDisplayPictStruct << 16
                                 | lpMPGDevice->dwMPEGDisplayIndex << 12));
            }
            else
            {
                VIDOutD( 0xc00, (0x80 | lpMPGDevice->VIAMPGSurface.dwDisplayPictStruct << 16
                                 | lpMPGDevice->dwMPEGDisplayIndex << 12));
            }
            break;
    }

    if ((dwMpeg&0x80)!=0x80)
    {
        VIDOutD( 0xc00, dwMpeg);
    }
}

