 /*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

 

/* I N C L U D E S ---------------------------------------------------------- */
#include "via.h"
#include "viavid.h"
#include "regrec.h"
#include "ddmpeg.h"
#include "VidInit.h"
#include "VidStruc.h"   /* VIDDATA, video top level structure */
#include "debug.h"      /* for DBG_DD */
#include "engmgr.h"

/* E X T E R N   G L O B A L S ---------------------------------------------- */
/*extern PVIDDATA pVidData;*/

/* G L O B A L   V A R I A B L E S ------------------------------------------ */
static CLIENTINFO client_info[CLIENTMAX];
static unsigned long eng_owner[ENGINETYPE][ENG_ID_MAX];

/* F U N C T I O N ---------------------------------------------------------- */

static unsigned long FindMax(CLIENTINFO *pClient_info,unsigned long *pMax_idx)
{
	unsigned long i;
	unsigned long ret=ENGMGR_FAIL;
	DBG_DD(("\n")); 

    for(i=0;i<CLIENTMAX;i++)
    {
		if(pClient_info[i].pFunc !=0)
		{
			if(ret==ENGMGR_FAIL)
			{
				*pMax_idx=i;
				ret=ENGMGR_SUCCESS;

			}else if(pClient_info[i].uPriority>pClient_info[*pMax_idx].uPriority)
			{
				*pMax_idx=i;
			}
		}
	}
	return ret;
}

static unsigned long SortAllocateClient(unsigned long exclude_client,unsigned long *pClients)
{
	unsigned long cnt=0;
	unsigned long i,val=0;
    CLIENTINFO tmp_client_info[CLIENTMAX];
    DBG_DD(("\n")); 

	memcpy(tmp_client_info,client_info,sizeof(CLIENTINFO)*CLIENTMAX);

    for(i=0;i<CLIENTMAX;i++)
    {
		if(ENGMGR_FAIL==FindMax(&tmp_client_info[0],&val)) break;
        tmp_client_info[val].pFunc =0;
		*(pClients+cnt)=val;
		cnt++;
    }
	return cnt;
}

static void Notify_Client_To_Allocate(PVIDDATA pVidData, unsigned long exclude_client,unsigned long uEng_type,unsigned long uEng_id)
{
	unsigned long i;
	EngMgrCallBack *func;
	unsigned long clients[CLIENTMAX];
	unsigned long client_num;
	Call_Back_Para para;
	DBG_DD(("\n")); 

	client_num=SortAllocateClient(exclude_client,&clients[0]); /*dependent on client uPriority*/	
	for(i=0;i<client_num;i++)
	{
		if(clients[i]!=exclude_client)	
		{		
			para.uMessage_Type = ENGMGR_ALLOCATE_ENG_MSG;
			para.uClient_handle =clients[i];
			para.uSrcClient_Priority =client_info[exclude_client].uPriority;
 			para.uEng_Type =uEng_type;
			para.uEng_Id0 =uEng_id;	
			func=client_info[clients[i]].pFunc ;
			if(func(pVidData, &para)==ENGMGR_YES)
			{
				eng_owner[uEng_type][uEng_id]=clients[i];		
				break;
			}
		}
	}
	return;
}

static unsigned long Notify_Client_To_Free(PVIDDATA pVidData, unsigned long exclude_client,unsigned long uEng_type,unsigned long uEng_id,unsigned long *pEng_id_available)
{
	unsigned long ret=ENGMGR_FAIL;
	unsigned long i;
	EngMgrCallBack *func;
	Call_Back_Para para;
    DBG_DD(("\n")); 

    if(uEng_id==ANY_ENGINE_ID)
	{
		for(i=0;i<ENG_ID_MAX;i++)
		{
			if((eng_owner[uEng_type][i]!=ENGINE_MGR)&&(eng_owner[uEng_type][i]!=ENGINE_NOT_EXIST))
			{
				if(eng_owner[uEng_type][i]!=exclude_client)	
				{	
					para.uMessage_Type = ENGMGR_FREE_ENG_MSG;
					para.uClient_handle =eng_owner[uEng_type][i];
					para.uSrcClient_Priority =client_info[exclude_client].uPriority;
 					para.uEng_Type =uEng_type;
					para.uEng_Id0 =i;	
					func=client_info[eng_owner[uEng_type][i]].pFunc ;
					if(func(pVidData, &para)==ENGMGR_YES)
					{
						eng_owner[uEng_type][i]=exclude_client;
						ret=ENGMGR_SUCCESS;
     					break;
					}
				}
			}
		}
	}else
	{
		if((eng_owner[uEng_type][uEng_id]!=ENGINE_MGR)&&(eng_owner[uEng_type][uEng_id]!=ENGINE_NOT_EXIST))
		{
			if(eng_owner[uEng_type][uEng_id]!=exclude_client)	
			{	
				para.uMessage_Type = ENGMGR_FREE_ENG_MSG;
				para.uClient_handle =eng_owner[uEng_type][uEng_id];
				para.uSrcClient_Priority =client_info[exclude_client].uPriority;
 				para.uEng_Type  =uEng_type;
				para.uEng_Id0 =uEng_id;	
				func=client_info[eng_owner[uEng_type][uEng_id]].pFunc ;
				if(func(pVidData, &para)==ENGMGR_YES)
				{
					eng_owner[uEng_type][uEng_id]=exclude_client;
					*pEng_id_available=uEng_id;
					ret=ENGMGR_SUCCESS;
				}
			}
		}
	}
	return ret;
}

static unsigned long Notify_Client_To_Swap(PVIDDATA pVidData, unsigned long exclude_client,unsigned long uEng_type,unsigned long uEng_id_old,unsigned long uEng_id_new)
{
	unsigned long ret=ENGMGR_FAIL;
	EngMgrCallBack *func;
	unsigned long tmp;
	Call_Back_Para para;
	DBG_DD(("\n")); 

	if((eng_owner[uEng_type][uEng_id_new]!=ENGINE_MGR)&&(eng_owner[uEng_type][uEng_id_new]!=ENGINE_NOT_EXIST))
	{
	    if(eng_owner[uEng_type][uEng_id_new]!=exclude_client)
		{	
			para.uMessage_Type = ENGMGR_SWAP_ENG_MSG;
			para.uClient_handle =eng_owner[uEng_type][uEng_id_new];
			para.uSrcClient_Priority =client_info[exclude_client].uPriority;
 			para.uEng_Type = uEng_type;
			para.uEng_Id0 = uEng_id_old;	
			para.uEng_Id1 = uEng_id_new;
			func=client_info[eng_owner[uEng_type][uEng_id_new]].pFunc ;
			if(func(pVidData, &para)==ENGMGR_YES)
			{
				tmp=eng_owner[uEng_type][uEng_id_new];
				eng_owner[uEng_type][uEng_id_new]=exclude_client;
				eng_owner[uEng_type][uEng_id_old]=tmp;
				ret=ENGMGR_SUCCESS;
			}
		}
	}
	return ret;
}


static unsigned long Check_Available_Engine_ID(unsigned long uEng_type,unsigned long uEng_id,unsigned long *pEng_id_available)
{
	unsigned long i;
	unsigned long ret=ENGMGR_FAIL;
	DBG_DD(("\n")); 
	
	if(uEng_id==ANY_ENGINE_ID)
	{
		for(i=0;i<ENG_ID_MAX;i++)
		{
			if(eng_owner[uEng_type][i]==ENGINE_MGR)
			{
			   *pEng_id_available=i;
			   ret=ENGMGR_SUCCESS;
			   break;
			}
		}
	}else
	{
		if(eng_owner[uEng_type][uEng_id]==ENGINE_MGR)
		{
		   *pEng_id_available=uEng_id;
		   ret=ENGMGR_SUCCESS;
		}
	}
	return ret;
}


void EngMgr_Construct(void)
{
    unsigned long i,j;
    DBG_DD(("\n")); 
   
    for(i=0;i<CLIENTMAX;i++)
    {
	    client_info[i].pFunc=0;
    }
    for(i=0;i<ENGINETYPE;i++)
    {
	    for(j=0;j<ENG_ID_MAX;j++)
	    {
	        eng_owner[i][j]=ENGINE_NOT_EXIST;
	   }
   }
   /*The managed engines is as follows*/
   
   eng_owner[VID_ENGINE][0]=ENGINE_MGR;    /*V1: CLE266,VT3259*/
   eng_owner[VID_ENGINE][1]=ENGINE_MGR;    /*V3: CLE266,VT3205,VT3204,VT3259 */
   eng_owner[HQV_ENGINE][0]=ENGINE_MGR;    /*1st HQV: CLE266,VT3205,VT3204,VT3259*/
   eng_owner[HQV_ENGINE][1]=ENGINE_MGR;    /*2nd HQV: VT3259*/
   eng_owner[CAP_ENGINE][0]=ENGINE_MGR;    /*Capture0: CLE266,VT3259*/
   eng_owner[CAP_ENGINE][1]=ENGINE_MGR;    /*Capture1: CLE266,VT3259*/
   eng_owner[MPG_ENGINE][0]=ENGINE_MGR;    /*CLE266,VT3205,VT3204,VT3259*/
   eng_owner[MPG_ENGINE][1]=ENGINE_MGR;    /*VT3259?*/
   return;
}


unsigned long EngMgr_RegClient(EngMgr_RegClient_Para *pInput)
{
	unsigned long ret=ENGMGR_FAIL;
	unsigned long i;	
	DBG_DD(("\n")); 
	
	for(i=0;i<CLIENTMAX;i++)
	{
		if(client_info[i].pFunc==0)
		{
           client_info[i].pFunc = pInput->func;
		   client_info[i].uPriority = pInput->uPriority;
           *pInput->pClient_handle=i;
           DBG_DD((" *pInput->pClient_handle = %ld \n",*pInput->pClient_handle)); 
		   ret=ENGMGR_SUCCESS;
		   break;
		}
	}
	return ret;
}


unsigned long EngMgr_UnRegClient(PVIDDATA pVidData, EngMgr_UnRegClient_Para *pInput)
{
	unsigned long i,j,ret=ENGMGR_FAIL;
	EngMgr_FreeEngine_Para para;
	DBG_DD(("\n")); 
	if(pInput->uClient_handle > CLIENTMAX)
         return ret;
	if(client_info[pInput->uClient_handle].pFunc!=0)
	{
	    for(i=0;i<ENGINETYPE;i++)
		{
		    for(j=0;j<ENG_ID_MAX;j++)
			{
			    if(eng_owner[i][j]==pInput->uClient_handle)
				{
					para.uClient_handle = pInput->uClient_handle;
					para.uEng_type = i;
					para.uEng_id = j;
					EngMgr_FreeEngine(pVidData, &para);
				}
			}
		}
		client_info[pInput->uClient_handle].pFunc=0 ;
		ret=ENGMGR_SUCCESS;
	}
    
    return ret;
}


unsigned long EngMgr_AdjustClientPriority(EngMgr_AdjustClientPriority_Para *pInput)
{
	unsigned long ret=ENGMGR_FAIL;
	DBG_DD(("\n")); 
	if(pInput->uClient_handle > CLIENTMAX)
         return ret;
	if(client_info[pInput->uClient_handle].pFunc!=0)
	{
		client_info[pInput->uClient_handle].uPriority =pInput->uPriority ;
	}
	return ret;
}


unsigned long EngMgr_CheckEngine(EngMgr_CheckEngine_Para *pInput)
{
	unsigned long eng_id_available;
	unsigned long ret;
	DBG_DD(("\n")); 

	ret=ENGMGR_FAIL;
	if(ENGMGR_SUCCESS==Check_Available_Engine_ID(pInput->uEng_type,pInput->uEng_id,&eng_id_available))
	{
		ret=ENGMGR_SUCCESS;
	}
	return ret;
}


unsigned long EngMgr_AllocateEngine(PVIDDATA pVidData, EngMgr_AllocateEngine_Para *pInput)
{
	unsigned long eng_id_available=0;
	unsigned long ret;
    DBG_DD(("\n")); 
    
	DBG_DD((" Eng#[%ld]-[%ld] :\n",pInput->uEng_type,pInput->uEng_id)); 
	if(ENGMGR_FAIL==Check_Available_Engine_ID(pInput->uEng_type,pInput->uEng_id,&eng_id_available))
	{
		if(ENGMGR_SUCCESS==Notify_Client_To_Free(pVidData, pInput->uClient_handle,pInput->uEng_type,pInput->uEng_id,&eng_id_available))
		{ 
			*pInput->pEng_id_actual=eng_id_available;
			ret=ENGMGR_SUCCESS;
		}
		else
		{
		    ret=ENGMGR_FAIL;
	    }
	}else
	{
		eng_owner[pInput->uEng_type][eng_id_available]=pInput->uClient_handle;
		*pInput->pEng_id_actual=eng_id_available;
		ret=ENGMGR_SUCCESS;
	}
	return ret;
}


unsigned long EngMgr_FreeEngine(PVIDDATA pVidData, EngMgr_FreeEngine_Para *pInput)
{
	unsigned long ret;
	DBG_DD(("\n")); 
	
	DBG_DD((" Eng#[%ld]-[%ld] :\n",pInput->uEng_type,pInput->uEng_id)); 
	if(eng_owner[pInput->uEng_type][pInput->uEng_id]==pInput->uClient_handle)
	{
		eng_owner[pInput->uEng_type][pInput->uEng_id]=ENGINE_MGR;
		Notify_Client_To_Allocate(pVidData, pInput->uClient_handle,pInput->uEng_type,pInput->uEng_id);
		ret=ENGMGR_SUCCESS;
	}else 
	{
		ret=ENGMGR_FAIL;
	}
	return ret;
}


unsigned long EngMgr_SwapEngine(PVIDDATA pVidData, EngMgr_SwapEngine_Para *pInput )
{
	unsigned long eng_id_available;
	unsigned long ret=ENGMGR_FAIL;
	DBG_DD(("\n")); 
		
	DBG_DD((" Swap Eng#[%ld]-[%ld] to [%ld] :\n",pInput->uEng_type,pInput->uEng_id_src,pInput->uEng_id_dst)); 
	if(ENGMGR_FAIL==Check_Available_Engine_ID(pInput->uEng_type,pInput->uEng_id_dst,&eng_id_available))
	{
	    if(ENGMGR_SUCCESS==Notify_Client_To_Swap(pVidData, pInput->uClient_handle,pInput->uEng_type,pInput->uEng_id_src,pInput->uEng_id_dst))
		{
		    eng_owner[pInput->uEng_type][pInput->uEng_id_dst]=pInput->uClient_handle;
			ret=ENGMGR_SUCCESS;
		}
	}else
	{
	    eng_owner[pInput->uEng_type][pInput->uEng_id_src]=ENGINE_MGR;
		eng_owner[pInput->uEng_type][eng_id_available]=pInput->uClient_handle;
		ret=ENGMGR_SUCCESS;
	}
	return ret;
}


unsigned long EngCallback_MPEG(PVIDDATA pVidData, Call_Back_Para *para)
{
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;    
    
    DBG_DD(("\n"));
    switch(para->uMessage_Type)
    {
        case ENGMGR_FREE_ENG_MSG:
        {
            switch(para->uEng_Type)
            {
                case VID_ENGINE:
                {
                    DBG_DD(("ENGMGR_FREE_ENG_MSG: VID_ENGINE[%ld] \n",para->uEng_Id0));                    
                }    
                    break;                    
                
                case HQV_ENGINE:
                {
                    DBG_DD(("ENGMGR_FREE_ENG_MSG: HQV_ENGINE[%ld] \n",para->uEng_Id0));                                        
                }
                    break;
                
                default:
                    break;                    
            }
        }/*ENGMGR_FREE_ENG_MSG*/
            break;
        case ENGMGR_ALLOCATE_ENG_MSG:
        {
            switch(para->uEng_Type)
            {
                case VID_ENGINE:
                {
                    DBG_DD(("ENGMGR_ALLOCATE_ENG_MSG: VID_ENGINE[%ld] \n",para->uEng_Id0));
                    if (para->uEng_Id0==0)/*V1*/
                    {
                        switch ( lpVIAGraphicInfo->dwDeviceID )
                        {
                            case VIA_DEVICE_CLE1:
                            case VIA_DEVICE_CLE2:
                                /*MPEG+V3 HIDE on SND*/
                                if ( ~(pVidData->VidStatus.dwVideoFlagMPEG[0]&VIDEO_SHOW) &&
                                     ~(pVidData->VidStatus.dwVideoFlagMPEG[0]&VIDEO_1_INUSE) &&
                                      (pVidData->VidStatus.dwVideoFlagMPEG[0]&MPEG_ON_SND) )
                                {
                                    pVidData->VidStatus.dwVideoFlagMPEG[0] &= ~VIDEO_3_INUSE;
                                    pVidData->VidStatus.dwVideoFlagMPEG[0] |= VIDEO_1_INUSE;
                                    pVidData->dwVideoEngineSwitch = TRUE;
                                    DBG_DD((" Upd_MPEG!! Switch V3 to V1 !! \n"));
                                    
                                    UpdateOverlay(pVidData, &pVidData->UpdateOverlayMPEGBackup);
                        
                                    Macro_VidREGRec(VIDREGREC_RESET_COUNTER, 0,0);
                                    DBG_DD(("ENGMGR_YES!!\n"));
                                    return ENGMGR_YES;
                                }
                                
                                break;
                            default:
                                break;
                        }
                    }
                }/*VID_ENGINE*/
                    break;    
                case HQV_ENGINE:
                {
                    DBG_DD(("ENGMGR_ALLOCATE_ENG_MSG: HQV_ENGINE[%ld] \n",para->uEng_Id0));
                    if (para->uEng_Id0==0)/*HQV0*/
                    {
                        switch ( lpVIAGraphicInfo->dwDeviceID )
                        {
                            case VIA_DEVICE_CLE1:
                            case VIA_DEVICE_CLE2:
                                Macro_VidREGRec(VIDREGREC_RESET_COUNTER, 0,0);
                                DBG_DD(("ENGMGR_YES!!\n"));
                                return ENGMGR_YES;
                            break;
                            default:
                                break;
                        }                                
                    }
                }/*HQV_ENGINE*/
                    break;
                default:
                    break;
            }/*para->uEng_Type*/
        }/*ENGMGR_ALLOCATE_ENG_MSG*/
            break;
        case ENGMGR_SWAP_ENG_MSG:
        {
            switch(para->uEng_Type)
            {
                case VID_ENGINE:
                {
                    DBG_DD(("ENGMGR_SWAP_ENG_MSG: swap VID_ENGINE[%ld] to VID_ENGINE[%ld] \n",para->uEng_Id0,para->uEng_Id1));   
                    if ( (para->uEng_Id0==1) && (para->uEng_Id1==0) )/*switch V1 to V3*/
                    {
                        switch ( lpVIAGraphicInfo->dwDeviceID )
                        {
                            case VIA_DEVICE_CLE1:
                            case VIA_DEVICE_CLE2:
                                if ( (pVidData->VidStatus.dwVideoFlagMPEG[0]&VIDEO_SHOW) && 
                                     (pVidData->VidStatus.dwVideoFlagMPEG[0]&MPEG_ON_SND) )
                                {
                                    DBG_DD((" No! => coz MPEG+V1+HQV is show on SND screen!  \n"));
                                    break;
                                }
                                else
                                {
                                    pVidData->VidStatus.dwVideoFlagMPEG[0] &= ~VIDEO_1_INUSE;
                                    pVidData->VidStatus.dwVideoFlagMPEG[0] |= VIDEO_3_INUSE;
                                    pVidData->dwVideoEngineSwitch = TRUE;  
                                    DBG_DD((" Upd_MPEG!! Switch V1 to V3 !! \n"));
                            
                                    UpdateOverlay(pVidData, &pVidData->UpdateOverlayMPEGBackup);
                         
                                    Macro_VidREGRec(VIDREGREC_RESET_COUNTER, 0,0);
                                    DBG_DD(("ENGMGR_YES!!\n"));
                                    return ENGMGR_YES;
                                }
                            default:
                                break;
                        }
                    }
                    else if ( (para->uEng_Id0==0) && (para->uEng_Id1==1) )/*Switch V3 to V1*/
                    {
                        switch ( lpVIAGraphicInfo->dwDeviceID )
                        {
                            case VIA_DEVICE_CLE1:
                            case VIA_DEVICE_CLE2:
                            {
                                pVidData->VidStatus.dwVideoFlagMPEG[0] &= ~VIDEO_3_INUSE;
                                pVidData->VidStatus.dwVideoFlagMPEG[0] |= VIDEO_1_INUSE;
                                pVidData->dwVideoEngineSwitch = TRUE;  
                                DBG_DD((" Upd_MPEG!! Switch V3 to V1 !! \n"));
                            
                                UpdateOverlay(pVidData, &pVidData->UpdateOverlayMPEGBackup);
                         
                                Macro_VidREGRec(VIDREGREC_RESET_COUNTER, 0,0);          
                                DBG_DD(("ENGMGR_YES!!\n"));
                                return ENGMGR_YES;                      
                            }                            
                            default:
                                break;
                        }
                     
                    }/*Switch V3 to V1*/
                }/*VID_ENGINE*/    
                    break;                    
                
                case HQV_ENGINE:
                {
                    DBG_DD(("ENGMGR_SWAP_ENG_MSG: swap HQV_ENGINE[%ld] to HQV_ENGINE[%ld] \n",para->uEng_Id0,para->uEng_Id1));   
                    if ( (para->uEng_Id0==1) && (para->uEng_Id1==0) )/*switch HQV0 to HQV1*/
                    {
                        switch ( lpVIAGraphicInfo->dwDeviceID )
                        {
                            default:
                                break;
                        }
                    }
                    else if ( (para->uEng_Id0==0) && (para->uEng_Id1==1) )/*switch HQV1 to HQV0*/
                    {
                        switch ( lpVIAGraphicInfo->dwDeviceID )
                        {
                            default:
                                break;
                        }                     
                    }
                }/*HQV_ENGINE*/
                    break;
                
                default:
                    break;
            }/*para->uEng_Type*/
        }/*ENGMGR_SWAP_ENG_MSG*/
            
            break;
        default:
            break;
    }
    DBG_DD(("ENGMGR_NO!!\n"));
    return ENGMGR_NO;
}

unsigned long EngCallback_SW(PVIDDATA pVidData, Call_Back_Para *para)
{
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;
    
    DBG_DD(("\n"));
    switch(para->uMessage_Type)
    {
        case ENGMGR_FREE_ENG_MSG:
        {
            switch(para->uEng_Type)
            {
                case VID_ENGINE:
                {
                    DBG_DD(("ENGMGR_FREE_ENG_MSG: VID_ENGINE[%ld] \n",para->uEng_Id0));   
                }    
                    break;                    
                
                case HQV_ENGINE:
                {
                    DBG_DD(("ENGMGR_FREE_ENG_MSG: HQV_ENGINE[%ld] \n",para->uEng_Id0));
                    if (para->uEng_Id0==0)/*HQV0*/
                    {
                        switch ( lpVIAGraphicInfo->dwDeviceID )
                        {
                            case VIA_DEVICE_CLE1:
                            case VIA_DEVICE_CLE2:
                                if (para->uSrcClient_Priority > NORMAL_PRIORITY)
                                {                            
                                    Macro_VidREGRec(VIDREGREC_RESET_COUNTER, 0,0);
                                    DBG_DD(("ENGMGR_YES!!\n"));
                                    return ENGMGR_YES;                  
                                }
                                break;
                            default:
                                break;
                        }
                    }                    
                }
                    break;
                
                default:
                    break;
                    
            }
        }/*ENGMGR_FREE_ENG_MSG*/   
            break;
        case ENGMGR_ALLOCATE_ENG_MSG:
        {
            switch(para->uEng_Type)
            {
                case VID_ENGINE:
                {
                    DBG_DD(("ENGMGR_ALLOCATE_ENG_MSG: VID_ENGINE[%ld] \n",para->uEng_Id0));
                    if (para->uEng_Id0==0)/*V1*/
                    {
                        switch ( lpVIAGraphicInfo->dwDeviceID )
                        {
                            case VIA_DEVICE_CLE1:
                            case VIA_DEVICE_CLE2:
                                /*SW+V3 HIDE on SND*/
                                if ( ~(pVidData->VidStatus.dwVideoFlagSW[0]&VIDEO_SHOW) &&
                                     ~(pVidData->VidStatus.dwVideoFlagSW[0]&VIDEO_1_INUSE) &&
                                      (pVidData->VidStatus.dwVideoFlagSW[0]&SW_ON_SND) )
                                {
                                    pVidData->VidStatus.dwVideoFlagSW[0] &= ~VIDEO_3_INUSE;
                                    pVidData->VidStatus.dwVideoFlagSW[0] |= VIDEO_1_INUSE;
                                    pVidData->dwVideoEngineSwitch = TRUE;
                                    DBG_DD((" Upd_Video!! Switch V3 to V1 !! \n"));
                                    
                                    UpdateOverlay(pVidData, &pVidData->UpdateOverlaySWBackup);
                          
                                    Macro_VidREGRec(VIDREGREC_RESET_COUNTER, 0,0);
                                    DBG_DD(("ENGMGR_YES!!\n"));
                                    return ENGMGR_YES;
                                }
                                
                                break;
                            default:
                                break;
                        }
                    }
                }/*VID_ENGINE*/
                    break;    
                case HQV_ENGINE:
                {
                    DBG_DD(("ENGMGR_ALLOCATE_ENG_MSG: HQV_ENGINE[%ld] \n",para->uEng_Id0));
                    if (para->uEng_Id0==0)/*HQV0*/
                    {
                        switch ( lpVIAGraphicInfo->dwDeviceID )
                        {
                            case VIA_DEVICE_CLE1:
                            case VIA_DEVICE_CLE2:
                                Macro_VidREGRec(VIDREGREC_RESET_COUNTER, 0,0);                                
                                DBG_DD(("ENGMGR_YES!!\n"));
                                return ENGMGR_YES;
                            break;
                            default:
                                break;
                        }                                
                    }
                }/*HQV_ENGINE*/
                    break;
                default:
                    break;
            }/*para->uEng_Type*/
        }/*ENGMGR_ALLOCATE_ENG_MSG*/
            break;
        case ENGMGR_SWAP_ENG_MSG:
        {
            switch(para->uEng_Type)
            {
                case VID_ENGINE:
                {
                    DBG_DD(("ENGMGR_SWAP_ENG_MSG: swap VID_ENGINE[%ld] to VID_ENGINE[%ld] \n",para->uEng_Id0,para->uEng_Id1));   
                    if ( (para->uEng_Id0==1) && (para->uEng_Id1==0) )/*switch V1 to V3*/
                    {
                        switch ( lpVIAGraphicInfo->dwDeviceID )
                        {
                            case VIA_DEVICE_CLE1:
                            case VIA_DEVICE_CLE2:
                                if ( (pVidData->VidStatus.dwVideoFlagSW[0]&VIDEO_SHOW) && 
                                     (pVidData->VidStatus.dwVideoFlagSW[0]&SW_ON_SND) )
                                {
                                    DBG_DD((" No! => coz SW+V1 is show on SND screen!  \n"));
                                    break;
                                }
                                else
                                {
                                    pVidData->VidStatus.dwVideoFlagSW[0] &= ~VIDEO_1_INUSE;
                                    pVidData->VidStatus.dwVideoFlagSW[0] |= VIDEO_3_INUSE;
                                    pVidData->dwVideoEngineSwitch = TRUE;  
                                    DBG_DD((" Upd_Video!! Switch V1 to V3 !! \n"));
                            
                                    fpUpd_Video(pVidData);
                          
                                    Macro_VidREGRec(VIDREGREC_RESET_COUNTER, 0,0);
                                    DBG_DD(("ENGMGR_YES!!\n"));
                                    return ENGMGR_YES;
                                }
                            default:
                                break;
                        }
                    }
                    else if ( (para->uEng_Id0==0) && (para->uEng_Id1==1) )/*Switch V3 to V1*/
                    {
                        switch ( lpVIAGraphicInfo->dwDeviceID )
                        {
                            case VIA_DEVICE_CLE1:
                            case VIA_DEVICE_CLE2:
                            {
                                pVidData->VidStatus.dwVideoFlagSW[0] &= ~VIDEO_3_INUSE;
                                pVidData->VidStatus.dwVideoFlagSW[0] |= VIDEO_1_INUSE;
                                pVidData->dwVideoEngineSwitch = TRUE;  
                                DBG_DD((" Upd_Video!! Switch V3 to V1 !! \n"));
                            
                                UpdateOverlay(pVidData, &pVidData->UpdateOverlaySWBackup);
                        
                                Macro_VidREGRec(VIDREGREC_RESET_COUNTER, 0,0);          
                                DBG_DD(("ENGMGR_YES!!\n"));
                                return ENGMGR_YES;                      
                            }                            
                            default:
                                break;
                        }
                     
                    }/*Switch V3 to V1*/
                }/*VID_ENGINE*/    
                    break;                    
                
                case HQV_ENGINE:
                {
                    DBG_DD(("ENGMGR_SWAP_ENG_MSG: swap HQV_ENGINE[%ld] to HQV_ENGINE[%ld] \n",para->uEng_Id0,para->uEng_Id1));   
                    if ( (para->uEng_Id0==1) && (para->uEng_Id1==0) )/*switch HQV0 to HQV1*/
                    {
                        switch ( lpVIAGraphicInfo->dwDeviceID )
                        {
                            default:
                                break;
                        }
                    }
                    else if ( (para->uEng_Id0==0) && (para->uEng_Id1==1) )/*switch HQV1 to HQV0*/
                    {
                        switch ( lpVIAGraphicInfo->dwDeviceID )
                        {
                            default:
                                break;
                        }                     
                    }
                }/*HQV_ENGINE*/
                    break;
                
                default:
                    break;
            }/*para->uEng_Type*/
        }/*ENGMGR_SWAP_ENG_MSG*/
            
            break;
        default:
            break;
    }
    DBG_DD(("ENGMGR_NO!!\n"));
    return ENGMGR_NO;
}



