 /*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

 
/* I N C L U D E S ---------------------------------------------------------- */
#ifdef XFree86Server
#include "xf86_ansic.h" /* for Bool */
#else
#include "via_types.h"  /* for Bool */
#endif

#include "via_memmgr.h"
#include "VidStruc.h"
#include "debug.h"
#include "ddmpeg.h"
#include "via_drm.h"

#ifdef HAVE_CONFIG_H
#include "config_via.h"
#endif

/* E X T E R N   G L O B A L S*/
/* extern PVIDDATA pVidData; */

/* G L O B A L S ------------------------------------------------------------ */
unsigned long   S2D_Count  = 0;
unsigned long   S3D_Count  = 0;
unsigned long   Video_Count = 0;

#ifdef XF86DRI
extern int  drm_fd;
#endif

extern VIDEOMEMORY     VidMemory;          /* video memory struct */

/* F U N C T I O N S---------------------------------------------------------- */

void PrintFBMem(PVIDDATA pVidData)
{
    OffMemRange *used,*unused;
    ViaOffScrnPtr lpMemLayOut = &(pVidData->MemLayOut);

    if(pVidData->AddrUnused0)
        lpMemLayOut->unused = &(pVidData->MemLayOutBufPool[pVidData->AddrUnused0 - 1]);
    else
        lpMemLayOut->unused = NULL;

    if(pVidData->AddrUsed0)
        lpMemLayOut->used = &(pVidData->MemLayOutBufPool[pVidData->AddrUsed0 - 1]);
    else
        lpMemLayOut->used = NULL;

    unused = lpMemLayOut->unused;
    used   = lpMemLayOut->used;

    DBG_DD(("\n"));
    DBG_DD(("via_memmgr.c : Used   ID, StartAddr,  EndAddr,     size,     type,     this,     next\n"));
    while (used != NULL)
    {
        DBG_DD(("via_memmgr.c : %08lx,  %08lx, %08lx, %08lx, %08x, %p, %p \n",
                  used->ID, used->StartAddr, used->EndAddr,used->size, used->type, used, used->nextarray));
        if(used->nextarray == 0)
            break;
        used = nextq(used);

    }

    DBG_DD(("\n"));
    DBG_DD(("via_memmgr.c : Unused ID, StartAddr,  EndAddr,     size,     type,     this,     next\n"));
    while (unused != NULL)
    {
        DBG_DD(("via_memmgr.c :  %08lx,  %08lx, %08lx, %08lx, %08x, %p, %p \n",
                  unused->ID,unused->StartAddr, unused->EndAddr,unused->size, unused->type,
                  unused, unused->nextarray));
        if(unused->nextarray == 0)
            break;
        unused = nextq(unused);

    }
}

void MergeL(OffMemRange * left , unsigned long size)
{
    left->EndAddr  = endq(left)  + size;
    left->size     = sizeq(left) + size;
}

void MergeR(OffMemRange * right, unsigned long Addr,unsigned long size)
{
    right->StartAddr = Addr;
    right->size      = sizeq(right) + size;
}

void MergeLR(OffMemRange *left, OffMemRange *right, unsigned long S_Addr,unsigned long size)
{
    left->size = sizeq(left) + sizeq(right) + size;
    left->EndAddr  = endq(right);
    left->nextarray = right->nextarray;
}

ViaMMReq* viaGetSurfaceID(PVIDDATA pVidData, int wSurfaceID)
{
    unsigned long handle=pVidData->VMI_Info.handle;
    LPVIDEOMEMORY lpVidMemory = &VidMemory;
    ViaMMReq *lpVideoSurface=NULL;
    
    switch(wSurfaceID)
    {
        case MPEG_REQ_SURFACE_ID:           
            lpVideoSurface = &lpVidMemory->MPEGMemRequest[handle]; 
            DBG_DD(("Surface type: MPEG %d surface! \n", pVidData->VMI_Info.handle));    
            break;         
        case SUBP_REQ_SURFACE_ID:
            lpVideoSurface = &lpVidMemory->SUBPMemRequest[handle];
            DBG_DD(("Surface type: SubPicture %d surface! \n", pVidData->VMI_Info.handle));
            break;         
        case HQV_REQ_SURFACE_ID:
            lpVideoSurface = &lpVidMemory->HQVMemRequest;          
            DBG_DD(("Surface type: HQV surface! \n"));   
            break;             
        case HQV2_REQ_SURFACE_ID:
            lpVideoSurface = &lpVidMemory->HQV2MemRequest;
            DBG_DD(("Surface type: HQV2 surface! \n")); 
            break;             
        case ALPHA_REQ_SURFACE_ID:
            lpVideoSurface = &lpVidMemory->ALPHAMemRequest;       
            DBG_DD(("Surface type: ALPHA surface! \n"));    
            break;         
        case SW_REQ_SURFACE_ID:
            lpVideoSurface = &lpVidMemory->SWMemRequest[handle];      
            DBG_DD(("Surface type: software %d surface! \n", pVidData->VMI_Info.handle));   
            break;         
        case VQ_REQ_SURFACE_ID:
            lpVideoSurface = &lpVidMemory->VQMemRequest;
            DBG_DD(("Surface type: virtual queue surface! \n"));
            break;
        case ROT_SW_REQ_SURFACE_ID:
            lpVideoSurface = &lpVidMemory->RotMemRequest;      
            DBG_DD(("Surface type: software surface for rotation! \n"));
            break;    
        default:
            DBG_DD(("Surface ID error\n"));       
            break;
    }

    return lpVideoSurface;
}


drm_via_mem_t* viaGetSurfaceID_DRM(PVIDDATA pVidData, int wSurfaceID)
{
    unsigned long handle = pVidData->VMI_Info.handle;
    LPVIDEOMEMORY lpVidMemory = &VidMemory;
    drm_via_mem_t *lpDRMSurface = NULL;
    
    switch(wSurfaceID)
    {
        case MPEG_REQ_SURFACE_ID:
		    lpDRMSurface = &lpVidMemory->MPEGfbRequest[handle];
		    DBG_DD(("Surface type: MPEG %d surface! (drm) \n", pVidData->VMI_Info.handle));
	        break;			
        case SUBP_REQ_SURFACE_ID:
		    lpDRMSurface = &lpVidMemory->SUBPfbRequest[handle];
		    DBG_DD(("Surface type: SubPicture %d surface! (drm) \n", pVidData->VMI_Info.handle));
	        break;			
        case HQV_REQ_SURFACE_ID:
		    lpDRMSurface = &lpVidMemory->HQVfbRequest;
		    DBG_DD(("Surface type: HQV surface! (drm) \n"));
	        break; 			
        case HQV2_REQ_SURFACE_ID:
	        lpDRMSurface = &lpVidMemory->HQV2fbRequest;	
		    DBG_DD(("Surface type: HQV2 surface! (drm) \n"));	 
	        break; 			
        case ALPHA_REQ_SURFACE_ID:
		    lpDRMSurface =  &lpVidMemory->ALPHAfbRequest;
		    DBG_DD(("Surface type: ALPHA surface! (drm) \n"));	
	        break;			
        case SW_REQ_SURFACE_ID:
		    lpDRMSurface = &lpVidMemory->SWfbRequest; 
		    DBG_DD(("Surface type: software surface! (drm) \n"));
	        break;			
        case VQ_REQ_SURFACE_ID:
		    lpDRMSurface = &lpVidMemory->VQfbRequest;
		    DBG_DD(("Surface type: virtual queue surface! (drm) \n"));
	        break;			
        case ROT_SW_REQ_SURFACE_ID:
            lpDRMSurface = &lpVidMemory->RotfbRequest;
            DBG_DD(("Surface type: software surface for rotation! (drm) \n"));
            break;  
	    default:		
		    DBG_DD(("Surface ID error (drm)\n"));	
		    break;
    }

    return lpDRMSurface;
}

unsigned long viaAllocVideoSurface(PVIDDATA pVidData, int wSurfaceID, unsigned long dwBufSize)
{
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;
    ViaMMReq *lpVideoSurface=NULL;
    drm_via_mem_t *lpDRMSurface=NULL;
    unsigned long dwBufOffset;  
        
    
    DBG_DD((" viaAllocVideoSurface()! \n"));

    if (!lpVIAGraphicInfo->DRMEnabled)
    {
        DBG_DD(("video surface management\n"));
        lpVideoSurface = viaGetSurfaceID(pVidData, wSurfaceID);

        lpVideoSurface->checkID = VIA_MID;
        lpVideoSurface->size    = dwBufSize;
        lpVideoSurface->type    = VIDMGR_TYPE_VIDEO;
        if ( MEM_OK != viaMgr_vid_ioctl(pVidData, VIAMGR_ALLOC_VIDEO, (char *)(lpVideoSurface) ) )
        {
            DBG_DD(("Can't allocate memory use Video surface management(Surface ID = %d)\n", wSurfaceID));
            return 0;
        }
        dwBufOffset = ALIGN_TO_32_BYTE(lpVideoSurface->offset);
    }
    else
    {
        lpDRMSurface = viaGetSurfaceID_DRM(pVidData, wSurfaceID);
        
        lpDRMSurface->context=1;
        lpDRMSurface->size=dwBufSize;
        lpDRMSurface->type= VIDEO;


#if defined(XF86DRI) 
        /* Both 2D and DDMPEG are user space drivers, they will use 
         *ioctl to communicate with DRM module. 
         */
        DBG_DD(("drm surfce management: by ioctl(DRM_IOCTL_VIA_ALLOCMEM). \n"));
        if ( ioctl( drm_fd, DRM_IOCTL_VIA_ALLOCMEM, lpDRMSurface) )
        {
            DBG_DD(("\nCan't allocate memory use DRM surface management!(Surface ID=%d)\n", wSurfaceID));
            return 0;
        }
#endif

        dwBufOffset =lpDRMSurface->offset;
    }

    return dwBufOffset ;

}

int viaFreeVideoSurface(PVIDDATA pVidData,int wSurfaceID)
{
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;
    ViaMMReq *lpVideoSurface=NULL;
    drm_via_mem_t *lpDRMSurface=NULL;   
    unsigned long ret=1;
    DBG_DD((" viaFreeVideoSurface()! \n"));
    
    if (!lpVIAGraphicInfo->DRMEnabled)
    {
        lpVideoSurface = viaGetSurfaceID(pVidData, wSurfaceID);
        
        if (lpVideoSurface->offset != 0)
        {
            lpVideoSurface->checkID = VIA_MID;
            lpVideoSurface->type    = VIDMGR_TYPE_VIDEO;
            if ( MEM_OK != viaMgr_vid_ioctl(pVidData, VIAMGR_FREE_VIDEO, (char *)(lpVideoSurface) ) )
            {
                DBG_DD((" Memory Free fail !\n"));
                return 1;
            }
            lpVideoSurface->offset = 0; 
            ret=0;
            DBG_DD((" viaFreeVideoSurface() done! \n"));
        }      
    }
    else 
    {
        lpDRMSurface = viaGetSurfaceID_DRM(pVidData, wSurfaceID);
        
        if (lpDRMSurface->offset != 0)
        {

#if defined(XF86DRI) 
            /* Both 2D and DDMPEG are user space drivers, 
             *they should use ioctl to communicate with DRM module.
             */
            DBG_DD(("drm surfce management: by ioctl(DRM_IOCTL_VIA_FREEMEM). \n"));
            if ( ioctl( drm_fd, DRM_IOCTL_VIA_FREEMEM, lpDRMSurface) )
            {
                DBG_DD(("\nMemory Free fail !\n"));
                return 1;
            }
#endif
            lpDRMSurface->offset= 0;
            DBG_DD((" viaFreeVideoSurface() done (drm)! \n"));
            ret=0;
        }
     }
return ret;     
}


OffMemRange * viaAllocSurface(PVIDDATA pVidData, int *size ,int alignment)
{
    int new_size ;
    unsigned long old_addr;
    OffMemRange *fq,*sq,*bq=NULL;
    Bool isFind = FALSE ;

    ViaOffScrnPtr lpMemLayOut = &(pVidData->MemLayOut);

    int nextnullptr=0;
    DBG_DD(("via_memmgr.c : viaAllocSurface \n"));
    /* new_size = (*size + alignment - 1) & ~(alignment-1) ;
       aligap = new_size - *size; */
    new_size = *size;
    if(pVidData->AddrUnused0)
        lpMemLayOut->unused = &(pVidData->MemLayOutBufPool[pVidData->AddrUnused0 - 1]);
    else
        lpMemLayOut->unused = NULL;

    if(pVidData->AddrUsed0)
        lpMemLayOut->used = &(pVidData->MemLayOutBufPool[pVidData->AddrUsed0 - 1]);
    else
        lpMemLayOut->used = NULL;
    if( lpMemLayOut->unused == NULL && lpMemLayOut->used == NULL )
    {
        LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;
        int ScrnIndex = 0;
        nextnullptr = 0;
        while(pVidData->MemLayOutBufPool[nextnullptr].StartAddr)
            nextnullptr++;
        pVidData->AddrUnused0 = nextnullptr + 1;
        pVidData->AddrUsed0   = 0;
        lpMemLayOut->unused = &(pVidData->MemLayOutBufPool[nextnullptr]);
        memset(lpMemLayOut->unused, 0, sizeof(OffMemRange));
        lpMemLayOut->unused->StartAddr = lpVIAGraphicInfo->Screen[ScrnIndex].VideoHeapBase;
        lpMemLayOut->unused->EndAddr   = lpVIAGraphicInfo->Screen[ScrnIndex].VideoHeapEnd;
        lpMemLayOut->unused->size      = lpMemLayOut->unused->EndAddr - lpMemLayOut->unused->StartAddr + 1;
        lpMemLayOut->unused->nextarray = 0;
        lpMemLayOut->used              = NULL;
        PrintFBMem(pVidData);
    }
    fq = lpMemLayOut->unused;
    bq = fq;
    while ( fq )
    {
        /* we must find the smaller-suitable memory for allocation */
        if ( (endq(fq) - startq(fq) + 1 ) >= new_size )
        {
            isFind = TRUE;
            break;
        }
        bq = fq ;
        fq = nextq(fq) ;
    }

    if (!isFind)
    {
        DBG_DD(("via_memmgr.c : viaAllocSurface : No surface available\n"));
        return NULL ;
    }
    else
    {
        old_addr = startq(fq);
        fq->StartAddr = startq(fq) + new_size ;
        fq->size      = sizeq(fq) - new_size;

        /* No memory left in this queue */
        if (startq(fq) == endq(fq) + 1)
        {
            if ( bq == lpMemLayOut->unused )
            {
                pVidData->AddrUnused0 = fq->nextarray;
                fq = nextq(fq);
                deleteq(lpMemLayOut->unused);
                lpMemLayOut->unused = fq ;
            }
            else
            {
                bq->nextarray = fq->nextarray;
                deleteq(fq);
            }
        }

        if (lpMemLayOut->used == NULL )
        {
        nextnullptr = 0;
        while(pVidData->MemLayOutBufPool[nextnullptr].StartAddr)
            nextnullptr++;
        pVidData->AddrUsed0 = nextnullptr + 1;
        lpMemLayOut->used = &(pVidData->MemLayOutBufPool[nextnullptr]);
            lpMemLayOut->used->StartAddr = old_addr;
            lpMemLayOut->used->EndAddr   = old_addr + new_size - 1;
            lpMemLayOut->used->size      = new_size;
            lpMemLayOut->used->nextarray = 0;
            DBG_DD(("via_memmgr.c : viaAllocSurface : First Used \n"));
            return (lpMemLayOut->used);
        }

        sq = lpMemLayOut->used ;

        while (sq->nextarray != 0)
             sq = nextq(sq);
        nextnullptr = 0;
        while(pVidData->MemLayOutBufPool[nextnullptr].StartAddr)
            nextnullptr++;
        sq->nextarray = nextnullptr + 1;
        sq = nextq(sq);

        sq->nextarray = 0;
        sq->StartAddr = old_addr;
        sq->EndAddr   = old_addr + new_size - 1;
        sq->size      = new_size;
        DBG_DD(("via_memmgr.c : viaAllocSurface : 2\n"));
        return (sq);
    }


}

Bool viaFreeSurface(PVIDDATA pVidData, unsigned long S_Addr,int size,unsigned char ctype)
{
    Bool isFind;
    OffMemRange *this_free,*fq,*bq;
    OffMemRange *rightq=NULL,*leftq=NULL;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;
    int ScrnIndex = 0;   
    Bool fm = FALSE, bm = FALSE, bRightest=FALSE;
    Bool bLeftest=FALSE;/*left(lower addr)-->right(higher addr)*/

    ViaOffScrnPtr lpMemLayOut = &(pVidData->MemLayOut);

    int nextnullptr=0;
    
    if(pVidData->AddrUnused0)
        lpMemLayOut->unused = &(pVidData->MemLayOutBufPool[pVidData->AddrUnused0 - 1]);
    else
        lpMemLayOut->unused = NULL;

    if(pVidData->AddrUsed0)
        lpMemLayOut->used = &(pVidData->MemLayOutBufPool[pVidData->AddrUsed0 - 1]);
    else
        lpMemLayOut->used = NULL;

    DBG_DD(("via_memmgr.c : viaFreeSurface : \n"));
    isFind = FALSE;
    this_free = lpMemLayOut->used;
    fq = lpMemLayOut->unused;
    bq = NULL;

    bq = this_free;
    /* Find the queue to be deleted */
    while (this_free != NULL) 
    {
        if (startq(this_free) == S_Addr && size ==sizeq(this_free) && typeq(this_free) == ctype)
        {
            isFind = TRUE;
            break;
        }
        else
        {
            bq = this_free;
            this_free = nextq(this_free);
        }
    }


    if ( ! isFind ) 
    {
        DBG_DD(("via_memmgr.c : viaFreeSurface : Warnning! Surface to be freed not found. \n"));
        return isFind ;
    }

    /* bq is the first queue in the link list */
    if (bq == this_free && isFind)
    {
        lpMemLayOut->used = nextq(bq) ;
        pVidData->AddrUsed0 = bq->nextarray;
        deleteq(this_free);
    /* bq -> this_free                        */
    }else if(bq != this_free && isFind) 
    {
        /*bq->next = nextq(this_free);*/
        bq->nextarray = this_free->nextarray;
        deleteq(this_free);
    }

    DBG_DD(("via_memmgr.c : viaFreeSurface : 2\n"));

    /* ====================================================================== */
    bq = fq; /* where fq is lpMemLayOut->unused */


    while( fq!=NULL )
    {
        if(fq->nextarray)
        {
            if ( (endq(fq)  == (S_Addr - 1)) && (pVidData->MemLayOutBufPool[fq->nextarray -1].StartAddr == (S_Addr + size)) )
            {
                bm = TRUE ;
                fm = TRUE ;
                leftq = fq;
                rightq = nextq(fq);
                break;
            }
        }
        /* fq -> this_free */
        if (endq(fq)  == (S_Addr - 1) )
        {
            bm = TRUE ;
            leftq = fq;
            break;
        }

        /* this_free -> fq */
        if ( startq(fq) == (S_Addr + size) ) 
        {
            fm = TRUE ;
            rightq = fq;
            break;
        }

        bq=fq;
        if (fq->nextarray == 0)
            break;
        fq = nextq(fq);
    }

    DBG_DD(("via_memmgr.c : viaFreeSurface : 3\n"));


    if ( fm && bm )
    {
        DBG_DD(("via_memmgr.c : viaFreeSurface : MergeLR\n"));
        MergeLR(leftq, rightq, S_Addr, size);
        deleteq(rightq);
    }
    else if ( fm )
    {
        DBG_DD(("via_memmgr.c : viaFreeSurface : MergeR\n"));
        MergeR(rightq ,S_Addr, size);
    }
    else if ( bm ) 
    {
        DBG_DD(("via_memmgr.c : viaFreeSurface : MergeL\n"));
        MergeL(leftq ,size);
    }

    /* Merged and return */
    if ( fm || bm )
    {
        fq=lpMemLayOut->unused;

        if(startq(fq)==                                   \
        lpVIAGraphicInfo->Screen[ScrnIndex].VideoHeapBase \
        && endq(fq)==                                     \
        lpVIAGraphicInfo->Screen[ScrnIndex].VideoHeapEnd) {
            pVidData->AddrUsed0=0;
            pVidData->AddrUnused0=0;
            memset(pVidData->MemLayOutBufPool,0,sizeof(pVidData->MemLayOutBufPool));
        }
        return isFind ;
    }

    /* sort it when insert unused queue */
    fq = lpMemLayOut->unused;
    bq = nextq(fq);

    /* fq -> NULL */
    if ( bq == NULL )
    {
        OffMemRange * this_add;
        nextnullptr = 0;
        while(pVidData->MemLayOutBufPool[nextnullptr].StartAddr)
            nextnullptr++;
        this_add = &(pVidData->MemLayOutBufPool[nextnullptr]);
        this_add->StartAddr = S_Addr;
        this_add->EndAddr   = S_Addr + size -1;
        this_add->type      = ctype;
        this_add->size      = size;


        if ( startq(fq) > S_Addr ) 
        {
            lpMemLayOut->unused = this_add;
            this_add->nextarray = pVidData->AddrUnused0;            
            pVidData->AddrUnused0 = nextnullptr + 1;
        }
        else
        {
            fq->nextarray    = nextnullptr + 1;
            this_add->nextarray = 0;
        }
        return isFind;
    }
    /* fq -> bq */
    else
    {
        while (fq != NULL && bq != NULL)
        {
            if ( (startq(fq) < S_Addr) && (startq(bq) > S_Addr) )
                break;
            else if((startq(fq) > S_Addr))
            {
                bLeftest = TRUE;
                break;
            }
            else 
            {
                fq = nextq(fq);
                bq = nextq(bq);
            }
        }
    }

    if ( bq == NULL )
       bRightest = TRUE;

    if (lpMemLayOut->unused == NULL) 
    {
        nextnullptr = 0;
        while(pVidData->MemLayOutBufPool[nextnullptr].StartAddr)
            nextnullptr++;
        lpMemLayOut->unused = &(pVidData->MemLayOutBufPool[nextnullptr]);
        pVidData->AddrUnused0 = nextnullptr + 1;
        bq= lpMemLayOut->unused;
        bq->nextarray = 0;
    }
    else /* fq -> bq and bq = NULL*/
    { 
        if ( bRightest ) 
        {
            nextnullptr = 0;
            while(pVidData->MemLayOutBufPool[nextnullptr].StartAddr)
                nextnullptr++;
            bq = &(pVidData->MemLayOutBufPool[nextnullptr]);
            bq->nextarray= 0;
            fq->nextarray = nextnullptr+1;
            bq->EndAddr = S_Addr+size-1;
            bq->StartAddr = S_Addr;
            bq->size = size;
        }
        else if(bLeftest)
        {/*S_Addr is at the lowest address*/
            OffMemRange * this_add;
            nextnullptr = 0;
            while(pVidData->MemLayOutBufPool[nextnullptr].StartAddr)
                nextnullptr++;
            this_add = &(pVidData->MemLayOutBufPool[nextnullptr]);
            this_add->StartAddr = S_Addr;
            this_add->EndAddr   = S_Addr + size -1;
            this_add->type      = ctype;
            this_add->size      = size;
            this_add->nextarray = pVidData->AddrUnused0;            
            lpMemLayOut->unused = this_add;
            pVidData->AddrUnused0 = nextnullptr + 1;
        }
        else /* fq -> bq */
        { 
            OffMemRange * cq;
            nextnullptr = 0;
            while(pVidData->MemLayOutBufPool[nextnullptr].StartAddr)
                nextnullptr++;
            cq = &(pVidData->MemLayOutBufPool[nextnullptr]);
            cq->nextarray = fq->nextarray;            
            fq->nextarray = nextnullptr+1;
            cq->EndAddr = S_Addr+size-1;
            cq->StartAddr = S_Addr;
            cq->size = size;
        }
    }


    return isFind ;
    /* ====================================================================== */
}


int viaMgr_vid_ioctl(PVIDDATA pVidData, unsigned int cmd, char *arg)
{
    DBG_DD(("via_memmgr.c: IOCTL : cmd = %08x \n",cmd));
    switch (cmd)
    {
        case VIAMGR_ALLOC_2D:
        case VIAMGR_ALLOC_3D:
        case VIAMGR_ALLOC_VIDEO:
        {
            ViaMMReq MMReq;
            int req_size;
            OffMemRange *offset;

            memcpy(&MMReq ,(ViaMMReq *)arg,sizeof(ViaMMReq));

            if (MMReq.checkID != VIA_MID)
            {
                DBG_DD(("via_memmgr.c: fail ID code.\n"));
                return MEM_ERROR;
            }

            DBG_DD(("via_memmgr.c: Alloc type = %08x\n",MMReq.type));
            req_size = MMReq.size;
            if( (offset=viaAllocSurface(pVidData, &req_size,0 )) == NULL)
                return(MEM_ERROR);


            switch (MMReq.type) 
            {
                case VIDMGR_TYPE_2D :
                    offset->ID = S2D_Count;
                    S2D_Count ++;
                    break;
                case VIDMGR_TYPE_3D :
                    offset->ID = S3D_Count;
                    S3D_Count ++;
                    break;
                case VIDMGR_TYPE_VIDEO :
                    offset->ID = Video_Count ;
                    Video_Count ++;
                    break;
                default :
                    DBG_DD(("via_memmgr.c: Invalid buffer type! \n"));
                    return(MEM_ERROR);
            }

            offset->type = MMReq.type;
            offset->priority = MMReq.priority;
            offset->capability = MMReq.capability;
            MMReq.offset = offset->StartAddr;
            MMReq.ID     = offset->ID;
            memcpy((ViaMMReq *)arg,&MMReq,sizeof(ViaMMReq));

            PrintFBMem(pVidData);
            return MEM_OK;
        }

        case VIAMGR_FREE_2D:
        case VIAMGR_FREE_3D:
        case VIAMGR_FREE_VIDEO:
        {
            ViaMMReq fm ;

            memcpy(&fm ,(ViaMMReq *)arg,sizeof(ViaMMReq));
            if (fm.checkID != VIA_MID)
            {
                DBG_DD(("via_memmgr.c: fail ID code.\n"));
                return(MEM_ERROR);
            }
            DBG_DD(("via_memmgr.c:  VIAMGR_FREE type = %08x ,size = %lx\n",
                     fm.type,fm.size));

            if(!viaFreeSurface(pVidData, fm.offset,(int)fm.size,fm.type))
            {
                return(MEM_ERROR);
            }

            switch (fm.type) 
            {
                case VIDMGR_TYPE_2D :
                    S2D_Count --;
                    break;
                case VIDMGR_TYPE_3D :
                    S3D_Count --;
                    break;
                case VIDMGR_TYPE_VIDEO :
                    Video_Count --;
                    break;
                default :
                    DBG_DD(("via_memmgr.c: Invalid buffer type! \n"));
                    return(MEM_ERROR);
            }

            PrintFBMem(pVidData);
            return MEM_OK;
        }

        default :
            DBG_DD(("via_memmgr.c:  Invalid ioctl cmd! \n"));
            return(MEM_ERROR);
    } /* end of switch */

    return MEM_OK;
}
