 /*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
 

/* I N C L U D E S ------------------------------------------------------------ */
#ifdef XFree86Server
#include "xf86_ansic.h" /* for Bool */
#else
#include "via_types.h"  /* for Bool */
#endif


#include "via.h"        /* for H/W register definition */
#include "Vinline.h"
#include "viampg.h"
#include "VidInit.h"
#include "VidStruc.h"   /* VIDDATA, video top level structure */
#include "ovlfCME.h"
#include "vregCME.h"
#include "vdefCME.h"
#include "debug.h"


extern unsigned int VideoInitial;

/*
 * DEFINE
 */


/* 
 * FUNCTION  DECLARATION
 */
void vfHM(PVIDDATA pVidData,unsigned long dwFunSel);
void vfCM(unsigned long dwIndex,unsigned long dwData,PVIDDATA pVidData);
void vfCommandManager(PVIDDATA pVidData);
void vfCMHQVW(unsigned long dwIndex,unsigned long dwData,PVIDDATA pVidData);
unsigned long vfCMHQVR(unsigned long dwIndex,PVIDDATA pVidData);
void vfCMV1(PVIDDATA pVidData);
void vfCMV1_HQV(PVIDDATA pVidData);
void vfCMV3(PVIDDATA pVidData);
void vfCMV3_HQV(PVIDDATA pVidData);
void vfCMSetPosition_V1_CME(PVIDDATA pVidData);
void vfCMSetPosition_V3_CME(PVIDDATA pVidData);


/* 
 * EXTERN FUNCTIONS
 */
extern void vfCMAGP(unsigned long dwIndex,unsigned long dwData,PVIDDATA pVidData);
extern unsigned char* vfRequestAGPbuffer(PVIDDATA pVidData, unsigned long dwAGPHeaderType,\
                                         unsigned long dwHeader5Reg);
extern void vfFireAGPcommand(PVIDDATA pVidData);
extern void UpdYInterpolationFlag(LPVIAGRAPHICINFO lpVIAGraphicInfo, unsigned long dwSrcWidth, \
            unsigned long dwVideoFlag, unsigned long *pdwMiniCtl, unsigned long *pdwCompose);

/* 
 * GLOBALS
 */


/* 
 * FUNCTION
 */

/*
 * In:      pjVideo,Video MMIO base address
 *          dwVideoFlag,Video control flag
 *          dwIndex,Video register offset
 * Out:     dwRegValue,Video register value
 * Ret:
 * Desc:    HQV register read function
 */
unsigned long vfCMHQVR(unsigned long dwIndex,PVIDDATA pVidData)
{
    unsigned char* pjVideo = pVidData->Vhwinfo.pjVideo;
    unsigned long handle=pVidData->VMI_Info.handle;
    PVIDDEVICEINFO pVDevInfo = &pVidData->VDevInfo;
    unsigned long dwVideoFlag   = pVidData->VidCtl.dwVideoFlag[handle];
    unsigned long dwRegValue    = 0;

    switch (pVDevInfo->dwDeviceID)
    {
        case VIA_DEVICE_VT3259:
        case VIA_DEVICE_VT3324:
        case VIA_DEVICE_VT3293:    
		case VIA_DEVICE_VT3353:
            if (dwVideoFlag & VIDEO_1_INUSE)
            {
                dwRegValue = dMMRD(pjVideo, dwIndex);
            }
            else if (dwVideoFlag)
            {
                dwIndex|=REG_HQV1_INDEX;
                dwRegValue = dMMRD(pjVideo, dwIndex);
            }
            else
            { 

            }
            break;

        default:
            dwRegValue = dMMRD(pjVideo, dwIndex);
            break;
    }

    return dwRegValue;
}

/*
 * In:      pjVideo,Video MMIO base address
 *          dwVideoFlag,Video control flag
 *          dwIndex,Video register offset
 *          dwData,Value to be set to Video register
 *          dwAGPRequest,AGP command buffer allocated flag
 * Out:
 * Ret:
 * Desc:    HQV register write function
 */
void vfCMHQVW(unsigned long dwIndex,unsigned long dwData,PVIDDATA pVidData)
{
    unsigned long handle=pVidData->VMI_Info.handle;
    unsigned long dwVideoFlag   = pVidData->VidCtl.dwVideoFlag[handle];
    unsigned char* pjVideo = pVidData->Vhwinfo.pjVideo;

    if (dwVideoFlag)
    {
        if (dwVideoFlag & VIDEO_1_INUSE)
        {
            if (dwVideoFlag & VIDEO_VQ_INUSE)
            {
                dwIndex += REG_VQ_OFFSET;
            }
            DBG_DD(("vfcmCME.c:W0 %08lx %08lx dwVideoFlag=%lx\n",dwIndex,dwData,dwVideoFlag));
        }
        else
        {
            dwIndex|=REG_HQV1_INDEX;
            if (dwVideoFlag & VIDEO_VQ_INUSE)
            {
                dwIndex += REG_VQ_OFFSET;
            }
	      DBG_DD(("vfcmCME.c:W1 %08lx %08lx dwVideoFlag=%lx\n",dwIndex,dwData,dwVideoFlag));
        }
        dMMWR(pjVideo, dwIndex, dwData);
    }
    else 
    {
 
    }
}

/*
 * In:      pjVideo,Video MMIO base address
 *          dwVideoFlag,Video control flag
 *          dwIndex,Video register offset
 *          dwData,Value to be set to Video register
 *          dwAGPRequest,AGP command buffer allocated flag
 * Out:
 * Ret:
 * Desc:    Video register write function
 */
void vfCM(unsigned long dwIndex,unsigned long dwData,PVIDDATA pVidData)
{
    unsigned long handle=pVidData->VMI_Info.handle;
    unsigned long dwVideoFlag   = pVidData->VidCtl.dwVideoFlag[handle];
    unsigned char* pjVideo = pVidData->Vhwinfo.pjVideo;

    /*
     * if hw supports VQ, we use VQ  for both sw and hw decoding case
     * otherwise in PIP mpeg2 and mpeg1 case, time delay introduced by mpeg2 VQ 
     * has sync problems with none time delay in sw decoding 
     */
    if (dwVideoFlag & VIDEO_VQ_INUSE)
    {
        dwIndex += REG_VQ_OFFSET;
    }
    dMMWR(pjVideo, dwIndex, dwData);        
}

/*
 * In:      pjVideo,Video MMIO base address
 *          dwVideoFlag,Video control flag
 *          dwFunSel,Video wait type
 * Out:
 * Ret:
 * Desc:    Video wait function
 */
void vfHM(PVIDDATA pVidData,unsigned long dwFunSel)
{
    unsigned long handle=pVidData->VMI_Info.handle;
    unsigned long dwVideoFlag   = pVidData->VidCtl.dwVideoFlag[handle];
    unsigned char* pjVideo = pVidData->Vhwinfo.pjVideo;

    if (dwVideoFlag & VIDEO_VQ_INUSE)
    {
        dwFunSel |= HM_WAIT_VIRTUAL_QUEUE;
    }

    switch (dwFunSel) 
    {
    case HM_WAIT_FIRE:
        if (dwVideoFlag & VIDEO_1_INUSE)
        {
        	WaitVideoCommandFire(pjVideo);
        }
        else
        {	
            WaitVideo3CommandFire(pjVideo);
        }    
        break;
    case HM_WAIT_SUBFLIP_FIRE:
        if (dwVideoFlag & VIDEO_1_INUSE)
        {
        	vfWaitSUBFlip(pjVideo);
        	WaitVideoCommandFire(pjVideo);
        }
        else
        {	
            vfWaitSUB1Flip(pjVideo);
            WaitVideo3CommandFire(pjVideo);
        }    
        break;
    case HM_WAIT_SUBFLIP:
        if (dwVideoFlag & VIDEO_1_INUSE)
        {
        	vfWaitSUBFlip(pjVideo);
        }
        else
        {	
            vfWaitSUB1Flip(pjVideo);
        }    
        break;
    case HM_WAIT_HQV_DONE:
        if (dwVideoFlag & VIDEO_1_INUSE)
        {
        	WaitHQVDone(pjVideo);
        }
        else
        {	
            vfWaitHQV1Done(pjVideo);
        }    
        break;
    case HM_WAIT_HQVDONE_FIRE:
        if (dwVideoFlag & VIDEO_1_INUSE)
        {
        	WaitHQVDone(pjVideo);
        	WaitVideoCommandFire(pjVideo);
        }
        else
        {	
            vfWaitHQV1Done(pjVideo);
        	WaitVideo3CommandFire(pjVideo);
        }    
        break;

    case HW_WAIT_HQV_FIFOEMPTY:
        if (dwVideoFlag & VIDEO_1_INUSE)
        {
        	vfWaitHQVFIFOEmpty(pjVideo);
        }
        else
        {	
            vfWaitHQV1FIFOEmpty(pjVideo);
        }    
        break;
    case HW_WAIT_HQV_FINISH:
        if (dwVideoFlag & VIDEO_1_INUSE)
        {
        	vfWaitHQVFinish(pjVideo);
        }
        else
        {	
            vfWaitHQV1Finish(pjVideo);
        }    
        break;
    case HM_WAIT_FIRE_VQ:
        /* Use vfCM instead of dMMWR, so that AGP command won't go PCI path.*/
        if (dwVideoFlag & VIDEO_1_INUSE)
        {
            vfCM(REG_HW_WAIT_CME, VAL_HM_WAIT_V1_FIRE, pVidData);
        }
        else
        {	
            vfCM(REG_HW_WAIT_CME, VAL_HM_WAIT_V3_FIRE, pVidData);
        }    
        break;
    case HM_WAIT_SUBFLIP_FIRE_VQ:
        if (dwVideoFlag & VIDEO_1_INUSE)
        {
            vfCM(REG_HW_WAIT_CME, VAL_HM_WAIT_V1_SUBFLIP, pVidData);
            vfCM(REG_HW_WAIT_CME, VAL_HM_WAIT_V1_FIRE, pVidData);
        }
        else
        {	
            vfCM(REG_HW_WAIT_CME, VAL_HM_WAIT_V3_SUBFLIP, pVidData);
            vfCM(REG_HW_WAIT_CME, VAL_HM_WAIT_V3_FIRE, pVidData);
        }    
        break;
    case HM_WAIT_SUBFLIP_VQ:
        if (dwVideoFlag & VIDEO_1_INUSE)
        {
            vfCM(REG_HW_WAIT_CME, VAL_HM_WAIT_V1_SUBFLIP, pVidData);
        }
        else
        {	
            vfCM(REG_HW_WAIT_CME, VAL_HM_WAIT_V3_SUBFLIP, pVidData);
        }    
        break;
    case HM_WAIT_HQV_DONE_VQ:/* SW FLIP */
        if (dwVideoFlag & VIDEO_1_INUSE)
        {
            vfCM(REG_HW_WAIT_CME, VAL_HM_WAIT_V1_HQV_SW_FLIP, pVidData);
        }
        else
        {	
            vfCM(REG_HW_WAIT_CME, VAL_HM_WAIT_V3_HQV_SW_FLIP, pVidData);
        }    
        break;
    case HM_WAIT_HQVDONE_FIRE_VQ:
        if (dwVideoFlag & VIDEO_1_INUSE)
        {
            vfCM(REG_HW_WAIT_CME, VAL_HM_WAIT_V1_HQV_SW_FLIP, pVidData);
            vfCM(REG_HW_WAIT_CME, VAL_HM_WAIT_V1_FIRE, pVidData);
        }
        else
        {	
            vfCM(REG_HW_WAIT_CME, VAL_HM_WAIT_V3_HQV_SW_FLIP, pVidData);
            vfCM(REG_HW_WAIT_CME, VAL_HM_WAIT_V3_FIRE, pVidData);
        }
        break;

    case HW_WAIT_HQV_FIFOEMPTY_VQ:
        if (dwVideoFlag & VIDEO_1_INUSE)
        {
            vfCM(REG_HW_WAIT_CME, VAL_HM_WAIT_V1_HQV_FLIP_FIFO, pVidData);
        }
        else
        {	
            vfCM(REG_HW_WAIT_CME, VAL_HM_WAIT_V3_HQV_FLIP_FIFO, pVidData);
        }    
        break;
    /* HW wait for HQV finished. */
    case HW_WAIT_HQV_FINISH_VQ:
        if (dwVideoFlag&VIDEO_1_INUSE)
        {
            vfCM(REG_HW_WAIT_CME, VAL_HM_WAIT_V1_HQV_DONE, pVidData);
        }
        else
        {
            vfCM(REG_HW_WAIT_CME, VAL_HM_WAIT_V3_HQV_DONE, pVidData);
        }
        break;
    case HM_WAIT_3D_IDLE:
    case HM_WAIT_3D_IDLE_VQ:
        vfWait3DIdle(pjVideo);
        break;
    case HM_WAIT_VQ_EMPTY:
    case HM_WAIT_VQ_EMPTY_VQ:
        vfWaitVirtualQueueEmpty(pjVideo);
        break;
    default:
        DBG_DD(("vfHM :Undefined wait command!\n"));
        break;
    }
}


/*
 * In:      dwVideoFlag,Video control flag
 *          lphwinfo,Video register values to be set.
 * Out:
 * Ret:
 * Desc:    Video registers write function, used for Video,Capture,MPEG
 */
void vfCMV3(PVIDDATA pVidData)
{    
    LPVIDEOHWINFO lphwinfo  = &pVidData->Vhwinfo;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;

    if (vfCMHQVR(SUBP_CONTROL_STRIDE, pVidData) & SUBP_HQV_ENABLE)
    {
        vfHM(pVidData,HM_WAIT_SUBFLIP_FIRE);
    }
    else
    {
        vfHM(pVidData,HM_WAIT_FIRE);
    }   
    

    /* 
     * In rotation case, we will update the frame buffer starting address, stride,
     * fetch count, and etc, after the video is rotated. So there is no need to 
     * set these registers here in rotation case, to do this can help to reduce 
     * garbage or flicker (rotation image overlaps unrotation image) 
     */
    if(!lpVIAGraphicInfo->rotate)
    {
        /* 0x2A4,1,5 */
        vfCM(V3_STARTADDR_0, (unsigned long)lphwinfo->dwVSrcAddr[0],pVidData);
        lphwinfo->dwV3SrcAddr[0]=lphwinfo->dwVSrcAddr[0];
        /* 0x2A8,X,6 */
        vfCM(V3_STARTADDR_1, (unsigned long)lphwinfo->dwVSrcAddr[1],pVidData);
        lphwinfo->dwV3SrcAddr[1]=lphwinfo->dwVSrcAddr[1];
        /* 0x29C,X,7 */
        vfCM(V3_STARTADDR_2, (unsigned long)lphwinfo->dwVSrcAddr[2],pVidData);
        lphwinfo->dwV3SrcAddr[2]=lphwinfo->dwVSrcAddr[2];
        /* 0x2AC,2    Capture set this register in UpdateVPort_CME() */
        vfCM(V3_STRIDE, lphwinfo->dwVPitch,pVidData); 
        /* 0x2B0,3,1 */
        vfCM(REG_V3_WIN_START_X_Y, lphwinfo->dwWinStartXY,pVidData);
        /* 0x2B4,4,2 */
        vfCM(REG_V3_WIN_END_X_Y, lphwinfo->dwWinEndXY,pVidData);
        /* 0x2B8,5,4 */
        vfCM(V3_ALPHA_FETCH_COUNT, lphwinfo->dwFetch,pVidData);
    }

    /* 
     * No need to set these registers here because we will update them after the
     * video is rotated to degree 90 and 270. 
     */
    if ((lpVIAGraphicInfo->rotate == 0) || (lpVIAGraphicInfo->rotate == 2))
    {
        /* 0x2BC,10,9 */
        vfCM(V3_ZOOM_CONTROL, lphwinfo->dwzoomCtl,pVidData);
        /* 0x27C,6,3 */
        vfCM(REG_V3_DISPLAY_COUNT, lphwinfo->dwVDisplayCount,pVidData);
    }
    
    /* 0x2C0,9,8 */
    vfCM(V3_MINI_CONTROL, lphwinfo->dwminiCtl,pVidData);
    /* 0x278,7,13 */
    vfCM(ALPHA_V3_FIFO_CONTROL, lphwinfo->dwV3FIFO,pVidData);
    /* 0x268,8,14 */
    vfCM(ALPHA_V3_PREFIFO_CONTROL, lphwinfo->dwV3PreFIFO,pVidData);
    /* 0x280,11,10 */
    vfCM(REG_V3_COLOR_KEY, lphwinfo->dwColorKey,pVidData);
    
    /*0x228,13,12 */
    vfCM(V_CHROMAKEY_LOW, lphwinfo->dwChromaKeyLow,pVidData);  
    /* 0x22c,12,11 */
    vfCM(V_CHROMAKEY_HIGH, lphwinfo->dwChromaKeyHigh,pVidData);
}

/*
 * In:      dwVideoFlag,Video control flag
 *          lphwinfo,Video register values to be set.
 * Out:
 * Ret:
 * Desc:    Video registers write function
 */
void vfCMV3_HQV(PVIDDATA pVidData)
{    
    LPVIDEOHWINFO lphwinfo  = &pVidData->Vhwinfo;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;   
    
    if (vfCMHQVR(SUBP_CONTROL_STRIDE, pVidData) & SUBP_HQV_ENABLE)
    {
        vfHM(pVidData,HM_WAIT_SUBFLIP_FIRE);
    }
    else
    {
        vfHM(pVidData,HM_WAIT_FIRE);
    } 

    /* 
     * In rotation case, we will update the frame buffer starting address, stride, 
     * fetch count, and etc, after the video is rotated. So there is no need to 
     * set these registers here in rotation case, to do this can help to reduce 
     * garbage or flicker (rotation image overlaps unrotation image) 
     */
    if(!lpVIAGraphicInfo->rotate)
    {
        /* 0x2A4,2,8,5 */
        vfCM(V3_STARTADDR_0, (unsigned long)lphwinfo->dwVSrcAddr[0],pVidData);
        lphwinfo->dwV3SrcAddr[0]=lphwinfo->dwVSrcAddr[0];
        /* 0x2A8,3,9,6 */
        vfCM(V3_STARTADDR_1, (unsigned long)lphwinfo->dwVSrcAddr[1],pVidData);
        lphwinfo->dwV3SrcAddr[1]=lphwinfo->dwVSrcAddr[1];
        /* 0x29C,4,10,7 */
        vfCM(V3_STARTADDR_2, (unsigned long)lphwinfo->dwVSrcAddr[2],pVidData);
        lphwinfo->dwV3SrcAddr[2]=lphwinfo->dwVSrcAddr[2];
        /* 0x2AC,6,X,8  Capture set this register in UpdateVPort_CME() */
        vfCM(V3_STRIDE, lphwinfo->dwVPitch,pVidData);
        /* 0x2B0,9,3,12 */
        vfCM(REG_V3_WIN_START_X_Y, lphwinfo->dwWinStartXY,pVidData);         
        /* 0x2B4,10,4,13 */
        vfCM(REG_V3_WIN_END_X_Y, lphwinfo->dwWinEndXY,pVidData);
        /* 0x2B8,11,7,14 */
        vfCM(V3_ALPHA_FETCH_COUNT, lphwinfo->dwFetch,pVidData);
    }

    /* 
     * No need to set these registers here because we will update them after the 
     * video is rotated to degree 90 and 270. 
     */
    if ((lpVIAGraphicInfo->rotate == 0) || (lpVIAGraphicInfo->rotate == 2))
    {
        /* 0x2BC,16,12,19 */
        vfCM(V3_ZOOM_CONTROL, lphwinfo->dwzoomCtl,pVidData);
        /* 0x27C,12,5,15 */
        vfCM(REG_V3_DISPLAY_COUNT, lphwinfo->dwVDisplayCount,pVidData);
    }
    
    /* 0x2C0,15,11,18 */
    vfCM(V3_MINI_CONTROL, lphwinfo->dwminiCtl,pVidData);
    /* 0x278,13,16,16 */
    vfCM(ALPHA_V3_FIFO_CONTROL, lphwinfo->dwV3FIFO,pVidData); 
    /* 0x268,14,17,17 */
    vfCM(ALPHA_V3_PREFIFO_CONTROL, lphwinfo->dwV3PreFIFO,pVidData); 
    /* 0x280,18,13,20 */
    vfCM(REG_V3_COLOR_KEY, lphwinfo->dwColorKey,pVidData);
    /*0x228,20,15,22  Not use currently*/
    vfCM(V_CHROMAKEY_LOW, lphwinfo->dwChromaKeyLow,pVidData);      
    /* 0x22c,19,14,21 Not use currently */
    vfCM(V_CHROMAKEY_HIGH, lphwinfo->dwChromaKeyHigh,pVidData);    

    /* HQV1 */
    /* 0x380 */
    vfCM(HQV_SRC_DATA_OFFSET_CONTROL1|REG_HQV1_INDEX, lphwinfo->dwHQVSrcDataOffsetStart, pVidData);
    /* 0x388 */
    vfCM(HQV_SRC_DATA_OFFSET_CONTROL3|REG_HQV1_INDEX, lphwinfo->dwHQVSrcDataOffsetEnd, pVidData);

    if(lpVIAGraphicInfo->dwDeviceID == VIA_DEVICE_VT3353)
    {
        /* VT3353 has register for subpicture start/end setting */
        /* 0x384 */
        vfCM(HQV_SRC_DATA_OFFSET_CONTROL2|REG_HQV1_INDEX, lphwinfo->dwHQVSrcDataOffsetStart, pVidData);
        /* 0x38c */
        vfCM(HQV_SRC_DATA_OFFSET_CONTROL4|REG_HQV1_INDEX, lphwinfo->dwHQVSrcDataOffsetEnd, pVidData);
    }
    /* 0x3B8 */
    vfCM(HQV_DEFAULT_VIDEO_COLOR|REG_HQV1_INDEX, HQV_FIX_COLOR, pVidData); 

    /* 0x3cc         Only use with HQV scaling up */
    vfCM(REG_HQV_SRC_OFFSET|REG_HQV1_INDEX, lphwinfo->dwHQVOffset,pVidData);
    /* 0x3d4,2,X,3   Only for Video and MPEG S/W flip use */
    vfCM(HQV_SRC_STARTADDR_Y|REG_HQV1_INDEX, (unsigned long)lphwinfo->dwHQVSrcAddr[0],pVidData);
    /* 0x3d8,X,X,4   Only for Video and MPEG S/W flip use */
    vfCM(HQV_SRC_STARTADDR_U|REG_HQV1_INDEX, (unsigned long)lphwinfo->dwHQVSrcAddr[1],pVidData);
    /* 0x3dc,1,1,1 */
    vfCM(REG_HQV_TILE_CSC_CTL_CME|REG_HQV1_INDEX, lphwinfo->dwHQVTCGDCtl,pVidData);                
    /* 0x3e0,5,X,9   Capture set this register in UpdateVPort_CME() */
    vfCM(HQV_SRC_FETCH_LINE|REG_HQV1_INDEX, lphwinfo->dwHQVSrcCountWH,pVidData);       
    /* 0x3e4,X,2,2   Only for Capture and MPEG use,we don't use this in Video */
    vfCM(REG_HQV_MOTIONADAPTIVE_CTL_CME|REG_HQV1_INDEX, lphwinfo->dwHQVDeinterlaceCtl,pVidData);
    /* 0x3e8,17,6 */ 
    if((lpVIAGraphicInfo->dwDeviceID == VIA_DEVICE_VT3293) ||
       (lpVIAGraphicInfo->dwDeviceID == VIA_DEVICE_VT3353))

    {
        vfCM(HQV_H_SCALE_CONTORL|REG_HQV1_INDEX, lphwinfo->dwHQVScaleCtlH,pVidData);
        vfCM(HQV_V_SCALE_CONTORL|REG_HQV1_INDEX, lphwinfo->dwHQVScaleCtlV,pVidData);
    }
    else
    {
        vfCM(HQV_MINI_CONTROL|REG_HQV1_INDEX, lphwinfo->dwHQVMiniZoomCtl,pVidData);    
    }
    	
    /* 0x3ec */
    vfCM(HQV_DST_STARTADDR0|REG_HQV1_INDEX, lphwinfo->dwHQVDstAddr[0],pVidData);                
    /* 0x3f0 */
    vfCM(HQV_DST_STARTADDR1|REG_HQV1_INDEX, lphwinfo->dwHQVDstAddr[1],pVidData);
    /* 0x3fc */
    vfCM(HQV_DST_STARTADDR2|REG_HQV1_INDEX, lphwinfo->dwHQVDstAddr[2],pVidData);                

    /* 0x3f4,7,X,10  //Capture set this register in UpdateVPort_CME() */
    vfCM(HQV_DST_STRIDE|REG_HQV1_INDEX, lphwinfo->dwHQVDstPitch,pVidData);            
    /* 0x3f8,8,X,11  //Capture set this register in UpdateVPort_CME() */
    vfCM(HQV_SRC_STRIDE|REG_HQV1_INDEX, lphwinfo->dwHQVSrcPitch|
                                        VAL_HQV_SW_FLIP_QUEUE_ENABLE_CME,pVidData);
}

/*
 * In:      dwVideoFlag,Video control flag
 *          lphwinfo,Video register values to be set.
 * Out:
 * Ret:
 * Desc:    Video registers write function
 */
void vfCMV1(PVIDDATA pVidData)
{    
    LPVIDEOHWINFO lphwinfo  = &pVidData->Vhwinfo;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;    
    
    
    if (vfCMHQVR(SUBP_CONTROL_STRIDE, pVidData) & SUBP_HQV_ENABLE)
    {
        vfHM(pVidData,HM_WAIT_SUBFLIP_FIRE);
    }
    else
    {
        vfHM(pVidData,HM_WAIT_FIRE);
    }


    /* 
     * In rotation case, we will update the frame buffer starting address, stride,
     * fetch count, and etc, after the video is rotated. So there is no need to 
     * set these registers here in rotation case, to do this can help to reduce 
     * garbage or flicker (rotation image overlaps unrotation image) 
     */
    if(!lpVIAGraphicInfo->rotate)
    {
        /* 0x254,1,5 */
        vfCM(V1_STARTADDR0, (unsigned long)lphwinfo->dwVSrcAddr[0],pVidData);         
        /* 0x238,X,6 */
        vfCM(V1_STARTADDR1, (unsigned long)lphwinfo->dwVSrcAddr[1],pVidData);         
        /* 0x248,X,7 */
        vfCM(V1_STARTADDR2, (unsigned long)lphwinfo->dwVSrcAddr[2],pVidData);   
        /* 0x23C,2 */
        vfCM(V1_STRIDE, lphwinfo->dwVPitch,pVidData);              
        /* 0x240,3,1 */
        vfCM(REG_V1_WIN_START_X_Y, lphwinfo->dwWinStartXY,pVidData);         
        /* 0x244,4,2 */
        vfCM(REG_V1_WIN_END_X_Y, lphwinfo->dwWinEndXY,pVidData);           
        /* 0x234,5,4 */
        vfCM(V12_QWORD_PER_LINE, lphwinfo->dwFetch,pVidData);   
    }

    /* 
     * No need to set these registers here because we will update them after the 
     * video is rotated to degree 90 and 270. 
     */
    if ((lpVIAGraphicInfo->rotate == 0) || (lpVIAGraphicInfo->rotate == 2))
    {
        /* 0x24C,9,9 */
        vfCM(V1_ZOOM_CONTROL, lphwinfo->dwzoomCtl,pVidData);
        /* 0x26C,6,3 */
        vfCM(REG_V1_DISPLAY_COUNT, lphwinfo->dwVDisplayCount,pVidData);
    }
    
    /* 0x2C0,8,8 */
    vfCM(V1_MINI_CONTROL, lphwinfo->dwminiCtl,pVidData);        
    /* 0x258,7,14 */
    vfCM(V_FIFO_CONTROL, lphwinfo->dwV1FIFO,pVidData);
    /* 0x220,10,10 */
    vfCM(V_COLOR_KEY, lphwinfo->dwColorKey,pVidData);                
    /* 0x264,11,11   Only for v1 use LCD */
    vfCM(SND_COLOR_KEY, lphwinfo->dwLCDColorKey,pVidData);            
    /* 0x228,13,13   Not use currently */
    vfCM(V_CHROMAKEY_LOW, lphwinfo->dwChromaKeyLow,pVidData);            
    /* 0x22c,12,12   Not use currently */
    vfCM(V_CHROMAKEY_HIGH, lphwinfo->dwChromaKeyHigh,pVidData);            
}

/*
 * In:      dwVideoFlag,Video control flag
 *          lphwinfo,Video register values to be set.
 * Out:
 * Ret:
 * Desc:    Video registers write function
 */
void vfCMV1_HQV(PVIDDATA pVidData)
{    
    LPVIDEOHWINFO lphwinfo  = &pVidData->Vhwinfo;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;

    if (vfCMHQVR(SUBP_CONTROL_STRIDE, pVidData) & SUBP_HQV_ENABLE)
    {
        vfHM(pVidData,HM_WAIT_SUBFLIP_FIRE);
    }
    else
    {
        vfHM(pVidData,HM_WAIT_FIRE);
    }


    /* 
     * In rotation case, we will update the frame buffer starting address, stride,
     * fetch count, and etc, after the video is rotated. So there is no need to 
     * set these registers here in rotation case, to do this can help to reduce 
     * garbage or flicker (rotation image overlaps unrotation image) 
     */
    if(!lpVIAGraphicInfo->rotate)
    {
        /* 0x254,1,8,5 */
        vfCM(V1_STARTADDR0, (unsigned long)lphwinfo->dwVSrcAddr[0],pVidData);         
        /* 0x238,X,9,6 */
        vfCM(V1_STARTADDR1, (unsigned long)lphwinfo->dwVSrcAddr[1],pVidData);         
        /* 0x248,X,10,7 */
        vfCM(V1_STARTADDR2, (unsigned long)lphwinfo->dwVSrcAddr[2],pVidData);   
        /* 0x23C,6,X,8 */
        vfCM(V1_STRIDE, lphwinfo->dwVPitch,pVidData);              
        /* 0x240,9,3,12 */
        vfCM(REG_V1_WIN_START_X_Y, lphwinfo->dwWinStartXY,pVidData);         
        /* 0x244,10,4,13 */
        vfCM(REG_V1_WIN_END_X_Y, lphwinfo->dwWinEndXY,pVidData);           
        /* 0x234,11,7,14 */
        vfCM(V12_QWORD_PER_LINE, lphwinfo->dwFetch,pVidData);   
    }

    /* 
     * No need to set these registers here because we will update them after the 
     * video is rotated to degree 90 and 270. 
     */
    if ((lpVIAGraphicInfo->rotate == 0) || (lpVIAGraphicInfo->rotate == 2))
    {
        /* 0x24C,15,12,18 */
        vfCM(V1_ZOOM_CONTROL, lphwinfo->dwzoomCtl,pVidData);
        /* 0x26C,12,5,15 */
        vfCM(REG_V1_DISPLAY_COUNT, lphwinfo->dwVDisplayCount,pVidData);                
    }
    
    /* 0x2C0,14,11,17 */
    vfCM(V1_MINI_CONTROL, lphwinfo->dwminiCtl,pVidData);        
    /* 0x258,13,17,16 */
    vfCM(V_FIFO_CONTROL, lphwinfo->dwV1FIFO,pVidData);    
    /* 0x220,17,13,19 */
    vfCM(V_COLOR_KEY, lphwinfo->dwColorKey,pVidData);                
    /* 0x264,18,14,20    Only for v1 use LCD */
    vfCM(SND_COLOR_KEY, lphwinfo->dwLCDColorKey,pVidData);            
    /* 0x228,20,16,22    Not use currently */
    vfCM(V_CHROMAKEY_LOW, lphwinfo->dwChromaKeyLow,pVidData);            
    /* 0x22c,19,15,21    Not use currently */
    vfCM(V_CHROMAKEY_HIGH, lphwinfo->dwChromaKeyHigh,pVidData);            

    /* HQV0 */
    /* 0x380 */
    vfCM(HQV_SRC_DATA_OFFSET_CONTROL1, lphwinfo->dwHQVSrcDataOffsetStart, pVidData);
    /* 0x388 */
    vfCM(HQV_SRC_DATA_OFFSET_CONTROL3, lphwinfo->dwHQVSrcDataOffsetEnd, pVidData);

    if(lpVIAGraphicInfo->dwDeviceID == VIA_DEVICE_VT3353)
    {
        /* VT3353 has register for subpicture start/end setting */
        /* 0x384 */
        vfCM(HQV_SRC_DATA_OFFSET_CONTROL2, lphwinfo->dwHQVSrcDataOffsetStart, pVidData);
        /* 0x38c */
        vfCM(HQV_SRC_DATA_OFFSET_CONTROL4, lphwinfo->dwHQVSrcDataOffsetEnd, pVidData);
    }
    /* 0x3B8 */
    vfCM(HQV_DEFAULT_VIDEO_COLOR, HQV_FIX_COLOR, pVidData); 

    /* 0x3cc         Only use with HQV scaling up */
    vfCM(REG_HQV_SRC_OFFSET, lphwinfo->dwHQVOffset,pVidData);
    /* 0x3d4,2,X,3 */
    vfCM(HQV_SRC_STARTADDR_Y, (unsigned long)lphwinfo->dwHQVSrcAddr[0],pVidData);                
    /* 0x3d8,X,X,4 */
    vfCM(HQV_SRC_STARTADDR_U, (unsigned long)lphwinfo->dwHQVSrcAddr[1],pVidData);                
    /* 0x3dc,1,1,1 */
    vfCM(REG_HQV_TILE_CSC_CTL_CME, lphwinfo->dwHQVTCGDCtl,pVidData);                
    /* 0x3e0,5,X,9 */
    vfCM(HQV_SRC_FETCH_LINE, lphwinfo->dwHQVSrcCountWH,pVidData);                
    /* 0x3e4,X,2,2 */
    vfCM(REG_HQV_MOTIONADAPTIVE_CTL_CME, lphwinfo->dwHQVDeinterlaceCtl,pVidData);
    /* 0x3e8,16,6 */
    
    if((lpVIAGraphicInfo->dwDeviceID == VIA_DEVICE_VT3293) || 
       (lpVIAGraphicInfo->dwDeviceID == VIA_DEVICE_VT3353))
    {
        vfCM(HQV_H_SCALE_CONTORL, lphwinfo->dwHQVScaleCtlH,pVidData);
        vfCM(HQV_V_SCALE_CONTORL, lphwinfo->dwHQVScaleCtlV,pVidData);
    }
    else
    {
        /* since 3D engine does all the zoom-in/zoom-out,better leave HQV not do anything*/
        vfCM(HQV_MINI_CONTROL, lphwinfo->dwHQVMiniZoomCtl,pVidData); 
    }
    /* 0x3ec */
    vfCM(HQV_DST_STARTADDR0, lphwinfo->dwHQVDstAddr[0],pVidData);                
    /* 0x3f0 */
    vfCM(HQV_DST_STARTADDR1, lphwinfo->dwHQVDstAddr[1],pVidData);
    /* 0x3fc */
    vfCM(HQV_DST_STARTADDR2, lphwinfo->dwHQVDstAddr[2],pVidData);                
    /* 0x3f4,7,X,10 */
    vfCM(HQV_DST_STRIDE, lphwinfo->dwHQVDstPitch,pVidData);                
    /* 0x3f8,8,X,11 */
    vfCM(HQV_SRC_STRIDE, lphwinfo->dwHQVSrcPitch|
                         VAL_HQV_SW_FLIP_QUEUE_ENABLE_CME,pVidData);
}

/*
 * In:      dwVideoFlag,Video control flag
 *          lphwinfo,Video register values to be set.
 * Out:
 * Ret:
 * Desc:    Video registers write function
 */
void vfCMSetPosition_V1_CME(PVIDDATA pVidData)
{    
    unsigned long handle=pVidData->VMI_Info.handle;
    unsigned long dwVideoFlag       = pVidData->VidCtl.dwVideoFlag[handle];
    LPVIDEOHWINFO lphwinfo  = &pVidData->Vhwinfo;
    if (dwVideoFlag&VIDEO_HQV_INUSE)
    {
        vfHM(pVidData,HM_WAIT_SUBFLIP_FIRE);
    }
    else
    {
        vfHM(pVidData,HM_WAIT_FIRE);        
    }

    /* 0x240 */
    vfCM(REG_V1_WIN_START_X_Y, lphwinfo->dwWinStartXY,pVidData);         
    /* 0x244 */
    vfCM(REG_V1_WIN_END_X_Y, lphwinfo->dwWinEndXY,pVidData);           
    /* 0x298 */
    vfCM(V_COMPOSE_MODE, lphwinfo->dwCompose|V1_COMMAND_FIRE,pVidData);              
    /* 0x3d0 */
    if (dwVideoFlag&VIDEO_HQV_INUSE)
    {
        vfCMHQVW(HQV_CONTROL, lphwinfo->dwHQVCtl|HQV_SUB_FLIP,pVidData);
    }
}

/*
 * In:      dwVideoFlag,Video control flag
 *          phwinfo,Video register values to be set.
 * Out:
 * Ret:
 * Desc:    Video registers write function
 */
void vfCMSetPosition_V3_CME(PVIDDATA pVidData)
{    
    unsigned long handle=pVidData->VMI_Info.handle;
    unsigned long dwVideoFlag       = pVidData->VidCtl.dwVideoFlag[handle];
    LPVIDEOHWINFO lphwinfo  = &pVidData->Vhwinfo;

    if (dwVideoFlag&VIDEO_HQV_INUSE)
    {
        vfHM(pVidData,HM_WAIT_SUBFLIP_FIRE);
    }
    else
    {
        vfHM(pVidData,HM_WAIT_FIRE);        
    }

    /* 0x240 */
    vfCM(REG_V3_WIN_START_X_Y, lphwinfo->dwWinStartXY,pVidData);         
    /* 0x244 */
    vfCM(REG_V3_WIN_END_X_Y, lphwinfo->dwWinEndXY,pVidData);           
    /* 0x298 */
    vfCM(V_COMPOSE_MODE, lphwinfo->dwCompose|V3_COMMAND_FIRE,pVidData);              
    /* 0x3d0 */
    if (dwVideoFlag&VIDEO_HQV_INUSE)
    {
        vfCMHQVW(HQV_CONTROL, lphwinfo->dwHQVCtl|HQV_SUB_FLIP,pVidData);
    }
}


/*
 * In:      dwVideoFlag,Video control flag
 * Out:
 * Ret:
 * Desc:    Video registers write function
 */
void vfCommandManager(PVIDDATA pVidData)
{
    unsigned long handle=pVidData->VMI_Info.handle;
    unsigned long dwVideoFlag       = pVidData->VidCtl.dwVideoFlag[handle];
    unsigned long dwFunSel          = pVidData->VidCtl.dwCtlCMFlag;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;
    LPOVERLAYRECORD lpOverlayRecord = &pVidData->OverlayRecordSet.OverlayRecordSW[handle];
    OV_RECTL rSrc                   = lpOverlayRecord->dwOverlaySrcRect;
    unsigned long dwSrcWidth        = 0;
    unsigned long dwCompose         = 0;
    dwSrcWidth = (unsigned long)(rSrc.right - rSrc.left);
    
    if (dwVideoFlag&VIDEO_HQV_INUSE)
    {
        dwFunSel |= CTL_CM_USE_HQV_CME;  /* 0x00100000 */
    }

    if (dwVideoFlag&VIDEO_1_INUSE)
    {
        dwFunSel |= CTL_CM_USE_V1_CME; /* 0x00010000 */
    }
    if (dwVideoFlag&VIDEO_1_ON_SND)/*added for Duo View*/
    {
        dwFunSel |= CTL_CM_USE_UseDuoView; /* 0x01000000 */
    }


    /* 
     * In duoview mode, Upd_Video call this function twice for V1, V3
     * While some thing can't be isolated for V1, V3
     * So at firt time to update overlay we force to update V1 and V3 
     */
    if(VideoInitial==FALSE) 
    {
        VideoInitial=TRUE;
        lpVIAGraphicInfo->OverlayToUpdate=(VIDEO_1|VIDEO_3);
    }
    switch (dwFunSel) 
    {
    case CM_UPDATEOVERLAY_V1_CME:  /* 0x00010001 */
        if(lpVIAGraphicInfo->OverlayToUpdate&VIDEO_1)
            vfCMV1(pVidData);
        break;
    case CM_UPDATEOVERLAY_V1_HQV_CME: /* 0x00110001 */
        if(lpVIAGraphicInfo->OverlayToUpdate&VIDEO_1)
            vfCMV1_HQV(pVidData);
        break;
    case CM_UPDATEOVERLAY_V1_HQV_DUOVIEW_CME:/*New Branch for DuoView*/
    {
        unsigned long tmpzoom=0, tmpmini=0, tmphqvmini=0;
        if(pVidData->Vhwinfo.dwzoomCtlV1DuoView != 0)
        {
            tmpzoom = pVidData->Vhwinfo.dwzoomCtl;
            tmpmini = pVidData->Vhwinfo.dwminiCtl;
            pVidData->Vhwinfo.dwzoomCtl = \
                    (pVidData->Vhwinfo.dwzoomCtl&0xffff0000)|
                    pVidData->Vhwinfo.dwzoomCtlV1DuoView;
            pVidData->Vhwinfo.dwminiCtl = V1_X_INTERPOLY;

            UpdYInterpolationFlag(lpVIAGraphicInfo, dwSrcWidth, dwVideoFlag, &(pVidData->Vhwinfo.dwminiCtl), &dwCompose);
        }

        if(lpVIAGraphicInfo->OverlayToUpdate&VIDEO_1)
        {
            if(pVidData->Vhwinfo.dwScalDuoV1ZoomCtl != 0)
            {
                tmpzoom = pVidData->Vhwinfo.dwzoomCtl;
                if(pVidData->Vhwinfo.dwScalDuoV1ZoomCtl & 0xFFFF0000)
                {
                    pVidData->Vhwinfo.dwzoomCtl &= 0x0000ffff;
                    pVidData->Vhwinfo.dwzoomCtl |= pVidData->Vhwinfo.dwScalDuoV1ZoomCtl & 0xFFFF0000;
                }
                if(pVidData->Vhwinfo.dwScalDuoV1ZoomCtl & 0x0000ffff)
                {
                    pVidData->Vhwinfo.dwzoomCtl &= 0xFFFF0000;
                    pVidData->Vhwinfo.dwzoomCtl |= pVidData->Vhwinfo.dwScalDuoV1ZoomCtl & 0x0000ffff;
                }
                UpdYInterpolationFlag(lpVIAGraphicInfo, dwSrcWidth, dwVideoFlag, &(pVidData->Vhwinfo.dwminiCtl), &dwCompose);
            }
            if(pVidData->Vhwinfo.dwScalDuoHQVMiniZoomCtl !=0)
            {
                tmphqvmini = pVidData->Vhwinfo.dwHQVMiniZoomCtl;
                pVidData->Vhwinfo.dwHQVMiniZoomCtl = pVidData->Vhwinfo.dwScalDuoHQVMiniZoomCtl;
            }
            vfCMV1_HQV(pVidData);
            if(pVidData->Vhwinfo.dwScalDuoV1ZoomCtl != 0) 
                pVidData->Vhwinfo.dwzoomCtl = tmpzoom;
            if(pVidData->Vhwinfo.dwScalDuoHQVMiniZoomCtl !=0) 
                pVidData->Vhwinfo.dwHQVMiniZoomCtl = tmphqvmini;
        }

        if(pVidData->Vhwinfo.dwzoomCtlV1DuoView != 0)
        {
            pVidData->Vhwinfo.dwzoomCtl = tmpzoom;
            pVidData->Vhwinfo.dwminiCtl = tmpmini;
        }
        if(lpVIAGraphicInfo->OverlayToUpdate&VIDEO_3)
        {
            if(pVidData->Vhwinfo.dwScalDuoV3ZoomCtl != 0)
            {
                tmpzoom = pVidData->Vhwinfo.dwzoomCtl;
                if(pVidData->Vhwinfo.dwScalDuoV3ZoomCtl & 0xFFFF0000)
                {
                    pVidData->Vhwinfo.dwzoomCtl &= 0x0000ffff;
                    pVidData->Vhwinfo.dwzoomCtl |= pVidData->Vhwinfo.dwScalDuoV3ZoomCtl & 0xFFFF0000;
                }
                if(pVidData->Vhwinfo.dwScalDuoV3ZoomCtl & 0x0000ffff)
                {
                    pVidData->Vhwinfo.dwzoomCtl &= 0xFFFF0000;
                    pVidData->Vhwinfo.dwzoomCtl |= pVidData->Vhwinfo.dwScalDuoV3ZoomCtl & 0x0000ffff;
                }
            }
            vfCMV3(pVidData);
            if(pVidData->Vhwinfo.dwScalDuoV3ZoomCtl != 0) 
                pVidData->Vhwinfo.dwzoomCtl = tmpzoom;
        }
    }
        break;

    case CM_UPDATEOVERLAY_V3_CME:  /* 0x00000001 */
        if(lpVIAGraphicInfo->OverlayToUpdate&VIDEO_3)
            vfCMV3(pVidData);
        break;
    case CM_UPDATEOVERLAY_V3_HQV_CME: /* 0x00100001 */
        if(lpVIAGraphicInfo->OverlayToUpdate&VIDEO_3)
            vfCMV3_HQV(pVidData);
        break;
    case CM_UPDATEOVERLAY_DuoView_CME: /* 0x01000001 */
        if(lpVIAGraphicInfo->OverlayToUpdate&VIDEO_1)
            vfCMV1(pVidData);
        pVidData->VidCtl.dwVideoFlag[handle]&=~VIDEO_1_INUSE;
        if(lpVIAGraphicInfo->OverlayToUpdate&VIDEO_3)
            vfCMV3(pVidData);
        pVidData->VidCtl.dwVideoFlag[handle]|=VIDEO_1_INUSE;
        break;
    case CM_UPDATEOVERLAY_DuoView_HQV_CME:
        if(lpVIAGraphicInfo->OverlayToUpdate&VIDEO_1)
            vfCMV1_HQV(pVidData);
        pVidData->VidCtl.dwVideoFlag[handle]&=~VIDEO_1_INUSE;
        if(lpVIAGraphicInfo->OverlayToUpdate&VIDEO_3)
            vfCMV3_HQV(pVidData);
        pVidData->VidCtl.dwVideoFlag[handle]|=VIDEO_1_INUSE;
        break;    
    case CM_SETPOSITION_V1_CME: /* 0x00010002 */
    case CM_SETPOSITION_V1_HQV_CME: /* 0x00110002 */
        if(lpVIAGraphicInfo->OverlayToUpdate&VIDEO_1)
            vfCMSetPosition_V1_CME(pVidData);
        break;
    case CM_SETPOSITION_V3_CME: /* 0x00000002 */
    case CM_SETPOSITION_V3_HQV_CME: /* 0x00100002 */
        if(lpVIAGraphicInfo->OverlayToUpdate&VIDEO_3)
            vfCMSetPosition_V3_CME(pVidData);
        break;
    case CM_SETPOSITION_DuoView_CME: /* 0x010000002 */
    case CM_SETPOSITION_DuoView_HQV_CME: /* 0x01100002 */
        if(lpVIAGraphicInfo->OverlayToUpdate&VIDEO_1)
            vfCMSetPosition_V1_CME(pVidData);
    	pVidData->VidCtl.dwVideoFlag[handle]&=~VIDEO_1_INUSE;
        if(lpVIAGraphicInfo->OverlayToUpdate&VIDEO_3)
            vfCMSetPosition_V3_CME(pVidData);
        pVidData->VidCtl.dwVideoFlag[handle]|=VIDEO_1_INUSE;
        break;    
    case CM_FLIP_V1_CME:
        break;
    case CM_FLIP_V1_HQV_CME:
        break;
    case CM_FLIP_V3_CME:
        break;
    case CM_FLIP_V3_HQV_CME:
        break;
    default:
        break;
    }
}
