 /*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
 

/*************************************************************************
 *
 *  File:       via_accel.c
 *  Content:    2D acceleration function for VIA/S3G UniChrome
 *
 ************************************************************************/

#include "Xarch.h"
#include "xaalocal.h"
#include "xaarop.h"
#include "miline.h"
#include "via_driver.h"
#include "via_regs.h"
#include "via.h"
#include "via_rotate.h" /* for rotate feature.*/
#include "agpctl.h"

extern Bool b3DRegsInitialized; 

extern int gVIAEntityIndex;
/*Function declare*/
extern void AddWaitCmdInv(volatile unsigned int ** ppCmd, unsigned int ChipID, unsigned int Flags);


#define GET_AGP_CMD_BUF(name)   unsigned int _##name[400];  \
    unsigned int * name = _##name;                                              \
    while(((unsigned long)name) & 0xf){                                          \
        name++;}

#define ISSUE_2D_COMMAND(pVia, isH2, pcmd, addr, value)  \
do{                                                                                             \
    if(pVia->IsPCI)                                                                         \
    {                                                                                               \
        VIASETREG(addr, value);                                                     \
    }                                                                                                 \
    else                                                                                            \
    {                                                                                                   \
        if(isH2)                                                                                    \
        {                                                                                               \
            ADD2DCmd_H2(pcmd, addr, value);                                                          \
        }                                                                                               \
        else                                                                                           \
        {                                                                                                 \
            ADD2DCmd_INVI(pcmd, addr, value);                                    \
        }                                                                                                   \
    }                                                                                                       \
}while(0)
#define ADD_AGP_HEADER(pVia, isH2, pcmd, cnt)                               \
do{                                                                                                         \
    if(!(pVia->IsPCI))                                                                              \
    {                                                                                                       \
        if(isH2)                                                                                        \
        {                                                                                                   \
            ;                                                                                               \
        }                                                                                                   \
        else                                                                                               \
        {                                                                                                   \
            ADDCmdHeader0_INVI(pcmd, cnt);                                             \
        }                                                                                                    \
    }                                                                                                       \
}while(0)

#define ISSUE_2D_DATA(pVia, isH2, pBuf, size, DestBase)                          \
do{                                                                              \
    volatile unsigned int* tmp = pBuf;                                           \
    if(pVia->IsPCI)                                                              \
    {                                                                            \
        DestBase = pVia->BltBase;                                                \
    }                                                                            \
    else                                                                         \
    {                                                                            \
        if(isH2)                                                                 \
        {                                                                        \
            Add2DHdr4Cmd_H2(pBuf, size, 0);                                      \
            pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size += pBuf - tmp;         \
            pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size += (size);             \
            DestBase = (unsigned char *)pBuf;                                    \
            ALIGN_H2_DATA(pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,          \
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);    \
            pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = TRUE;               \
        }                                                                        \
        else                                                                     \
        {                                                                        \
            ADDCmdHeader1_INVI(pBuf, size, 0);                                   \
            pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size += pBuf - tmp;         \
            pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size += (size);             \
            DestBase = (unsigned char *)pBuf;                                    \
            pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = TRUE;               \
        }                                                                        \
    }                                                                            \
}while(0)
 
static void VIASetupForScreenToScreenCopy(
    ScrnInfoPtr pScrn,
    int xdir,
    int ydir,
    int rop,
    unsigned planemask,
    int trans_color);

static void VIASubsequentScreenToScreenCopy(
    ScrnInfoPtr pScrn,
    int x1,
    int y1,
    int x2,
    int y2,
    int w,
    int h);

static void VIASetupForSolidFill(
    ScrnInfoPtr pScrn,
    int color,
    int rop,
    unsigned planemask);

static void VIASubsequentSolidFillRect(
    ScrnInfoPtr pScrn,
    int x,
    int y,
    int w,
    int h);

static void VIASetupForMono8x8PatternFill(
    ScrnInfoPtr pScrn,
    int pattern0,
    int pattern1,
    int fg,
    int bg,
    int rop,
    unsigned planemask);

static void VIASubsequentMono8x8PatternFillRect(
    ScrnInfoPtr pScrn,
    int patOffx,
    int patOffy,
    int x,
    int y,
    int w,
    int h);

static void VIASetupForColor8x8PatternFill(
    ScrnInfoPtr pScrn,
    int patternx,
    int patterny,
    int rop,
    unsigned planemask,
    int trans_color);

static void VIASubsequentColor8x8PatternFillRect(
    ScrnInfoPtr pScrn,
    int patOffx,
    int patOffy,
    int x,
    int y,
    int w,
    int h);


static void VIASetupForCPUToScreenColorExpandFill(
    ScrnInfoPtr pScrn,
    int fg,
    int bg,
    int rop,
    unsigned planemask);

static void VIASubsequentCPUToScreenColorExpandFill(
    ScrnInfoPtr pScrn,
    int x,
    int y,
    int w,
    int h,
    int skipleft);

static void VIASetupForScreenToScreenColorExpand(
    ScrnInfoPtr pScrn,
    int bg,
    int fg,
    int rop,
    unsigned planemask);

static void VIASubsequentScreenToScreenColorExpand(
    ScrnInfoPtr pScrn,
    int x,
    int y,
    int w,
    int h,
    int srcx,
    int srcy,
    int offset);

static void VIASetupForImageWrite(
    ScrnInfoPtr pScrn,
    int rop,
    unsigned planemask,
    int trans_color,
    int bpp,
    int depth);

static void VIASubsequentImageWriteRect(
    ScrnInfoPtr pScrn,
    int x,
    int y,
    int w,
    int h,
    int skipleft);

static void
VIASetupForSolidLine(
    ScrnInfoPtr pScrn,
    int color,
    int rop,
    unsigned int planemask);

static void
VIASubsequentSolidTwoPointLine(
    ScrnInfoPtr pScrn,
    int x1,
    int y1,
    int x2,
    int y2,
    int flags);

static void
VIASubsequentSolidHorVertLine(
    ScrnInfoPtr pScrn,
    int x,
    int y,
    int len,
    int dir);

static void
VIASetupForDashedLine(
    ScrnInfoPtr pScrn,
    int fg,
    int bg,
    int rop,
    unsigned int planemask,
    int length,
    unsigned char *pattern);

static void
VIASubsequentDashedTwoPointLine(
    ScrnInfoPtr pScrn,
    int x1,
    int y1,
    int x2,
    int y2,
    int flags,
    int phase);

static void VIASetClippingRectangle(
    ScrnInfoPtr pScrn,
    int x1,
    int y1,
    int x2,
    int y2);

static void VIADisableClipping( ScrnInfoPtr );
static void VIARestoreAccelState(ScrnInfoPtr pScrn);

extern unsigned int via_xf86_version_current;


static void
VIASetClippingRectangle_M1(
    ScrnInfoPtr pScrn,
    int left,   /* X1 */
    int top,    /* Y1 */
    int right,  /* X2 */
    int bottom) /* Y2 */
{
    VIAPtr pVia = VIAPTR(pScrn);    
    VIABIOSInfoPtr  pBIOSInfo;
    pBIOSInfo = pVia->pBIOSInfo;    
    CARD32 dwGEMode=0;
    volatile unsigned int* pBuf;
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 3);  

    switch (pScrn->bitsPerPixel)
    {
        case 16:
            dwGEMode |= VIA_GEM_16bpp;
            break;
        case 32:
            dwGEMode |= VIA_GEM_32bpp;
            break;
        default:
            dwGEMode |= VIA_GEM_8bpp;
            break;
    }        

#ifdef DEBUG_EXTRA
    ErrorF("ClipRect, (%4d,%4d)-(%4d,%4d) \n", x1, y1, x2, y2 );
#endif
    
    /* Set BPP and Pitch */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GEMODE_M1, dwGEMode);    
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_CLIPTL_M1, ((top << 16) | left));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_CLIPBR_M1, ((bottom << 16) | right)); 
    
    pVia->SavedCmd |= VIA_GEC_CLIP_ENABLE;
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        
}


static void
VIASetupForScreenToScreenCopy_M1(
    ScrnInfoPtr pScrn,
    int xdir,
    int ydir,
    int rop,
    unsigned planemask,
    int trans_color)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd, dwGEMode = 0;
    volatile unsigned int* pBuf;

#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif

    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    if (trans_color != -1)
    {
        ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 3);      
    }
    else
    {
        ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 2);    
    }

    
    switch (pScrn->bitsPerPixel)
    {
        case 16:
            dwGEMode |= VIA_GEM_16bpp;
            break;
        case 32:
            dwGEMode |= VIA_GEM_32bpp;
            break;
        default:
            dwGEMode |= VIA_GEM_8bpp;
            break;
    }

#ifdef X_HAVE_XAAGETROP
    cmd = VIA_GEC_BLT | (XAAGetCopyROP(rop) << 24);
#else
    cmd = VIA_GEC_BLT | (XAACopyROP[rop] << 24);
#endif    

    if (pVia->IsHWRotateEnabled)
    {                
        ConvertDirection(pScrn, xdir, ydir, &cmd);            
    }
    else        
    {
        if (xdir < 0)
            cmd |= VIA_GEC_DECX;

        if (ydir < 0)
            cmd |= VIA_GEC_DECY;    
    }    
    
    /* Set BPP and Pitch */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GEMODE_M1, dwGEMode);
    
    pVia->SavedCmd = cmd;

    if (trans_color != -1)
    {
        /* Transparent Bitblt */
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCCOLORKEY_M1, trans_color);        
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_KEYCONTROL_M1, 0x4000);        
    }
    else
    {
        /* Disable Transparent Bitblt */
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_KEYCONTROL_M1, 0x0);
    }

    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASubsequentScreenToScreenCopy_M1(
    ScrnInfoPtr pScrn,
    int x1,
    int y1,
    int x2,
    int y2,
    int w,
    int h)
{    
    VIAPtr pVia = VIAPTR(pScrn);    
    volatile unsigned int* pBuf;
#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 7);

    WaitIdle();

    if (pVia->IsHWRotateEnabled)
    {                        
        ConvertSize(pScrn, &w, &h);
        ConvertCoordinatesTwo(pScrn, &x1, &y1, &x2, &y2, w, h);        
    }   

    if (!w || !h)
        return;

    if (pVia->SavedCmd & VIA_GEC_DECX) {
        x1 += (w - 1);
        x2 += (w - 1);
    }

    if (pVia->SavedCmd & VIA_GEC_DECY) {
        y1 += (h - 1);
        y2 += (h - 1);
    }

    /* Set Src and Dst base address and pitch, pitch is qword */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH_M1, ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCPOS_M1, ((y1 << 16) | x1));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTPOS_M1, ((y2 << 16) | x2));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION_M1, (((h - 1) << 16) | (w - 1)));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD_M1, pVia->SavedCmd);
	
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);}


static void
VIASetupForSolidFill_M1(
    ScrnInfoPtr pScrn,
    int color,
    int rop,
    unsigned planemask)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd, dwGEMode=0;
    volatile unsigned int* pBuf;

#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 1);
	
    switch (pScrn->bitsPerPixel)
    {
        case 16:
            dwGEMode |= VIA_GEM_16bpp;
            break;
        case 32:
            dwGEMode |= VIA_GEM_32bpp;
            break;
        default:
            dwGEMode |= VIA_GEM_8bpp;
            break;
    }        

    cmd = VIA_GEC_BLT | VIA_GEC_FIXCOLOR_PAT | 
#ifdef X_HAVE_XAAGETROP
    (XAAGetPatternROP(rop) << 24);
#else
    (XAAPatternROP[rop] << 24);
#endif                

    /* Set BPP and Pitch */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GEMODE_M1, dwGEMode);    

    pVia->SavedCmd = cmd;    
    pVia->SavedFgColor = color;    
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASubsequentSolidFillRect_M1(
    ScrnInfoPtr pScrn,
    int x,
    int y,
    int w,
    int h)
{
    VIAPtr pVia = VIAPTR(pScrn);
    volatile unsigned int* pBuf;

#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 7);

    if (pVia->IsHWRotateEnabled)
    {            
        ConvertSize(pScrn, &w, &h);
        ConvertCoordinatesOne(pScrn, &x, &y, w, h);        
    }

    if (!w || !h)
        return;

    /* Set Src and Dst base address and pitch, pitch is qword */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH_M1, ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTPOS_M1, ((y << 16) | x));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION_M1, (((h - 1) << 16) | (w - 1)));       
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_MONOPATFGC_M1, pVia->SavedFgColor);     
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD_M1, pVia->SavedCmd);
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASetupForMono8x8PatternFill_M1(
    ScrnInfoPtr pScrn,
    int pattern0,
    int pattern1,
    int fg,
    int bg,
    int rop,
    unsigned planemask)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd, dwGEMode = 0;
    volatile unsigned int* pBuf;
#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif

    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 1);
	
    switch (pScrn->bitsPerPixel)
    {
        case 16:
            dwGEMode |= VIA_GEM_16bpp;
            break;
        case 32:
            dwGEMode |= VIA_GEM_32bpp;
            break;
        default:
            dwGEMode |= VIA_GEM_8bpp;
            break;
    }    

    cmd = VIA_GEC_BLT | VIA_GEC_PAT_REG | VIA_GEC_PAT_MONO |
#ifdef X_HAVE_XAAGETROP
    (XAAGetPatternROP(rop) << 24);
#else
    (XAAPatternROP[rop] << 24);
#endif

    if (bg == -1) 
    {
        /* transparent mono pattern */
        cmd |= VIA_GEC_MPAT_TRANS;
    }        

    /* Set BPP and Pitch */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GEMODE_M1, dwGEMode);
    pVia->SavedCmd = cmd;
    pVia->SavedFgColor = fg;
    pVia->SavedBgColor = bg;
    pVia->SavedPattern0 = pattern0;
    pVia->SavedPattern1 = pattern1;
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASubsequentMono8x8PatternFillRect_M1(
    ScrnInfoPtr pScrn,
    int patOffx,
    int patOffy,
    int x,
    int y,
    int w,
    int h)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  dwPatOffset;
    volatile unsigned int* pBuf;

#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif

    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 11);
	
    if (pVia->IsHWRotateEnabled)
    {        
        ConvertSize(pScrn, &w, &h);
        ConvertCoordinatesOne(pScrn, &x, &y, w, h);  
    }
    
    if (!w || !h)
        return;

    dwPatOffset = ((patOffy & 0x7)  << 29) | ((patOffx & 0x7) << 26);

    /* Set Src and Dst base address and pitch, pitch is qword */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH_M1, ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTPOS_M1, ((y << 16) | x));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION_M1, (((h - 1) << 16) | (w - 1)));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PATADDR_M1, dwPatOffset);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_MONOPATFGC_M1, pVia->SavedFgColor);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_MONOPATBGC_M1, pVia->SavedBgColor);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_MONOPAT0_M1, pVia->SavedPattern0);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_MONOPAT1_M1, pVia->SavedPattern1);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD_M1, pVia->SavedCmd);
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}

/* xaa Color8x8 cache bug fixed in cvs tag xf4 3 99 14 */ 
static void
VIASetupForColor8x8PatternFill_M1(
    ScrnInfoPtr pScrn,
    int patternx,
    int patterny,
    int rop,
    unsigned planemask,
    int trans_color)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd, dwGEMode = 0;
    volatile unsigned int* pBuf;

#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 1);
	
    switch (pScrn->bitsPerPixel)
    {
        case 16:
            dwGEMode |= VIA_GEM_16bpp;
            break;
        case 32:
            dwGEMode |= VIA_GEM_32bpp;
            break;
        default:
            dwGEMode |= VIA_GEM_8bpp;
            break;
    }

#ifdef X_HAVE_XAAGETROP
    cmd = VIA_GEC_BLT | (XAAGetPatternROP(rop) << 24);
#else
    cmd = VIA_GEC_BLT | (XAAPatternROP[rop] << 24);
#endif    

    /* Set BPP and Pitch */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GEMODE_M1, dwGEMode);

    pVia->SavedCmd = cmd;
    pVia->SavedPatternAddr = (patternx * pVia->Bpp + patterny * pVia->Bpl);
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASubsequentColor8x8PatternFillRect_M1(
    ScrnInfoPtr pScrn,
    int patOffx,
    int patOffy,
    int x,
    int y,
    int w,
    int h)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  dwPatAddr;
    volatile unsigned int* pBuf;

#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif

    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 7);
    if (pVia->IsHWRotateEnabled)
    {        
        ConvertSize(pScrn, &w, &h);
        ConvertCoordinatesOne(pScrn, &x, &y, w, h);  
    }
    
    if (!w || !h)
        return;

    dwPatAddr = (pVia->SavedPatternAddr >> 3) |
                ((patOffy & 0x7)  << 29) | ((patOffx & 0x7) << 26);

    /* Set Src and Dst base address and pitch, pitch is qword */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH_M1, ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTPOS_M1, ((y << 16) | x));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION_M1, (((h - 1) << 16) | (w - 1)));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PATADDR_M1, dwPatAddr);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD_M1, pVia->SavedCmd);
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASetupForSolidLine_M1(
    ScrnInfoPtr pScrn,
    int color,
    int rop,
    unsigned int planemask)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd, dwGEMode= 0;
    volatile unsigned int* pBuf;
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 3);

#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif
    switch (pScrn->bitsPerPixel)
    {
        case 16:
            dwGEMode |= VIA_GEM_16bpp;
            break;
        case 32:
            dwGEMode |= VIA_GEM_32bpp;
            break;
        default:
            dwGEMode |= VIA_GEM_8bpp;
            break;
    }        

    /* we move VIA_GEC_LINE from here to the place firing command */
#ifdef X_HAVE_XAAGETROP
    cmd = VIA_GEC_FIXCOLOR_PAT | (XAAGetPatternROP(rop) << 24);
#else
    cmd = VIA_GEC_FIXCOLOR_PAT | (XAAPatternROP[rop] << 24);
#endif        

    /* Set BPP and Pitch */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GEMODE_M1, dwGEMode);

    pVia->SavedCmd = cmd;
    pVia->SavedFgColor = color;    
    
    /* set solid line pattern */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_MONOPAT0_M1, 0xFF);    
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_MONOPATFGC_M1, pVia->SavedFgColor);    
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASubsequentSolidTwoPointLine_M1(
    ScrnInfoPtr pScrn,
    int x1,
    int y1,
    int x2,
    int y2,
    int flags)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd;
    int     dx, dy, tmp, error = 1;
    volatile unsigned int* pBuf;

#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif

    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 8);
    if (pVia->IsHWRotateEnabled)
    {          
        ConvertCoordinatesTwo(pScrn, &x1, &y1, &x2, &y2, 0, 0);
    } 
       
    cmd = pVia->SavedCmd | VIA_GEC_LINE;

    dx = x2 - x1;
    if (dx < 0) 
    {
        dx = -dx;
        cmd |= VIA_GEC_DECX;            /* line will be drawn from right */
        error = 0;
    }

    dy = y2 - y1;
    if (dy < 0) 
    {
        dy = -dy;
        cmd |= VIA_GEC_DECY;            /* line will be drawn from bottom */
    }

    if (dy > dx) 
    {
        tmp  = dy;
        dy = dx;
        dx = tmp;                       /* Swap 'dx' and 'dy' */
        cmd |= VIA_GEC_Y_MAJOR;         /* Y major line */
    }

    if (flags & OMIT_LAST) 
    {
        cmd |= VIA_GEC_LASTPIXEL_OFF;
    }

    /* Set Src and Dst base address and pitch, pitch is qword */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH_M1, ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));
    
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_LINE_K1K2_M1, ((((dy << 1) & 0x3fff) << 16)|
              (((dy - dx) << 1) & 0x3fff)));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_LINE_XY_M1, ((y1 << 16) | x1));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION_M1, dx);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_LINE_ERROR_M1, (((dy << 1) - dx - error) & 0x3fff));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD_M1, cmd);  
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


/* Subsequent XAA solid horizontal and vertical lines */
static void
VIASubsequentSolidHorVertLine_M1(
    ScrnInfoPtr pScrn,
    int x,
    int y,
    int len,
    int dir)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo;
    pBIOSInfo = pVia->pBIOSInfo;    
    int LineDegree = dir;
    volatile unsigned int* pBuf;
    
#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif

    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 6);
    if (pVia->IsHWRotateEnabled)
    {           
        ConvertCoordinatesOne(pScrn, &x, &y, 0, 0);
        
        switch (pBIOSInfo->HWRotate)
        {
            case VIA_ROTATE_DEGREE_90:
                if (LineDegree == DEGREES_0) 
                {
                    LineDegree = DEGREES_270;
                }
                else if (LineDegree == DEGREES_270) 
                {
                    LineDegree = DEGREES_0;
                    x -= len;
                }
                break;
            case VIA_ROTATE_DEGREE_180:
                if (LineDegree == DEGREES_0) 
                {                    
                    x -= len;
                }
                else if (LineDegree == DEGREES_270) 
                {                    
                    y -= len;
                }                
                break;
            case VIA_ROTATE_DEGREE_270:
                if (LineDegree == DEGREES_0) 
                {
                    LineDegree = DEGREES_270;
                    y -= len;
                }
                else if (LineDegree == DEGREES_270) 
                {
                    LineDegree = DEGREES_0;
                }
                break;
        }
    }    

    /* Set Src and Dst base address and pitch, pitch is qword */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH_M1, ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));

    
    if (LineDegree == DEGREES_0)
    {
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTPOS_M1, ((y << 16) | x));
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION_M1, (len - 1));
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD_M1, pVia->SavedCmd | VIA_GEC_BLT);
    }
    else
    {
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTPOS_M1, ((y << 16) | x));
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION_M1, ((len - 1) << 16));
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD_M1, pVia->SavedCmd | VIA_GEC_BLT);
    }    
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}

static void
VIASetupForDashedLine_M1(
    ScrnInfoPtr pScrn,
    int fg,
    int bg,
    int rop,
    unsigned int planemask,
    int length,
    unsigned char *pattern)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  pat = *(CARD32 *)pattern;
    CARD32  cmd, dwGEMode = 0;
    volatile unsigned int* pBuf;

#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif

    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 4);
	
    switch (pScrn->bitsPerPixel)
    {
        case 16:
            dwGEMode |= VIA_GEM_16bpp;
            break;
        case 32:
            dwGEMode |= VIA_GEM_32bpp;
            break;
        default:
            dwGEMode |= VIA_GEM_8bpp;
            break;
    }
    
    /* Set BPP and Pitch */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GEMODE_M1, dwGEMode);

#ifdef X_HAVE_XAAGETROP
    cmd = VIA_GEC_LINE | VIA_GEC_FIXCOLOR_PAT | (XAAGetPatternROP(rop) << 24);
#else
    cmd = VIA_GEC_LINE | VIA_GEC_FIXCOLOR_PAT | (XAAPatternROP[rop] << 24);
#endif

    if (bg == -1) 
    {
        /* transparent mono pattern */
        cmd |= VIA_GEC_MPAT_TRANS;
    }

    pVia->SavedCmd = cmd;
    pVia->SavedFgColor = fg;
    pVia->SavedBgColor = bg;

    switch (length)
    {
        case  2: pat |= pat <<  2; /* fall through */
        case  4: pat |= pat <<  4; /* fall through */
        case  8: pat |= pat <<  8; /* fall through */
        case 16: pat |= pat << 16;
    }

    pVia->SavedPattern0 = pat;

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_MONOPATFGC_M1, pVia->SavedFgColor);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_MONOPATBGC_M1, pVia->SavedBgColor);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_MONOPAT0_M1, pVia->SavedPattern0);
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASubsequentDashedTwoPointLine_M1(
    ScrnInfoPtr pScrn,
    int x1,
    int y1,
    int x2,
    int y2,
    int flags,
    int phase)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd;
    int     dx, dy, tmp, error = 1;
    volatile unsigned int* pBuf;

#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif

    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 8);
	
    if (pVia->IsHWRotateEnabled)
    {                
        ConvertCoordinatesTwo(pScrn, &x1, &y1, &x2, &y2, 0, 0);
    } 

    cmd = pVia->SavedCmd;

    dx = x2 - x1;
    if (dx < 0) {
        dx = -dx;
        cmd |= VIA_GEC_DECX;            /* line will be drawn from right */
        error = 0;
    }

    dy = y2 - y1;
    if (dy < 0) {
        dy = -dy;
        cmd |= VIA_GEC_DECY;            /* line will be drawn from bottom */
    }

    if (dy > dx) {
        tmp  = dy;
        dy = dx;
        dx = tmp;                       /* Swap 'dx' and 'dy' */
        cmd |= VIA_GEC_Y_MAJOR;         /* Y major line */
    }

    if (flags & OMIT_LAST) {
        cmd |= VIA_GEC_LASTPIXEL_OFF;
    }

    /* Set Src and Dst base address and pitch, pitch is qword */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH_M1, ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));
    
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_LINE_K1K2_M1, ((((dy << 1) & 0x3fff) << 16)|
              (((dy - dx) << 1) & 0x3fff)));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_LINE_XY_M1, ((y1 << 16) | x1));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION_M1, dx);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_LINE_ERROR_M1, (((dy << 1) - dx - error) & 0x3fff) | 0xFF0000);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD_M1, cmd);
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASetupForCPUToScreenColorExpandFill_M1(
    ScrnInfoPtr pScrn,
    int fg,
    int bg,
    int rop,
    unsigned planemask)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd, dwGEMode = 0;
    volatile unsigned int* pBuf;

#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif

    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 2);
	
    switch (pScrn->bitsPerPixel)
    {
        case 16:
            dwGEMode |= VIA_GEM_16bpp;
            break;
        case 32:
            dwGEMode |= VIA_GEM_32bpp;
            break;
        default:
            dwGEMode |= VIA_GEM_8bpp;
            break;
    }

    cmd = VIA_GEC_BLT | VIA_GEC_SRC_SYS | VIA_GEC_SRC_MONO |
          VIA_GEC_MONO_PACK | VIA_GEC_MONO_BYTE |
#ifdef X_HAVE_XAAGETROP
    (XAAGetCopyROP(rop) << 24);
#else
    (XAACopyROP[rop] << 24);
#endif

    if (bg == -1) {
        cmd |= VIA_GEC_MSRC_TRANS;
    }  
    
    if (pVia->IsHWRotateEnabled)
    {        
        Enable2DEngineRotate_M1(pScrn, &dwGEMode, &cmd);
    }        

    /* Set BPP and Pitch */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GEMODE_M1, dwGEMode);

    pVia->SavedCmd = cmd;
    pVia->SavedFgColor = fg;
    pVia->SavedBgColor = bg;

    /* Disable Transparent Bitblt */ 
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_KEYCONTROL_M1, 0x0);   
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASubsequentCPUToScreenColorExpandFill_M1(
    ScrnInfoPtr pScrn,
    int x,
    int y,
    int w,
    int h,
    int skipleft)
{
    VIAPtr pVia = VIAPTR(pScrn);
    XAAInfoRecPtr xaaptr = GET_XAAINFORECPTR_FROM_SCRNINFOPTR(pScrn);
    volatile unsigned int* pBuf;
    unsigned int size;

#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif            
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    /* XAA will be sending bitmap data next.  */
    /* We should probably wait for empty/idle here. */
    if (skipleft) {
        VIASetClippingRectangle_M1(pScrn, (x + skipleft), y, (x + w - 1), (y + h -1));        
    }     
    
    if (pVia->IsHWRotateEnabled)
    {            
        ConvertSize(pScrn, &w, &h);
        ConvertCoordinatesOne(pScrn, &x, &y, w, h);          
        pVia->SavedCmd |= VIA_GEC_ROT;
    }     
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 9);
         
    /* Set Src and Dst base address and pitch, pitch is qword */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH_M1, ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));    
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCPOS_M1, 0);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTPOS_M1, ((y << 16) | x));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION_M1, (((h - 1) << 16) | (w - 1)));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_FGCOLOR_M1, pVia->SavedFgColor);    
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_BGCOLOR_M1, pVia->SavedBgColor);   
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD_M1, pVia->SavedCmd);    
	
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = (pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf);
    size = ((w + 31) >> 5) * h;
	ISSUE_2D_DATA(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, size, xaaptr->ColorExpandBase);
}


static void
VIASetupForImageWrite_M1(
    ScrnInfoPtr pScrn,
    int rop,
    unsigned planemask,
    int trans_color,
    int bpp,
    int depth)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd, dwGEMode = 0;
    volatile unsigned int* pBuf;
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    if (trans_color != -1)
    {
        ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 3);
    }
    else
    {
        ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 2);
    }
    switch (bpp)
    {
        case 16:
            dwGEMode |= VIA_GEM_16bpp;
            break;
        case 32:
            dwGEMode |= VIA_GEM_32bpp;
            break;
        default:
            dwGEMode |= VIA_GEM_8bpp;
            break;
    }

#ifdef X_HAVE_XAAGETROP
    cmd = VIA_GEC_BLT | VIA_GEC_SRC_SYS | (XAAGetCopyROP(rop) << 24);
#else
    cmd = VIA_GEC_BLT | VIA_GEC_SRC_SYS | (XAACopyROP[rop] << 24);
#endif   

    if (pVia->IsHWRotateEnabled)
    {        
        Enable2DEngineRotate_M1(pScrn, &dwGEMode, &cmd);
    }
    
    /* Set BPP and Pitch */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GEMODE_M1, dwGEMode);

    pVia->SavedCmd = cmd;
    pVia->SavedBpp = bpp >> 3;

    if (trans_color != -1)
    {
        /* Transparent Bitblt */
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCCOLORKEY_M1, trans_color);        
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_KEYCONTROL_M1, 0x4000);
        
    }
    else
    {
        /* Disable Transparent Bitblt */
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_KEYCONTROL_M1, 0x0);
    }
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASubsequentImageWriteRect_M1(
    ScrnInfoPtr pScrn,
    int x,
    int y,
    int w,
    int h,
    int skipleft)
{
    VIAPtr  pVia = VIAPTR(pScrn);    
    volatile unsigned int* pBuf;
    unsigned int size;
    XAAInfoRecPtr xaaptr = GET_XAAINFORECPTR_FROM_SCRNINFOPTR(pScrn);
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    if (skipleft) {
        VIASetClippingRectangle_M1(pScrn, (x + skipleft), y, (x + w - 1), (y + h -1));
    }

    if (pVia->IsHWRotateEnabled)
    {        
        ConvertSize(pScrn, &w, &h);
        ConvertCoordinatesOne(pScrn, &x, &y, w, h);  
        pVia->SavedCmd |= VIA_GEC_ROT;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 7);

    /* Set Src and Dst base address and pitch, pitch is qword */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH_M1, ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCPOS_M1, 0);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTPOS_M1, ((y << 16) | x));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION_M1, (((h - 1) << 16) | (w - 1)));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD_M1, pVia->SavedCmd);
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = (pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf);
    size = ((w * pVia->SavedBpp) + 3) >> 2;

	ISSUE_2D_DATA(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, size, xaaptr->ImageWriteBase);
}



void
VIAInitialize2DEngine(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  dwVQStartAddr, dwVQEndAddr;
    CARD32  dwVQLen, dwVQStartL, dwVQEndL, dwVQStartEndH;
    CARD32  dwGEMode;

    /* init 2D engine regs to reset 2D engine */
    VIASETREG(0x04, 0x0);
    VIASETREG(0x08, 0x0);
    VIASETREG(0x0c, 0x0);
    VIASETREG(0x10, 0x0);
    VIASETREG(0x14, 0x0);
    VIASETREG(0x18, 0x0);
    VIASETREG(0x1c, 0x0);
    VIASETREG(0x20, 0x0);
    VIASETREG(0x24, 0x0);
    VIASETREG(0x28, 0x0);
    VIASETREG(0x2c, 0x0);
    VIASETREG(0x30, 0x0);
    VIASETREG(0x34, 0x0);
    VIASETREG(0x38, 0x0);
    VIASETREG(0x3c, 0x0);
    VIASETREG(0x40, 0x0);

    if (pVia->Chipset == VIA_VX800)
    {
        VIASETREG(0x44, 0x0);
        VIASETREG(0x48, 0x0);
        VIASETREG(0x4C, 0x0);
        VIASETREG(0x50, 0x0);
        VIASETREG(0x54, 0x0);
        VIASETREG(0x58, 0x0);
        VIASETREG(0x5C, 0x0);
    }

    if((pVia->Chipset!=VIA_K8M800)&&(b3DRegsInitialized==0))   
    {
        /*need to handle differently otherwise vexp will hang with K8M800*/
        /* Init AGP and VQ regs */
        switch( pVia->Chipset )  
        {
            case VIA_K8M890: 
            case VIA_P4M900:
                VIASETREG(0x41c, 0x00100000);
                VIASETREG(0x420, 0x680A0000);
                VIASETREG(0x420, 0x02000000);
                break; 

            default: 
                VIASETREG(0x43c, 0x00100000);
                VIASETREG(0x440, 0x00000000);
                VIASETREG(0x440, 0x00333004);
                VIASETREG(0x440, 0x60000000);
                VIASETREG(0x440, 0x61000000);
                VIASETREG(0x440, 0x62000000);
                VIASETREG(0x440, 0x63000000);
                VIASETREG(0x440, 0x64000000);
                VIASETREG(0x440, 0x7D000000);
                VIASETREG(0x43c, 0xfe020000);
                VIASETREG(0x440, 0x00000000);
                break;
        }  
        
        if (pVia->VQStart != 0)
        {
            /* Enable VQ */
            dwVQStartAddr = pVia->VQStart;
            dwVQEndAddr = pVia->VQEnd;
            switch ( pVia->Chipset )
            {
                case VIA_K8M890:
                case VIA_P4M900:  
                    dwVQStartL = 0x70000000 | (dwVQStartAddr & 0xFFFFFF);
                    dwVQEndL = 0x71000000 | (dwVQEndAddr & 0xFFFFFF);
                    dwVQStartEndH = 0x72000000 | ((dwVQStartAddr & 0xFF000000) >> 24) |
                            ((dwVQEndAddr & 0xFF000000) >> 16);
                    dwVQLen = 0x73000000 | (VIA_VQ_SIZE >> 3);
                    break;

                default:
                    dwVQStartL = 0x50000000 | (dwVQStartAddr & 0xFFFFFF);
                    dwVQEndL = 0x51000000 | (dwVQEndAddr & 0xFFFFFF);
                    dwVQStartEndH = 0x52000000 | ((dwVQStartAddr & 0xFF000000) >> 24) |
                            ((dwVQEndAddr & 0xFF000000) >> 16);
                    dwVQLen = 0x53000000 | (VIA_VQ_SIZE >> 3);
                    break; 
            }
        
            switch ( pVia->Chipset )
            { 
                case VIA_K8M890:
                case VIA_P4M900: 
                    VIASETREG(0x41c, 0x00100000);
                    VIASETREG(0x420, dwVQStartEndH);
                    VIASETREG(0x420, dwVQStartL);
                    VIASETREG(0x420, dwVQEndL);
                    VIASETREG(0x420, dwVQLen);
                    VIASETREG(0x420, 0x74301001);
                    VIASETREG(0x420, 0x00000000);
                    break;
                
                default: 
                    VIASETREG(0x43c, 0x00fe0000);
                    VIASETREG(0x440, 0x080003fe);
                    VIASETREG(0x440, 0x0a00027c);
                    VIASETREG(0x440, 0x0b000260);
                    VIASETREG(0x440, 0x0c000274);
                    VIASETREG(0x440, 0x0d000264);
                    VIASETREG(0x440, 0x0e000000);
                    VIASETREG(0x440, 0x0f000020);
                    VIASETREG(0x440, 0x1000027e);
                    VIASETREG(0x440, 0x110002fe);
                    VIASETREG(0x440, 0x200f0060);

                    VIASETREG(0x440, 0x00000006);
                    VIASETREG(0x440, 0x40008c0f);
                    VIASETREG(0x440, 0x44000000);
                    VIASETREG(0x440, 0x45080c04);
                    VIASETREG(0x440, 0x46800408);

                    VIASETREG(0x440, dwVQStartEndH);
                    VIASETREG(0x440, dwVQStartL);
                    VIASETREG(0x440, dwVQEndL);
                    VIASETREG(0x440, dwVQLen);
                    break;
            }
        }
        else
        {
            /* Diable VQ */
            switch ( pVia->Chipset )
            {
                case VIA_K8M890: 	      
                case VIA_P4M900:  
                    VIASETREG(0x41c, 0x00100000);
                    VIASETREG(0x420, 0x74301000);
                    break;
                
                default: 
                    VIASETREG(0x43c, 0x00fe0000);
                    VIASETREG(0x440, 0x00000004);
                    VIASETREG(0x440, 0x40008c0f);
                    VIASETREG(0x440, 0x44000000);
                    VIASETREG(0x440, 0x45080c04);
                    VIASETREG(0x440, 0x46800408);
                    break;
            }   
        }
    }

    dwGEMode = 0;

    switch (pScrn->bitsPerPixel)
    {
        case 16:
            dwGEMode |= VIA_GEM_16bpp;
            break;
        case 32:
            dwGEMode |= VIA_GEM_32bpp;
            break;
        default:
            dwGEMode |= VIA_GEM_8bpp;
            break;
    }

    
    if (pVia->Chipset == VIA_VX800)
    {
        /* Set BPP and Pitch */
        VIASETREG(VIA_REG_GEMODE_M1, dwGEMode);
        /* Set Src and Dst base address and pitch, pitch is qword */
        VIASETREG(VIA_REG_SRCBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
        VIASETREG(VIA_REG_DSTBASE_M1, 0x0 + (pScrn->fbOffset >> 3));
        VIASETREG(VIA_REG_PITCH_M1, ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
                  (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));
    }
    else
    {
        VIASETREG(VIA_REG_GEMODE, dwGEMode);

        /* Set Src and Dst base address and pitch, pitch is qword */
        VIASETREG(VIA_REG_SRCBASE, 0x0 + (pScrn->fbOffset >> 3));
        VIASETREG(VIA_REG_DSTBASE, 0x0 + (pScrn->fbOffset >> 3));
        VIASETREG(VIA_REG_PITCH, VIA_PITCH_ENABLE |
                  ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
                  (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));

        /* Set rotation temporary memory base and pitch. */
        if (pVia->IsHWRotateEnabled)
        {
            VIASETREG(VIA_REG_ROT_TMP_ADDR, pVia->RotateFBStart);
            VIASETREG(VIA_REG_ROT_TMP_PITCH, (pScrn->displayWidth * (pScrn->bitsPerPixel/8))>>3);
        }
    }   
}



/* Acceleration init function, sets up pointers to our accelerated functions */
Bool
VIAInitAccel(ScreenPtr pScreen)
{
    ScrnInfoPtr     pScrn = xf86Screens[pScreen->myNum];
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    ScrnInfoPtr pScrn0 = xf86Screens[0];
    VIAPtr pVia0 = VIAPTR(pScrn0);
    XAAInfoRecPtr   xaaptr;
    BoxRec AvailFBArea;
    DevUnion* pPriv;
    VIAEntPtr pVIAEnt;

    /* reserve for frame buffer device driver */
    pVia->FBFreeEnd -= VIA_FB_CURSOR_SIZE;

    if(!pVia->IsSecondary)
    {
        pVia->VQStart = 0;
        if (((pVia->FBFreeEnd - pVia->FBFreeStart) >= VIA_VQ_SIZE) &&                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
            pVia->VQEnable)
        {
            /* Reserved space for VQ */
            pVia->VQStart = pVia->FBFreeEnd - VIA_VQ_SIZE;
            pVia->VQEnd = pVia->VQStart + VIA_VQ_SIZE - 1;
            pVia->FBFreeEnd -= VIA_VQ_SIZE;
        }

        /* Reserved space for rotate temporary memory */
        if (pVia->Chipset != VIA_VX800)
        {
            if ((pVia->IsHWRotateEnabled) || (pVia->DynamicRotation))
            {        
                pVia->RotateFBStart = pVia->FBFreeEnd - (pScrn->virtualX * pScrn->virtualY * (pScrn->bitsPerPixel/8));
                pVia->RotateFBEnd = pVia->RotateFBStart + (pScrn->virtualX * pScrn->virtualY * (pScrn->bitsPerPixel/8)) - 1;
                pVia->FBFreeEnd -= (pScrn->virtualX * pScrn->virtualY * (pScrn->bitsPerPixel/8));
            }
        }
    }
    /* The second Scrn just need to copy the reserved buffer address from the first scrn */
    else
    {
        pVia->VQStart = pVia0->VQStart;
        pVia->VQEnd = pVia0->VQEnd;
        if (pVia->Chipset != VIA_VX800)
        {
            pVia->RotateFBStart = pVia0->RotateFBStart;
            pVia->RotateFBEnd = pVia0->RotateFBEnd;
        }
    }

    VIAInitialize2DEngine(pScrn);

    if (pScrn->depth == 8) {
        pVia->PlaneMask = 0xFF;
    }
    else if (pScrn->depth == 15) {
        pVia->PlaneMask = 0x7FFF;
    }
    else if (pScrn->depth == 16) {
        pVia->PlaneMask = 0xFFFF;
    }
    else if (pScrn->depth == 24) {
        pVia->PlaneMask = 0xFFFFFF;
    }

    /* General acceleration flags */
    if (!(xaaptr = pVia->AccelInfoRec = XAACreateInfoRec()))
        return FALSE;

    xaaptr->Flags = PIXMAP_CACHE |
                    OFFSCREEN_PIXMAPS |
                    LINEAR_FRAMEBUFFER |
                    MICROSOFT_ZERO_LINE_BIAS |
                    0;

    if (pScrn->bitsPerPixel == 8)
        xaaptr->CachePixelGranularity = 128;

    xaaptr->Sync = VIAAccelSync;    

    /* Clipping */
    if (pVia->Chipset == VIA_VX800)
    {   
        /* Marked the following code temportary.*/
        if ((!pVia->IsHWRotateEnabled) && (!pVia->DynamicRotation))
        {
            xaaptr->SetClippingRectangle = VIASetClippingRectangle_M1;
            xaaptr->DisableClipping = VIADisableClipping;
            xaaptr->ClippingFlags = HARDWARE_CLIP_SOLID_FILL |
                                    HARDWARE_CLIP_SOLID_LINE |
                                    HARDWARE_CLIP_DASHED_LINE |
                                    HARDWARE_CLIP_SCREEN_TO_SCREEN_COPY |
                                    HARDWARE_CLIP_MONO_8x8_FILL |
                                    HARDWARE_CLIP_COLOR_8x8_FILL |
                                    HARDWARE_CLIP_SCREEN_TO_SCREEN_COLOR_EXPAND |
                                    0;
        }

        /* ScreenToScreen copies */
        xaaptr->SetupForScreenToScreenCopy = VIASetupForScreenToScreenCopy_M1;
        xaaptr->SubsequentScreenToScreenCopy = VIASubsequentScreenToScreenCopy_M1;
        xaaptr->ScreenToScreenCopyFlags = NO_PLANEMASK | ROP_NEEDS_SOURCE;

        /* Solid filled rectangles */
        xaaptr->SetupForSolidFill = VIASetupForSolidFill_M1;
        xaaptr->SubsequentSolidFillRect = VIASubsequentSolidFillRect_M1;
        xaaptr->SolidFillFlags = NO_PLANEMASK | ROP_NEEDS_SOURCE;

        /* Mono 8x8 pattern fills */
        xaaptr->SetupForMono8x8PatternFill = VIASetupForMono8x8PatternFill_M1;
        xaaptr->SubsequentMono8x8PatternFillRect =
                VIASubsequentMono8x8PatternFillRect_M1;
        xaaptr->Mono8x8PatternFillFlags = NO_PLANEMASK |
                                          HARDWARE_PATTERN_PROGRAMMED_BITS |
                                          HARDWARE_PATTERN_PROGRAMMED_ORIGIN |
                                          BIT_ORDER_IN_BYTE_MSBFIRST |
                                          0;

        if (via_xf86_version_current > XF86_VERSION_NUMERIC(4,3,99,14,0))
        {
            /* Color 8x8 pattern fills */
            xaaptr->SetupForColor8x8PatternFill = VIASetupForColor8x8PatternFill_M1;
            xaaptr->SubsequentColor8x8PatternFillRect =
                    VIASubsequentColor8x8PatternFillRect_M1;
            xaaptr->Color8x8PatternFillFlags = NO_PLANEMASK |
                                            NO_TRANSPARENCY |
                                            HARDWARE_PATTERN_PROGRAMMED_BITS |
                                            HARDWARE_PATTERN_PROGRAMMED_ORIGIN |
                                            0;
        }  

        
        /* Solid lines */
        xaaptr->SetupForSolidLine = VIASetupForSolidLine_M1;
        xaaptr->SubsequentSolidTwoPointLine = VIASubsequentSolidTwoPointLine_M1;        
        xaaptr->SubsequentSolidHorVertLine = VIASubsequentSolidHorVertLine_M1;    
        xaaptr->SolidBresenhamLineErrorTermBits = 14;
        xaaptr->SolidLineFlags = NO_PLANEMASK | ROP_NEEDS_SOURCE;

        /* dashed line */
        xaaptr->SetupForDashedLine = VIASetupForDashedLine_M1;
        xaaptr->SubsequentDashedTwoPointLine = VIASubsequentDashedTwoPointLine_M1;
        xaaptr->DashPatternMaxLength = 8;
        xaaptr->DashedLineFlags = NO_PLANEMASK |
                                  ROP_NEEDS_SOURCE |
                                  LINE_PATTERN_POWER_OF_2_ONLY |
                                  LINE_PATTERN_MSBFIRST_LSBJUSTIFIED |
                                  0;

        /* CPU to Screen color expansion */
        xaaptr->CPUToScreenColorExpandFillFlags = NO_PLANEMASK |
                                                  CPU_TRANSFER_PAD_DWORD |
                                                  SCANLINE_PAD_DWORD |
                                                  BIT_ORDER_IN_BYTE_MSBFIRST |
                                                  LEFT_EDGE_CLIPPING |
                                                  ROP_NEEDS_SOURCE |
                                                  0;

        xaaptr->SetupForCPUToScreenColorExpandFill =
                VIASetupForCPUToScreenColorExpandFill_M1;
        xaaptr->SubsequentCPUToScreenColorExpandFill =
                VIASubsequentCPUToScreenColorExpandFill_M1;

        /* ImageWrite */
        xaaptr->ImageWriteFlags = NO_PLANEMASK |
                                  CPU_TRANSFER_PAD_DWORD |
                                  SCANLINE_PAD_DWORD |
                                  BIT_ORDER_IN_BYTE_MSBFIRST |
                                  LEFT_EDGE_CLIPPING |
                                  ROP_NEEDS_SOURCE |
                                  NO_GXCOPY |
                                  /*SYNC_AFTER_IMAGE_WRITE |*/
                                  0;
        
        xaaptr->SetupForImageWrite = VIASetupForImageWrite_M1;
        xaaptr->SubsequentImageWriteRect = VIASubsequentImageWriteRect_M1;
        xaaptr->ImageWriteBase = pVia->BltBase;
        xaaptr->ImageWriteRange = VIA_MMIO_BLTSIZE;
        
    }
    else
    {
        xaaptr->SetClippingRectangle = VIASetClippingRectangle;
        xaaptr->DisableClipping = VIADisableClipping;
        xaaptr->ClippingFlags = HARDWARE_CLIP_SOLID_FILL |
                                HARDWARE_CLIP_SOLID_LINE |
                                HARDWARE_CLIP_DASHED_LINE |
                                HARDWARE_CLIP_SCREEN_TO_SCREEN_COPY |
                                HARDWARE_CLIP_MONO_8x8_FILL |
                                HARDWARE_CLIP_COLOR_8x8_FILL |
                                HARDWARE_CLIP_SCREEN_TO_SCREEN_COLOR_EXPAND |
                                0;
        /* ScreenToScreen copies */
        xaaptr->SetupForScreenToScreenCopy = VIASetupForScreenToScreenCopy;
        xaaptr->SubsequentScreenToScreenCopy = VIASubsequentScreenToScreenCopy;
        xaaptr->ScreenToScreenCopyFlags = NO_PLANEMASK | ROP_NEEDS_SOURCE;

        /* Solid filled rectangles */
        xaaptr->SetupForSolidFill = VIASetupForSolidFill;
        xaaptr->SubsequentSolidFillRect = VIASubsequentSolidFillRect;
        xaaptr->SolidFillFlags = NO_PLANEMASK | ROP_NEEDS_SOURCE;

        /* Mono 8x8 pattern fills */
        xaaptr->SetupForMono8x8PatternFill = VIASetupForMono8x8PatternFill;
        xaaptr->SubsequentMono8x8PatternFillRect =
                VIASubsequentMono8x8PatternFillRect;
        xaaptr->Mono8x8PatternFillFlags = NO_PLANEMASK |
                                          HARDWARE_PATTERN_PROGRAMMED_BITS |
                                          HARDWARE_PATTERN_PROGRAMMED_ORIGIN |
                                          BIT_ORDER_IN_BYTE_MSBFIRST |
                                          0;

        /* xaa Color8x8 cache bug fixed in cvs tag xf4 3 99 14 */ 
        if (via_xf86_version_current > XF86_VERSION_NUMERIC(4,3,99,14,0))
        {
            /* Color 8x8 pattern fills */
            xaaptr->SetupForColor8x8PatternFill = VIASetupForColor8x8PatternFill;
            xaaptr->SubsequentColor8x8PatternFillRect =
                    VIASubsequentColor8x8PatternFillRect;
            xaaptr->Color8x8PatternFillFlags = NO_PLANEMASK |
                                            NO_TRANSPARENCY |
                                            HARDWARE_PATTERN_PROGRAMMED_BITS |
                                            HARDWARE_PATTERN_PROGRAMMED_ORIGIN |
                                            0;
        }    

        /* 2D Rotate Engine doesn't support line function. */
        if ((!pVia->IsHWRotateEnabled) && (!pVia->DynamicRotation))
        { 
            /* Solid lines */
            xaaptr->SetupForSolidLine = VIASetupForSolidLine;
            xaaptr->SubsequentSolidTwoPointLine = VIASubsequentSolidTwoPointLine;        
            xaaptr->SubsequentSolidHorVertLine = VIASubsequentSolidHorVertLine;    
            xaaptr->SolidBresenhamLineErrorTermBits = 14;
            xaaptr->SolidLineFlags = NO_PLANEMASK | ROP_NEEDS_SOURCE;

            /* dashed line */
            xaaptr->SetupForDashedLine = VIASetupForDashedLine;
            xaaptr->SubsequentDashedTwoPointLine = VIASubsequentDashedTwoPointLine;
            xaaptr->DashPatternMaxLength = 8;
            xaaptr->DashedLineFlags = NO_PLANEMASK |
                                      ROP_NEEDS_SOURCE |
                                      LINE_PATTERN_POWER_OF_2_ONLY |
                                      LINE_PATTERN_MSBFIRST_LSBJUSTIFIED |
                                      0;
        }

        /* CPU to Screen color expansion */
        xaaptr->CPUToScreenColorExpandFillFlags = NO_PLANEMASK |
                                                  CPU_TRANSFER_PAD_DWORD |
                                                  SCANLINE_PAD_DWORD |
                                                  BIT_ORDER_IN_BYTE_MSBFIRST |
                                                  LEFT_EDGE_CLIPPING |
                                                  ROP_NEEDS_SOURCE |
                                                  0;

        xaaptr->SetupForCPUToScreenColorExpandFill =
                VIASetupForCPUToScreenColorExpandFill;
        xaaptr->SubsequentCPUToScreenColorExpandFill =
                VIASubsequentCPUToScreenColorExpandFill;
        xaaptr->ColorExpandBase = pVia->BltBase;
        xaaptr->ColorExpandRange = (64 * 1024);

        /* ImageWrite */
        xaaptr->ImageWriteFlags = NO_PLANEMASK |
                                  CPU_TRANSFER_PAD_DWORD |
                                  SCANLINE_PAD_DWORD |
                                  BIT_ORDER_IN_BYTE_MSBFIRST |
                                  LEFT_EDGE_CLIPPING |
                                  ROP_NEEDS_SOURCE |
                                  NO_GXCOPY |
                                  /*SYNC_AFTER_IMAGE_WRITE |*/
                                  0;
        
        xaaptr->SetupForImageWrite = VIASetupForImageWrite;
        xaaptr->SubsequentImageWriteRect = VIASubsequentImageWriteRect;
        xaaptr->ImageWriteBase = pVia->BltBase;
        xaaptr->ImageWriteRange = (64 * 1024);
                                  
    }
                               
    if(xf86IsEntityShared(pScrn->entityList[0]))
    {
        pPriv = xf86GetEntityPrivate(pScrn->entityList[0],
        gVIAEntityIndex);
        pVIAEnt = pPriv->ptr;
    
        /*if there are more than one devices sharing this entity, we
        have to assign this call back, otherwise the XAA will be
        disabled */
        if(pVIAEnt->HasSecondary || pVIAEnt->BypassSecondary)
            xaaptr->RestoreAccelState = VIARestoreAccelState;
    }                              


    /* Set up screen parameters. */
    pVia->Bpp = pScrn->bitsPerPixel >> 3;
    pVia->Bpl = pScrn->displayWidth * pVia->Bpp;

    if((pVia->directRenderingEnabled))
    {
        /* We reserve space for pixel cache */
        pVia->ScissB = (pVia->FBFreeStart + VIA_PIXMAP_CACHE_SIZE) / pVia->Bpl;        
    }
    else
    {
        /* We reserve space for pixel cache */        
        pVia->ScissB = ((pVia->FBFreeEnd) / pVia->Bpl) - 1;        
    }

    if (pVia->ScissB > 2047)
        pVia->ScissB = 2047;

    /* Finally, we set up the video memory space available to the pixmap cache. */

    AvailFBArea.x1 = 0;
    AvailFBArea.y1 = 0;
    AvailFBArea.x2 = pScrn->displayWidth;        

    /* If the screen is rotated, we have to modify the height of availFBArea. */
    if (pVia->DynamicRotation)
    {
        AvailFBArea.y2 = pBIOSInfo->VDisplay; 

        /* When dynamic rotation is enabled, the framebuffer size will be VirtualX x VirtualX. */
        pVia->FBFreeStart = pVia->Bpl * pBIOSInfo->VirtualX;
    }
    else if (pVia->IsHWRotateEnabled)   
    {        
        AvailFBArea.y2 = pBIOSInfo->VDisplay; 
        pVia->FBFreeStart = pVia->Bpl * pBIOSInfo->VDisplay;
    }
    else /* Non-rotate */
    {
        AvailFBArea.y2 = pVia->ScissB;
        pVia->FBFreeStart = pVia->Bpl * pVia->ScissB;
    }
    
    if(!pVia->IsSecondary)
    {
         if (pVia->pScreensPublicInfo->UseHwCursor)
         {
             /* Always reserve cursor buffer after the first scrn */
             pVia->FBFreeStart = (pVia->FBFreeStart + 0x3FF) & 0xFFFFFC00;
             pVia->pScreensPublicInfo->CursorInfo.HwCursorBufStart = pVia->FBFreeStart;
             pVia->FBFreeStart += VIA_CURSOR_SIZE;
             pVia->pScreensPublicInfo->CursorInfo.HIBufStart = pVia->FBFreeStart;
             pVia->FBFreeStart += VIA_CURSOR_SIZE;
         }
         pVia->FBFreeStart = (pVia->FBFreeStart + 0xF) & 0xFFFFFFF0;
         pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf = (unsigned int*)(pVia0->FBBase + pVia->FBFreeStart);
         pVia->FBFreeStart += VIA_AGP_CMD_BUFFER_SIEZ;
         pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
         pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2 = TRUE;
         if(H5_UMA_CHIPID||H6_UMA_CHIPID)
         {
             pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2 = FALSE;
         }
         pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
     }

    /*
    *   Initialization of the XFree86 framebuffer manager is done via
    *   Bool xf86InitFBManager(ScreenPtr pScreen, BoxPtr FullBox) 
    *   FullBox represents the area of the framebuffer that the manager
    *   is allowed to manage. This is typically a box with a width 
    *   of pScrn->displayWidth and a height of as many lines as can be fit
    *   within the total video memory
    */
    xf86InitFBManager(pScreen, &AvailFBArea);
    
    xf86DrvMsg(pScrn->scrnIndex, X_INFO,
               "Frame Buffer From (%d,%d) To (%d,%d)\n",
               AvailFBArea.x1, AvailFBArea.y1, AvailFBArea.x2, AvailFBArea.y2);
    xf86DrvMsg(pScrn->scrnIndex, X_INFO,
               "Using %d lines for offscreen memory.\n",
               AvailFBArea.y2);    
        
    return XAAInit(pScreen, xaaptr);
}


/* The sync function for the GE */
void VIAAccelSync(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);

    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }

    WaitIdle();
}


/* These are the ScreenToScreen bitblt functions. We support all ROPs, all
 * directions.
 *
 */

static void
VIASetupForScreenToScreenCopy(
    ScrnInfoPtr pScrn,
    int xdir,
    int ydir,
    int rop,
    unsigned planemask,
    int trans_color)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd, dwGEMode=0;
    volatile unsigned int* pBuf;

{
#ifdef  WAIT_IDEL_FOR2D
WaitIdle();
#endif
}
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    if (trans_color != -1)
    {
        ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 3);
    }
    else
    {
        ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 2);
    }
    
    switch (pScrn->bitsPerPixel)
    {
    case 16:
        dwGEMode |= VIA_GEM_16bpp;
        break;
    case 32:
        dwGEMode |= VIA_GEM_32bpp;
        break;
    default:
        dwGEMode |= VIA_GEM_8bpp;
        break;
    }

#ifdef X_HAVE_XAAGETROP
    cmd = VIA_GEC_BLT | (XAAGetCopyROP(rop) << 24);
#else
    cmd = VIA_GEC_BLT | (XAACopyROP[rop] << 24);
#endif

    if (xdir < 0)
        cmd |= VIA_GEC_DECX;

    if (ydir < 0)
        cmd |= VIA_GEC_DECY;    

    /*  Set rotate parameters.
     *  GEMode and cmd could be modified after calling this function. 
     */    
    Enable2DEngineRotate(pScrn, &dwGEMode, &cmd);
    
    /* Set BPP and Pitch */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GEMODE, dwGEMode);
    
    pVia->SavedCmd = cmd;

    if (trans_color != -1) {
        /* Transparent Bitblt */
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCCOLORKEY, trans_color);
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_KEYCONTROL, 0x4000);
    }
    else {
        /* Disable Transparent Bitblt */
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_KEYCONTROL, 0x0);
    }

    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASubsequentScreenToScreenCopy(
    ScrnInfoPtr pScrn,
    int x1,
    int y1,
    int x2,
    int y2,
    int w,
    int h)
{
    VIAPtr pVia = VIAPTR(pScrn);
    volatile unsigned int* pBuf;
{
#ifdef  WAIT_IDEL_FOR2D
WaitIdle();
#endif
}
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    if(!pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2)
    {
        AddWaitCmdInv(&pBuf, (unsigned int)pVia->ChipId, FLAG_WAIT_2D_IDLE);
    }
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 7);

    if (!w || !h)
        return;

    if (pVia->SavedCmd & VIA_GEC_DECX) {
        x1 += (w - 1);
        x2 += (w - 1);
    }

    if (pVia->SavedCmd & VIA_GEC_DECY) {
        y1 += (h - 1);
        y2 += (h - 1);
    }
    
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH, VIA_PITCH_ENABLE |
              ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCPOS, ((y1 << 16) | x1));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTPOS, ((y2 << 16) | x2));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION, (((h - 1) << 16) | (w - 1)));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD, pVia->SavedCmd);

    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


/*
 * SetupForSolidFill is also called to set up for lines.
 */

static void
VIASetupForSolidFill(
    ScrnInfoPtr pScrn,
    int color,
    int rop,
    unsigned planemask)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd, dwGEMode=0;
    volatile unsigned int* pBuf;
{
#ifdef  WAIT_IDEL_FOR2D
WaitIdle();
#endif
}
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 1);
    switch (pScrn->bitsPerPixel)
    {
    case 16:
        dwGEMode |= VIA_GEM_16bpp;
        break;
    case 32:
        dwGEMode |= VIA_GEM_32bpp;
        break;
    default:
        dwGEMode |= VIA_GEM_8bpp;
        break;
    }        

    cmd = VIA_GEC_BLT | VIA_GEC_FIXCOLOR_PAT | 
#ifdef X_HAVE_XAAGETROP
    (XAAGetPatternROP(rop) << 24);
#else
    (XAAPatternROP[rop] << 24);
#endif

    /*  Set rotate parameters.
     *  GEMode and cmd could be modified after calling this function. 
     */    
    Enable2DEngineRotate(pScrn, &dwGEMode, &cmd);
    
    /* Set BPP and Pitch */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GEMODE, dwGEMode);

    pVia->SavedCmd = cmd;
    pVia->SavedFgColor = color;
	
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);

}


static void
VIASubsequentSolidFillRect(
    ScrnInfoPtr pScrn,
    int x,
    int y,
    int w,
    int h)
{
    VIAPtr pVia = VIAPTR(pScrn);
    volatile unsigned int* pBuf;

#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif

    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 7);

    if (!w || !h)
        return;

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH, VIA_PITCH_ENABLE |
              ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTPOS, ((y << 16) | (x & 0xFFFF)));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION, (((h - 1) << 16) | (w - 1)));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_FGCOLOR, pVia->SavedFgColor);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD, pVia->SavedCmd);

    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


/*
 * The meaning of the two pattern paremeters to Setup & Subsequent for
 * Mono8x8Patterns varies depending on the flag bits.  We specify
 * HW_PROGRAMMED_BITS, which means our hardware can handle 8x8 patterns
 * without caching in the frame buffer.  Thus, Setup gets the pattern bits.
 * There is no way with BCI to rotate an 8x8 pattern, so we do NOT specify
 * HW_PROGRAMMED_ORIGIN.  XAA wil rotate it for us and pass the rotated
 * pattern to both Setup and Subsequent.  If we DID specify PROGRAMMED_ORIGIN,
 * then Setup would get the unrotated pattern, and Subsequent gets the
 * origin values.
 */

static void
VIASetupForMono8x8PatternFill(
    ScrnInfoPtr pScrn,
    int pattern0,
    int pattern1,
    int fg,
    int bg,
    int rop,
    unsigned planemask)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd, dwGEMode=0;
    volatile unsigned int* pBuf;
{
#ifdef  WAIT_IDEL_FOR2D
WaitIdle();
#endif
}

    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 1);

    switch (pScrn->bitsPerPixel)
    {
    case 16:
        dwGEMode |= VIA_GEM_16bpp;
        break;
    case 32:
        dwGEMode |= VIA_GEM_32bpp;
        break;
    default:
        dwGEMode |= VIA_GEM_8bpp;
        break;
    }    

    cmd = VIA_GEC_BLT | VIA_GEC_PAT_REG | VIA_GEC_PAT_MONO |
#ifdef X_HAVE_XAAGETROP
    (XAAGetPatternROP(rop) << 24);
#else
    (XAAPatternROP[rop] << 24);
#endif

    if (bg == -1) {
        /* transparent mono pattern */
        cmd |= VIA_GEC_MPAT_TRANS;
    }

    /*  Set rotate parameters.
     *  GEMode and cmd could be modified after calling this function. 
     */    
    Enable2DEngineRotate(pScrn, &dwGEMode, &cmd);
    
    /* Set BPP and Pitch */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GEMODE, dwGEMode);

    pVia->SavedCmd = cmd;
    pVia->SavedFgColor = fg;
    pVia->SavedBgColor = bg;
    pVia->SavedPattern0 = pattern0;
    pVia->SavedPattern1 = pattern1;

    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASubsequentMono8x8PatternFillRect(
    ScrnInfoPtr pScrn,
    int patOffx,
    int patOffy,
    int x,
    int y,
    int w,
    int h)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  dwPatOffset;
    volatile unsigned int* pBuf;

{
#ifdef  WAIT_IDEL_FOR2D
WaitIdle();
#endif
}
    if (!w || !h)
        return;

    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 11);

    dwPatOffset = ((patOffy & 0x7)  << 29) | ((patOffx & 0x7) << 26);

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH, VIA_PITCH_ENABLE |
              ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTPOS, ((y << 16) | x));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION, (((h - 1) << 16) | (w - 1)));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PATADDR, dwPatOffset);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_FGCOLOR, pVia->SavedFgColor);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_BGCOLOR, pVia->SavedBgColor);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_MONOPAT0, pVia->SavedPattern0);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_MONOPAT1, pVia->SavedPattern1);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD, pVia->SavedCmd);

    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}

/* xaa Color8x8 cache bug fixed in cvs tag xf4 3 99 14 */ 
static void
VIASetupForColor8x8PatternFill(
    ScrnInfoPtr pScrn,
    int patternx,
    int patterny,
    int rop,
    unsigned planemask,
    int trans_color)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd, dwGEMode=0;
    volatile unsigned int* pBuf;
{
#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif
}
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 1);
	
    switch (pScrn->bitsPerPixel)
    {
    case 16:
        dwGEMode |= VIA_GEM_16bpp;
        break;
    case 32:
        dwGEMode |= VIA_GEM_32bpp;
        break;
    default:
        dwGEMode |= VIA_GEM_8bpp;
        break;
    }

#ifdef X_HAVE_XAAGETROP
    cmd = VIA_GEC_BLT | (XAAGetPatternROP(rop) << 24);
#else
    cmd = VIA_GEC_BLT | (XAAPatternROP[rop] << 24);
#endif

    /*  Set rotate parameters.
     *  GEMode and cmd could be modified after calling this function. 
     */    
    Enable2DEngineRotate(pScrn, &dwGEMode, &cmd);
    
    /* Set BPP and Pitch */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GEMODE, dwGEMode);
    
    pVia->SavedCmd = cmd;
    pVia->SavedPatternAddr = (patternx * pVia->Bpp + patterny * pVia->Bpl);

    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASubsequentColor8x8PatternFillRect(
    ScrnInfoPtr pScrn,
    int patOffx,
    int patOffy,
    int x,
    int y,
    int w,
    int h)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  dwPatAddr;
    volatile unsigned int* pBuf;
{
#ifdef  WAIT_IDEL_FOR2D
WaitIdle();
#endif
}
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 7);
	
    if (!w || !h)
        return;

    dwPatAddr = (pVia->SavedPatternAddr >> 3) |
                ((patOffy & 0x7)  << 29) | ((patOffx & 0x7) << 26);

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH, VIA_PITCH_ENABLE |
              ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTPOS, ((y << 16) | x));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION, (((h - 1) << 16) | (w - 1)));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PATADDR, dwPatAddr);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD, pVia->SavedCmd);

    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASetupForCPUToScreenColorExpandFill(
    ScrnInfoPtr pScrn,
    int fg,
    int bg,
    int rop,
    unsigned planemask)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd, dwGEMode=0;
    volatile unsigned int* pBuf;
{
#ifdef  WAIT_IDEL_FOR2D
WaitIdle();
#endif
}
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 2);
    switch (pScrn->bitsPerPixel)
    {
    case 16:
        dwGEMode |= VIA_GEM_16bpp;
        break;
    case 32:
        dwGEMode |= VIA_GEM_32bpp;
        break;
    default:
        dwGEMode |= VIA_GEM_8bpp;
        break;
    }

    cmd = VIA_GEC_BLT | VIA_GEC_SRC_SYS | VIA_GEC_SRC_MONO |
          VIA_GEC_MONO_PACK | VIA_GEC_MONO_BYTE |
#ifdef X_HAVE_XAAGETROP
    (XAAGetCopyROP(rop) << 24);
#else
    (XAACopyROP[rop] << 24);
#endif

    if (bg == -1) {
        cmd |= VIA_GEC_MSRC_TRANS;
    }
    
    /*  Set rotate parameters.
     *  GEMode and cmd could be modified after calling this function. 
     */    
    Enable2DEngineRotate(pScrn, &dwGEMode, &cmd);
    
    /* Set BPP and Pitch */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GEMODE, dwGEMode);

    pVia->SavedCmd = cmd;
    pVia->SavedFgColor = fg;
    pVia->SavedBgColor = bg;

    /* Disable Transparent Bitblt */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_KEYCONTROL, 0x0);

    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASubsequentCPUToScreenColorExpandFill(
    ScrnInfoPtr pScrn,
    int x,
    int y,
    int w,
    int h,
    int skipleft)
{
    VIAPtr pVia = VIAPTR(pScrn);
    XAAInfoRecPtr xaaptr = GET_XAAINFORECPTR_FROM_SCRNINFOPTR(pScrn);
    volatile unsigned int* pBuf;
    unsigned int size;
{
#ifdef  WAIT_IDEL_FOR2D
WaitIdle();
#endif
}
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    /* XAA will be sending bitmap data next.  */
    /* We should probably wait for empty/idle here. */

    if (skipleft) {
        VIASetClippingRectangle(pScrn, (x + skipleft), y, (x + w - 1), (y + h -1));
    }

    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 9);

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH, VIA_PITCH_ENABLE |
              ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCPOS, 0);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTPOS, ((y << 16) | x));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION, (((h - 1) << 16) | (w - 1)));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_FGCOLOR, pVia->SavedFgColor);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_BGCOLOR, pVia->SavedBgColor);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD, pVia->SavedCmd);

    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = (pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf);
    size = ((w + 31) >> 5) * h;
	ISSUE_2D_DATA(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, size, xaaptr->ColorExpandBase);
}

static void
VIASetupForScreenToScreenColorExpand(
    ScrnInfoPtr pScrn,
    int fg,
    int bg,
    int rop,
    unsigned int planemask)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd, dwGEMode=0;
    volatile unsigned int* pBuf;
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 1);

    switch (pScrn->bitsPerPixel)
    {
    case 16:
        dwGEMode |= VIA_GEM_16bpp;
        break;
    case 32:
        dwGEMode |= VIA_GEM_32bpp;
        break;
    default:
        dwGEMode |= VIA_GEM_8bpp;
        break;
    }
    
    /* Set BPP and Pitch */
    VIASETREG(VIA_REG_GEMODE, dwGEMode);

#ifdef X_HAVE_XAAGETROP
    cmd = VIA_GEC_BLT | VIA_GEC_SRC_MONO | (XAAGetCopyROP(rop) << 24);
#else
    cmd = VIA_GEC_BLT | VIA_GEC_SRC_MONO | (XAACopyROP[rop] << 24);
#endif

    if (bg == -1) {
        cmd |= VIA_GEC_MSRC_TRANS;
    }

    pVia->SavedCmd = cmd;
    pVia->SavedFgColor = fg;
    pVia->SavedBgColor = bg;

    /* Disable Transparent Bitblt */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_KEYCONTROL, 0x0);

    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASubsequentScreenToScreenColorExpand(
    ScrnInfoPtr pScrn,
    int x,
    int y,
    int w,
    int h,
    int srcx,
    int srcy,
    int offset)
{
    VIAPtr pVia = VIAPTR(pScrn);
    volatile unsigned int* pBuf;
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 11);

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH, VIA_PITCH_ENABLE |
              ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE, ((((srcy * pScrn->displayWidth) + srcx) *
                            (pScrn->bitsPerPixel >> 3)) >> 3) + (pScrn->fbOffset >> 3));
    srcy = 0;

    srcx = srcx % (128 / pScrn->bitsPerPixel);

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCPOS, ((srcy << 16) |
              (srcx * pScrn->bitsPerPixel + offset)));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTPOS, ((y << 16) | x));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION, (((h - 1) << 16) | (w - 1)));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_FGCOLOR, pVia->SavedFgColor);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_BGCOLOR, pVia->SavedBgColor);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD, pVia->SavedCmd);

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE, 0x0 + (pScrn->fbOffset >> 3));

    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}

static void
VIASetupForImageWrite(
    ScrnInfoPtr pScrn,
    int rop,
    unsigned planemask,
    int trans_color,
    int bpp,
    int depth)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd, dwGEMode=0;
    volatile unsigned int* pBuf;
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    if (trans_color != -1) 
    {
        ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 3);
    }
    else
    {
        ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 2);
    }

    switch (bpp)
    {
    case 16:
        dwGEMode |= VIA_GEM_16bpp;
        break;
    case 32:
        dwGEMode |= VIA_GEM_32bpp;
        break;
    default:
        dwGEMode |= VIA_GEM_8bpp;
        break;
    }

#ifdef X_HAVE_XAAGETROP
    cmd = VIA_GEC_BLT | VIA_GEC_SRC_SYS | (XAAGetCopyROP(rop) << 24);
#else
    cmd = VIA_GEC_BLT | VIA_GEC_SRC_SYS | (XAACopyROP[rop] << 24);
#endif

    
    /*  Set rotate parameters.
     *  GEMode and cmd could be modified after calling this function. 
     */    
    Enable2DEngineRotate(pScrn, &dwGEMode, &cmd);
    
    /* Set BPP and Pitch */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GEMODE, dwGEMode);

    pVia->SavedCmd = cmd;
    pVia->SavedBpp = bpp >> 3;

    if (trans_color != -1) {
        /* Transparent Bitblt */
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCCOLORKEY, trans_color);
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_KEYCONTROL, 0x4000);
    }
    else {
        /* Disable Transparent Bitblt */
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_KEYCONTROL, 0x0);
    }
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASubsequentImageWriteRect(
    ScrnInfoPtr pScrn,
    int x,
    int y,
    int w,
    int h,
    int skipleft)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    volatile unsigned int* pBuf;
    unsigned int size;
    XAAInfoRecPtr xaaptr = GET_XAAINFORECPTR_FROM_SCRNINFOPTR(pScrn);

    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
   if (skipleft) {
        VIASetClippingRectangle(pScrn, (x + skipleft), y, (x + w - 1), (y + h -1));
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 7);

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH, VIA_PITCH_ENABLE |
              ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCPOS, 0);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTPOS, ((y << 16) | x));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION, (((h - 1) << 16) | (w - 1)));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD, pVia->SavedCmd);

    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = (pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf);
    size = ((w * h * pVia->SavedBpp) + 3) >> 2;

	ISSUE_2D_DATA(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, size, xaaptr->ImageWriteBase);
}


/* Setup for XAA solid lines. */
static void
VIASetupForSolidLine(
    ScrnInfoPtr pScrn,
    int color,
    int rop,
    unsigned int planemask)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd, dwGEMode=0;
    volatile unsigned int* pBuf;
{
#ifdef  WAIT_IDEL_FOR2D
WaitIdle();
#endif
}
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 3);

    switch (pScrn->bitsPerPixel)
    {
    case 16:
        dwGEMode |= VIA_GEM_16bpp;
        break;
    case 32:
        dwGEMode |= VIA_GEM_32bpp;
        break;
    default:
        dwGEMode |= VIA_GEM_8bpp;
        break;
    }        

    /* we move VIA_GEC_LINE from here to the place firing command */
#ifdef X_HAVE_XAAGETROP
    cmd = VIA_GEC_FIXCOLOR_PAT | (XAAGetPatternROP(rop) << 24);
#else
    cmd = VIA_GEC_FIXCOLOR_PAT | (XAAPatternROP[rop] << 24);
#endif    

    /* Set BPP and Pitch */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GEMODE, dwGEMode);

    pVia->SavedCmd = cmd;
    pVia->SavedFgColor = color;

    /* set solid line pattern */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_MONOPAT0, 0xFF);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_FGCOLOR, pVia->SavedFgColor);
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASubsequentSolidTwoPointLine(
    ScrnInfoPtr pScrn,
    int x1,
    int y1,
    int x2,
    int y2,
    int flags)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd;
    int     dx, dy, tmp, error = 1;  
    volatile unsigned int* pBuf;
{
#ifdef  WAIT_IDEL_FOR2D
WaitIdle();
#endif
}
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 8);
       
    cmd = pVia->SavedCmd | VIA_GEC_LINE;

    dx = x2 - x1;
    if (dx < 0) {
        dx = -dx;
        cmd |= VIA_GEC_DECX;            /* line will be drawn from right */
        error = 0;
    }

    dy = y2 - y1;
    if (dy < 0) {
        dy = -dy;
        cmd |= VIA_GEC_DECY;            /* line will be drawn from bottom */
    }

    if (dy > dx) {
        tmp  = dy;
        dy = dx;
        dx = tmp;                       /* Swap 'dx' and 'dy' */
        cmd |= VIA_GEC_Y_MAJOR;         /* Y major line */
    }

    if (flags & OMIT_LAST) {
        cmd |= VIA_GEC_LASTPIXEL_OFF;
    }

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH, VIA_PITCH_ENABLE |
              ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_LINE_K1K2, ((((dy << 1) & 0x3fff) << 16)|
              (((dy - dx) << 1) & 0x3fff)));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_LINE_XY, ((y1 << 16) | x1));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION, dx);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_LINE_ERROR, (((dy << 1) - dx - error) & 0x3fff));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD, cmd);
	
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


/* Subsequent XAA solid horizontal and vertical lines */
static void
VIASubsequentSolidHorVertLine(
    ScrnInfoPtr pScrn,
    int x,
    int y,
    int len,
    int dir)
{
    VIAPtr  pVia = VIAPTR(pScrn); 
    volatile unsigned int* pBuf;

{
#ifdef  WAIT_IDEL_FOR2D
WaitIdle();
#endif
}
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                              pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 6);

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH, VIA_PITCH_ENABLE |
              ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));

    if (dir == DEGREES_0) {
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTPOS, ((y << 16) | x));
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION, (len - 1));
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD, pVia->SavedCmd | VIA_GEC_BLT);
    }
    else {
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTPOS, ((y << 16) | x));
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION, ((len - 1) << 16));
        ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD, pVia->SavedCmd | VIA_GEC_BLT);
    }    
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}

static void
VIASetupForDashedLine(
    ScrnInfoPtr pScrn,
    int fg,
    int bg,
    int rop,
    unsigned int planemask,
    int length,
    unsigned char *pattern)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  pat = *(CARD32 *)pattern;
    CARD32  cmd, dwGEMode=0;  
    volatile unsigned int* pBuf;

#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif

    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 4);

    switch (pScrn->bitsPerPixel)
    {
    case 16:
        dwGEMode |= VIA_GEM_16bpp;
        break;
    case 32:
        dwGEMode |= VIA_GEM_32bpp;
        break;
    default:
        dwGEMode |= VIA_GEM_8bpp;
        break;
    }
    
    /* Set BPP and Pitch */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GEMODE, dwGEMode);

#ifdef X_HAVE_XAAGETROP
    cmd = VIA_GEC_LINE | VIA_GEC_FIXCOLOR_PAT | (XAAGetPatternROP(rop) << 24);
#else
    cmd = VIA_GEC_LINE | VIA_GEC_FIXCOLOR_PAT | (XAAPatternROP[rop] << 24);
#endif

    if (bg == -1) {
        /* transparent mono pattern */
        cmd |= VIA_GEC_MPAT_TRANS;
    }

    pVia->SavedCmd = cmd;
    pVia->SavedFgColor = fg;
    pVia->SavedBgColor = bg;

    switch (length) {
        case  2: pat |= pat <<  2; /* fall through */
        case  4: pat |= pat <<  4; /* fall through */
        case  8: pat |= pat <<  8; /* fall through */
        case 16: pat |= pat << 16;
    }

    pVia->SavedPattern0 = pat;

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_FGCOLOR, pVia->SavedFgColor);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_BGCOLOR, pVia->SavedBgColor);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_MONOPAT0, pVia->SavedPattern0);
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASubsequentDashedTwoPointLine(
    ScrnInfoPtr pScrn,
    int x1,
    int y1,
    int x2,
    int y2,
    int flags,
    int phase)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD32  cmd;
    int     dx, dy, tmp, error = 1;  
    volatile unsigned int* pBuf;

#ifdef  WAIT_IDEL_FOR2D
    WaitIdle();
#endif

    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 8);
    
    cmd = pVia->SavedCmd;

    dx = x2 - x1;
    if (dx < 0) {
        dx = -dx;
        cmd |= VIA_GEC_DECX;            /* line will be drawn from right */
        error = 0;
    }

    dy = y2 - y1;
    if (dy < 0) {
        dy = -dy;
        cmd |= VIA_GEC_DECY;            /* line will be drawn from bottom */
    }

    if (dy > dx) {
        tmp  = dy;
        dy = dx;
        dx = tmp;                       /* Swap 'dx' and 'dy' */
        cmd |= VIA_GEC_Y_MAJOR;         /* Y major line */
    }

    if (flags & OMIT_LAST) {
        cmd |= VIA_GEC_LASTPIXEL_OFF;
    }

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_SRCBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DSTBASE, 0x0 + (pScrn->fbOffset >> 3));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_PITCH, VIA_PITCH_ENABLE |
              ((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) |
              (((pScrn->displayWidth * pScrn->bitsPerPixel >> 3) >> 3) << 16));

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_LINE_K1K2, ((((dy << 1) & 0x3fff) << 16)|
              (((dy - dx) << 1) & 0x3fff)));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_LINE_XY, ((y1 << 16) | x1));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_DIMENSION, dx);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_LINE_ERROR, (((dy << 1) - dx - error) & 0x3fff) | 0xFF0000);
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GECMD, cmd);
    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void
VIASetClippingRectangle(
    ScrnInfoPtr pScrn,
    int x1,
    int y1,
    int x2,
    int y2)
{
    VIAPtr pVia = VIAPTR(pScrn);
    CARD32 dwGEMode=0;
    volatile unsigned int* pBuf;
    if(pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire)
    {
        FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                          pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
        pVia->pScreensPublicInfo->AgpCmd.Agp_Need_Fire = FALSE;
        pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = 0;
    }
    pBuf = pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    ADD_AGP_HEADER(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, 3);
    switch (pScrn->bitsPerPixel)
    {
        case 16:
            dwGEMode |= VIA_GEM_16bpp;
            break;
        case 32:
            dwGEMode |= VIA_GEM_32bpp;
            break;
        default:
            dwGEMode |= VIA_GEM_8bpp;
            break;
    }        

#ifdef DEBUG_EXTRA
    ErrorF("ClipRect, (%4d,%4d)-(%4d,%4d) \n", x1, y1, x2, y2 );
#endif

    /*  Set rotate parameters.
     *  GEMode and cmd could be modified after calling this function. 
     */
    Enable2DEngineRotate(pScrn, &dwGEMode, &(pVia->SavedCmd));
    
    /* Set BPP and Pitch */
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_GEMODE, dwGEMode);

    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_CLIPTL, ((y1 << 16) | x1));
    ISSUE_2D_COMMAND(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, pBuf, VIA_REG_CLIPBR, ((y2 << 16) | x2));
    
    pVia->SavedCmd |= VIA_GEC_CLIP_ENABLE;

    pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size = pBuf - pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf;
    FLUSE_AGP_CMD(pVia, pVia->pScreensPublicInfo->AgpCmd.Agp_IsH2, 
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Buf,
                      pVia->pScreensPublicInfo->AgpCmd.Agp_Cmd_Size);
}


static void VIADisableClipping(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);

#ifdef DEBUG_EXTRA
    ErrorF("Kill ClipRect\n");
#endif

    pVia->SavedCmd &= ~VIA_GEC_CLIP_ENABLE;
}

/* This callback is required for multihead cards using XAA */
static void VIARestoreAccelState(ScrnInfoPtr pScrn)
{

}
