/*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/*************************************************************************
 *
 *  File:       via_cursor.c
 *  Content:    Hardware cursor support for VIA/S3G UniChrome
 *
 ************************************************************************/

#include "via_driver.h"
#include "via.h"
#include "cursorstr.h"

#define	USESwScaling

static void VIALoadCursorImage(ScrnInfoPtr pScrn, unsigned char *src);
void VIASetCursorPosition(ScrnInfoPtr pScrn, int x, int y);
static void VIASetCursorColors(ScrnInfoPtr pScrn, int bg, int fg);
static Bool VIAUseHWCursor(ScreenPtr pScreen, CursorPtr pCurs);
/*forARGB cursor using hardware icon after starting X*/
static Bool VIAHiUseHWCursorARGB(ScreenPtr pScreen,CursorPtr pCursor);
static void VIAHiLoadCursorARGB (ScrnInfoPtr pScrn, CursorPtr pCurs);
extern void VIACheckCursorTypeToUse(ScrnInfoPtr pScrn);

void VIALoadIconImage(ScrnInfoPtr pScrn, CARD32* src);

/*change color cursor size from 32x32 to 64x64*/
#ifndef USEHwIconSimulateHwCursor
#ifdef ARGB_CURSOR
Bool CallWhileARGB = TRUE;
#endif
#endif

void VIAARGBCursorInit(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    CARD32 dwHiControl;
    
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAARGBCursorInit\n"));

    switch(pBIOSInfo->Chipset )
    {
        case VIA_CX700:
        case VIA_CN750:
        case VIA_P4M890:
        case VIA_P4M900:
        case VIA_VX800:
            /* set 0 as transparent color key */
            VIASETREG(PRIM_HI_TRANSCOLOR, 0);
            VIASETREG(PRIM_HI_FIFO,0x0D000D0F);
            VIASETREG(PRIM_HI_INVTCOLOR, 0X00FFFFFF);
            VIASETREG(V327_HI_INVTCOLOR, 0X00FFFFFF);
            dwHiControl = VIAGETREG(PRIM_HI_CTRL);
            
            /* Turn cursor off. */
            VIASETREG(PRIM_HI_CTRL, dwHiControl & 0xFFFFFFFA);                    

            /* two hi use the same memory*/
            VIASETREG(PRIM_HI_FBOFFSET, pVia->pScreensPublicInfo->CursorInfo.HIBufStart);
            VIASETREG(HI_FBOFFSET, pVia->pScreensPublicInfo->CursorInfo.HIBufStart);
            
            /* set 0 as transparent color key */
            VIASETREG(HI_TRANSPARENT_COLOR, 0);
            VIASETREG(HI_INVTCOLOR, 0X00FFFFFF);
            VIASETREG(ALPHA_V3_PREFIFO_CONTROL, 0xE0000);
            VIASETREG(ALPHA_V3_FIFO_CONTROL, 0xE0F0000);
            dwHiControl = VIAGETREG(HI_CONTROL);
            
            /* Turn cursor off. */
            VIASETREG(HI_CONTROL, dwHiControl & 0xFFFFFFFA);
            break;
            
        default:               
            VIASETREG(HI_FBOFFSET, pVia->pScreensPublicInfo->CursorInfo.HIBufStart);
            VIASETREG(HI_TRANSPARENT_COLOR, 0);
            VIASETREG(HI_INVTCOLOR, 0X00FFFFFF);
            VIASETREG(ALPHA_V3_PREFIFO_CONTROL, 0xE0000);
            VIASETREG(ALPHA_V3_FIFO_CONTROL, 0xE0F0000);
            dwHiControl = VIAGETREG(HI_CONTROL);

            /* Turn cursor off. */
            VIASETREG(HI_CONTROL, dwHiControl & 0xFFFFFFFA);                         
    }
}

Bool
VIAHWCursorInit(ScreenPtr pScreen)
{
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    VIAPtr pVia = VIAPTR(pScrn);
    xf86CursorInfoPtr infoPtr;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAHWCursorInit\n"));
    infoPtr = xf86CreateCursorInfoRec();
    if (!infoPtr)
        return FALSE;

    pVia->CursorInfoRec = infoPtr;

    infoPtr->MaxWidth = MAX_CURS;
    infoPtr->MaxHeight = MAX_CURS;
    infoPtr->Flags = HARDWARE_CURSOR_SOURCE_MASK_INTERLEAVE_64 |
                     HARDWARE_CURSOR_AND_SOURCE_WITH_MASK |
                     HARDWARE_CURSOR_TRUECOLOR_AT_8BPP |
                     HARDWARE_CURSOR_INVERT_MASK |
                     HARDWARE_CURSOR_BIT_ORDER_MSBFIRST|
                     0;
    
    infoPtr->SetCursorColors = VIASetCursorColors;
    infoPtr->SetCursorPosition = VIASetCursorPosition;
    infoPtr->LoadCursorImage = VIALoadCursorImage;
    infoPtr->HideCursor = VIAHideCursor;
    infoPtr->ShowCursor = VIAShowCursor;
    infoPtr->UseHWCursor = VIAUseHWCursor;

  
#ifdef ARGB_CURSOR
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "We will use HWCursor ARGB\n"));
    
    infoPtr->UseHWCursorARGB = VIAHiUseHWCursorARGB;
    infoPtr->LoadCursorARGB = VIAHiLoadCursorARGB;
#endif
   
    /*for NoAccel case*/
    if (pVia->NoAccel)
    {
        if(!pVia->IsSecondary)
        {
            pVia->FBFreeStart = (pVia->FBFreeStart + 0x3FF) & 0xFFFFFC00;
            pVia->pScreensPublicInfo->CursorInfo.HwCursorBufStart = pVia->FBFreeStart;
            pVia->FBFreeStart += VIA_CURSOR_SIZE;
            pVia->pScreensPublicInfo->CursorInfo.HIBufStart = pVia->FBFreeStart;
            pVia->FBFreeStart += VIA_CURSOR_SIZE;
        }
    }

#ifndef USEHwIconSimulateHwCursor
    if(pVia->pScreensPublicInfo->UseHwCursor)
        VIASETREG(VIA_REG_CURSOR_MODE, pVia->pScreensPublicInfo->CursorInfo.HwCursorBufStart & 0x3FFFF00);
#endif

    pVia->pScreensPublicInfo->CursorInfo.CursorBG = 0xffffff;
    pVia->pScreensPublicInfo->CursorInfo.CursorFG = 0x0;

#ifdef ARGB_CURSOR
    VIAARGBCursorInit(pScrn);
#endif

    return xf86InitCursor(pScreen, infoPtr);
}


static Bool VIAUseHWCursor(ScreenPtr pScreen, CursorPtr pCurs)
{
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    VIAPtr pVia = VIAPTR(pScrn);

    pVia->pScreensPublicInfo->CursorInfo.width
		= (pCurs->bits->width > MAX_CURS) ? MAX_CURS : pCurs->bits->width;
    pVia->pScreensPublicInfo->CursorInfo.height
		= (pCurs->bits->height > MAX_CURS) ? MAX_CURS : pCurs->bits->height;
    
    /*Need to confine hotx, hoty to MAX_CURS?*/
    pVia->pScreensPublicInfo->CursorInfo.hotx = pCurs->bits->xhot;
    pVia->pScreensPublicInfo->CursorInfo.hoty = pCurs->bits->yhot;
	
    return pVia->pScreensPublicInfo->UseHwCursor ? TRUE: FALSE;
}


void VIAShowCursor(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    CARD32 dwCursorMode;
    unsigned long* Screen_IGA_Map = pVia->pVidData->VIAGraphicInfo.Screen_IGA_Map;

#ifdef USEHwIconSimulateHwCursor
    switch(pBIOSInfo->Chipset )
    {
        case VIA_CX700:
        case VIA_CN750:
        case VIA_P4M890:
        case VIA_P4M900:
        case VIA_VX800:
             /* Turn on Hardware icon Cursor */
             if(!pBIOSInfo->DuoView)
            {
                switch(Screen_IGA_Map[pScrn->scrnIndex])
                {
                     /* Turn on Hardware icon Cursor */
                    case IGA1:
                        VIASETREG(PRIM_HI_CTRL, 0x36000005);
                        break;
                    case IGA2:
                        VIASETREG(HI_CONTROL, 0xB6000005);
                        break;
                }
            }
            else
            {
                VIASETREG(PRIM_HI_CTRL, 0x36000005);
                VIASETREG(HI_CONTROL, 0xb6000005);
            }
             break;
        
        default:
            switch(Screen_IGA_Map[pScrn->scrnIndex])
            {
            /* Turn on Hardware icon Cursor */
            case IGA1:
                VIASETREG(HI_CONTROL, 0x76000005);
                break;                  
            case IGA2:                    
                VIASETREG(HI_CONTROL, 0xf6000005);
                break;
            }
    }
#else

#ifdef ARGB_CURSOR
    if(CallWhileARGB )
    {      
        switch(pBIOSInfo->Chipset )
        {
            case VIA_CX700:
            case VIA_CN750:
            case VIA_P4M890:
            case VIA_P4M900:
            case VIA_VX800:
                 /* Turn on Hardware icon Cursor */
                 if(!pBIOSInfo->DuoView)
                {
                    switch(Screen_IGA_Map[pScrn->scrnIndex])
                    {
                         /* Turn on Hardware icon Cursor */
                        case IGA1:
                            VIASETREG(PRIM_HI_CTRL, 0x36000005);
                            break;
                        case IGA2:
                            VIASETREG(HI_CONTROL, 0xB6000005);
                            break;
                    }
                }
                else
                {
                    VIASETREG(PRIM_HI_CTRL, 0x36000005);
                    VIASETREG(HI_CONTROL, 0xb6000005);
                }
                 break;
            
            default:
                switch(Screen_IGA_Map[pScrn->scrnIndex])
                {
                     /* Turn on Hardware icon Cursor */
                case IGA1:
                    VIASETREG(HI_CONTROL, 0x76000005);
                    break;                  
                case IGA2:                    
                    VIASETREG(HI_CONTROL, 0xf6000005);
                    break;
                }
        }
    }
    else
#endif
    {
        if ((pBIOSInfo->Chipset == VIA_CX700) ||
            (pBIOSInfo->Chipset == VIA_CN750) ||
            (pBIOSInfo->Chipset == VIA_VX800))
        {
            /* Set color to white and black. */
            /* Use 30 bits [29:0] to specify HW cursor background/foreground color. */
            VIASetCursorColors(pScrn, 0x3fffffff, 0);
        }

        dwCursorMode = VIAGETREG(VIA_REG_CURSOR_MODE);
        dwCursorMode &=~0x80000003;
        /* Turn on Hardware Cursor */
        if(Screen_IGA_Map[pScrn->scrnIndex] == IGA1)
        VIASETREG(VIA_REG_CURSOR_MODE, dwCursorMode | 0x1);
        else
        VIASETREG(VIA_REG_CURSOR_MODE, dwCursorMode | 0x80000001);
    }
#endif
}


void VIAHideCursor(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    CARD32 dwCursorMode;
    CARD32 dwHiControl;
    unsigned long* Screen_IGA_Map = pVia->pVidData->VIAGraphicInfo.Screen_IGA_Map;

#ifdef USEHwIconSimulateHwCursor
    switch(pVia->pBIOSInfo->Chipset )
    {
        case VIA_CX700:
        case VIA_CN750:
        case VIA_P4M890:
        case VIA_P4M900:
        case VIA_VX800:
            if(!pBIOSInfo->DuoView)
            {
                switch(Screen_IGA_Map[pScrn->scrnIndex])
                {
                case IGA1:
                    dwHiControl = VIAGETREG(PRIM_HI_CTRL);
                    /* Turn hardware icon cursor off. */
                    VIASETREG(PRIM_HI_CTRL, dwHiControl & 0xFFFFFFFA);
                    break;
                case IGA2:
                    dwHiControl = VIAGETREG(HI_CONTROL);
                    /* Turn hardware icon cursor off. */
                    VIASETREG(HI_CONTROL, dwHiControl & 0xFFFFFFFA);
                    break;
                }
            }
            else
            {
                dwHiControl = VIAGETREG(PRIM_HI_CTRL);
                /* Turn hardware icon cursor off. */
                VIASETREG(PRIM_HI_CTRL, dwHiControl & 0xFFFFFFFA);
                dwHiControl = VIAGETREG(HI_CONTROL);
                /* Turn hardware icon cursor off. */
                VIASETREG(HI_CONTROL, dwHiControl & 0xFFFFFFFA);
            }
            break;
        default:
            dwHiControl = VIAGETREG(HI_CONTROL);
            /* Turn hardware icon cursor off. */
            VIASETREG(HI_CONTROL, dwHiControl & 0xFFFFFFFA);
    }
#else

#ifdef ARGB_CURSOR
    if(CallWhileARGB )
    { 
        switch(pVia->pBIOSInfo->Chipset )
        {
            case VIA_CX700:
            case VIA_CN750:
            case VIA_P4M890:
            case VIA_P4M900:
            case VIA_VX800:
                if(!pBIOSInfo->DuoView)
                {
                    switch(Screen_IGA_Map[pScrn->scrnIndex])
                    {
                    case IGA1:
                        dwHiControl = VIAGETREG(PRIM_HI_CTRL);
                        /* Turn hardware icon cursor off. */
                        VIASETREG(PRIM_HI_CTRL, dwHiControl & 0xFFFFFFFA);
                        break;
                    case IGA2:
                        dwHiControl = VIAGETREG(HI_CONTROL);
                        /* Turn hardware icon cursor off. */
                        VIASETREG(HI_CONTROL, dwHiControl & 0xFFFFFFFA);
                        break;
                    }
                }
                else
                {
                    dwHiControl = VIAGETREG(PRIM_HI_CTRL);
                    /* Turn hardware icon cursor off. */
                    VIASETREG(PRIM_HI_CTRL, dwHiControl & 0xFFFFFFFA);
                    dwHiControl = VIAGETREG(HI_CONTROL);
                    /* Turn hardware icon cursor off. */
                    VIASETREG(HI_CONTROL, dwHiControl & 0xFFFFFFFA);
                }
                break;
            default:
                dwHiControl = VIAGETREG(HI_CONTROL);
                /* Turn hardware icon cursor off. */
                VIASETREG(HI_CONTROL, dwHiControl & 0xFFFFFFFA);
        }
    }
    else
#endif
    { 
        dwCursorMode = VIAGETREG(VIA_REG_CURSOR_MODE);
        /* Turn cursor off. */
        VIASETREG(VIA_REG_CURSOR_MODE, dwCursorMode & 0xFFFFFFFE);

        /* after showing X shape cursor switch to use hardware icon  engine*/
#ifdef ARGB_CURSOR
        CallWhileARGB = TRUE;
#endif
    }
#endif

}

void VIAGetHWCursorRealPosition(ScrnInfoPtr pScrn, int iga_path, int x, int y, int frameX0, int frameY0,
        int *xpos, int *ypos, unsigned char *xoff, unsigned char *yoff)
{
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    IGASETTINGINFO *IGASettingInfo = NULL;
    int ScreenSizeH, ScreenSizeV;
    int Hotx = 0, Hoty = 0;
    int ActiveDevs = 0;
    
    switch (iga_path)
    {
        case IGA1:
            IGASettingInfo = &pBIOSInfo->IGA1SettingInfo;
            break;
        case IGA2:
            IGASettingInfo = &pBIOSInfo->IGA2SettingInfo;
            break;
    }

    Hotx = pVia->pScreensPublicInfo->CursorInfo.hotx;
    Hoty = pVia->pScreensPublicInfo->CursorInfo.hoty;

    if (IGASettingInfo->IsPanning)
    {
        ScreenSizeH = pBIOSInfo->HDisplay;
        ScreenSizeV = pBIOSInfo->VDisplay;
    }
    else if (IGASettingInfo->IsDownScaling)
    {
        ScreenSizeH = pBIOSInfo->HorDownScaleSize;
        ScreenSizeV = pBIOSInfo->VerDownScaleSize;
        x += frameX0;
        y += frameY0;
        x = (x * ScreenSizeH) / pBIOSInfo->HDisplay;
        y = (y * ScreenSizeV) / pBIOSInfo->VDisplay;
        Hotx = (Hotx * ScreenSizeH) / pBIOSInfo->HDisplay;
        Hoty = (Hoty * ScreenSizeV) / pBIOSInfo->VDisplay;
    }
    else
    {
        ScreenSizeH = pBIOSInfo->VirtualX;
        ScreenSizeV = pBIOSInfo->VirtualY;
        x += frameX0;
        y += frameY0;
    }

    /*compute x*/
    if(x > ScreenSizeH-1)
    {
        *xpos = ScreenSizeH-1;
        *xoff = 0;
    }
    else if (x < 0) 
    {
        *xpos = 0;
        *xoff = (-x) & 0xFF;
        if (Hotx > 0)
        {
            if (*xoff >= Hotx)
            {
                *xoff = Hotx;
            }
        }
        else
        {
            *xoff = 0; 
        }
    } 
    else 
    {
        *xpos = x;
        *xoff = 0;
    }

    /*compute y*/
    if(y > ScreenSizeV-1)
    {
        *ypos = ScreenSizeV-1;
        *yoff = 0;
    }
    else if (y < 0) 
    {
        *ypos = 0;
        *yoff = (-y) & 0xFF;
        if (Hoty > 0)
        {
            if (*yoff >= Hoty)
            {
                *yoff = Hoty;
            }
        }
        else
        {
            *yoff = 0; 
        }
    } 
    else 
    {
        *ypos = y;
        *yoff = 0;
    }
}

#ifdef USEHwIconSimulateHwCursor
static void VIAMonoToArgbTrans(ScrnInfoPtr pScrn, unsigned char* src, CARD32* argb)
{
    VIAPtr pVia = VIAPTR(pScrn);
    int i, j, k, bits;
    unsigned int bg, fg;
    unsigned char *AndPlane;
    unsigned char *XorPlane;
    unsigned int pitch = pVia->CursorInfoRec->MaxWidth / 8;
    CARD32 *tmp = argb;

    /* bg and fg should set alpha value to 0xff, then it can show on screens */
    bg = 0xff000000 | pVia->pScreensPublicInfo->CursorInfo.CursorBG;
    fg = 0xff000000 | pVia->pScreensPublicInfo->CursorInfo.CursorFG;
    XorPlane = src;
    AndPlane = XorPlane + pitch;
    for(i = 0; i < pVia->pScreensPublicInfo->CursorInfo.height; i++)
    {
        for(j = 0; j < pitch; j++)
        {
            for(k = 7; k >= 0; k--)
            {
                bits = ((((AndPlane[j] >> k) & 0x1) << 1) | ((XorPlane[j] >> k) & 0x1));
                switch(bits & 0x03)
                {
                    case 0:
                        *tmp = bg;
                        break;
                    case 1:
                        *tmp = fg;
                        break;
                    case 2:
                    case 3:
                        *tmp = 0x0;
                        break;
                }
                tmp++;
            }
        }
        XorPlane += pitch * 2;
        AndPlane = XorPlane + pitch;
    }
}
#endif


static void VIALoadCursorImage(ScrnInfoPtr pScrn, unsigned char* src)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    ScrnInfoPtr pScrn0 = xf86Screens[0];
    VIAPtr pVia0 = VIAPTR(pScrn0);
    WaitIdle();
   
#ifndef USEHwIconSimulateHwCursor
#ifdef ARGB_CURSOR
    CallWhileARGB = FALSE; 
#endif
#endif

#ifdef USEHwIconSimulateHwCursor
    memcpy(pVia0->FBBase + pVia->pScreensPublicInfo->CursorInfo.HwCursorBufStart, src, MAX_CURS * MAX_CURS/8*2 );
    memset((CARD8 *)pVia0->FBBase + pVia->pScreensPublicInfo->CursorInfo.HIBufStart, 0, 
               (pVia->CursorInfoRec->MaxWidth * pVia->CursorInfoRec->MaxHeight * 4));
    VIAMonoToArgbTrans(pScrn, src, (CARD32 *)(pVia0->FBBase + pVia->pScreensPublicInfo->CursorInfo.HIBufStart));
    VIALoadIconImage(pScrn, (CARD32 *)(pVia0->FBBase + pVia->pScreensPublicInfo->CursorInfo.HIBufStart));
#else

    /* Upload the cursor image to the frame buffer. */
    if(!pVia->IsSecondary)
    {        
        memcpy(pVia->FBBase + pVia->pScreensPublicInfo->CursorInfo.HwCursorBufStart, src, MAX_CURS * MAX_CURS / 8 * 2);
    }
    else
    {
        /* mono color cursor is simple, it use two bit one pixel. so it use HxV/8*2 Bytes*/
        memcpy(pVia0->FBBase + pVia->pScreensPublicInfo->CursorInfo.HwCursorBufStart, src, MAX_CURS * MAX_CURS / 8 * 2);
    }
#endif
}

void VIASetCursorPosition(ScrnInfoPtr pScrn, int x, int y)
{
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    int             xpos, ypos;    
    unsigned char   xoff, yoff;
    unsigned long* Screen_IGA_Map = pVia->pVidData->VIAGraphicInfo.Screen_IGA_Map;

#ifdef USEHwIconSimulateHwCursor
    switch(pBIOSInfo->Chipset )
    {
        case VIA_CX700:
        case VIA_CN750:
        case VIA_P4M890:
        case VIA_P4M900:
        case VIA_VX800:
            if(!pBIOSInfo->DuoView)
            {
                switch(Screen_IGA_Map[pScrn->scrnIndex])
                {
                case IGA1:
                    /*Get IGA1 cursor position*/
                    VIAGetHWCursorRealPosition(pScrn, IGA1, x, y, pScrn->frameX0, pScrn->frameY0, &xpos, &ypos, &xoff, &yoff);
                    /*set hi position*/
                    VIASETREG(PRIM_HI_POSSTART, ((xpos << 16) | (ypos & 0x07ff)));
                    VIASETREG(PRIM_HI_CENTEROFFSET, ((xoff<< 16) | (yoff & 0x07ff)));
                    break;
                case IGA2:
                    /*Get IGA2 cursor position*/
                    VIAGetHWCursorRealPosition(pScrn, IGA2, x, y, pScrn->frameX0, pScrn->frameY0, &xpos, &ypos, &xoff, &yoff);
                    /*set hi position*/
                    VIASETREG(HI_POSSTART, ((xpos << 16) | (ypos & 0x07ff)));
                    VIASETREG(HI_CENTEROFFSET, ((xoff<< 16) | (yoff & 0x07ff)));
                    break;
                }
            }
            else
            {
                /*Get IGA1 cursor position*/
                VIAGetHWCursorRealPosition(pScrn, IGA1, x, y, pScrn->frameX0, pScrn->frameY0, &xpos, &ypos, &xoff, &yoff);

                /*set hi position*/
                VIASETREG(PRIM_HI_POSSTART, ((xpos << 16) | (ypos & 0x07ff)));
                VIASETREG(PRIM_HI_CENTEROFFSET, ((xoff<< 16) | (yoff & 0x07ff)));

                /*Get IGA2 cursor position*/
                VIAGetHWCursorRealPosition(pScrn, IGA2, x, y, pScrn->frameX0, pScrn->frameY0, &xpos, &ypos, &xoff, &yoff);
                
                /*set hi position*/
                VIASETREG(HI_POSSTART, ((xpos << 16) | (ypos & 0x07ff)));
                VIASETREG(HI_CENTEROFFSET, ((xoff<< 16) | (yoff & 0x07ff)));
            }
            break;
        default:
            /*Get IGA cursor position*/
            VIAGetHWCursorRealPosition(pScrn, (int)Screen_IGA_Map[pScrn->scrnIndex], x, y, pScrn->frameX0, pScrn->frameY0, &xpos, &ypos, &xoff, &yoff);
            
            /*set hi position*/
            VIASETREG(HI_POSSTART, ((xpos << 16) | (ypos & 0x07ff)));
            VIASETREG(HI_CENTEROFFSET, ((xoff<< 16) | (yoff & 0x07ff)));
            break;
    }
#else
		
#ifdef ARGB_CURSOR
    if(CallWhileARGB)
    { 
        switch(pBIOSInfo->Chipset )
        {
            case VIA_CX700:
            case VIA_CN750:
            case VIA_P4M890:
            case VIA_P4M900:
            case VIA_VX800:
                if(!pBIOSInfo->DuoView)
                {
                    switch(Screen_IGA_Map[pScrn->scrnIndex])
                    {
                    case IGA1:
                        /*Get IGA1 cursor position*/
                        VIAGetHWCursorRealPosition(pScrn, IGA1, x, y, pScrn->frameX0, pScrn->frameY0, &xpos, &ypos, &xoff, &yoff);
                        /*set hi position*/
                        VIASETREG(PRIM_HI_POSSTART, ((xpos << 16) | (ypos & 0x07ff)));
                        VIASETREG(PRIM_HI_CENTEROFFSET, ((xoff<< 16) | (yoff & 0x07ff)));
                        break;
                    case IGA2:
                        /*Get IGA2 cursor position*/
                        VIAGetHWCursorRealPosition(pScrn, IGA2, x, y, pScrn->frameX0, pScrn->frameY0, &xpos, &ypos, &xoff, &yoff);
                        /*set hi position*/
                        VIASETREG(HI_POSSTART, ((xpos << 16) | (ypos & 0x07ff)));
                        VIASETREG(HI_CENTEROFFSET, ((xoff<< 16) | (yoff & 0x07ff)));
                        break;
                    }
                }
                else
                {
                    /*Get IGA1 cursor position*/
                    VIAGetHWCursorRealPosition(pScrn, IGA1, x, y, pScrn->frameX0, pScrn->frameY0, &xpos, &ypos, &xoff, &yoff);

                    /*set hi position*/
                    VIASETREG(PRIM_HI_POSSTART, ((xpos << 16) | (ypos & 0x07ff)));
                    VIASETREG(PRIM_HI_CENTEROFFSET, ((xoff<< 16) | (yoff & 0x07ff)));

                    /*Get IGA2 cursor position*/
                    VIAGetHWCursorRealPosition(pScrn, IGA2, x, y, pScrn->frameX0, pScrn->frameY0, &xpos, &ypos, &xoff, &yoff);
                    
                    /*set hi position*/
                    VIASETREG(HI_POSSTART, ((xpos << 16) | (ypos & 0x07ff)));
                    VIASETREG(HI_CENTEROFFSET, ((xoff<< 16) | (yoff & 0x07ff)));
                }
                break;
            default:
                /*Get IGA cursor position*/
                VIAGetHWCursorRealPosition(pScrn, (int)Screen_IGA_Map[pScrn->scrnIndex], x, y, pScrn->frameX0, pScrn->frameY0, &xpos, &ypos, &xoff, &yoff);
                
                /*set hi position*/
                VIASETREG(HI_POSSTART, ((xpos << 16) | (ypos & 0x07ff)));
                VIASETREG(HI_CENTEROFFSET, ((xoff<< 16) | (yoff & 0x07ff)));
                break;
        }
    }
    else
#endif
    {
        VIAGetHWCursorRealPosition(pScrn, (int)Screen_IGA_Map[pScrn->scrnIndex], x, y, pScrn->frameX0, pScrn->frameY0, &xpos, &ypos, &xoff, &yoff);
        VIASETREG(VIA_REG_CURSOR_ORG, ((xoff << 16) | (yoff & 0x003f)));
        VIASETREG(VIA_REG_CURSOR_POS, ((xpos << 16) | (ypos & 0x07ff)));
   }
#endif
}


static void VIASetCursorColors(ScrnInfoPtr pScrn, int bg, int fg)
{
    ScrnInfoPtr pScrn0 = xf86Screens[0];
    VIAPtr pVia0 = VIAPTR(pScrn0);

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASetCursorColors\n"));
#ifdef USEHwIconSimulateHwCursor
    /* 
     * Because Xserver always load cursor image before set cursor color, if
     * use hardware icon simulate hardware cursor, hardware cursor should
     * retransfer and reload new hardware icon image.
     */
    if((fg != pVia0->pScreensPublicInfo->CursorInfo.CursorFG)
        || (bg != pVia0->pScreensPublicInfo->CursorInfo.CursorBG))
    {
        pVia0->pScreensPublicInfo->CursorInfo.CursorFG = fg;
        pVia0->pScreensPublicInfo->CursorInfo.CursorBG = bg;
        VIAMonoToArgbTrans(pScrn, (unsigned char *)(pVia0->FBBase + pVia0->pScreensPublicInfo->CursorInfo.HwCursorBufStart),
                               (CARD32 *)(pVia0->FBBase + pVia0->pScreensPublicInfo->CursorInfo.HIBufStart));
        VIALoadIconImage(pScrn, (CARD32 *)(pVia0->FBBase + pVia0->pScreensPublicInfo->CursorInfo.HIBufStart));
    }
#else
    VIASETREG(VIA_REG_CURSOR_FG, fg);
    VIASETREG(VIA_REG_CURSOR_BG, bg);
#endif
}

#ifdef ARGB_CURSOR

/*enable ALPHA cursor */
static Bool VIAHiUseHWCursorARGB(ScreenPtr pScreen, CursorPtr pCurs)
{
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    VIAPtr pVia = VIAPTR(pScrn);

    pVia->pScreensPublicInfo->CursorInfo.width
		= (pCurs->bits->width > MAX_CURS) ? MAX_CURS : pCurs->bits->width;
    pVia->pScreensPublicInfo->CursorInfo.height
		= (pCurs->bits->height > MAX_CURS) ? MAX_CURS : pCurs->bits->height;
    
    /*Need to confine hotx, hoty to MAX_CURS?*/
    pVia->pScreensPublicInfo->CursorInfo.hotx = pCurs->bits->xhot;
    pVia->pScreensPublicInfo->CursorInfo.hoty = pCurs->bits->yhot;
	
    VIACheckCursorTypeToUse(pScrn);
    /* Switch SW/HW cursor dynamically. */
    if (pVia->pScreensPublicInfo->UseHwCursor)
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}


static void VIAHiLoadCursorARGB (ScrnInfoPtr pScrn, CursorPtr pCurs)
{
    VIAPtr pVia = VIAPTR(pScrn);
    ScrnInfoPtr pScrn0 = xf86Screens[0];
    VIAPtr pVia0 = VIAPTR(pScrn0);
	int x, y;
    CARD32 *image = pCurs->bits->argb;
    CARD32 *IconStartAddr = (CARD32 *)(pVia0->FBBase + pVia->pScreensPublicInfo->CursorInfo.HIBufStart);

    if (!pCurs->bits->argb)
        return;

    for (y = 0; y < pVia->pScreensPublicInfo->CursorInfo.height; y++)
    {
        for (x = 0; x < pVia->pScreensPublicInfo->CursorInfo.width; x++)
        {
            *IconStartAddr++ = *image++;
        }
        
        /* pad to the right with transparent */
        for (; x < pVia->CursorInfoRec->MaxWidth; x++)
        {   
            *IconStartAddr++ = 0;
        }
    }

    /* pad below with transparent */
    for (; y < pVia->CursorInfoRec->MaxHeight; y++)
    {  
        for (x = 0; x < pVia->CursorInfoRec->MaxWidth; x++)
        {
            *IconStartAddr++ = 0;
        }
    }

    VIALoadIconImage(pScrn, (CARD32 *)(pVia0->FBBase + pVia->pScreensPublicInfo->CursorInfo.HIBufStart));
}

#endif

#ifdef USESwScaling
/* Use linear interpolation algorithm to scale up/down cursor image. */
int VIAARGBCursorSWScaling(VIAPtr pVia, unsigned char *pSrc, unsigned char *pDest,
	                     int SrcWidth, int SrcHeight, int DestWidth, int DestHeight)
{
    int xSrc, ySrc, xDest, yDest;
    int xParam, yParam;
    int i;
    int Bpp = 4;
    unsigned char *pPrevLine, *pNextLine;
    unsigned char *pTmp;
    unsigned char *pLeftUp, *pRightUp, *pLeftDown, *pRightDown;

    for(yDest = 0; yDest <= DestHeight; yDest++)
    {
        pTmp = pDest + (pVia->CursorInfoRec->MaxWidth * yDest * Bpp);
	    ySrc = yDest * SrcHeight / DestHeight;
	    yParam = DestHeight - (yDest * SrcHeight % DestHeight);
	    pPrevLine = pSrc + (pVia->CursorInfoRec->MaxWidth * ySrc * Bpp);
        ySrc++;

        if(yParam == DestHeight)
            pNextLine = pPrevLine;
        else
            pNextLine = pSrc + (pVia->CursorInfoRec->MaxWidth * ySrc * Bpp);
		
        for(xDest = 0; xDest <= DestWidth; xDest++)
        {
            xSrc = xDest * SrcWidth / DestWidth * Bpp;
            xParam = DestWidth - xDest * SrcWidth % DestWidth;
            pLeftUp = pPrevLine + xSrc;
            pRightUp = pLeftUp + Bpp;
            pLeftDown = pNextLine + xSrc;
            pRightDown = pLeftDown + Bpp;
            if ( xParam == DestWidth )
            {
                pRightUp = pLeftUp;
                pRightDown = pLeftDown;
            }
            for(i = 0; i < Bpp; i++)
            {
                *pTmp++ = (unsigned char)(unsigned int)(
                    (xParam * yParam * ((*pLeftUp++) - *pRightUp - *pLeftDown + *pRightDown) 
                    + DestWidth * yParam * (*pRightUp++)
                    + DestHeight * xParam * (*pLeftDown++) 
                    + (DestWidth * DestHeight - DestHeight * xParam - DestWidth * yParam) * (*pRightDown++)
                    + DestWidth * DestHeight / 2)
                    / ( DestWidth * DestHeight ));
            }
        }	
    }
    return 0;
}
#endif

void VIADoIconScaling(VIAPtr pVia, int iga_path, CARD32* src, int IconOffset)
{
    VIAPtr pVia0 = VIAPTR(xf86Screens[0]);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    IGASETTINGINFO *pIGASettingInfo = NULL;
    CARD32 *IconStartAddr = NULL;
    int x, y;
    CARD32 lSrcPitch, dwSrcWidth, dwSrcHeight;
    CARD32 newHIwidth,newHIheight;
    CARD32 newHActive, newVActive;
    RECTL rScreenDest;
    RECTL rIconSrc;

    switch (iga_path)
    {
        case IGA1:
            pIGASettingInfo = &pBIOSInfo->IGA1SettingInfo;
            break;
        case IGA2:
            pIGASettingInfo = &pBIOSInfo->IGA2SettingInfo;
            break;
    }
    
    rScreenDest.left = rScreenDest.top = 0;

    newHActive = rScreenDest.right - rScreenDest.left;
    newVActive = rScreenDest.bottom - rScreenDest.top;
    newHIwidth = pVia->pScreensPublicInfo->CursorInfo.width * newHActive / (pBIOSInfo->CrtcHDisplay);
    newHIheight = pVia->pScreensPublicInfo->CursorInfo.height * newVActive / (pBIOSInfo->CrtcVDisplay);
#ifdef USESwScaling
    VIAARGBCursorSWScaling(pVia, (unsigned char *)src, (unsigned char *)(pVia0->FBBase + IconOffset),
        (pVia->pScreensPublicInfo->CursorInfo.width - 1), (pVia->pScreensPublicInfo->CursorInfo.height - 1),
        (newHIwidth - 1), (newHIheight - 1));
#else            
    rIconSrc.left = rIconSrc.top = 0;
    rIconSrc.right = pVia->pScreensPublicInfo->CursorInfo.width;
    rIconSrc.bottom = pVia->pScreensPublicInfo->CursorInfo.height;
    dwSrcWidth = dwSrcHeight = MAX_CURS;
    /*Note the alpha cursor should be 32bpp */
    lSrcPitch = MAX_CURS * 4;
    /* do cursor image adjust*/
#endif
    IconStartAddr = (CARD32 *)(pVia0->FBBase + IconOffset);
    for (y = 0; y < newHIheight; y++)
    {
        for (x = 0; x < newHIwidth; x++)
        {
            IconStartAddr++ ;
        }            
        /* pad to the right with transparent */
        for (; x < MAX_CURS; x++)
        {   
            *IconStartAddr++ = 0;
        }
    }

    /* pad below with transparent */
    for (; y < MAX_CURS; y++)
    {  
        for (x = 0; x < (MAX_CURS); x++)
        {
            *IconStartAddr++ = 0;
        }
    }
}

void VIALoadIconImage(ScrnInfoPtr pScrn, CARD32* src)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    int     IconOnIGA1IsHandled = FALSE, IconOnIGA2IsHandled = FALSE;
    unsigned long* Screen_IGA_Map = pVia->pVidData->VIAGraphicInfo.Screen_IGA_Map;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIALoadIconImage\n"));

    if (pBIOSInfo->DuoView)
    {
        if (!IconOnIGA1IsHandled)
        {
            VIASETREG(PRIM_HI_FBOFFSET, pVia->pScreensPublicInfo->CursorInfo.HIBufStart);
        }
        if (!IconOnIGA2IsHandled)
        {
            VIASETREG(HI_FBOFFSET, pVia->pScreensPublicInfo->CursorInfo.HIBufStart);
        }
    }
    else
    {
        switch(pBIOSInfo->Chipset)
        {
            case VIA_CX700:
            case VIA_CN750:
            case VIA_P4M890:
            case VIA_P4M900:
            case VIA_VX800:
                if (Screen_IGA_Map[pScrn->scrnIndex] == IGA1)
                {
                    VIASETREG(PRIM_HI_FBOFFSET, pVia->pScreensPublicInfo->CursorInfo.HIBufStart);
                }
                else
                {
                    VIASETREG(HI_FBOFFSET, pVia->pScreensPublicInfo->CursorInfo.HIBufStart);
                }
                break;
            default:
                /*One HW Icon case: Duo view case uses SW cursor!*/
				VIASETREG(HI_FBOFFSET, pVia->pScreensPublicInfo->CursorInfo.HIBufStart);
                break;
        }
    }
}

