/*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
#include "via_driver.h"
#include "via_dvi.h"
#include "hw.h"

/* Extern Functions Prototypes Declaration */
extern void write_reg_mask(CARD8 index, int io_port, CARD8 data, CARD8 mask);
extern int SearchModeSetting(int ModeInfoIndex);
extern void fill_crtc_timing(VIABIOSInfoPtr pBIOSInfo, struct crt_mode_table *crt_table, int mode_array, int bpp_byte, int set_iga);
extern __inline__ CARD8 read_reg(int io_port, CARD8 index);
extern __inline__ void write_reg(CARD8 index, CARD16 io_port, CARD8 data);
extern void EnableSecondDisplayChannel();
extern int g_DeviceI2CPort;
extern Bool i2cWriteByte(VIABIOSInfoPtr pBIOSInfo, CARD8 slave_addr, CARD8 index, CARD8 data);
extern Bool i2cReadByte(VIABIOSInfoPtr pBIOSInfo, CARD8 slave_addr, CARD8 index, CARD8 *pData);
extern void delayIn_usec(int usec);
extern void load_reg(int timing_value, int load_reg_num, struct io_register *reg, int io_type);
extern void VIASetDisplayChannel(VIABIOSInfoPtr pBIOSInfo);
extern void load_crtc_shadow_timing(VIABIOSInfoPtr pBIOSInfo, struct display_timing mode_timing, struct display_timing panel_timing);
extern void load_fetch_count_reg(int h_addr, int bpp_byte, int set_iga);

CEA_TIMING_FORMAT CEATimingTable[] = {
    { 640,  480},    /*  640 x  480P @ 60 Hz    */
    { 720,  480},    /*  720 x  480P @ 60 Hz    */
    { 720,  480},    /*  720 x  480P @ 60 Hz    */
    {1280,  720},    /* 1280 x  720P @ 60 Hz    */
    {1920, 1080},    /* 1920 x 1080i @ 60 Hz    */
    { 720,  480},    /*  720 x  480i @ 60 Hz    */
    { 720,  480},    /*  720 x  480i @ 60 Hz    */
    { 720,  240},    /*  720 x  240P @ 60 Hz    */
    { 720,  240},    /*  720 x  240P @ 60 Hz    */
    {2880,  480},    /* 2880 x  480i @ 60 Hz    */
    {2880,  480},    /* 2880 x  480i @ 60 Hz    */
    {2880,  240},    /* 2880 x  240P @ 60 Hz    */
    {2880,  240},    /* 2880 x  240P @ 60 Hz    */
    {1440,  480},    /* 1440 x  480P @ 60 Hz    */
    {1440,  480},    /* 1440 x  480P @ 60 Hz    */
    {1920, 1080},    /* 1920 x 1080P @ 60 Hz    */
    { 720,  576},    /*  720 x  576P @ 50 Hz    */
    { 720,  576},    /*  720 x  576P @ 50 Hz    */ 
    {1280,  720},    /* 1280 x  720P @ 50 Hz    */
    {1920, 1080},    /* 1920 x 1080i @ 50 Hz    */
    { 720,  576},    /*  720 x  576i @ 50 Hz    */
    { 720,  576},    /*  720 x  576i @ 50 Hz    */
    { 720,  288},    /*  720 x  288P @ 50 Hz    */
    { 720,  288},    /*  720 x  288P @ 50 Hz    */
    {2880,  576},    /* 2880 x  576i @ 50 Hz    */
    {2880,  576},    /* 2880 x  576i @ 50 Hz    */
    {2880,  288},    /* 2880 x  288P @ 50 Hz    */
    {2880,  288},    /* 2880 x  288P @ 50 Hz    */
    {1440,  576},    /* 1440 x  576P @ 50 Hz    */
    {1440,  576},    /* 1440 x  576P @ 50 Hz    */
    {1920, 1080},    /* 1920 x 1080P @ 50 Hz    */
    {1920, 1080},    /* 1920 x 1080P @ 24 Hz    */
    {1920, 1080},    /* 1920 x 1080P @ 25 Hz    */
    {1920, 1080}     /* 1920 x 1080P @ 30 Hz    */
};


void VIADVISetMode(VIABIOSInfoPtr pBIOSInfo)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIADVISetMode!!\n"));

    VIASetDFPMode(pBIOSInfo);
    VIAEnableDFP(pBIOSInfo);
}


void VIADVIPatchSkew(VIABIOSInfoPtr pBIOSInfo, int DIPort)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIADVIPatchSkew: DIPort=%d\n", DIPort));
    
    switch (DIPort)
    {
        case VIA_DI_DVP0:
            VIADVIPatchSkew_DVP0(pBIOSInfo);
            break;

        case VIA_DI_DVP1:
            VIADVIPatchSkew_DVP1(pBIOSInfo);
            break;
            
        case VIA_DI_DFPLOW:
            VIADVIPatchSkew_DFPLOW(pBIOSInfo);            
            break;

        case VIA_DI_DFPHIGH:
            break;

        default:
            break;
    }

    /* Replace the default patch if get user DPA settings from xorg: */
    if (pBIOSInfo->UserDPASettingInfo.HasUserSetting_Gfx)
    {
        VIASetDPA_Gfx(DIPort, &(pBIOSInfo->UserDPASettingInfo.GfxDPA));
    }
}

void VIADVIPatchSkew_DVP0(VIABIOSInfoPtr pBIOSInfo)
{
    switch (pBIOSInfo->Chipset)
    {
        case VIA_P4M890:
        {
            if((pBIOSInfo->CrtcHDisplay==1600)&&(pBIOSInfo->CrtcVDisplay==1200))
                write_reg_mask(CR96, VIACR, 0x03, BIT0+BIT1+BIT2);
            else
                write_reg_mask(CR96, VIACR, 0x07, BIT0+BIT1+BIT2);
            break;
        }

        case VIA_P4M900:
        {
            write_reg_mask(CR96, VIACR, 0x02, BIT0+BIT1+BIT2);
            break;
        }

        default:
        {
           break;
        }
    }
}

void VIADVIPatchSkew_DVP1(VIABIOSInfoPtr pBIOSInfo)
{
    return;
}

void VIADVIPatchSkew_DFPLOW(VIABIOSInfoPtr pBIOSInfo)
{
    switch (pBIOSInfo->Chipset)
    {
        case VIA_K8M890:
        {
            write_reg_mask(CR99, VIACR, 0x03, BIT0+BIT1);
            break;
        }
        
        case VIA_P4M900:        
        {
            write_reg_mask(CR99, VIACR, 0x09, BIT0+BIT1+BIT2+BIT3);
            break;
        }

        case VIA_P4M890:        
        {
            write_reg_mask(CR99, VIACR, 0x0F, BIT0+BIT1+BIT2+BIT3);
            break;
        }

        default:
        {
            break;
        }
    }
}

void VIASetDVIOutputPath(VIABIOSInfoPtr pBIOSInfo, int set_iga, int output_interface)
{
    switch(output_interface)
    {
        case VIA_DI_DVP0:
            write_reg_mask(CR6B, VIACR, 0x01, BIT0);

            if (set_iga == IGA1)
            {
                write_reg_mask(CR96, VIACR, 0x00, BIT4);
                write_reg_mask(CR6C, VIACR, 0x21, BIT0+BIT5+BIT7);
            }
            else
            {
                write_reg_mask(CR96, VIACR, 0x10, BIT4);
                write_reg_mask(CR6C, VIACR, 0xA1, BIT0+BIT5+BIT7);
            }
                 
            write_reg_mask(SR1E, VIASR, 0xC0, BIT7+BIT6);
            break;

        case VIA_DI_DVP1:
            if (pBIOSInfo->Chipset == VIA_CLE266)
            {
                if (set_iga== IGA1)
                    write_reg_mask(CR93, VIACR, 0x21, BIT0+BIT5+BIT7);
                else
                    write_reg_mask(CR93, VIACR, 0xA1, BIT0+BIT5+BIT7);
            }
            else
            {
                if (set_iga== IGA1)
                    write_reg_mask(CR9B, VIACR, 0x00, BIT4);
                else
                    write_reg_mask(CR9B, VIACR, 0x10, BIT4);
            }

            write_reg_mask(SR1E, VIASR, 0x30, BIT4+BIT5);
            break;

        case VIA_DI_DFPHIGH:
            if (pBIOSInfo->Chipset != VIA_CLE266)
            {
                if (set_iga== IGA1)
                {
                    write_reg_mask(CR96, VIACR, 0x00, BIT4);
                    write_reg_mask(CR97, VIACR, 0x03, BIT0+BIT1+BIT4);
                }
                else
                {
                    write_reg_mask(CR96, VIACR, 0x10, BIT4);
                    write_reg_mask(CR97, VIACR, 0x13, BIT0+BIT1+BIT4);
                }
            }        
            write_reg_mask(SR2A, VIASR, 0x0C, BIT2+BIT3);
            break;
            
        case VIA_DI_DFPLOW:
            if (pBIOSInfo->Chipset == VIA_CLE266)
            {
                break;
            }

            if (set_iga == IGA1)
            {
                write_reg_mask(CR99, VIACR, 0x00, BIT4);
                write_reg_mask(CR9B, VIACR, 0x00, BIT4);
            }
            else
            {                
                write_reg_mask(CR99, VIACR, 0x10, BIT4);
                write_reg_mask(CR9B, VIACR, 0x10, BIT4);
            }

            write_reg_mask(SR2A, VIASR, 0x03, BIT0+BIT1);
            break;

        case VIA_DI_TMDS:
            if (set_iga == IGA1)
            {
                write_reg_mask(CR99, VIACR, 0x00, BIT4);
            }
            else
            {
                write_reg_mask(CR99, VIACR, 0x10, BIT4);
            }
            
            break;
    }
}

int VIATMDSIdentify_VT1632(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   W_Buffer[2], R_Buffer[4];
    I2CDevPtr       dev;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIATMDSIdentify_VT1632\n"));
    
    /* Sense VT1632 TMDS Transmiter */
    if (xf86I2CProbeAddress(pBIOSInfo->I2C_Port2, 0x10))
    {
        dev = xf86CreateI2CDevRec();
        dev->DevName = "TMDS";
        dev->SlaveAddr = 0x10;
        dev->pI2CBus = pBIOSInfo->I2C_Port2;

        if (xf86I2CDevInit(dev))
        {
            W_Buffer[0] = 0x00;
            xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,4);

            if (R_Buffer[0] == 0x06 && R_Buffer[1] == 0x11)
            {
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED,
                                 "Found VIA TMDS Transmiter!\n"));
                if (R_Buffer[2] == 0x92 && R_Buffer[3] == 0x31)
                {
                    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIA: VT1632!\n"));
                    pBIOSInfo->TMDSSettingInfo.ChipID = VIA_VT3192;
                    pBIOSInfo->TMDSSettingInfo.I2CPort = PORT_INDEX_I2C_31;                    
                    
                    xf86DestroyI2CDevRec(dev,TRUE);

                    return FOUND_EXTERNAL_DEV_BY_I2C;
                }
                else
                {
                    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIA: Unknow Chip!!\n"));
                    xf86DestroyI2CDevRec(dev,TRUE);
                }
            }
            else
            {
                xf86DestroyI2CDevRec(dev,TRUE);
            }
        }
        else
        {
            xf86DestroyI2CDevRec(dev,TRUE);
        }
    }

    /* Not found by I2C, use GPIO to try again: */
    /* CLE266 has no Low-Swing device */
    if (pBIOSInfo->Chipset != VIA_CLE266)
    {
        GPIOI2C_Initial(pBIOSInfo, 0x10);
        GPIOI2C_Read(pBIOSInfo, 0, R_Buffer, 4);
        if (R_Buffer[0] == 0x06 && R_Buffer[1] == 0x11)
        {
            if (R_Buffer[2] == 0x92 && R_Buffer[3] == 0x31)
            {
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED,
                                 "Found TMDS Transmiter VT1632.\n"));
                pBIOSInfo->TMDSSettingInfo.ChipID = VIA_VT3192;
                pBIOSInfo->TMDSSettingInfo.I2CPort = PORT_INDEX_GPIO_2C; 
                
                return FOUND_EXTERNAL_DEV_BY_GPIO;
            }
        }
    }

    return NOT_FOUND_DEV;
}



Bool VIATMDSIdentify_VT1632A(VIABIOSInfoPtr pBIOSInfo, TMDSSETTINGINFOPTR pTMDSSettingInfo, CARD32 PortNum)
{
    CARD8 Buffer[2];

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIATMDSIdentify_VT1632A: Try Port No. 0x%x\n", PortNum));
    
    /* Sense VT1632 TMDS Transmiter */
    pTMDSSettingInfo->I2CPort = PortNum;
    pTMDSSettingInfo->I2CAddress = 0x10;

    /* Check vendor ID first: */
    if (!GPIOI2CRead_TMDS(pBIOSInfo, pTMDSSettingInfo, 0x00, &(Buffer[0])))
    {
        return FALSE;
    }
    
    if (!GPIOI2CRead_TMDS(pBIOSInfo, pTMDSSettingInfo, 0x01, &(Buffer[1])))
    {
        return FALSE;
    }

    if (!((Buffer[0] == 0x06) && (Buffer[1] == 0x11)))
    {        
        return FALSE;        
    }

    /* Vendor ID is correct, so continue to check Chip ID: */
    if (!GPIOI2CRead_TMDS(pBIOSInfo, pTMDSSettingInfo, 0x02, &(Buffer[0])))
    {
        return FALSE;
    }
    
    if (!GPIOI2CRead_TMDS(pBIOSInfo, pTMDSSettingInfo, 0x03, &(Buffer[1])))
    {
        return FALSE;
    }

    if ((Buffer[0] == 0x92) && (Buffer[1] == 0x31))
    {
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "Found VIA VT1632 TMDS on port %x!\n", PortNum));
        pTMDSSettingInfo->ChipID = VIA_VT3192;
        return TRUE;
    }

    return FALSE;
}


int VIATMDSIdentify_IntegratedTMDS(VIABIOSInfoPtr pBIOSInfo)
{
    if (pBIOSInfo->Chipset == VIA_VX800)
    {
        if(VIAGetRevisionOfVX800() > REVISION_VX800_B)
            return NOT_FOUND_DEV;
    }

    /* VT3324 supports integrated TMDS. */
    if (((pBIOSInfo->Chipset == VIA_CX700) ||
         (pBIOSInfo->Chipset == VIA_CN750) ||
         (pBIOSInfo->Chipset == VIA_VX800)) &&
        ((pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_DVI_ONLY) || 
         (pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_LCD_DVI)))
    {
        /* 1. Integrated TMDS can't be sensed, so we direct assign the TMDS chip ID
              according to gfx chip.
           2. Integrated TMDS use I2C port 31 to get edid. */
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "Found Integrated TMDS.\n"));
        pBIOSInfo->TMDSSettingInfo.ChipID = VIA_INTEGRATED_TMDS;
        if (pBIOSInfo->Chipset == VIA_VX800)
        {        
            pBIOSInfo->TMDSSettingInfo.I2CPort = PORT_INDEX_GPIO_3D;
            pBIOSInfo->GPIOI2CInfo.bGPIOPort = PORT_INDEX_GPIO_3D;
        }
        else
        {
            pBIOSInfo->TMDSSettingInfo.I2CPort = PORT_INDEX_I2C_31;
        }
        return FOUND_INTERNAL_DEV;
    }

    return NOT_FOUND_DEV;
}

void VIATMDSRegInit_VT1632(VIABIOSInfoPtr pBIOSInfo)
{
    TMDSSETTINGINFOPTR pTMDSSettingInfo = &(pBIOSInfo->TMDSSettingInfo);

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIATMDSRegInit_VT1632\n"));
    
    /* If hotkey, we don't fill any value. VGA BIOS will take care it. */        
    if (pBIOSInfo->Hotkey)    
        return;
   
    pTMDSSettingInfo->I2CAddress = 0x10;
    
    if (pTMDSSettingInfo->BusWidth == VIA_DI_24BIT)
    {
        /* DVI bus width is 24 Bits, we can select to use single or dual edge */
        /* by option. */
        if (pTMDSSettingInfo->IsDualEdge)
        {
            GPIOI2CWrite_TMDS(pBIOSInfo, pTMDSSettingInfo, 0x08, 0x3F);
        }
        else
        {
            GPIOI2CWrite_TMDS(pBIOSInfo, pTMDSSettingInfo, 0x08, 0x37);
        }
    }
    else
    {
        /* DVI bus width is 12 Bits, we should use dual edge. */
        GPIOI2CWrite_TMDS(pBIOSInfo, pTMDSSettingInfo, 0x08, 0x3B);
    }
}


void VIATMDSRegInit(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   sr2a, sr1e, sr3e = 0, cr93 = 0;

    sr1e = read_reg(VIASR, SR1E);
    sr2a = read_reg(VIASR, SR2A);
    
    VIADIPortPadOn(pBIOSInfo->DFPDIPort);
    
    if (pBIOSInfo->Chipset == VIA_CLE266)
    {
        /*=* CR93[7] DI1 Data Source Selection: 1 = DSP2.
             CR93[5] DI1 Clock Source: 1 = internal.
             CR93[4] DI1 Clock Polarity.
             CR93[3:1] DI1 Clock Adjust. CR93[0] DI1 enable *=*/
        cr93 = read_reg(VIACR, CR93);
        write_reg(CR93, VIACR, 0x01);
    }
    else
    {
        /* SR3E[1]Multi-function selection:
           0 = Emulate I2C and DDC bus by GPIO2/3/4. */
        sr3e = read_reg(VIASR, SR3E);
        write_reg_mask(SR3E, VIASR, 0x0, BIT5);
    }
    
    switch (pBIOSInfo->TMDSSettingInfo.ChipID)
    {
        case VIA_VT3192:
            VIATMDSRegInit_VT1632(pBIOSInfo);
            break;

        default:
            break;
    }

    /* Restore status */
    write_reg(SR1E, VIASR, sr1e);
    write_reg(SR2A, VIASR, sr2a);
            
    if (pBIOSInfo->Chipset == VIA_CLE266)
    {
        write_reg(CR93, VIACR, cr93);
    }
    else
    {
        write_reg(SR3E, VIASR, sr3e);
    }
}


void VIAInitDVIDIPort(VIABIOSInfoPtr pBIOSInfo, int IdentifyState)
{
    if (pBIOSInfo->DFPDIPort != VIA_DI_NONE)
    {
        /* Do nothing, because DFP DIPort can be assigned by option. */
        return;
    }
        
    switch (pBIOSInfo->Chipset)
    {
        case VIA_CLE266:
             pBIOSInfo->DFPDIPort = VIA_DI_DVP1; 
             break;

       case VIA_CX700:
       case VIA_CN750:
       case VIA_VX800:
            /* Add TMDS(VT1632A) on AMR with DVP1 support in VT3324*/
            if (IdentifyState == FOUND_INTERNAL_DEV)
            {
            	/*Integrated TMDS*/
                pBIOSInfo->DFPDIPort = VIA_DI_TMDS;
            }
            else 
            {
                /*External TMDS*/
                pBIOSInfo->DFPDIPort = VIA_DI_DVP1;            
            }
            break;
            
        case VIA_K8M890:
        case VIA_P4M900:
        case VIA_P4M890:
            /* TMDS on PCIE, we set DFPLOW as default. */
            pBIOSInfo->DFPDIPort = VIA_DI_DFPLOW;            
            break;

        default:
            if (IdentifyState == FOUND_EXTERNAL_DEV_BY_I2C)
            {
                /* TMDS on AMR, we set DVP0 as default. */
                pBIOSInfo->DFPDIPort = VIA_DI_DVP0;
            }
            else
            {
                /* TMDS on AGP, we set DVP1 as default. */
                pBIOSInfo->DFPDIPort = VIA_DI_DVP1;
            }
            
            break;
    }
}

Bool VIATMDSIdentify(VIABIOSInfoPtr pBIOSInfo)
{
    int             IdentifyState = NOT_FOUND_DEV;
    unsigned char   sr2a = 0, sr1e = 0, sr3e = 0, cr93 = 0;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIATMDSIdentify\n"));

    switch (pBIOSInfo->Chipset)
    {
        case VIA_K8M890:
            /*=* DFP Low Pad on *=*/
            sr2a = read_reg(VIASR, SR2A);
            write_reg_mask(SR2A, VIASR, 0x03, BIT0+BIT1);
            break;

        case VIA_P4M900:
        case VIA_P4M890:
            /* DFP Low Pad on */
            sr2a = read_reg(VIASR, SR2A);
            write_reg_mask(SR2A, VIASR, 0x03, BIT0+BIT1);

            /* DVP0 Pad on */
            sr1e = read_reg(VIASR, SR1E);
            write_reg_mask(SR1E, VIASR, 0xC0, BIT6+BIT7);
            break;
               
        default:
            /*=* DVP0/DVP1 Pad on *=*/
            sr1e = read_reg(VIASR, SR1E);
            write_reg_mask(SR1E, VIASR, 0xF0, BIT4+BIT5+BIT6+BIT7);
            /*=* SR3E[1]Multi-function selection:
                 0 = Emulate I2C and DDC bus by GPIO2/3/4. *=*/
            sr3e = read_reg(VIASR, SR3E);
            write_reg_mask(SR3E, VIASR, 0x0, BIT5);
            break;
    }
    
    if (pBIOSInfo->Chipset == VIA_CLE266)
    {
        /*=* CR93[7] DI1 Data Source Selection: 1 = DSP2.
             CR93[5] DI1 Clock Source: 1 = internal.
             CR93[4] DI1 Clock Polarity.
             CR93[3:1] DI1 Clock Adjust. CR93[0] DI1 enable *=*/
        cr93 = read_reg(VIACR, CR93);
        write_reg(CR93, VIACR, 0x01);
    }
    /*add the default setting*/
    pBIOSInfo->TMDSSettingInfo.HDMINativesize = M1280x720;    
    
    if (VIATMDSIdentify_VT1632A(pBIOSInfo, &(pBIOSInfo->TMDSSettingInfo), PORT_INDEX_GPIO_2C))
    {
        pBIOSInfo->GPIOI2CInfo.bGPIOPort = PORT_INDEX_GPIO_2C;
        IdentifyState = FOUND_EXTERNAL_DEV_BY_GPIO;
    }
    else if (VIATMDSIdentify_VT1632A(pBIOSInfo, &(pBIOSInfo->TMDSSettingInfo), PORT_INDEX_I2C_31))
    {
        pBIOSInfo->GPIOI2CInfo.bGPIOPort = PORT_INDEX_I2C_31;
        IdentifyState = FOUND_EXTERNAL_DEV_BY_I2C;
    }
    else
    {
        IdentifyState = NOT_FOUND_DEV;
    }
    
    if (IdentifyState == NOT_FOUND_DEV)
    {
        IdentifyState = VIATMDSIdentify_IntegratedTMDS(pBIOSInfo);
    }

    /* Restore status */
    switch (pBIOSInfo->Chipset)
    {
        case VIA_K8M890:
            write_reg(SR2A, VIASR, sr2a);
            break;

        case VIA_P4M900:
        case VIA_P4M890:
            write_reg(SR2A, VIASR, sr2a);
            write_reg(SR1E, VIASR, sr1e);
            break;

        default: 
            write_reg(SR1E, VIASR, sr1e);
            write_reg(SR3E, VIASR, sr3e);
            break;
    }

    if (pBIOSInfo->Chipset == VIA_CLE266)
    {
        write_reg(CR93, VIACR, cr93);
    }

    
    if (IdentifyState == NOT_FOUND_DEV)
    {
        return FALSE;
    }
    else
    {
        VIAInitDVIDIPort(pBIOSInfo, IdentifyState);
        return TRUE;
    }
}

/*=*
 *
 * Bool VIASenseDVI(pBIOSInfo)
 *
 *     - Sense DVI Connector
 *
 * Return Type:    Bool
 *
 * The Definition of Input Value:
 *
 *                 VIABIOSInfoPtr
 *
 * The Definition of Return Value:
 *
 *                 DVI Attached  - TRUE
 *                 DVI Not Attached - FALSE
 *=*/
Bool VIASenseDVI(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   sr1e, cr6b = 0, cr91, cr93 = 0;
    unsigned char   SlaveAddr;
    Bool            ret = FALSE;
    I2CDevPtr       dev;
    unsigned char   W_Buffer[1];
    unsigned char   R_Buffer[1];

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIASenseDVI\n"));
    
    if (pBIOSInfo->NoDDCValue)
        return TRUE;
  
    if (pBIOSInfo->Chipset == VIA_CLE266)
    {
        /*=* DI1 Pad on *=*/
        sr1e = read_reg(VIASR, SR1E);
        write_reg_mask(SR1E, VIASR, 0x30, BIT4+BIT5);

        /*=* CR6B[0]VCK Input Selection: 1 = External clock. *=*/
        cr6b = read_reg(VIACR, CR6B);
        write_reg_mask(CR6B, VIACR, 0x08, BIT0+BIT1+BIT2+BIT3);

        /*=* CR91[4] VDD On [3] Data On [2] VEE On [1] Back Light Off
                 [0] Software Control Power Sequence *=*/
        cr91 = read_reg(VIACR, CR91);
        write_reg(CR91, VIACR, 0x1D);

        /*=* CR93[7] DI1 Data Source Selection: 1 = DSP2.
             CR93[5] DI1 Clock Source: 1 = internal.
             CR93[4] DI1 Clock Polarity.
             CR93[3:1] DI1 Clock Adjust. CR93[0] DI1 enable *=*/
        cr93 = read_reg(VIACR, CR93);
        write_reg(CR93, VIACR, 0x01);

        switch (pBIOSInfo->TMDSSettingInfo.ChipID)
        {
            case VIA_VT3192:
                SlaveAddr = 0x10;
                break;
            default:
                return ret;
                break;
        }

        if (xf86I2CProbeAddress(pBIOSInfo->I2C_Port2, SlaveAddr))
        {
            dev = xf86CreateI2CDevRec();
            dev->DevName = "TMDS";
            dev->SlaveAddr = SlaveAddr;
            dev->pI2CBus = pBIOSInfo->I2C_Port2;

            if (xf86I2CDevInit(dev))
            {
                W_Buffer[0] = 0x09;
                xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                xf86DestroyI2CDevRec(dev,TRUE);
                if (R_Buffer[0] & 0x04)
                    ret = TRUE;
            }
            else
                xf86DestroyI2CDevRec(dev,TRUE);
        }

        /* Restore status */
        write_reg(SR1E, VIASR, sr1e);
        write_reg(CR6B, VIACR, cr6b);
        write_reg(CR91, VIACR, cr91);
        write_reg(CR93, VIACR, cr93);
    }

    if (!ret)
    {
        if(VIAQueryDVIEDIDType(pBIOSInfo))
        {
            ret = TRUE;
        }
    }

    return ret;
}

int VIAQueryDVIEDIDType(VIABIOSInfoPtr pBIOSInfo)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIAQueryDVIEDIDType\n"));
    
    CARD8 Buffer[2];

    pBIOSInfo->TMDSSettingInfo.I2CAddress = 0xA0;
    
    if (!GPIOI2CRead_TMDS(pBIOSInfo, &(pBIOSInfo->TMDSSettingInfo), 0x00, &(Buffer[0])))    
        return FALSE;
    
    
    if (!GPIOI2CRead_TMDS(pBIOSInfo, &(pBIOSInfo->TMDSSettingInfo), 0x01, &(Buffer[1])))    
        return FALSE;    

    if ((Buffer[0] == 0) && (Buffer[1] == 0xFF))
            return 1; /* Found EDID1 Table */
    else
    {
        pBIOSInfo->TMDSSettingInfo.I2CAddress = 0xA2;
        if (!GPIOI2CRead_TMDS(pBIOSInfo, &(pBIOSInfo->TMDSSettingInfo), 0x00, &(Buffer[0])))                
            return FALSE;
        
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Buffer[0]=%d\n", Buffer[0]));
        
        if (Buffer[0] == 0x20)
            return 2; /* Found EDID2 Table */
        else
            return 0; /* Found No EDID Table */
    }

}


CARD8 VIAGetPanelSizeFromEDID(VIABIOSInfoPtr pBIOSInfo)
{        
    DisplayModePtr      modes;
    xf86MonPtr          edid_mon;      
    int                 i, max_h, max_v;

    unsigned char       W_Buffer[2];
    unsigned char       *R_Buffer;    
    
    R_Buffer = xcalloc(1,sizeof(unsigned char) * (128));
    
    pBIOSInfo->TMDSSettingInfo.I2CAddress = 0xA0;
    
    /* Read EDID v1.1 Block */
    for (i=0; i<128; i++)
    {
        GPIOI2CRead_TMDS(pBIOSInfo, &(pBIOSInfo->TMDSSettingInfo), i, R_Buffer+i);
    }    

    /* Parse the EDID block */
    edid_mon = xf86InterpretEDID(pBIOSInfo->scrnIndex,R_Buffer);
    
    if (edid_mon)
    {             
        /*Find max resolution from Established timing*/
        VIAFindMaxResFromEstablishedTiming(edid_mon, &max_h, &max_v);

        /*Find max resolution from Standard timing*/
        for (i = 0; i < 8; i++)
        {
            if (edid_mon->timings2[i].hsize > max_h)
            {
                max_h = edid_mon->timings2[i].hsize;
                max_v = edid_mon->timings2[i].vsize;
            }                
        }

        /*Find max resolution from Detail timing*/
        for (i = 0; i < 4; i++)
        {
            if ((edid_mon->det_mon[i].type == DT) && 
                (edid_mon->det_mon[i].section.d_timings.h_active > max_h))
            {
                max_h = edid_mon->det_mon[i].section.d_timings.h_active;
                max_v = edid_mon->det_mon[i].section.d_timings.v_active;
            }                
        }
        
        /* Correct monitor size to avoid invalid mode. */
        VIACorrectMonitorSize(&max_h, &max_v);            

        /* To prevent to get a wrong EDID data. We should assign a default value.*/
        if ((max_h == 0) || (max_v == 0))
        {
            max_h = 1024;
            max_v = 768;
        }

        pBIOSInfo->TMDSSettingInfo.DFPSize = VIAGetModeIndex(max_h, max_v);        
    }                                
         
    
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "DVI Panel Size ID = %x\n", pBIOSInfo->TMDSSettingInfo.DFPSize));
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "TMDS use mode, modes->HDisplay=%d, modes->VDisplay=%d\n", modes->HDisplay, modes->VDisplay));
    

}

/*=*
 *
 * unsigned char VIAGetPanelInfo(VIABIOSInfoPtr pBIOSInfo)
 *
 *     - Get Panel Size
 *
 * Return Type:    unsigned char
 *
 * The Definition of Input Value:
 *
 *                 VIABIOSInfoPtr
 *
 * The Definition of Return Value:
 *
 *                 0 - 640x480
 *                 1 - 800x600
 *                 2 - 1024x768
 *                 3 - 1280x768
 *                 4 - 1280x1024
 *                 5 - 1400x1050
 *                 6 - 1600x1200
 *                 0xFF - Not Supported Panel Size
 *=*/
unsigned char VIAGetPanelInfo(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   sr1e, sr3e = 0, cr6b = 0, cr91, cr93 = 0, cr9b = 0;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAGetPanelInfo\n"));

    if (pBIOSInfo->TMDSSettingInfo.DFPSize == M_INVALID)
    {
        if (pBIOSInfo->Chipset == VIA_CLE266)
        {
            /*=* DI1 Pad on *=*/
            sr1e = read_reg(VIASR, SR1E);
            write_reg_mask(SR1E, VIASR, 0xF0, BIT4+BIT5+BIT6+BIT7);

            /*=* CR6B[0]VCK Input Selection: 1 = External clock. *=*/
            cr6b = read_reg(VIACR, CR6B);
            write_reg_mask(CR6B, VIACR, 0x08, BIT0+BIT1+BIT2+BIT3);

            /*=* CR91[4] VDD On [3] Data On [2] VEE On [1] Back Light Off
                     [0] Software Control Power Sequence *=*/
            cr91 = read_reg(VIACR, CR91);
            write_reg(CR91, VIACR, 0x1D);

            /*=* CR93[7] DI1 Data Source Selection: 1 = DSP2.
                 CR93[5] DI1 Clock Source: 1 = internal.
                 CR93[4] DI1 Clock Polarity.
                 CR93[3:1] DI1 Clock Adjust. CR93[0] DI1 enable *=*/
            cr93 = read_reg(VIACR, CR93);
            write_reg(CR93, VIACR, 0x01);
        }
        else
        {
            /*=* DVP0/DVP1 Pad on *=*/
            sr1e = read_reg(VIASR, SR1E);
            write_reg_mask(SR1E, VIASR, 0xF0, BIT4+BIT5+BIT6+BIT7);

            /*=* SR3E[1]Multi-function selection:
                        0 = Emulate I2C and DDC bus by GPIO2/3/4. *=*/
            sr3e = read_reg(VIASR, SR3E);
            write_reg_mask(SR3E, VIASR, 0x0, BIT5);

            /*=* CR91[4] VDD On [3] Data On [2] VEE On [1] Back Light Off
                     [0] Software Control Power Sequence *=*/
            cr91 = read_reg(VIACR, CR91);
            write_reg(CR91, VIACR, 0x1D);

            /*=* CR9B[4] DVP1 Data Source Selection: 1 = From secondary display.
                 CR9B[2:0] DVP1 Clock Adjust *=*/
            cr9b = read_reg(VIACR, CR9B);
            write_reg(CR9B, VIACR, 0x1F);
        }

        switch (VIAQueryDVIEDIDType(pBIOSInfo))
        {
            case 1:
            case 2:
                VIAGetPanelSizeFromEDID(pBIOSInfo);
                break;
                           
            default:
                break;
        }

        /* Restore status */
        if (pBIOSInfo->Chipset == VIA_CLE266)
        {
            write_reg(SR1E, VIASR, sr1e);
            write_reg(CR6B, VIACR, cr6b);
            write_reg(CR91, VIACR, cr91);
            write_reg(CR93, VIACR, cr93);
        }
        else
        {
            write_reg(SR1E, VIASR, sr1e);
            write_reg(SR3E, VIASR, sr3e);
            write_reg(CR91, VIACR, cr91);
            write_reg(CR9B, VIACR, cr9b);
        }
    }
    
    VIAGetModeSizeByIndex(pBIOSInfo->TMDSSettingInfo.DFPSize, &pBIOSInfo->TMDSSettingInfo.PanelSizeH, &pBIOSInfo->TMDSSettingInfo.PanelSizeV);
    
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "DFPSize = %d\n", pBIOSInfo->TMDSSettingInfo.DFPSize));
    return (unsigned char)(pBIOSInfo->TMDSSettingInfo.DFPSize);
}

Bool VIASetDFPMode(VIABIOSInfoPtr pBIOSInfo)
{
    struct io_register      *reg = NULL;
    int                     load_reg_num=0;
    int                     reg_value=0;
    int                     offset;
	
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIASetDFPMode\n"));

    VideoSetMode(pBIOSInfo, pBIOSInfo->TMDSSettingInfo.IGAPath);

    if ((pBIOSInfo->IsSecondary)||(pBIOSInfo->DuoView))
    {
        /* Second Display Channel Enable */
        EnableSecondDisplayChannel();
        write_reg_mask(CR91, VIACR, 0x0, BIT5);
    }

    /* Disable LCD Scaling */
    if(pBIOSInfo->TMDSSettingInfo.IGAPath == IGA2)
    {
        write_reg(CR79, VIACR, 0x0);
    }

    /* Enable Interlace mode. */
    if (pBIOSInfo->IsEnableInterlaceMode)
    {
        if (pBIOSInfo->TMDSSettingInfo.IGAPath == IGA1)
        {
            write_reg_mask(CR33, VIACR, BIT6, BIT6);
        }
        else if (pBIOSInfo->TMDSSettingInfo.IGAPath == IGA2)
        {
            write_reg_mask(CR67, VIACR, BIT5, BIT5);
        }
    }
        
    if(pBIOSInfo->EPIA_DVI && pBIOSInfo->DuoView == FALSE && pBIOSInfo->SAMM== FALSE)
    {
        VideoSetMode(pBIOSInfo, pBIOSInfo->TMDSSettingInfo.IGAPath);
        switch(pBIOSInfo->TMDSSettingInfo.ModeIndex)
        {
            case M640x480:
                offset = 80;
                break;
            case M800x480:
            case M800x600:
                offset = 110;
                break;
            case M1024x768:
                offset = 150;
                break;
            case M1280x768:
            case M1280x800:
            case M1280x1024:
            case M1440x1050:
                offset = 190;
                break;
            case M1600x1200:
                offset = 250;
                break;
            case M1366x768:
            case M1360x768:
                offset = 212;                             
                break;
            default:
                offset = 140;
                break;
        }
        /* Offset for simultaneous */
        reg_value = offset;
        load_reg_num = offset_reg.iga2_offset_reg.reg_num;
        reg = offset_reg.iga2_offset_reg.reg;
        load_reg(reg_value, load_reg_num, reg, VIACR);
       
        write_reg_mask(CR6B, VIACR, 0x08, BIT3);
        write_reg_mask(CR6A, VIACR, 0x00, BIT6);
        write_reg_mask(CR6A, VIACR, 0x40, BIT6);
        write_reg_mask(SR16, VIASR, 0x40, BIT6);
    }

    switch (pBIOSInfo->Chipset)
    {
        case VIA_CLE266:
            if (pBIOSInfo->TMDSSettingInfo.IGAPath == IGA2)
            {
                write_reg_mask(CR93, VIACR, 0x89, BIT0+BIT1+BIT2+BIT3+BIT7);
            }
            else
            {
                write_reg_mask(CR93, VIACR, 0x09, BIT0+BIT1+BIT2+BIT3+BIT7);
            }
            break;
        case VIA_KM400:
            /* Set DVP1 clock skew 0x9B[2:0] and data source 0x9B[4] */
            if (pBIOSInfo->ChipRev < 5)   /* KM400 Ax */
            {
                if (pBIOSInfo->TMDSSettingInfo.IGAPath == IGA2)
                {
                    write_reg(CR9B, VIACR, 0x1F);
                }
                else
                {
                    write_reg(CR9B, VIACR, 0x0F);
                }
            }
            else
            {
                if (pBIOSInfo->TMDSSettingInfo.IGAPath == IGA2)
                {
                    write_reg(CR9B, VIACR, 0x17);
                }
                else
                {
                    write_reg(CR9B, VIACR, 0x07);
                }
            }
            break;
        case VIA_K8M800:
        case VIA_PM800:
        case VIA_P4M800PRO:
            if (pBIOSInfo->TMDSSettingInfo.IGAPath == IGA2)
            {
                write_reg(CR9B, VIACR, 0x11);
            }
            else
            {
                if(pBIOSInfo->IsDemoBoard)
                {
                    write_reg(CR9B, VIACR, 0x01);
                }
                else
            	{
                    write_reg_mask(CR97, VIACR, 0x01, BIT0);
            	}
            }
            break;
        default:
            break;
    }

    VIASetOutputPath(pBIOSInfo, VIA_DEVICE_DFP, pBIOSInfo->TMDSSettingInfo.IGAPath, pBIOSInfo->DFPDIPort);

    /* Adjust DPA Setting for clock skew problem: */
    VIADVIPatchSkew(pBIOSInfo, pBIOSInfo->DFPDIPort);

    /* Set display channel. */
    if ((pBIOSInfo->Chipset == VIA_CX700)||
        (pBIOSInfo->Chipset == VIA_CN750)||
        (pBIOSInfo->Chipset == VIA_VX800))
    {
        VIASetDisplayChannel(pBIOSInfo);
    }
    
    return TRUE;
}

/* If Enable DFP, turn on pad */
void VIAEnableDFP(VIABIOSInfoPtr pBIOSInfo)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAEnableDFP\n"));
    
    VIABIOSInfoPtr  pVia = pBIOSInfo;

    if (pBIOSInfo->DFPDIPort & VIA_DI_DVP0)
    {
        VGAOUT8(0x3c4, 0x1E);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0xC0);
    }
    if (pBIOSInfo->DFPDIPort & VIA_DI_DVP1)
    {
        VGAOUT8(0x3c4, 0x1E);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x30);
    }
    if (pBIOSInfo->DFPDIPort & VIA_DI_DFPHIGH)
    {
        VGAOUT8(0x3c4, 0x2A);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x0C);
    }
    if (pBIOSInfo->DFPDIPort & VIA_DI_DFPLOW)
    {
        VGAOUT8(0x3c4, 0x2A);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x03);
    }
    if(pBIOSInfo->EPIA_DVI)      /*Support DVI on EPIA*/
    {
        write_reg(CR91, VIACR, 0x1f);
        write_reg(CR88, VIACR, 0x00);
    }
    if (pBIOSInfo->DFPDIPort & VIA_DI_TMDS)
    {
        VGAOUT8(0x3d4, 0x91);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0x7F);   /* Turn on Display period in the panel path. */
        
        VGAOUT8(0x3d4, 0xD2);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0xF7);   /* Turn on TMDS power. */
    }
}

/* If Disable DFP, turn off pad */
void VIADisableDFP(VIABIOSInfoPtr pBIOSInfo)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIADisableDFP\n"));
    
    VIABIOSInfoPtr  pVia = pBIOSInfo;

    if (pBIOSInfo->DFPDIPort & VIA_DI_DVP0)
    {
        VGAOUT8(0x3c4, 0x1E);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0x3F);
    }

    if (pBIOSInfo->DFPDIPort & VIA_DI_DVP1)
    {
        VGAOUT8(0x3c4, 0x1E);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xCF);
    }

    if (pBIOSInfo->DFPDIPort & VIA_DI_DFPHIGH)
    {
        VGAOUT8(0x3c4, 0x2A);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xF3);
    }

    if (pBIOSInfo->DFPDIPort & VIA_DI_DFPLOW)
    {
        VGAOUT8(0x3c4, 0x2A);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xFC);
    }

    if (pBIOSInfo->DFPDIPort & VIA_DI_TMDS)
    {
        VGAOUT8(0x3d4, 0xD2);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) | 0x08);   /* Turn off TMDS power. */
    }
}

void VideoSetMode(VIABIOSInfoPtr pBIOSInfo, int set_iga)
{
    struct  VideoModeTable *VideoMode = NULL;
    struct display_timing   mode_crt_reg;
    struct crt_mode_table   *mode_crt_table = NULL;
    
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VideoSetMode: Set IGA%d Mode!\n", set_iga));

    VideoMode = &CLE266Modes[SearchModeSetting(pBIOSInfo->TMDSSettingInfo.ModeIndex)];
    mode_crt_table = VideoMode->crtc;
    mode_crt_reg = mode_crt_table->crtc;

    fill_crtc_timing(pBIOSInfo, mode_crt_table, VideoMode->mode_array, (pBIOSInfo->bitsPerPixel >> 3), set_iga);
    
    if(pBIOSInfo->EPIA_DVI && (pBIOSInfo->DuoView == FALSE))
    {
        fill_crtc_timing(pBIOSInfo, mode_crt_table, VideoMode->mode_array, (pBIOSInfo->bitsPerPixel >> 3), IGA2);

        if(pBIOSInfo->TMDSSettingInfo.ModeIndex == M640x480 && pBIOSInfo->FoundRefresh == 60)
        {
            /* The border is 8 pixels. */
            mode_crt_reg.hor_blank_start = mode_crt_reg.hor_blank_start - 8;

            /* Blanking time should add left and right borders. */
            mode_crt_reg.hor_blank_end = mode_crt_reg.hor_blank_end + 16;
        }
        
        load_crtc_shadow_timing(pBIOSInfo, mode_crt_reg, mode_crt_reg);       /* Fill shadow registers */
        load_fetch_count_reg(mode_crt_reg.hor_addr, 4, IGA2);           /* Fetch count for simultaneous*/        
    }
    
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VideoSetMode --Done--\n"));
}

unsigned char VIAGetDeviceDetectDVI(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   ret = 0x0;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAGetDeviceDetectDVI\n"));

    if(pBIOSInfo->TMDSSettingInfo.ChipID)
    {
        if(VIASenseDVI(pBIOSInfo))
        {
            ret = VIA_DEVICE_DFP;
            pBIOSInfo->DVIAttach = TRUE;
            DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "DVI has Attachment.\n"));
            if(pBIOSInfo->TMDSSettingInfo.DFPSize == M_INVALID)
                VIAGetPanelInfo(pBIOSInfo);
        }
        else
        {
            pBIOSInfo->DVIAttach = FALSE;
            DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "DVI hasn't Attachment.\n"));
        }
    }

    return ret;
}

