/*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
 


#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <linux/version.h>
#include "via_driver.h"
#ifdef RANDRV12_TEST_ENABLE
#include "globals.h"
#include "debug.h"      /* for DBG_DD */
#include "xf86RAC.h"
#include "xf86Priv.h"
#include "xf86Crtc.h"
#include "xf86RandR12.h"
#include "via_bios.h"
#include "via_rotate.h"
#include "via.h"
#include "viamode.h"
#include "X11/Xatom.h"


static Atom lcd_display_atom;
static Atom lcd_display_name_atoms[2];
static char *lcd_display_names[] = {
    "center",
    "expand",    
};
#define LCD_DISPLAY_NUM   2
#define LCD_DISPLAY_NAME  "display_mode"


extern struct _VIASUPPORTMODE VIASupportMode[];

extern Bool VIAModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode);
extern void VIAFindMaxResFromEstablishedTiming(xf86MonPtr MonitorConf, int* MaxH, int* MaxV);
extern void VIAEnableLCD(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo);
extern void VIADisableLCD(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo);
extern void VIALCDSetMode(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo);
extern void VIAEnableDFP(VIABIOSInfoPtr pBIOSInfo);
extern void VIADisableDFP(VIABIOSInfoPtr pBIOSInfo);
extern Bool VIASetDFPMode(VIABIOSInfoPtr pBIOSInfo); 


/* filter those modes which we can't support */
int via_driver_valid_mode(ScrnInfoPtr pScrn, DisplayModePtr mode)
{
    VIAPtr  pVia = VIAPTR(pScrn);   
    CARD32  mode_status;
    float   my_refresh_rate;
    
    /* Check Clock Range */
    if(mode->Clock > 400000)
        mode_status = MODE_CLOCK_HIGH;        

    if(mode->Clock < 25000)
        mode_status = MODE_CLOCK_LOW;

    /* KHz -> Hz */
    my_refresh_rate = (mode->Clock*1000)/(mode->HTotal*mode->VTotal);
    if ((my_refresh_rate > 58) && (my_refresh_rate < 62))
    {
        mode_status = MODE_OK;
    }
    else if ((my_refresh_rate > 73) && (my_refresh_rate < 77))
    {
        mode_status = MODE_OK;
    }
    else if ((my_refresh_rate > 83) && (my_refresh_rate < 87))
    {
        mode_status = MODE_OK;
    }
    else if ((my_refresh_rate > 98) && (my_refresh_rate < 102))
    {
        mode_status = MODE_OK;
    }
    else if ((my_refresh_rate > 118) && (my_refresh_rate < 122))
    {
        mode_status = MODE_OK;
    }
    
    if (mode_status == MODE_OK)
    {                   
        /* Check Resolution Range */
        switch (mode->HDisplay)
        {
            case 480:
                if (mode->VDisplay == 640)
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
                
            case 640:
                if (mode->VDisplay == 480)
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 720:
                if (mode->VDisplay == 480 || mode->VDisplay == 576)
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 800:
                if (mode->VDisplay == 600 || mode->VDisplay == 480)
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 848:
                if (mode->VDisplay == 480)
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 856:
                if (mode->VDisplay == 480)
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 960:
                if (mode->VDisplay == 600)
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 1000:
                if (mode->VDisplay == 600)
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 1088:
                if (mode->VDisplay == 612)
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 1024:
                if (mode->VDisplay == 512 || mode->VDisplay == 576 ||
                    mode->VDisplay == 768  || mode->VDisplay == 600)
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 1152:
                if ((mode->VDisplay == 864) || (mode->VDisplay == 720))
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 1200:
                if (mode->VDisplay == 720)
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 1280:
                if (mode->VDisplay == 720 || mode->VDisplay == 768 || mode->VDisplay == 800 ||
                    mode->VDisplay == 960 || mode->VDisplay == 1024 || mode->VDisplay == 600)
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 1366:
                if (mode->VDisplay == 768)
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 1360:
                if (mode->CrtcVDisplay == 768)
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 1400:
                if (mode->VDisplay == 1050)
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 1440:
                if ((mode->VDisplay == 1050) || (mode->VDisplay == 900))
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 1600:
                if ((mode->VDisplay == 1200) || (mode->VDisplay == 900) || 
                    (mode->VDisplay == 1024))
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 1680:
                if ((mode->VDisplay == 1050) || (mode->VDisplay == 600))
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 1792:
                if (mode->VDisplay == 1344)
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 1856:
                if (mode->VDisplay == 1392)
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 1920:
                if ((pVia->Chipset != VIA_CLE266) &&
                    ((mode->VDisplay == 1440) || 
                     (mode->VDisplay == 1080) ||
                     (mode->VDisplay == 1200)))
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;
            case 2048:
                if (mode->VDisplay == 1536)
                {
                    mode_status = MODE_OK;
                }
                else
                {
                    mode_status = MODE_BAD_VVALUE;
                }
                break;

            default:
                mode_status = MODE_BAD_HVALUE;
                break;
        }                
    } 
    else
    {
        mode_status = MODE_BAD_VSCAN;
    }
    
    return mode_status;
}


/* Only reserved those modes which we have */
void via_correct_refresh(xf86OutputPtr output, DisplayModePtr adjusted_mode)
{
    ScrnInfoPtr         pScrn = output->scrn;
    VIAPtr              pVia = VIAPTR(pScrn);   
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    
    float   my_refresh_rate;    
    
    /* KHz -> Hz */
    my_refresh_rate = (adjusted_mode->Clock*1000)/(adjusted_mode->HTotal*adjusted_mode->VTotal);

    if ((my_refresh_rate > 58) && (my_refresh_rate < 62))
    {
        pBIOSInfo->FoundRefresh = pBIOSInfo->OptRefresh = 60;
    }
    else if ((my_refresh_rate > 73) && (my_refresh_rate < 77))
    {
        pBIOSInfo->FoundRefresh = pBIOSInfo->OptRefresh = 75;
    }
    else if ((my_refresh_rate > 83) && (my_refresh_rate < 87))
    {
        pBIOSInfo->FoundRefresh = pBIOSInfo->OptRefresh = 85;
    }
    else if ((my_refresh_rate > 98) && (my_refresh_rate < 102))
    {
        pBIOSInfo->FoundRefresh = pBIOSInfo->OptRefresh = 100;
    }
    else if ((my_refresh_rate > 118) && (my_refresh_rate < 122))
    {
        pBIOSInfo->FoundRefresh = pBIOSInfo->OptRefresh = 125;
    }
    else
    {
        pBIOSInfo->FoundRefresh = pBIOSInfo->OptRefresh = 60;
    }
}


/* Set IGA1's starting address by position */
void via_set_iga1_starting_address(ScrnInfoPtr pScrn, CARD32 x, CARD32 y)
{
    DEBUG(ErrorF("via_set_iga1_starting_address\n"));
    VIAPtr          pVia = VIAPTR(pScrn);  
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    CARD32 startAddr; 
    
    startAddr = (y * pScrn->displayWidth + x) * (pScrn->bitsPerPixel / 8);
     
    /* Transform to physical address */
    startAddr = (startAddr + pScrn->fbOffset) >> 1;
    
    write_reg(CR0C, VIACR, startAddr >> 8);
    write_reg(CR0D, VIACR, startAddr & 0xFF);    
    write_reg_mask(CR48, VIACR, (startAddr>>24) & 0x1F, 0x1F);
    write_reg(CR34, VIACR, startAddr >> 16);
}

/* Set IGA2's starting address by position */
void via_set_iga2_starting_address(ScrnInfoPtr pScrn, CARD32 x, CARD32 y)
{
    DEBUG(ErrorF("via_set_iga2_starting_address\n"));
    VIAPtr          pVia = VIAPTR(pScrn);  
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    CARD32 startAddr; 
    
    startAddr = (y * pScrn->displayWidth + x) * (pScrn->bitsPerPixel / 8);
     
    /* Quadword */    
    startAddr = (startAddr + pScrn->fbOffset) >> 3;

    /* The register value start at bit 1. */
    startAddr <<= 1;    
    
    write_reg_mask(CR62, VIACR, startAddr, 0xFE);
    write_reg(CR63, VIACR, startAddr>>8);    
    write_reg(CR64, VIACR, startAddr>>16);    
    write_reg_mask(CRA3, VIACR, (startAddr >> 24) & 0x07, 0x07);
}



void via_set_iga1_starting_address_by_addr(ScrnInfoPtr pScrn, int shadow_addr)
{
    DEBUG(ErrorF("via_set_iga1_starting_address\n"));
    VIAPtr          pVia = VIAPTR(pScrn);  
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    int startAddr = shadow_addr; 
    
    /* Transform to physical address */
    startAddr = (startAddr + pScrn->fbOffset) >> 1;
    
    write_reg(CR0C, VIACR, startAddr >> 8);
    write_reg(CR0D, VIACR, startAddr & 0xFF);    
    write_reg_mask(CR48, VIACR, (startAddr>>24) & 0x1F, 0x1F);
    write_reg(CR34, VIACR, startAddr >> 16);
}

void via_set_iga2_starting_address_by_addr(ScrnInfoPtr pScrn, int shadow_addr)
{
    DEBUG(ErrorF("via_set_iga2_starting_address\n"));
    VIAPtr          pVia = VIAPTR(pScrn);  
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    int startAddr = shadow_addr;         
     
    /* Quadword */    
    startAddr = (startAddr + pScrn->fbOffset) >> 3;

    /* The register value start at bit 1. */
    startAddr <<= 1;    
    
    write_reg_mask(CR62, VIACR, startAddr, 0xFE);
    write_reg(CR63, VIACR, startAddr>>8);    
    write_reg(CR64, VIACR, startAddr>>16);    
    write_reg_mask(CRA3, VIACR, (startAddr >> 24) & 0x07, 0x07);
}




/* Create a mode record according to panel id. */
DisplayModePtr via_create_panel_mode_record(ScrnInfoPtr pScrn, int panelID)
{
    DEBUG(ErrorF("via_create_panel_mode_record, panel id = 0x%x\n", panelID));
    
    DisplayModePtr  panel_mode;
    panel_mode = xnfalloc(sizeof(DisplayModeRec));
    memset(panel_mode, 0, sizeof(*panel_mode));
    
    struct VideoModeTable   *panel_mode_set;
    struct crt_mode_table   *panel_crtc_table;
    struct display_timing   panel_crtc_timing;
    
    /* Get LCD timing. */
    panel_mode_set = &LCDModes[SearchLCDModeIndex(VIAGetModeIndexByPanelSize(panelID))];    /* Get panel table */
    panel_crtc_table = panel_mode_set->crtc;
    panel_crtc_timing = panel_crtc_table->crtc;
            
    panel_mode->HDisplay   = panel_crtc_timing.hor_addr;
    panel_mode->VDisplay   = panel_crtc_timing.ver_addr;
    panel_mode->HSyncStart = panel_crtc_timing.hor_sync_start;
    panel_mode->HSyncEnd   = panel_crtc_timing.hor_sync_start + panel_crtc_timing.hor_sync_end;
    panel_mode->HTotal     = panel_crtc_timing.hor_total;
    panel_mode->VSyncStart = panel_crtc_timing.ver_sync_start;
    panel_mode->VSyncEnd   = panel_crtc_timing.ver_sync_start + panel_crtc_timing.ver_sync_end;
    panel_mode->VTotal     = panel_crtc_timing.ver_total;
    panel_mode->Clock      = panel_crtc_table->clk/1000;
    panel_mode->type       = M_T_PREFERRED;

    xf86SetModeDefaultName(panel_mode);
    
    return panel_mode;
}


/* Use CRT timing to create a fake EDID */
DisplayModePtr via_create_a_mode_record(ScrnInfoPtr pScrn, int modeIndex)
{    
    DisplayModePtr  crt_mode;
    crt_mode = xnfalloc(sizeof(DisplayModeRec));
    memset(crt_mode, 0, sizeof(*crt_mode));
    
    struct VideoModeTable   *crt_mode_set;
    struct crt_mode_table   *crt_crtc_table;
    struct display_timing   crt_crtc_timing;
        
    crt_mode_set = &CLE266Modes[SearchModeSetting(modeIndex)];    /* Get panel table */
    crt_crtc_table = crt_mode_set->crtc;
    crt_crtc_timing = crt_crtc_table->crtc;
            
    crt_mode->HDisplay   = crt_crtc_timing.hor_addr;
    crt_mode->VDisplay   = crt_crtc_timing.ver_addr;
    crt_mode->HSyncStart = crt_crtc_timing.hor_sync_start;
    crt_mode->HSyncEnd   = crt_crtc_timing.hor_sync_start + crt_crtc_timing.hor_sync_end;
    crt_mode->HTotal     = crt_crtc_timing.hor_total;
    crt_mode->VSyncStart = crt_crtc_timing.ver_sync_start;
    crt_mode->VSyncEnd   = crt_crtc_timing.ver_sync_start + crt_crtc_timing.ver_sync_end;
    crt_mode->VTotal     = crt_crtc_timing.ver_total;
    crt_mode->Clock      = crt_crtc_table->clk/1000;
    crt_mode->type       = M_T_DRIVER;

    xf86SetModeDefaultName(crt_mode);

    return crt_mode;
}

Bool via_get_device_status(ScrnInfoPtr pScrn, int deviceType)
{
    DEBUG(ErrorF("via_get_device_status\n"));  

    xf86CrtcConfigPtr   via_crtc_config = XF86_CRTC_CONFIG_PTR (pScrn);
    int     i;
    Bool    IsEnable;
    
    for(i = 0; i < via_crtc_config->num_output; i++)
    {
        switch (deviceType)
        {
            case VIA_DEVICE_CRT1:
                if (!strcmp(via_crtc_config->output[i]->name, "CRT"))
                {
                    if (via_crtc_config->output[i]->crtc)                
                    {
                        DEBUG(ErrorF("Output CRT is Enable\n"));  
                        IsEnable = TRUE;
                    }
                    else
                    {                
                        DEBUG(ErrorF("Output CRT is Disable\n"));
                        IsEnable = FALSE;           
                    }
                }                    
                break;

            case VIA_DEVICE_DFP:
                if (!strcmp(via_crtc_config->output[i]->name, "DVI"))
                {
                    if (via_crtc_config->output[i]->crtc)                
                    {
                        DEBUG(ErrorF("Output DVI is Enable\n"));  
                        IsEnable = TRUE;
                    }
                    else
                    {                
                        DEBUG(ErrorF("Output DVI is Disable\n"));
                        IsEnable = FALSE;           
                    }
                }                    
                break;

            case VIA_DEVICE_LCD:
                if (!strcmp(via_crtc_config->output[i]->name, "LCD"))
                {
                    if (via_crtc_config->output[i]->crtc)                
                    {
                        DEBUG(ErrorF("Output LCD is Enable\n"));  
                        IsEnable = TRUE;
                    }
                    else
                    {                
                        DEBUG(ErrorF("Output LCD is Disable\n"));
                        IsEnable = FALSE;           
                    }
                }         
                break;
        }
    }

    return IsEnable;
}


/* If the device is in-active, we should release its iga. */
static void via_release_iga(VIABIOSInfoPtr  pBIOSInfo, int deviceType)
{
    switch (deviceType)
    {
        case VIA_DEVICE_CRT1:
            /* Release IGA resource of CRT   */        
            switch (pBIOSInfo->CRTSettingInfo.IGAPath)
            {
                case IGA1:
                    if (pBIOSInfo->IGA1SettingInfo.IsActive) {
                        pBIOSInfo->CRTSettingInfo.IGAPath = 0;
                        pBIOSInfo->IGA1SettingInfo.IsActive = FALSE;
                    }
                    break;
                        
                case IGA2:
                    if (pBIOSInfo->IGA2SettingInfo.IsActive) {
                        pBIOSInfo->CRTSettingInfo.IGAPath = 0;
                        pBIOSInfo->IGA2SettingInfo.IsActive = FALSE;
                    }
                    break;
            }                    
            break;

        case VIA_DEVICE_DFP:
            /* Release IGA resource of DFP   */        
            switch (pBIOSInfo->TMDSSettingInfo.IGAPath)
            {
                case IGA1:
                    if (pBIOSInfo->IGA1SettingInfo.IsActive) {
                        pBIOSInfo->TMDSSettingInfo.IGAPath = 0;
                        pBIOSInfo->IGA1SettingInfo.IsActive = FALSE;
                    }
                    break;
                        
                case IGA2:
                    if (pBIOSInfo->IGA2SettingInfo.IsActive) {
                        pBIOSInfo->TMDSSettingInfo.IGAPath = 0;
                        pBIOSInfo->IGA2SettingInfo.IsActive = FALSE;
                    }
                    break;
            }                    
            break;

        case VIA_DEVICE_LCD:
            /* Release IGA resource of LCD   */        
            switch (pBIOSInfo->LVDSSettingInfo.IGAPath)
            {
                case IGA1:
                    if (pBIOSInfo->IGA1SettingInfo.IsActive) {
                        pBIOSInfo->LVDSSettingInfo.IGAPath = 0;
                        pBIOSInfo->IGA1SettingInfo.IsActive = FALSE;
                    }
                    break;
                        
                case IGA2:
                    if (pBIOSInfo->IGA2SettingInfo.IsActive) {
                        pBIOSInfo->LVDSSettingInfo.IGAPath = 0;
                        pBIOSInfo->IGA2SettingInfo.IsActive = FALSE;
                    }
                    break;
            }                    
            break;

    }
}



void via_set_output_mode(ScrnInfoPtr pScrn, int deviceType)
{
    VIAPtr              pVia = VIAPTR(pScrn);     
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    xf86CrtcConfigPtr   via_crtc_config = XF86_CRTC_CONFIG_PTR (pScrn);    
    int     i = 0;    
    int     index = 0xFF;
    
    for(i = 0; i < via_crtc_config->num_output; i++)
    {
        switch (deviceType)
        {
            case VIA_DEVICE_CRT1:
                if (!strcmp(via_crtc_config->output[i]->name, "CRT")) {
                    via_release_iga(pBIOSInfo, VIA_DEVICE_CRT1);
                    index = i;                                    
                }
                break;

            case VIA_DEVICE_DFP:
                if (!strcmp(via_crtc_config->output[i]->name, "DVI")) {
                    via_release_iga(pBIOSInfo, VIA_DEVICE_DFP);
                    index = i;                
                }
                break;

            case VIA_DEVICE_LCD:
                if (!strcmp(via_crtc_config->output[i]->name, "LCD")) {
                    via_release_iga(pBIOSInfo, VIA_DEVICE_LCD);
                    index = i;                   
                }
                break;

        }
    }  

    if ((index != 0xFF) && (via_crtc_config->output[index]->crtc))
    {
        xf86CrtcPtr crtc = via_crtc_config->output[index]->crtc;

        xf86CrtcSetMode(crtc, &crtc->desiredMode,
                        crtc->desiredRotation,
                        crtc->desiredX, crtc->desiredY);
    }
}



static int
get_desired_active_devices(ScrnInfoPtr     pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    xf86CrtcConfigPtr   via_crtc_config = XF86_CRTC_CONFIG_PTR (pScrn);
    int desiredActiveDev = 0;
    int i = 0;
    
    for(i = 0; i < via_crtc_config->num_output; i++)
    {
        if (via_crtc_config->output[i]->crtc)
        {
            if (!strcmp(via_crtc_config->output[i]->name, "CRT"))
                desiredActiveDev |= VIA_DEVICE_CRT1;
            
            if (!strcmp(via_crtc_config->output[i]->name, "DVI"))
                desiredActiveDev |= VIA_DEVICE_DFP;               
            
            if (!strcmp(via_crtc_config->output[i]->name, "LCD"))
                desiredActiveDev |= VIA_DEVICE_LCD;
                                          
        }
    }

    return desiredActiveDev;

}



/* If the device is in-active, we should release its iga. */
static void release_unused_iga_resource(ScrnInfoPtr pScrn)
{
    VIAPtr          pVia = VIAPTR(pScrn);  
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;

    switch (pBIOSInfo->IGA1SettingInfo.deviceActived)
    {
        case VIA_DEVICE_CRT1:
            if (!(get_desired_active_devices(pScrn) & VIA_DEVICE_CRT1))
            {                
                via_release_iga(pBIOSInfo, VIA_DEVICE_CRT1);            

                if (pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT1) {
                    pBIOSInfo->ActiveDevice -= VIA_DEVICE_CRT1;
                }
                    
            }
            break;
            
        case VIA_DEVICE_DFP:
            if (!(get_desired_active_devices(pScrn) & VIA_DEVICE_DFP))
            {
                via_release_iga(pBIOSInfo, VIA_DEVICE_DFP);

                if (pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP) {
                    pBIOSInfo->ActiveDevice -= VIA_DEVICE_DFP;
                }
            }
            break;
            
        case VIA_DEVICE_LCD:
            if (!(get_desired_active_devices(pScrn) & VIA_DEVICE_LCD))
            {
                via_release_iga(pBIOSInfo, VIA_DEVICE_LCD);

                if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) {
                    pBIOSInfo->ActiveDevice -= VIA_DEVICE_LCD;
                }
            }
            break;
    }

    switch (pBIOSInfo->IGA2SettingInfo.deviceActived)
    {
        case VIA_DEVICE_CRT1:
            if (!(get_desired_active_devices(pScrn) & VIA_DEVICE_CRT1))
            {                
                via_release_iga(pBIOSInfo, VIA_DEVICE_CRT1);            

                if (pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT1) {
                    pBIOSInfo->ActiveDevice -= VIA_DEVICE_CRT1;
                }
                    
            }
            break;
            
        case VIA_DEVICE_DFP:
            if (!(get_desired_active_devices(pScrn) & VIA_DEVICE_DFP))
            {
                via_release_iga(pBIOSInfo, VIA_DEVICE_DFP);

                if (pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP) {
                    pBIOSInfo->ActiveDevice -= VIA_DEVICE_DFP;
                }
            }
            break;
            
        case VIA_DEVICE_LCD:
            if (!(get_desired_active_devices(pScrn) & VIA_DEVICE_LCD))
            {
                via_release_iga(pBIOSInfo, VIA_DEVICE_LCD);

                if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) {
                    pBIOSInfo->ActiveDevice -= VIA_DEVICE_LCD;
                }
            }
            break;
    }


}




/* ======================================================================= */
/*                  Output Callback Functions                              */
/* ======================================================================= */

/*
* switch the level of DPMS. we should turn on/off the device in this function.
*/
static void via_crt_dpms(xf86OutputPtr output, int mode)
{
    DEBUG(ErrorF("via_crt_dpms, mode = %d\n", mode));
    
    ScrnInfoPtr     pScrn = output->scrn;
    VIAPtr          pVia = VIAPTR(pScrn);  
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    CARD8           cr36;        

    /* Clear DPMS setting */
    VGAOUT8(0x3d4, 0x36);
    cr36 = VGAIN8(0x3d5) & ~0x30;

    switch (mode)
    {
        case DPMSModeOn:        /* 0 */
            VGAOUT(0x3d4, 0x36);
            VGAOUT(0x3d5, cr36);                    
            break;
            
        case DPMSModeStandby:   /* 1 */
        case DPMSModeSuspend:   /* 2 */
        case DPMSModeOff:       /* 3 */       
            VGAOUT(0x3d4, 0x36);
            VGAOUT(0x3d5, cr36 | 0x30);      
            break;
            
        default:
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Invalid DPMS mode %d\n", mode);
            break;
    }

}


/* Saves the crtc's state for restoration on VT switch.*/
static void via_crt_save (xf86OutputPtr output)
{
    DEBUG(ErrorF("via_crt_save\n"));
    
    ScrnInfoPtr     pScrn = output->scrn;
    VIAPtr          pVia = VIAPTR(pScrn);   
}


/* Restore's the crtc's state at VT switch.*/
static void via_crt_restore (xf86OutputPtr output)
{
    DEBUG(ErrorF("via_crt_restore\n"));
    
    ScrnInfoPtr     pScrn = output->scrn;
    VIAPtr          pVia = VIAPTR(pScrn);   
}


/* To check whether the input mode is supported of not in our driver. */
static int via_crt_mode_valid(xf86OutputPtr output, DisplayModePtr mode)
{    
    ScrnInfoPtr     pScrn = output->scrn;       

    return via_driver_valid_mode(pScrn, mode);
}


/* To fix up the input mode, but I filter the modes whose the refresh rate we don't have. */
static Bool via_crt_mode_fixup(xf86OutputPtr output,
                                   DisplayModePtr mode,
                                   DisplayModePtr adjusted_mode)
{
    DEBUG(ErrorF("Call via_crt_mode_fixup\n"));

    ScrnInfoPtr         pScrn = output->scrn;
    VIAPtr              pVia = VIAPTR(pScrn);   
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;

    via_correct_refresh(output, adjusted_mode);
        
    return TRUE;
}


/* Set mode according to device type. */
static void via_crt_mode_set(xf86OutputPtr output,
                                 DisplayModePtr mode,
                                 DisplayModePtr adjusted_mode)
{
    DEBUG(ErrorF("via_crt_mode_set\n"));
    
    ScrnInfoPtr         pScrn = output->scrn;
    VIAPtr              pVia = VIAPTR(pScrn);   
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    xf86CrtcPtr         crtc = output->crtc;            

    /* If the current mode is different with the desired mode, we should change the setting. */
    if ((pBIOSInfo->CRTSettingInfo.HActive != adjusted_mode->HDisplay)
     || (pBIOSInfo->CRTSettingInfo.VActive != adjusted_mode->VDisplay))
    {
        pBIOSInfo->CRTSettingInfo.HActive = adjusted_mode->HDisplay;
        pBIOSInfo->CRTSettingInfo.VActive = adjusted_mode->VDisplay;
        pBIOSInfo->CRTSettingInfo.ModeIndex = VIAGetModeIndex(adjusted_mode->HDisplay, adjusted_mode->VDisplay);
    }
    
    VIAInitSettingInfo_IGA (pBIOSInfo, pBIOSInfo->CRTSettingInfo.IGAPath, FALSE, FALSE, adjusted_mode->HDisplay, adjusted_mode->VDisplay, VIA_DEVICE_CRT1);

    VIACRTSetMode(pBIOSInfo, pBIOSInfo->CRTSettingInfo.ModeIndex, pBIOSInfo->CRTSettingInfo.IGAPath);

    switch (pBIOSInfo->CRTSettingInfo.IGAPath)
    {
        case IGA1:
            VIAPrimaryDisplayOffsetPatch(pBIOSInfo);
            via_set_iga1_starting_address(pScrn, crtc->x, crtc->y);
            break;

        case IGA2:
            VIASecondDisplayOffsetPatch(pBIOSInfo);
            via_set_iga2_starting_address(pScrn, crtc->x, crtc->y);    
            break;            
    }    
}


/* To detect if the device is connect. */
static xf86OutputStatus via_crt_detect(xf86OutputPtr output)
{
    DEBUG(ErrorF("via_crt_detect\n"));
    
    ScrnInfoPtr         pScrn = output->scrn;
    VIAPtr              pVia = VIAPTR(pScrn);     
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    xf86OutputStatus    status;        
    xf86MonPtr          edid_mon;        
    CARD32              i, max_h, max_v;    

    /* This should be refined by use a better solution. */
    edid_mon = xf86OutputGetEDID (output, pVia->I2C_Port1);
    if (edid_mon)    
    {
        status = XF86OutputStatusConnected;   
        DEBUG(ErrorF("CRT Connected\n"));
    }
    else
    {
        status = XF86OutputStatusDisconnected;  
        DEBUG(ErrorF("CRT Dis-connected\n"));
    }
        
    return status;
}

static void via_crt_destroy (xf86OutputPtr output)
{
    DEBUG(ErrorF("via_crt_destroy\n"));
    
    if (output->driver_private)
        xfree (output->driver_private);
}


/* This will be done before set mode. Doing some preparations before set mode. */
void via_crt_prepare (xf86OutputPtr output)
{
    DEBUG(ErrorF("via_crt_prepare\n"));    

    ScrnInfoPtr         pScrn = output->scrn;
    VIAPtr              pVia = VIAPTR(pScrn);   
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;       
    int                 HActive = pBIOSInfo->CrtcHDisplay;
    int                 VActive = pBIOSInfo->CrtcVDisplay;    
    Bool                FoundIGA = FALSE;
    
    release_unused_iga_resource(pScrn);
    
    if ((!pBIOSInfo->IGA1SettingInfo.IsActive) 
     || ((pBIOSInfo->CRTSettingInfo.IGAPath == IGA1) && (pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT1)))
    {        
        pBIOSInfo->CRTSettingInfo.IGAPath = IGA1;
        pBIOSInfo->IGA1SettingInfo.IsActive = TRUE;
        FoundIGA = TRUE;
    }

    if (!FoundIGA)
    {
        if ((!pBIOSInfo->IGA2SettingInfo.IsActive) || (pBIOSInfo->CRTSettingInfo.IGAPath == IGA2))
        {        
            pBIOSInfo->CRTSettingInfo.IGAPath = IGA2;
            pBIOSInfo->IGA2SettingInfo.IsActive = TRUE;
        }
    }

    DEBUG(ErrorF("pBIOSInfo->CRTSettingInfo.IGAPath = %d\n", pBIOSInfo->CRTSettingInfo.IGAPath));

    VIAAdjustActiveSize_CRT(pBIOSInfo, &(pBIOSInfo->CRTSettingInfo), &HActive, &VActive);

    /* pBIOSInfo->frameX1 pBIOSInfo->frameY1 always remain the actual view size whenever rotation or not */
    pBIOSInfo->frameX1 = HActive - 1;
    pBIOSInfo->frameY1 = VActive - 1;
    
    pBIOSInfo->HDisplay = HActive;
    pBIOSInfo->VDisplay = VActive;
    VIAInitSettingInfo_CRT(pBIOSInfo, &(pBIOSInfo->CRTSettingInfo), VIA_DEVICE_CRT1 );

}



/* This will be done after set mode. */
void via_crt_commit (xf86OutputPtr output)
{
    DEBUG(ErrorF("via_crt_commit\n"));    

    ScrnInfoPtr         pScrn = output->scrn;
    VIAPtr              pVia = VIAPTR(pScrn);   
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    xf86CrtcPtr         crtc = output->crtc;            
    VIACrtcPrivatePtr   via_private_crtc = VIACrtcPrivate(crtc);
    
    if (via_private_crtc->RotateDegree > 0)
    {        
        if (pBIOSInfo->CRTSettingInfo.IGAPath == IGA1)
        {                            
            via_set_iga1_starting_address_by_addr(pScrn, via_private_crtc->RotateShadowOffset);
            
        }
        else if (pBIOSInfo->CRTSettingInfo.IGAPath == IGA2)
        {                
            via_set_iga2_starting_address_by_addr(pScrn, via_private_crtc->RotateShadowOffset);
        }                     
    }
        
    output->funcs->dpms (output, DPMSModeOn);

}


/* automatic select a mode to set */
DisplayModePtr via_crt_get_modes (xf86OutputPtr output)
{    
    DEBUG(ErrorF("via_crt_get_modes\n"));

    ScrnInfoPtr         pScrn = output->scrn;
    VIAPtr              pVia = VIAPTR(pScrn);  
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    DisplayModePtr      modes;
    xf86MonPtr          edid_mon;
    int                 i, max_h, max_v;    
        
    edid_mon = xf86OutputGetEDID (output, pVia->I2C_Port1);

    if (edid_mon)
    {                        
        xf86OutputSetEDID(output, edid_mon);    

        /*Find max resolution from Established timing*/
        VIAFindMaxResFromEstablishedTiming(edid_mon, &max_h, &max_v);

        /*Find max resolution from Standard timing*/
        for (i = 0; i < 8; i++)
        {
            if (edid_mon->timings2[i].hsize > max_h)
            {
                max_h = edid_mon->timings2[i].hsize;
                max_v = edid_mon->timings2[i].vsize;
            }                
        }

        /*Find max resolution from Detail timing*/
        for (i = 0; i < 4; i++)
        {
            if ((edid_mon->det_mon[i].type == DT) && 
                (edid_mon->det_mon[i].section.d_timings.h_active > max_h))
            {
                max_h = edid_mon->det_mon[i].section.d_timings.h_active;
                max_v = edid_mon->det_mon[i].section.d_timings.v_active;
            }                
        }

        for (i = 0; i < DET_TIMINGS; i++)
        {
            if (edid_mon->det_mon[i].type == DS_RANGES)
            {
                if (edid_mon->det_mon[i].section.ranges.max_clock != 0)
                {                    
                    pBIOSInfo->CRTSettingInfo.MaxPixelClock = edid_mon->det_mon[i].section.ranges.max_clock;
                }    
            }
        }

        
        /* Correct monitor size to avoid invalid mode. */
        VIACorrectMonitorSize(&max_h, &max_v);            

        /* To prevent to get a wrong EDID data. We should assign a default value.*/
        if ((max_h == 0) || (max_v == 0))
        {
            max_h = 1024;
            max_v = 768;
        }
        
        pBIOSInfo->CRTSettingInfo.MonitorSizeH = max_h;
        pBIOSInfo->CRTSettingInfo.MonitorSizeV = max_v;    

        modes = xf86OutputGetEDIDModes(output);
    }                                
    else
    {
        /* Create a Fake EDID */
        modes = via_create_a_mode_record(pScrn, M1024x768);
        
        edid_mon = xcalloc (1, sizeof (xf86Monitor));
        if (edid_mon)
        {
            /* Set wide sync ranges so we get all modes
             * handed to valid_mode for checking */
            edid_mon->det_mon[0].type = DS_RANGES;
            edid_mon->det_mon[0].section.ranges.min_v = 0;
            edid_mon->det_mon[0].section.ranges.max_v = 200;
            edid_mon->det_mon[0].section.ranges.min_h = 0;
            edid_mon->det_mon[0].section.ranges.max_h = 200;
            
            output->MonInfo = edid_mon;
        }
        
        /* If DDC fail, set a default value as monitor size. */
        pBIOSInfo->CRTSettingInfo.MonitorSizeH = 1024;
        pBIOSInfo->CRTSettingInfo.MonitorSizeV = 768;      
    }            
    
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Max CRT monitor Hor size = %d\n", pBIOSInfo->CRTSettingInfo.MonitorSizeH));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Max CRT monitor Ver size = %d\n", pBIOSInfo->CRTSettingInfo.MonitorSizeV));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "CRT use mode, modes->HDisplay=%d, modes->VDisplay=%d\n", modes->HDisplay, modes->VDisplay));

    return modes;
}



static const xf86OutputFuncsRec via_crt_output_funcs = {
    .dpms = via_crt_dpms,
    .save = NULL,
    .restore = NULL,
    .mode_valid = via_crt_mode_valid,
    .mode_fixup = via_crt_mode_fixup,
    .prepare = via_crt_prepare,
    .mode_set = via_crt_mode_set,
    .commit = via_crt_commit,
    .detect = via_crt_detect,    
    .get_modes = via_crt_get_modes,
    .destroy = via_crt_destroy,        
};



/* Create a device dependent properties */
static void
via_lvds_create_resources(xf86OutputPtr output)
{
    DEBUG(ErrorF("via_lvds_create_resources\n")); 
    
    ScrnInfoPtr pScrn = output->scrn;    
    VIAPtr          pVia = VIAPTR(pScrn);  
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    int             err;
    int             i;
    int             currentType;
    
    lcd_display_atom = MakeAtom(LCD_DISPLAY_NAME, sizeof(LCD_DISPLAY_NAME) - 1, TRUE);    

    for (i = 0; i < LCD_DISPLAY_NUM; i++)
    {
        lcd_display_name_atoms[i] = MakeAtom(lcd_display_names[i],
                                             strlen(lcd_display_names[i]),
                                             TRUE);
    }

    err = RRConfigureOutputProperty(output->randr_output, lcd_display_atom,
                     TRUE, FALSE, FALSE, 
                     LCD_DISPLAY_NUM, (INT32 *) lcd_display_name_atoms);

    if (err != 0) {
        xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                   "RRConfigureOutputProperty error, %d\n", err);
    }

    if (pBIOSInfo->LVDSSettingInfo.Center)
    {
        currentType = 0;
    }
    else
    {
        currentType = 1;
    }


    err = RRChangeOutputProperty(output->randr_output, lcd_display_atom,
                 XA_ATOM, 32, PropModeReplace, 1,
                 &lcd_display_name_atoms[1],
                 FALSE, TRUE);

    if (err != 0) {
    xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
           "failed to set panel fitting mode, %d\n", err);
    }   
}


static Bool
via_lvds_set_property(xf86OutputPtr output, Atom property,
                     RRPropertyValuePtr value)
{
    DEBUG(ErrorF("via_lvds_set_property\n"));  
        
    ScrnInfoPtr     pScrn = output->scrn;
    VIAPtr          pVia = VIAPTR(pScrn);  
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;


    if (property == lcd_display_atom)        
    {
        Atom    atom;
        char    *name;        

        if (value->type != XA_ATOM || value->format != 32 || value->size != 1)
            return FALSE;

        memcpy(&atom, value->data, 4);
        name = NameForAtom(atom);

        if (!strcmp(name, lcd_display_names[0]))
        {
            ErrorF("Apply center\n");
            pBIOSInfo->LVDSSettingInfo.Center = TRUE;
        }

        if (!strcmp(name, lcd_display_names[1]))
        {
            ErrorF("Apply expand\n");
            pBIOSInfo->LVDSSettingInfo.Center = FALSE;
        }        

        if (output->crtc)
        {
            xf86CrtcPtr crtc = output->crtc;
            if (crtc->enabled)
            {
                if (!xf86CrtcSetMode(crtc, &crtc->desiredMode,
                             crtc->desiredRotation,
                             crtc->desiredX, crtc->desiredY)) {
                    xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                           "Failed to set mode after panel fitting change!\n");
                    return FALSE;
                }
            }
        }
        return TRUE;
    }
}



static void via_lvds_dpms(xf86OutputPtr output, int mode)
{
    DEBUG(ErrorF("via_lvds_dpms, mode = %d\n", mode));
    
    ScrnInfoPtr     pScrn = output->scrn;
    VIAPtr          pVia = VIAPTR(pScrn);  
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    CARD8           cr36;
    
    switch (mode)
    {
        case DPMSModeOn:        /* 0 */
            VIAEnableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
            break;
            
        case DPMSModeStandby:   /* 1 */
        case DPMSModeSuspend:   /* 2 */
        case DPMSModeOff:       /* 3 */
            VIADisableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
            
            /* Release IGA resource of LCD   */
            if (!via_get_device_status(pScrn, VIA_DEVICE_LCD))
            {
                if ((pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP)
                 && (pBIOSInfo->TMDSSettingInfo.ChipID == VIA_INTEGRATED_TMDS))
                {
                    VGAOUT8(0x3D4, CRD2);
                    VGAOUT8(0x3D5, VGAIN8(0x3D5) | 0x30);
                }
            }     
            break;
            
        default:
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Invalid DPMS mode %d\n", mode);
            break;
    }

}


/* Saves the crtc's state for restoration on VT switch.*/
static void via_lvds_save (xf86OutputPtr output)
{
    DEBUG(ErrorF("via_lvds_save\n"));
    
    ScrnInfoPtr     pScrn = output->scrn;
    VIAPtr          pVia = VIAPTR(pScrn);   
}


/* Restore's the crtc's state at VT switch.*/
static void via_lvds_restore (xf86OutputPtr output)
{
    DEBUG(ErrorF("via_lvds_restore\n"));
    
    ScrnInfoPtr     pScrn = output->scrn;
    VIAPtr          pVia = VIAPTR(pScrn);   
}


/*
 * Callback for testing a video mode for a given output.
 *
 * This function should only check for cases where a mode can't be supported
 * on the pipe specifically, and not represent generic CRTC limitations.
 *
 * return MODE_OK if the mode is valid, or another MODE_* otherwise.
 */
static int via_lvds_mode_valid(xf86OutputPtr output, DisplayModePtr mode)
{    
    ScrnInfoPtr     pScrn = output->scrn;
    VIAPtr          pVia = VIAPTR(pScrn);  
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    float my_refresh_rate;

    /* Currently, we only support 60hz on LCD. */
    my_refresh_rate = (mode->Clock*1000)/(mode->HTotal*mode->VTotal);

    if ((my_refresh_rate > 61) || (my_refresh_rate < 59))
    {
        return MODE_BAD_VSCAN;
    }

    /* One panel size only can support one mode. */
    if ((pBIOSInfo->LVDSSettingInfo.PanelSizeX >= mode->HDisplay)        
      &&(pBIOSInfo->LVDSSettingInfo.PanelSizeY >= mode->VDisplay))
    {
        return MODE_OK;
    }        
    else
    {
        return MODE_NOMODE;
    }
}

static Bool via_lvds_mode_fixup(xf86OutputPtr output,
                                   DisplayModePtr mode,
                                   DisplayModePtr adjusted_mode)
{
    DEBUG(ErrorF("via_lvds_mode_fixup\n"));

    via_correct_refresh(output, adjusted_mode);
    
    return TRUE;
}

static void via_lvds_mode_set(xf86OutputPtr output,
                                 DisplayModePtr mode,
                                 DisplayModePtr adjusted_mode)
{
    DEBUG(ErrorF("via_lvds_mode_set\n"));
    
    ScrnInfoPtr         pScrn = output->scrn;
    VIAPtr              pVia = VIAPTR(pScrn);   
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    xf86CrtcPtr         crtc = output->crtc;                  

    if ((pBIOSInfo->LVDSSettingInfo.HActive != adjusted_mode->HDisplay)
     || (pBIOSInfo->LVDSSettingInfo.VActive != adjusted_mode->VDisplay))
    {
        pBIOSInfo->LVDSSettingInfo.HActive = adjusted_mode->HDisplay;
        pBIOSInfo->LVDSSettingInfo.VActive = adjusted_mode->VDisplay;
        pBIOSInfo->LVDSSettingInfo.ModeIndex = VIAGetModeIndex(adjusted_mode->HDisplay, adjusted_mode->VDisplay);
    }
    
    VIAInitSettingInfo_IGA (pBIOSInfo, pBIOSInfo->LVDSSettingInfo.IGAPath, FALSE, FALSE, adjusted_mode->HDisplay, adjusted_mode->VDisplay, VIA_DEVICE_LCD);

    VIALCDSetMode(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
    
    switch (pBIOSInfo->LVDSSettingInfo.IGAPath)
    {
        case IGA1:
            VIAPrimaryDisplayOffsetPatch(pBIOSInfo);
            via_set_iga1_starting_address(pScrn, crtc->x, crtc->y);
            break;

        case IGA2:
            VIASecondDisplayOffsetPatch(pBIOSInfo);
            via_set_iga2_starting_address(pScrn, crtc->x, crtc->y);    
            break;            
    }
    
}



static xf86OutputStatus via_lvds_detect(xf86OutputPtr output)
{
    DEBUG(ErrorF("via_lvds_detect\n"));
    
    ScrnInfoPtr         pScrn = output->scrn;
    VIAPtr              pVia = VIAPTR(pScrn);     
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    xf86OutputStatus    status;        
    xf86MonPtr          edid_mon;        
    CARD32              i, max_h, max_v;
    
    status = XF86OutputStatusConnected;        
    
    return status;
}

static void via_lvds_destroy (xf86OutputPtr output)
{
    DEBUG(ErrorF("via_crt_destroy\n"));
    
    if (output->driver_private)
        xfree (output->driver_private);
}


void via_lvds_prepare (xf86OutputPtr output)
{
    DEBUG(ErrorF("via_lvds_prepare\n"));

    ScrnInfoPtr         pScrn = output->scrn;
    VIAPtr              pVia = VIAPTR(pScrn);  
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    int                 HActive = pBIOSInfo->CrtcHDisplay;
    int                 VActive = pBIOSInfo->CrtcVDisplay;    

    release_unused_iga_resource(pScrn);
    
    /* Dispatch the IGA Path for LCD */
    switch(pBIOSInfo->IGA2SettingInfo.deviceActived)
    {                    
        case VIA_DEVICE_CRT1:
            ErrorF("CRT use also use IGA2\n");               
            via_set_output_mode(pScrn, VIA_DEVICE_CRT1);                
            break;
            
        case VIA_DEVICE_DFP:
            ErrorF("DVI use also use IGA2\n");         
            via_set_output_mode(pScrn, VIA_DEVICE_DFP);                
            break;        
    }                    

    pBIOSInfo->LVDSSettingInfo.IGAPath = IGA2;
    pBIOSInfo->IGA2SettingInfo.IsActive = TRUE;
    
    DEBUG(ErrorF("pBIOSInfo->LVDSSettingInfo.IGAPath = %d\n", pBIOSInfo->LVDSSettingInfo.IGAPath));

    VIAAdjustActiveSize_LCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo), &HActive, &VActive);    

    /* pBIOSInfo->frameX1 pBIOSInfo->frameY1 always remain the actual view size whenever rotation or not */
    pBIOSInfo->frameX1 = HActive - 1;
    pBIOSInfo->frameY1 = VActive - 1;
    
    pBIOSInfo->HDisplay = HActive;
    pBIOSInfo->VDisplay = VActive;
    VIAInitSettingInfo_LCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo), VIA_DEVICE_LCD);

}

void via_lvds_commit (xf86OutputPtr output)
{
    DEBUG(ErrorF("via_lvds_commit\n"));

    ScrnInfoPtr         pScrn = output->scrn;
    VIAPtr              pVia = VIAPTR(pScrn);   
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    xf86CrtcPtr         crtc = output->crtc;    
    VIACrtcPrivatePtr   via_private_crtc = VIACrtcPrivate(crtc);
    
    if (via_private_crtc->RotateDegree > 0)
    { 
        if (pBIOSInfo->LVDSSettingInfo.IGAPath == IGA1)
        {                
            via_set_iga1_starting_address_by_addr(pScrn, via_private_crtc->RotateShadowOffset);
        }
        else if (pBIOSInfo->LVDSSettingInfo.IGAPath == IGA2)
        {                
            via_set_iga2_starting_address_by_addr(pScrn, via_private_crtc->RotateShadowOffset);
        }                     
    }        
    
    output->funcs->dpms (output, DPMSModeOn);

    if ((pBIOSInfo->ActiveDevice == (VIA_DEVICE_DFP | VIA_DEVICE_LCD))
     && (pBIOSInfo->TMDSSettingInfo.ChipID == VIA_INTEGRATED_TMDS))
    {
        VGAOUT8(0x3D4, CRD2);
        VGAOUT8(0x3D5, VGAIN8(0x3D5) | 0x10);
    }    
}

DisplayModePtr via_lvds_get_modes (xf86OutputPtr output)
{    
    DEBUG(ErrorF("Call via_lvds_get_modes\n"));

    ScrnInfoPtr         pScrn = output->scrn;
    VIAPtr              pVia = VIAPTR(pScrn); 
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    DisplayModePtr      modes;
    xf86MonPtr          edid_mon;

    modes = via_create_panel_mode_record(pScrn, pBIOSInfo->LVDSSettingInfo.PanelSizeID);
    
    edid_mon = xcalloc (1, sizeof (xf86Monitor));
    if (edid_mon)
    {
        /* Set wide sync ranges so we get all modes
         * handed to valid_mode for checking */
        edid_mon->det_mon[0].type = DS_RANGES;
        edid_mon->det_mon[0].section.ranges.min_v = 0;
        edid_mon->det_mon[0].section.ranges.max_v = 200;
        edid_mon->det_mon[0].section.ranges.min_h = 0;
        edid_mon->det_mon[0].section.ranges.max_h = 200;
        
        output->MonInfo = edid_mon;
    }

    return modes;
}



static const xf86OutputFuncsRec via_lvds_output_funcs = {   
    .create_resources = via_lvds_create_resources,
    .set_property = via_lvds_set_property,   
    .dpms = via_lvds_dpms,
    .save = NULL,
    .restore = NULL,
    .mode_valid = via_lvds_mode_valid,
    .mode_fixup = via_lvds_mode_fixup,
    .prepare = via_lvds_prepare,
    .mode_set = via_lvds_mode_set,
    .commit = via_lvds_commit,
    .detect = via_lvds_detect,    
    .get_modes = via_lvds_get_modes,
    .destroy = via_lvds_destroy,        
};



static void via_tmds_dpms(xf86OutputPtr output, int mode)
{
    DEBUG(ErrorF("via_tmds_dpms, mode = %d\n", mode));
    
    ScrnInfoPtr     pScrn = output->scrn;
    VIAPtr          pVia = VIAPTR(pScrn);  
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    CARD8           cr36;                    
    
    switch (mode)
    {
        case DPMSModeOn:        /* 0 */               
            VIAEnableDFP(pBIOSInfo);     
            break;
            
        case DPMSModeStandby:   /* 1 */
        case DPMSModeSuspend:   /* 2 */
        case DPMSModeOff:       /* 3 */                           
            VIADisableDFP(pBIOSInfo); 
            break;
            
        default:
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Invalid DPMS mode %d\n", mode);
            break;
    }

}


/* Saves the crtc's state for restoration on VT switch.*/
static void via_tmds_save (xf86OutputPtr output)
{
    DEBUG(ErrorF("via_tmds_save\n"));
    
    ScrnInfoPtr     pScrn = output->scrn;
    VIAPtr          pVia = VIAPTR(pScrn);   
}


/* Restore's the crtc's state at VT switch.*/
static void via_tmds_restore (xf86OutputPtr output)
{
    DEBUG(ErrorF("via_tmds_restore\n"));
    
    ScrnInfoPtr     pScrn = output->scrn;
    VIAPtr          pVia = VIAPTR(pScrn);   
}


/*
 * Callback for testing a video mode for a given output.
 *
 * This function should only check for cases where a mode can't be supported
 * on the pipe specifically, and not represent generic CRTC limitations.
 *
 * return MODE_OK if the mode is valid, or another MODE_* otherwise.
 */
static int via_tmds_mode_valid(xf86OutputPtr output, DisplayModePtr mode)
{    
    ScrnInfoPtr     pScrn = output->scrn;
    
    return via_driver_valid_mode(pScrn, mode);;
    
}

static Bool via_tmds_mode_fixup(xf86OutputPtr output,
                                   DisplayModePtr mode,
                                   DisplayModePtr adjusted_mode)
{
    DEBUG(ErrorF("via_tmds_mode_fixup\n"));

    via_correct_refresh(output, adjusted_mode);
    
    return TRUE;
}

static void via_tmds_mode_set(xf86OutputPtr output,
                                 DisplayModePtr mode,
                                 DisplayModePtr adjusted_mode)
{
    DEBUG(ErrorF("via_tmds_mode_set\n"));
    
    ScrnInfoPtr         pScrn = output->scrn;
    VIAPtr              pVia = VIAPTR(pScrn);   
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    xf86CrtcPtr         crtc = output->crtc;       

    if ((pBIOSInfo->TMDSSettingInfo.HActive != adjusted_mode->HDisplay)
     || (pBIOSInfo->TMDSSettingInfo.VActive != adjusted_mode->VDisplay))
    {
        pBIOSInfo->TMDSSettingInfo.HActive = adjusted_mode->HDisplay;
        pBIOSInfo->TMDSSettingInfo.VActive = adjusted_mode->VDisplay;
        pBIOSInfo->TMDSSettingInfo.ModeIndex = VIAGetModeIndex(adjusted_mode->HDisplay, adjusted_mode->VDisplay);
    }
    
    VIAInitSettingInfo_IGA (pBIOSInfo, pBIOSInfo->TMDSSettingInfo.IGAPath, FALSE, FALSE, adjusted_mode->HDisplay, adjusted_mode->VDisplay, VIA_DEVICE_DFP);

    ErrorF("Active Device 0x%x\n", pBIOSInfo->ActiveDevice);
    VIASetDFPMode(pBIOSInfo);

    switch (pBIOSInfo->TMDSSettingInfo.IGAPath)
    {
        case IGA1:
            VIAPrimaryDisplayOffsetPatch(pBIOSInfo);
            via_set_iga1_starting_address(pScrn, crtc->x, crtc->y);
            break;

        case IGA2:
            VIASecondDisplayOffsetPatch(pBIOSInfo);
            via_set_iga2_starting_address(pScrn, crtc->x, crtc->y);    
            break;            
    }
}


static xf86OutputStatus via_tmds_detect(xf86OutputPtr output)
{
    DEBUG(ErrorF("Call via_tmds_detect\n"));
    
    ScrnInfoPtr         pScrn = output->scrn;
    VIAPtr              pVia = VIAPTR(pScrn);     
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    xf86OutputStatus    status;        
    xf86MonPtr          edid_mon;        
    CARD32              i, max_h, max_v;

    unsigned char       W_Buffer[2];
    unsigned char       *R_Buffer;    
    
    R_Buffer = xcalloc(1,sizeof(unsigned char) * (128));
        
    pBIOSInfo->TMDSSettingInfo.I2CAddress = 0xA0;
    
    /* Read EDID v1.1 Block */
    for (i=0; i<128; i++)
    {
        GPIOI2CRead_TMDS(pBIOSInfo, &(pBIOSInfo->TMDSSettingInfo), i, R_Buffer+i);
    }

    /* Parse the EDID block */
    edid_mon = xf86InterpretEDID(pScrn->scrnIndex,R_Buffer);
    
    if (edid_mon)
    {                        
        status = XF86OutputStatusConnected;        
    }                                
    else
    {
        status = XF86OutputStatusDisconnected;        
    }
        
    return status;
}

static void via_tmds_destroy (xf86OutputPtr output)
{
    DEBUG(ErrorF("via_tmds_destroy\n"));
    
    if (output->driver_private)
        xfree (output->driver_private);
}


void via_tmds_prepare (xf86OutputPtr output)
{
    DEBUG(ErrorF("via_tmds_prepare\n"));
    ScrnInfoPtr         pScrn = output->scrn;
    VIAPtr              pVia = VIAPTR(pScrn);  
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    int                 HActive = pBIOSInfo->CrtcHDisplay;
    int                 VActive = pBIOSInfo->CrtcVDisplay;    
    Bool                FoundIGA = FALSE;

    release_unused_iga_resource(pScrn);
    
    if ((!pBIOSInfo->IGA1SettingInfo.IsActive)
     || ((pBIOSInfo->TMDSSettingInfo.IGAPath == IGA1) && (pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP)))
    {        
        pBIOSInfo->TMDSSettingInfo.IGAPath = IGA1;
        pBIOSInfo->IGA1SettingInfo.IsActive = TRUE;
        FoundIGA = TRUE;
    }

    if (!FoundIGA)
    {
        if ((!pBIOSInfo->IGA2SettingInfo.IsActive) || (pBIOSInfo->TMDSSettingInfo.IGAPath == IGA2) )
        {        
            pBIOSInfo->TMDSSettingInfo.IGAPath = IGA2;
            pBIOSInfo->IGA2SettingInfo.IsActive = TRUE;
        }
    }

    DEBUG(ErrorF("pBIOSInfo->TMDSSettingInfo.IGAPath = %d\n", pBIOSInfo->TMDSSettingInfo.IGAPath));

    VIAAdjustActiveSize_DVI(pBIOSInfo, &(pBIOSInfo->TMDSSettingInfo), &HActive, &VActive);    
    /* pBIOSInfo->frameX1 pBIOSInfo->frameY1 always remain the actual view size whenever rotation or not */
    pBIOSInfo->frameX1 = HActive - 1;
    pBIOSInfo->frameY1 = VActive - 1;
    
    pBIOSInfo->HDisplay = HActive;
    pBIOSInfo->VDisplay = VActive;
    VIAInitSettingInfo_DVI(pBIOSInfo);
}

void via_tmds_commit (xf86OutputPtr output)
{
    DEBUG(ErrorF("via_tmds_commit\n"));

    ScrnInfoPtr         pScrn = output->scrn;
    VIAPtr              pVia = VIAPTR(pScrn);   
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    xf86CrtcPtr         crtc = output->crtc;          
    VIACrtcPrivatePtr   via_private_crtc = VIACrtcPrivate(crtc);
    
    if (via_private_crtc->RotateDegree > 0)
    { 
        if (pBIOSInfo->TMDSSettingInfo.IGAPath == IGA1)
        {                
            via_set_iga1_starting_address_by_addr(pScrn, via_private_crtc->RotateShadowOffset);
        }
        else if (pBIOSInfo->TMDSSettingInfo.IGAPath == IGA2)
        {                
            via_set_iga2_starting_address_by_addr(pScrn, via_private_crtc->RotateShadowOffset);
        }                     
    }        
    
    output->funcs->dpms (output, DPMSModeOn);

    if ((pBIOSInfo->ActiveDevice == (VIA_DEVICE_DFP | VIA_DEVICE_LCD))
     && (pBIOSInfo->TMDSSettingInfo.ChipID == VIA_INTEGRATED_TMDS))
    {
        VGAOUT8(0x3D4, CRD2);
        VGAOUT8(0x3D5, VGAIN8(0x3D5) | 0x10);
    }
    else if ((pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP)
          && (pBIOSInfo->TMDSSettingInfo.ChipID == VIA_INTEGRATED_TMDS))
    {
        VGAOUT8(0x3D4, CRD2);
        VGAOUT8(0x3D5, VGAIN8(0x3D5) | 0x30);
    }
}

DisplayModePtr via_tmds_get_modes (xf86OutputPtr output)
{    
    DEBUG(ErrorF("via_tmds_get_modes\n"));

    ScrnInfoPtr         pScrn = output->scrn;
    VIAPtr              pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    DisplayModePtr      modes;
    xf86MonPtr          edid_mon;  
    xf86MonPtr          fake_edid_mon;  
    int                 i, max_h, max_v;

    unsigned char       W_Buffer[2];
    unsigned char       *R_Buffer;    
    
    R_Buffer = xcalloc(1,sizeof(unsigned char) * (128));
    
    pBIOSInfo->TMDSSettingInfo.I2CAddress = 0xA0;
    
    /* Read EDID v1.1 Block */
    for (i=0; i<128; i++)
    {
        GPIOI2CRead_TMDS(pBIOSInfo, &(pBIOSInfo->TMDSSettingInfo), i, R_Buffer+i);
    }    

    /* Parse the EDID block */
    edid_mon = xf86InterpretEDID(pScrn->scrnIndex,R_Buffer);
    
    if (edid_mon)
    {                
        xf86OutputSetEDID(output, edid_mon);  

        /*Find max resolution from Established timing*/
        VIAFindMaxResFromEstablishedTiming(edid_mon, &max_h, &max_v);

        /*Find max resolution from Standard timing*/
        for (i = 0; i < 8; i++)
        {
            if (edid_mon->timings2[i].hsize > max_h)
            {
                max_h = edid_mon->timings2[i].hsize;
                max_v = edid_mon->timings2[i].vsize;
            }                
        }

        /*Find max resolution from Detail timing*/
        for (i = 0; i < 4; i++)
        {
            if ((edid_mon->det_mon[i].type == DT) && 
                (edid_mon->det_mon[i].section.d_timings.h_active > max_h))
            {
                max_h = edid_mon->det_mon[i].section.d_timings.h_active;
                max_v = edid_mon->det_mon[i].section.d_timings.v_active;
            }                
        }
        
        for (i = 0; i < DET_TIMINGS; i++)
        {
            if (edid_mon->det_mon[i].type == DS_RANGES)
            {
                if (edid_mon->det_mon[i].section.ranges.max_clock != 0)
                {                    
                    pBIOSInfo->TMDSSettingInfo.MaxPixelClock = edid_mon->det_mon[i].section.ranges.max_clock;
                }    
            }
        }
        
        /* Correct monitor size to avoid invalid mode. */
        VIACorrectMonitorSize(&max_h, &max_v);            

        /* To prevent to get a wrong EDID data. We should assign a default value.*/
        if ((max_h == 0) || (max_v == 0))
        {
            max_h = 1024;
            max_v = 768;
        }

        pBIOSInfo->TMDSSettingInfo.DFPSize = VIAGetModeIndex(max_h, max_v);

        modes = xf86OutputGetEDIDModes(output);
    }                                
    else
    {
        /* Create a Fake EDID */
        modes = via_create_a_mode_record(pScrn, M1024x768);
        
        fake_edid_mon = xcalloc (1, sizeof (xf86Monitor));
        if (fake_edid_mon)
        {
            /* Set wide sync ranges so we get all modes
             * handed to valid_mode for checking
             */
            fake_edid_mon->det_mon[0].type = DS_RANGES;
            fake_edid_mon->det_mon[0].section.ranges.min_v = 0;
            fake_edid_mon->det_mon[0].section.ranges.max_v = 200;
            fake_edid_mon->det_mon[0].section.ranges.min_h = 0;
            fake_edid_mon->det_mon[0].section.ranges.max_h = 200;
            
            output->MonInfo = fake_edid_mon;
        }
        /* If DDC fail, set a default value as monitor size. */
        pBIOSInfo->TMDSSettingInfo.DFPSize = M1024x768;
        ErrorF("Fail to read TMDS EDID\n");
    }                   
    
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "DVI Panel Size ID = %x\n", pBIOSInfo->TMDSSettingInfo.DFPSize));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "TMDS use mode, modes->HDisplay=%d, modes->VDisplay=%d\n", modes->HDisplay, modes->VDisplay));
    
    return modes;
}



static const xf86OutputFuncsRec via_tmds_output_funcs = {
    .dpms = via_tmds_dpms,
    .save = NULL,
    .restore = NULL,
    .mode_valid = via_tmds_mode_valid,
    .mode_fixup = via_tmds_mode_fixup,
    .prepare = via_tmds_prepare,
    .mode_set = via_tmds_mode_set,
    .commit = via_tmds_commit,
    .detect = via_tmds_detect,    
    .get_modes = via_tmds_get_modes,
    .destroy = via_tmds_destroy,        
};


void via_output_init(ScrnInfoPtr pScrn)
{
    DEBUG(ErrorF("via_output_init\n"));
    VIAPtr              pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    xf86OutputPtr       output_crt;    
    xf86OutputPtr       output_lvds;  
    xf86OutputPtr       output_tmds;   
    xf86OutputPtr       output_tv;   
    int i;

    xf86CrtcConfigPtr xf86_config = XF86_CRTC_CONFIG_PTR(pScrn);
    
    output_crt = xf86OutputCreate (pScrn, &via_crt_output_funcs, "CRT");       
        
    if (!output_crt)
    {
        ErrorF("xf86OutputCreate CRT Fail.\n");
        return;        
    }
    else
    {
        output_crt->possible_crtcs = 0x1 | 0x2;
        output_crt->possible_clones = 0;
        output_crt->interlaceAllowed = TRUE;
        output_crt->doubleScanAllowed = FALSE;
    }       
    
#if 1
    if (VIALVDSIdentify(pBIOSInfo))
    {
        output_lvds = xf86OutputCreate (pScrn, &via_lvds_output_funcs, "LCD");  
        if (!output_lvds)
        {
            ErrorF("xf86OutputCreate LCD Fail.\n");
            return;        
        }  
        else
        {
            output_lvds->possible_crtcs = 0x1 | 0x2;
            output_lvds->possible_clones = 0;
            output_lvds->interlaceAllowed = FALSE;
            output_lvds->doubleScanAllowed = FALSE;
        }
    }

    /* Detect TMDS Transmiter */
    if (VIATMDSIdentify(pBIOSInfo))
    {        
        if (pBIOSInfo->NoDDCValue)
        {            
            pBIOSInfo->TMDSSettingInfo.DFPSize = VIAGetModeIndex(pScrn->virtualX, pScrn->virtualY);
            VIAGetModeSizeByIndex(pBIOSInfo->TMDSSettingInfo.DFPSize, &pBIOSInfo->TMDSSettingInfo.PanelSizeH, &pBIOSInfo->TMDSSettingInfo.PanelSizeV);                        
        }        

        /* Create a output while the TMDS transmitter is exist. */
        output_tmds = xf86OutputCreate (pScrn, &via_tmds_output_funcs, "DVI");    
        if (!output_tmds)
        {
            ErrorF("xf86OutputCreate DVI Fail.\n");
            return;        
        }  
        else
        {
            output_tmds->possible_crtcs = 0x1 | 0x2;
            output_tmds->possible_clones = 0;
            output_tmds->interlaceAllowed = FALSE;
            output_tmds->doubleScanAllowed = FALSE;        
        }
    }

#endif    

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "amount of display output = %d\n", xf86_config->num_output));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "amount of display crtc = %d\n", xf86_config->num_crtc));
}


#endif

