 /*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
 

#ifndef _VIA_VIDEO_H_
#define _VIA_VIDEO_H_ 1

#include "xf86.h"
#include "capture.h"

/*
 * I N C L U D E S
 */
#define HW_3123

#define TRUE        1
#define FALSE       0


#include "videodev.h"

#define   XV_PORT_NUM       1
#define   XV_SWOV_PORTID    0
#define   XV_SWOV_PORTID2   1

#define   COMMAND_FOR_SWOV      XV_SWOV_PORTID
#define   COMMAND_FOR_SWOV2     XV_SWOV_PORTID2

/*
 *  Xv port specific struct, each port has it's own struct
 */
typedef struct {
    unsigned char  xv_portnum;
    long  brightness;
    long  saturation;
    long  contrast;
    long  hue;
    unsigned long  dwEncoding;
    RegionRec      clip;
    CARD32         colorKey;
    Time           offTime;
    Time           freeTime;
    VIAAUDCTRL     AudCtrl;

    /* Surface structure */
    DDSURFACEDESC SurfaceDesc;
    DDLOCK ddLock;

    /* file handle */
    int 			fd;
    struct video_capability     cap;

    /* attributes */
    struct video_picture    	pict;
    struct video_audio          audio;

    int                         *input;
    int                         *norm;
    int                         nenc,cenc;

    /* yuv to offscreen */
    struct video_window		yuv_win;

    /* store old video source & dst data */
    short old_src_w;
    short old_src_h;

    short old_drw_x;
    short old_drw_y;
    short old_drw_w;
    short old_drw_h;
    unsigned long old_dwUseExtendedFIFO;

    Bool  autopaint_colorkey;
} viaPortPrivRec, *viaPortPrivPtr;

typedef struct {
    unsigned long dwWidth;          /* On screen Width                  */
    unsigned long dwHeight;         /* On screen Height                 */
    unsigned long dwBPP;            /* Bits Per Pixel                   */
    unsigned long dwRefreshRate;    /* Refresh rate of the mode         */
}MODEINFO, * LPMODEINFO;

typedef struct{
    CARD32 interruptflag;         /* 200 */
    CARD32 ramtab;                /* 204 */
    CARD32 alphawin_hvstart;      /* 208 */
    CARD32 alphawin_size;         /* 20c */
    CARD32 alphawin_ctl;          /* 210 */
    CARD32 crt_startaddr;         /* 214 */
    CARD32 crt_startaddr_2;       /* 218 */
    CARD32 alphafb_stride ;       /* 21c */
    CARD32 color_key;             /* 220 */
    CARD32 alphafb_addr;          /* 224 */
    CARD32 chroma_low;            /* 228 */
    CARD32 chroma_up;             /* 22c */
    CARD32 video1_ctl;            /* 230 */
    CARD32 video1_fetch;          /* 234 */
    CARD32 video1y_addr1;         /* 238 */
    CARD32 video1_stride;         /* 23c */
    CARD32 video1_hvstart;        /* 240 */
    CARD32 video1_size;           /* 244 */
    CARD32 video1y_addr2;         /* 248 */
    CARD32 video1_zoom;           /* 24c */
    CARD32 video1_mictl;          /* 250 */
    CARD32 video1y_addr0;         /* 254 */
    CARD32 video1_fifo;           /* 258 */
    CARD32 video1y_addr3;         /* 25c */
    CARD32 hi_control;            /* 260 */
    CARD32 snd_color_key;         /* 264 */
    CARD32 v3alpha_prefifo;       /* 268 */
    CARD32 v1_source_w_h;         /* 26c */
    CARD32 hi_transparent_color;  /* 270 */
    CARD32 v_display_temp;        /* 274 :No use */
    CARD32 v3alpha_fifo;          /* 278 */
    CARD32 v3_source_width;       /* 27c */
    CARD32 dummy1;                /* 280 */
    CARD32 video1_CSC1;           /* 284 */
    CARD32 video1_CSC2;           /* 288 */
    CARD32 video1u_addr0;         /* 28c */
    CARD32 video1_opqctl;         /* 290 */
    CARD32 video3_opqctl;         /* 294 */
    CARD32 compose;               /* 298 */
    CARD32 dummy2;                /* 29c */
    CARD32 video3_ctl;            /* 2a0 */
    CARD32 video3_addr0;          /* 2a4 */
    CARD32 video3_addr1;          /* 2a8 */
    CARD32 video3_stribe;         /* 2ac */
    CARD32 video3_hvstart;        /* 2b0 */
    CARD32 video3_size;           /* 2b4 */
    CARD32 v3alpha_fetch;         /* 2b8 */
    CARD32 video3_zoom;           /* 2bc */
    CARD32 video3_mictl;          /* 2c0 */
    CARD32 video3_CSC1;           /* 2c4 */
    CARD32 video3_CSC2;           /* 2c8 */
    CARD32 v3_display_temp;       /* 2cc */
    CARD32 reserved[5];           /* 2d0 */
    CARD32 video1u_addr1;         /* 2e4 */
    CARD32 video1u_addr2;         /* 2e8 */
    CARD32 video1u_addr3;         /* 2ec */
    CARD32 video1v_addr0;         /* 2f0 */
    CARD32 video1v_addr1;         /* 2f4 */
    CARD32 video1v_addr2;         /* 2f8 */
    CARD32 video1v_addr3;         /* 2fc */
}  video_via_regs;

#endif  /* _VIA_VIDEO_H_ */
