/*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/*************************************************************************
 *
 *  File:       via_driver.c
 *  Content:    XFree86 4.0 for VIA/S3G UniChrome
 *
 ************************************************************************/
#include <stdlib.h>
#include <sys/socket.h>
#include <sys/poll.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <linux/version.h>

#if (defined(PMLINK))&&(LINUX_VERSION_CODE>132116)
#include <linux/netlink.h>
#include <linux/socket.h>
#include <linux/connector.h>
#endif

#include "via_driver.h"
#include "xf86RAC.h"
#include "xf86Priv.h"
#include "shadowfb.h"

#include "globals.h"
#define DPMS_SERVER
#include "extensions/dpms.h"


#include "via_video.h"
#include "videodev.h"
#include "VidInit.h"
#include "hw.h"
#include "via_vt1636.h"

#include "debug.h"      /* for DBG_DD */
#include "ddmpeg.h"
#include "capture.h"
#include "via.h"

#include "via_pmlink.h"  /*for power management*/
#ifdef XF86DRI
#include "dri.h"
#endif
#include "via_mergedfb.h" /*for mergedfb*/

#include "via_rotate.h" /* for rotate feature.*/
#include "VidStruc.h"

#ifdef RANDRV12_TEST_ENABLE
#include "xf86Crtc.h"
#include "xf86RandR12.h"
#endif

/*
*    fix the "Undefined Symbol:vgaHWddc1SetSpeed" problem on FC5(32/64) when starting X,
*    replace the function vgaHWddc1SetSpeed as VIAvgaHWddc1SetSpeed.
*/
#ifdef XORG_VERSION_CURRENT
/*
*    fix the "Undefined Symbol:vgaHWddc1SetSpeed" problem on FC7(32/64) when starting X,
*    assign the function Xorg 1.3.0 to Xorg 7.2.0.
*/
#if XORG_VERSION_CURRENT >= XF86_VERSION_NUMERIC(1,3,0,0,0)
#if XORG_VERSION_CURRENT <= XF86_VERSION_NUMERIC(2,0,0,0,0)
    #define XORG_VERSION_CURRENT XF86_VERSION_NUMERIC(7,2,0,0,0)
#endif
#endif



extern Bool    VIAnoPanoramiXExtension;

extern Bool VIASenseDVI(VIABIOSInfoPtr pBIOSInfo);
extern Bool VIASetLCDPanelInfo(VIABIOSInfoPtr pBIOSInfo, int panel_id);
extern void VIAPreInit_LVDS(VIABIOSInfoPtr pBIOSInfo);
extern void VIATMDSRegInit(VIABIOSInfoPtr pBIOSInfo);
extern void write_reg_mask(CARD8 index, int io_port, CARD8 data, CARD8 mask);
extern __inline__ CARD8 read_reg(int io_port, CARD8 index);

extern void via_crtc_init(ScrnInfoPtr pScrn, int pipe);
extern void via_output_init(ScrnInfoPtr pScrn);

extern VIDEOMEMORY VidMemory;
static Bool IsVidDataAllocated = FALSE; /* To prevent VidData allocated twice. */

/* for Update Cursor/Icon scaling image during switch mode */
extern void VIAHideCursor(ScrnInfoPtr pScrn);
extern void VIAShowCursor(ScrnInfoPtr pScrn);
extern void VIALoadIconImage(ScrnInfoPtr pScrn, CARD32 * src);

/*
 * prototypes
 */

static void VIAIdentify(int flags);
static Bool VIAProbe(DriverPtr drv, int flags);
static Bool VIAPreInit(ScrnInfoPtr pScrn, int flags);
static Bool VIAEnterVT(int scrnIndex, int flags);
static void VIALeaveVT(int scrnIndex, int flags);
static void VIASave(ScrnInfoPtr pScrn);
static void VIARestore(ScrnInfoPtr, vgaRegPtr, VIARegPtr);
static void VIAWriteMode(ScrnInfoPtr pScrn, vgaRegPtr, VIARegPtr);
static Bool VIAModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode);
static Bool VIACloseScreen(int scrnIndex, ScreenPtr pScreen);
static Bool VIASaveScreen(ScreenPtr pScreen, int mode);
static Bool VIAPMEvent(int scrnIndex, pmEvent event, Bool undo);
static void VIALoadPalette(ScrnInfoPtr pScrn, int numColors, int *indicies,
                           LOCO *colors, VisualPtr pVisual);
static Bool VIAScreenInit(int scrnIndex, ScreenPtr pScreen, int argc,
                          char **argv);
static int VIAInternalScreenInit(int scrnIndex, ScreenPtr pScreen);
static void VIAFreeScreen(int scrnIndex, int flags);
static ModeStatus VIAValidMode(int index, DisplayModePtr mode,
                               Bool verbose, int flags);
static void VIADPMS(ScrnInfoPtr pScrn, int mode, int flags);
static const OptionInfoRec * VIAAvailableOptions(int chipid, int busid);
static void VIAProbeChipset(ScrnInfoPtr pScrn, EntityInfoPtr pEnt);
static Bool VIAFilterMode(ScrnInfoPtr pScrn);
static Bool VIAProbeMonitorLimitation(ScrnInfoPtr pScrn);
static Bool VIAProbeBIOSInfo(ScrnInfoPtr pScrn);

static void VIAEnableMMIO(ScrnInfoPtr pScrn);
static Bool VIAMapMMIO(ScrnInfoPtr pScrn);
static Bool VIAMapFB(ScrnInfoPtr pScrn);
static void VIAUnmapMem(ScrnInfoPtr pScrn);
Bool VIADeviceSelection(ScrnInfoPtr pScrn);
Bool VIASetDisplayPath_SAMM(ScrnInfoPtr pScrn);
Bool VIADeviceDispatchDuoView(ScrnInfoPtr pScrn);
void VIASetDisplayPath(ScrnInfoPtr pScrn);
static void VIABuildModeList(ScrnInfoPtr pScrn);
void VIAFindMaxResFromEstablishedTiming(xf86MonPtr MonitorConf, int* MaxH, int* MaxV);

/* query NB to get frame buffer size */
static int VIAGetFBSize(ScrnInfoPtr pScrn);


/* functions and variables are used for power event handling */
#if (defined(PMLINK))&&(LINUX_VERSION_CODE>132116)
#define VIA_SCHEDULE_SLICE  1000  /*1000ms*/
#define VIA_POLL_TIMEOUT  900   /*900ms*/ 
#define NETLINK_CONNECTOR 11
static int sock_fd=-1;
static struct sockaddr_nl  local;
static int flag_lastrecv=0; 
CARD32 VIAPMHandler(OsTimerPtr timer,CARD32 now,pointer arg);
static int VIAPMLinkSendMSG(int sock_fd,struct cn_msg *msg);
static void VIASuspendRoutine(ScrnInfoPtr pScrn,struct cn_msg *msg);
static void VIAResumeRoutine(ScrnInfoPtr pScrn,struct cn_msg *msg);
static void VIAStandbyRoutine(ScrnInfoPtr pScrn,struct cn_msg *msg);
static int  VIAConfirmEvent(struct cn_msg *msg);
#endif  /* PMLINK **/ 

#if XORG_VERSION_CURRENT >= XF86_VERSION_NUMERIC(6,8,0,0,0)
    #if XORG_VERSION_CURRENT >= XF86_VERSION_NUMERIC(6,9,0,0,0)
    static Bool VIADriverFunc(ScrnInfoPtr pScrnInfo, xorgDriverFuncOp op, pointer data);
    #else
    static Bool VIADriverFunc(ScrnInfoPtr pScrn, xorgRRFuncFlags op, pointer data);
    #endif
#endif


void VIAInitialize3DEngine(ScrnInfoPtr pScrn);
Bool volatile b3DRegsInitialized = 0;

/* E X T E R N   F U N C T I O N S ------------------------------------------*/
extern Bool VIADRIFBInit(ScreenPtr pScreen, VIAPtr pVia);
extern void VIAEnableLCD(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo);
extern Bool VIAPostLCDNotDVO(VIABIOSInfoPtr pBIOSInfo);
extern Bool VIAPostLCDDVO(VIABIOSInfoPtr pBIOSInfo);
extern void VIADisableLCD(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo);
extern Bool VIATMDSIdentify(VIABIOSInfoPtr pBIOSInfo);
extern unsigned char VIAGetPanelInfo(VIABIOSInfoPtr pBIOSInfo);
extern void VIAEnableDFP(VIABIOSInfoPtr pBIOSInfo);
extern void VIADisableDFP(VIABIOSInfoPtr pBIOSInfo);
extern void VIAARGBCursorInit(ScrnInfoPtr pScrn);


/*---------------------------------------------------------------------------*/

DriverRec VIA =
{
    VIA_VERSION,
    DRIVER_NAME,
    VIAIdentify,
    VIAProbe,
    VIAAvailableOptions,
    NULL,
    0
};

/* Supported chipsets */

static SymTabRec VIAChipsets[] = {
    {VIA_CLE266,   "CLE266"},
    {VIA_KM400,    "KM400/KN400"},
    {VIA_K8M800,   "K8M800/K8N800"},
    {VIA_PM800,    "PM800/PM880/CN400"},
    {VIA_P4M800PRO,    "P4M800PRO"},
    {VIA_CX700,    "CX700"},
    {VIA_K8M890,   "K8M890"},
    {VIA_P4M890,   "P4M890"},
    {VIA_CN750,    "CN750"}, 
    {VIA_P4M900,   "P4M900"}, 
    {VIA_VX800,    "VX800"}, 
    {-1,            NULL }
};


/* This table maps a PCI device ID to a chipset family identifier. */

static PciChipsets VIAPciChipsets[] = {
    {VIA_CLE266,   PCI_CHIP_CLE3122,   RES_SHARED_VGA},
    {VIA_CLE266,   PCI_CHIP_CLE3022,   RES_SHARED_VGA},
    {VIA_KM400,    PCI_CHIP_VT7205,    RES_SHARED_VGA},
    {VIA_KM400,    PCI_CHIP_VT3205,    RES_SHARED_VGA},
    {VIA_K8M800,   PCI_CHIP_VT3204,    RES_SHARED_VGA},
    {VIA_PM800,    PCI_CHIP_VT3259,    RES_SHARED_VGA},
    {VIA_P4M800PRO, PCI_CHIP_VT3314,    RES_SHARED_VGA}, 
    {VIA_CX700,    PCI_CHIP_VT3324,    RES_SHARED_VGA},
    {VIA_K8M890,   PCI_CHIP_VT3336,    RES_SHARED_VGA},
    {VIA_P4M890,   PCI_CHIP_VT3327,    RES_SHARED_VGA},
    {VIA_CN750,    PCI_CHIP_VT3293,    RES_SHARED_VGA},
    {VIA_P4M900,   PCI_CHIP_VT3364,    RES_SHARED_VGA},
    {VIA_VX800,    PCI_CHIP_VT3353,    RES_SHARED_VGA},
    {-1,            -1,                RES_UNDEFINED}
};

int gVIAEntityIndex = -1;

typedef enum {
    OPTION_PCI_BURST,
    OPTION_PCI_RETRY,
    OPTION_NOACCEL,
    OPTION_SWCURSOR,
    OPTION_SHADOW_FB,
    OPTION_ROTATE,
    OPTION_ROTATE_TYPE,
    OPTION_INVERSE,    
    OPTION_USEBIOS,
    OPTION_VIDEORAM,
    OPTION_VIDEORAM1,
    OPTION_TWO_LCD,
    OPTION_ACTIVEDEVICE,
    OPTION_DISPLAY_HARDWARE_LAYOUT,
    OPTION_DVI_PORT,
    OPTION_DVI_DUAL_EDGE,
    OPTION_DVI_BUS_WIDTH,
    OPTION_SIMULTANEOUS,
    OPTION_LCD_PORT,
    OPTION_LCD_BUS_WIDTH,
    OPTION_CENTER,
    OPTION_REFRESH,
    OPTION_DISABLEVQ,
    OPTION_NODDCVALUE,
    OPTION_DEVICE_SWITCH_HOTKEY,
    OPTION_HQVFILTER_MANUALSELECT,
    OPTION_MPEG_HQV_VFILTER_MODE,
    OPTION_MPEG_HQV_HFILTER_MODE,
    OPTION_CAP_HQV_VFILTER_MODE,
    OPTION_CAP_HQV_HFILTER_MODE,
    OPTION_SETMPEG_FBNUMBER,
    OPTION_DRIXINERAMA,
    OPTION_NOMPEGAddOneLineOnBottom,
    OPTION_DeBlockingEnable,
    OPTION_MERGEDFB, /*enable mergedfb,switch to virtualscreen*/
    OPTION_NOVIAXINERAMA,/*and set screen layout*/
    OPTION_SCRN2POS,
    OPTION_EPIA_DVI,/* for TMDS on EPIA*/
    OPTION_VideoOnDevice, /*set video to work on which device for 3314,3204 etc.in duo view mode */
    OPTION_FORCE_LCD, /* Force to enable LCD without mobile version BIOS. */
    OPTION_LCDPANELID, /* simply LCD panel option. */
    OPTION_DRIVER_VERSION,
    OPTION_DPA_SETTING_DVP0,
    OPTION_DPA_SETTING_DVP1,
    OPTION_DPA_SETTING_DFPHIGH,
    OPTION_DPA_SETTING_DFPLOW,
    OPTION_DPA_SETTING_VT1636,
    OPTION_VIDEO_COLOR_KEY,
    OPTION_LCD_MSB_ENABLE,
    OPTION_INTERLACE_MODE,
    OPTION_FN_HOTKEY
} VIAOpts;


static OptionInfoRec VIAOptions[] =
{
    {OPTION_NOACCEL,    "NoAccel",      OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_SWCURSOR,   "SWCursor",     OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_SHADOW_FB,  "ShadowFB",     OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_ROTATE,     "Rotate",     OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_ROTATE_TYPE,"RotateType",     OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_INVERSE,    "Inverse",      OPTV_BOOLEAN, {0}, FALSE},    
    {OPTION_USEBIOS,    "UseBIOS",      OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_VIDEORAM,   "VideoRAM",     OPTV_INTEGER, {0}, FALSE},
    {OPTION_VIDEORAM1,  "VideoRAM1",    OPTV_INTEGER, {0}, FALSE},
    {OPTION_TWO_LCD,    "TwoLCD",       OPTV_BOOLEAN, {0}, FALSE},    
    {OPTION_ACTIVEDEVICE,   "ActiveDevice", OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_DISPLAY_HARDWARE_LAYOUT, "DisplayHardwareLayout",   OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_DVI_PORT,       "DVIPort",      OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_DVI_DUAL_EDGE,  "DVIDualEdge",  OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_DVI_BUS_WIDTH,  "DVIBusWidth",  OPTV_ANYSTR, {0}, FALSE},
    {OPTION_SIMULTANEOUS,   "Simultaneous", OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_LCD_PORT,       "LCDPort",      OPTV_ANYSTR, {0}, FALSE},
    {OPTION_LCD_BUS_WIDTH,  "LCDBusWidth",  OPTV_ANYSTR, {0}, FALSE},
    {OPTION_CENTER,         "Center",       OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_LCDPANELID,     "PanelID",      OPTV_INTEGER, {0}, FALSE},
    {OPTION_REFRESH,    "Refresh",      OPTV_INTEGER, {0}, FALSE},
    {OPTION_DISABLEVQ,  "DisableVQ",    OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_NODDCVALUE, "NoDDCValue",   OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_DEVICE_SWITCH_HOTKEY, "DeviceSwitchHotkey", OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_HQVFILTER_MANUALSELECT, "HQVFilterManualSelect",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_MPEG_HQV_VFILTER_MODE,   "MpegHQVVFilterMode", OPTV_INTEGER, {0}, FALSE},
    {OPTION_MPEG_HQV_HFILTER_MODE,   "MpegHQVHFilterMode", OPTV_INTEGER, {0}, FALSE},
    {OPTION_CAP_HQV_VFILTER_MODE,   "CapHQVVFilterMode", OPTV_INTEGER, {0}, FALSE},
    {OPTION_CAP_HQV_HFILTER_MODE,   "CapHQVHFilterMode", OPTV_INTEGER, {0}, FALSE},
    {OPTION_SETMPEG_FBNUMBER, "SetMpegFBNumber",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_DRIXINERAMA,  "DRIXINERAMA",    OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_NOMPEGAddOneLineOnBottom,   "NOMPEGAddOneLineOnBottom", OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_DeBlockingEnable,   "DeBlockingEnable", OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_MERGEDFB, "MergedFB",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_NOVIAXINERAMA, "NoVIAXinerama",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_SCRN2POS,   "Scrn2Pos", OPTV_ANYSTR, {0}, FALSE},   
    {OPTION_EPIA_DVI, "EPIA_DVI",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_VideoOnDevice,  "VideoOnDevice",       OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_FORCE_LCD, "ForceLCD", OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_SCRN2POS,   "Scrn2Pos", OPTV_ANYSTR, {0}, FALSE},
    {OPTION_DPA_SETTING_DVP0,       "DPASetting_DVP0",      OPTV_ANYSTR, {0}, FALSE},
    {OPTION_DPA_SETTING_DVP1,       "DPASetting_DVP1",      OPTV_ANYSTR, {0}, FALSE},
    {OPTION_DPA_SETTING_DFPHIGH,    "DPASetting_DFPHIGH",   OPTV_ANYSTR, {0}, FALSE},
    {OPTION_DPA_SETTING_DFPLOW,     "DPASetting_DFPLOW",    OPTV_ANYSTR, {0}, FALSE},
    {OPTION_DPA_SETTING_VT1636,     "DPASetting_VT1636",    OPTV_ANYSTR, {0}, FALSE},
    {OPTION_VIDEO_COLOR_KEY,        "VideoColorKey",  OPTV_INTEGER, {0}, FALSE},
    {OPTION_LCD_MSB_ENABLE,         "LCD_MSB_ENABLE",  OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_INTERLACE_MODE,         "Interlace",            OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_FN_HOTKEY,              "FnHotkey",             OPTV_ANYSTR, {0}, FALSE},
    {-1,                NULL,           OPTV_NONE,    {0}, FALSE}
};


static const char *vgaHWSymbols[] = {
    "vgaHWGetHWRec",
    "vgaHWSetMmioFuncs",
    "vgaHWSetStdFuncs",
    "vgaHWGetIOBase",
    "vgaHWSave",
    "vgaHWProtect",
    "vgaHWRestore",
    "vgaHWMapMem",
    "vgaHWUnmapMem",
    "vgaHWInit",
    "vgaHWSaveScreen",
    "vgaHWLock",
    "vgaHWUnlock",
    "vgaHWFreeHWRec",
    NULL
};


static const char *ramdacSymbols[] = {
    "xf86InitCursor",
    "xf86CreateCursorInfoRec",
    "xf86DestroyCursorInfoRec",
    NULL
};

static const char *vbeSymbols[] = {
    "VBEInit",
    "VBEGetVBEMode",
    "VBESetVBEMode",
    "vbeDoEDID",
    "vbeFree",
    NULL
};

static const char *vbeSymbols429900[] = {
    "VBEExtendedInit",
    "VBEGetVBEMode",
    "VBESetVBEMode",
    "vbeDoEDID",
    "vbeFree",
    NULL
};

static const char *ddcSymbols[] = {
    "xf86PrintEDID",
    "xf86DoEDID_DDC1",
    "xf86DoEDID_DDC2",
    "xf86SetDDCproperties",
    NULL
};


static const char *i2cSymbols[] = {
    "xf86CreateI2CBusRec",
    "xf86I2CBusInit",
    "xf86CreateI2CDevRec",
    "xf86I2CDevInit",
    "xf86I2CWriteRead",
    "xf86I2CFindDev",
    "xf86I2CProbeAddress",
    "xf86DestroyI2CDevRec",
    "xf86DestroyI2CBusRec",
    NULL
};

static const char *xaaSymbols[] = {
#ifdef X_HAVE_XAAGETROP
    "XAAGetCopyROP",
    "XAAGetCopyROP_PM",
    "XAAGetPatternROP",
#else
    "XAACopyROP",
    "XAACopyROP_PM",
    "XAAPatternROP",
#endif
    "XAACreateInfoRec",
    "XAADestroyInfoRec",
    "XAAInit",
    "XAAFillSolidRects",
    NULL
};

static const char *int10Symbols[] = {
    "xf86FreeInt10",
    "xf86InitInt10",
    NULL
};

static const char *shadowSymbols[] = {
    "ShadowFBInit",
    NULL
};

#ifdef USE_FB
static const char *fbSymbols[] = {
    "fbScreenInit",
    "fbPictureInit",
    NULL
};
#else
static const char *cfbSymbols[] = {
    "cfbScreenInit",
    "cfb16ScreenInit",
    "cfb24ScreenInit",
    "cfb24_32ScreenInit",
    "cfb32ScreenInit",
    "cfb16BresS",
    "cfb24BresS",
    NULL
};
#endif

/* for Via DVD and Video support */
/*
 * No use now, for old Xserver module code use
 */

#ifdef XFree86LOADER
#ifdef XF86DRI
static const char *drmSymbols[] = {
    "drmOpen",
    "drmClose",
    "drmAddMap",
    "drmAgpAcquire",
    "drmAgpAlloc",
    "drmAgpBase",
    "drmAgpBind",
    "drmAgpEnable",
    "drmAgpFree",
    "drmAgpGetMode",
    "drmAgpRelease",
    "drmCtlInstHandler",
    "drmGetInterruptFromBusID",
    "drmMap",
    "drmUnmap",
    NULL
};

static const char *driSymbols[] = {
    "DRICloseScreen",
    "DRICreateInfoRec",
    "DRIDestroyInfoRec",
    "DRIFinishScreenInit",
    "DRIGetSAREAPrivate",
    "DRILock",
    "DRIQueryVersion",
    "DRIScreenInit",
    "DRIUnlock",
    "GlxSetVisualConfigs",
    NULL
};
#endif

static MODULESETUPPROTO(VIASetup);

static XF86ModuleVersionInfo VIAVersRec = {
    "via",
    MODULEVENDORSTRING,
    MODINFOSTRING1,
    MODINFOSTRING2,
    XF86_VERSION_CURRENT,
    VERSION_MAJOR, VERSION_MINOR, PATCHLEVEL,
    ABI_CLASS_NONE,
    ABI_VIDEODRV_VERSION,
    MOD_CLASS_VIDEODRV,
    {0, 0, 0, 0}
};

XF86ModuleData viaModuleData = {&VIAVersRec, VIASetup, NULL};

unsigned int via_xf86_version_current;


#ifdef RANDRV12_TEST_ENABLE
static Bool
via_xf86crtc_resize (ScrnInfoPtr pScrn, int width, int height)
{
    DEBUG(ErrorF("via_xf86crtc_resize, width=%d, height=%d\n", width, height));
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;   
    CARD32 i;
        
    xf86CrtcConfigPtr xf86_config = XF86_CRTC_CONFIG_PTR(pScrn);
    
    pScrn->virtualX = width;
    pScrn->virtualY = height;       
        
    return TRUE;
}

static const xf86CrtcConfigFuncsRec via_xf86crtc_config_funcs = {
    via_xf86crtc_resize
};
#endif


void VIALoadVideoOptionDefault(VIAPtr pVia,LPVIAGRAPHICINFO lpVIAGraphicInfo) 
{
    lpVIAGraphicInfo->HQVFilter_ManualSelect = FALSE;
    lpVIAGraphicInfo->MpegHQV_VFilterMode = 0;
    lpVIAGraphicInfo->MpegHQV_HFilterMode = 0;
    lpVIAGraphicInfo->CapHQV_VFilterMode = 0;
    lpVIAGraphicInfo->CapHQV_HFilterMode = 0;
    lpVIAGraphicInfo->SetMpeg_FBNumber = FALSE;

    lpVIAGraphicInfo->NoMPEGAddOneLineOnBottom = FALSE;
    lpVIAGraphicInfo->DeBlockingEnable = FALSE;
    lpVIAGraphicInfo->dw3to2PullDownDisable = FALSE;

    return;
}

/* Load video option value from .VIAVIDEORC */
Bool VIALoadVideoOption(VIAPtr pVia,LPVIAGRAPHICINFO lpVIAGraphicInfo)
{
    FILE	*VIAVIDEORC = NULL;
    char 	filename[40], *home = NULL;
    Bool	ret = FALSE;
    char line[1024];
    char *value;

    int iValue;


    xf86Msg(X_INFO, "VIALoadVideoOption\n");

    home="/etc/X11";
    if (home)
    {
        filename[0] = '\0';
        strcpy(filename, home);
        strcat(filename, "/.VIAVIDEORC");
        if ((VIAVIDEORC = fopen(filename, "r")) == NULL)
        {
            xf86Msg(X_WARNING,
                    "No Video option file .VIAVIDEORC exist. \n");
            ret = FALSE;
        }
        else
        {
            /*
             *  Default video setting,
             *  Can be replaced by user-specified value later.
             */

            while (fgets (line, 1023, VIAVIDEORC))
            {
                line[strlen(line)-1]= (char) 0;

                if (line[0] == '#') continue;

                if ((value = strchr (line, ':')))
                {
                    *value = (char) 0;
                    value++;

                    if (strcmp(line,"HQVFILTER_MANUALSELECT")==0)
                    {
                      	if (atoi(value)==TRUE)
                     	{
                         	lpVIAGraphicInfo->HQVFilter_ManualSelect = TRUE;
                    	 		DBG_DD(("Option: HQV Filter Manual Select Enabled\n"));
                     	}
                     	else
                     	{
                          	lpVIAGraphicInfo->HQVFilter_ManualSelect = FALSE;
                        	DBG_DD(("Option: HQV Filter Manual Select Disabled\n"));
                    	}
                    }
                    else  if (strcmp(line,"MPEG_HQV_VFILTER_MODE")==0)
     	     		{
     	                lpVIAGraphicInfo->MpegHQV_VFilterMode = atoi(value);
     	            	DBG_DD(("MPEG HQV vertical filter mode: %d\n",lpVIAGraphicInfo->MpegHQV_VFilterMode));
       	     		}
                    else  if (strcmp(line,"MPEG_HQV_HFILTER_MODE")==0)
     	     		{
     	               	lpVIAGraphicInfo->MpegHQV_HFilterMode = atoi(value);
     	            	DBG_DD(("MPEG HQV horizontal filter mode: %d\n",lpVIAGraphicInfo->MpegHQV_HFilterMode));
       	     		}
                    else  if (strcmp(line,"CAP_HQV_VFILTER_MODE")==0)
     	     		{
     	               	lpVIAGraphicInfo->CapHQV_VFilterMode = atoi(value);
     	            	DBG_DD(("Capture HQV vertical filter mode: %d\n",lpVIAGraphicInfo->CapHQV_VFilterMode));
       	     		}
                    else  if (strcmp(line,"CAP_HQV_HFILTER_MODE")==0)
     	     		{
     	               	lpVIAGraphicInfo->CapHQV_HFilterMode = atoi(value);
     	            	DBG_DD(("Capture HQV Hertical filter mode: %d\n",lpVIAGraphicInfo->CapHQV_HFilterMode));
       	     		}
                    else  if (strcmp(line,"SETMPEG_FBNUMBER")==0)
     	     		{
     	               	if (atoi(value)==TRUE)
     	               	{
     	                   	lpVIAGraphicInfo->SetMpeg_FBNumber = TRUE;
   	                   		DBG_DD(("Option: Set MPEG decode frame buffer number Enable\n"));
     	              	}
     	              	else
     	              	{	
     	                   	lpVIAGraphicInfo->SetMpeg_FBNumber = FALSE;
      	                	DBG_DD(("Option: Set MPEG decode frame buffer number Disable\n"));
     	            	}
       	     		}
                    else  if (strcmp(line,"NOMPEGAddOneLineOnBottom")==0)
     	     		{
                        if (atoi(value)==TRUE)
                        {
                            lpVIAGraphicInfo->NoMPEGAddOneLineOnBottom = TRUE;
                            DBG_DD(("Option: No mpeg add one line on bottom\n"));
                        }
                        else
                        {
                            lpVIAGraphicInfo->NoMPEGAddOneLineOnBottom = FALSE;
                            DBG_DD(("Option: mpeg add one line on bottom\n"));
                        }
       	     		}
                    else  if (strcmp(line,"DeBlocking")==0) 
                    {
                        if (atoi(value)==TRUE)
                        {
							lpVIAGraphicInfo->DeBlockingEnable = TRUE;
                            DBG_DD(("Option: DeBlocking Enable!!\n"));                                                         
                        } 
                        else
                        {
							lpVIAGraphicInfo->DeBlockingEnable = FALSE;
                            DBG_DD(("Option: DeBlocking Disable!!\n")); 
                        }    
		      		}
                    else  if (strcmp(line,"3to2PullDownDisable")==0)
                    {
                        if (atoi(value)==TRUE)
                        {
                            lpVIAGraphicInfo->dw3to2PullDownDisable = TRUE;
                            DBG_DD(("Option: 3:2PullDown Disable!!\n"));
                        } 
                        else
                        {
                            lpVIAGraphicInfo->dw3to2PullDownDisable = FALSE;
                            DBG_DD(("Option: 3:2PullDown Enable!!\n")); 
                        }    
                    }
                    else  if (strcmp(line,"VideoRotation")==0)                      
                    {
                        lpVIAGraphicInfo->rotate= atoi(value);
                        DBG_DD(("Option:rotate: %d\n", lpVIAGraphicInfo->rotate));                         
                    }					
                }
            }

            ret = TRUE;
            fclose(VIAVIDEORC);
        }
    }
    return ret;

}

int GetIGAPath(VIABIOSInfoPtr pBIOSInfo)
{
    switch(pBIOSInfo->ActiveDevice)
    {
        case VIA_DEVICE_LCD:
          return pBIOSInfo->LVDSSettingInfo.IGAPath;
          
        case VIA_DEVICE_CRT1:        
          return pBIOSInfo->CRTSettingInfo.IGAPath;
          
        case VIA_DEVICE_DFP:
          return pBIOSInfo->TMDSSettingInfo.IGAPath;
          
        case VIA_DEVICE_CRT2:
          return pBIOSInfo->CRTSettingInfo2.IGAPath;
          
        default:
          return IGA1;
     }
} 

void FillGraphicInfo(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    PVIDDATA pVidData = pVia->pVidData;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;
    FILE*    pFilePointer;        /* for SAMM mode passing screen info */
    char     szBuffer[256];
    char*    pBuffer;
    int      icount = 0;
    int      i;
    /*Added for SAMM Mode*/
    unsigned char* FBBase;
    ScrnInfoPtr pScrn0 = xf86Screens[0];
    VIAPtr pVia0 = VIAPTR(pScrn0);
    DBG_DD(("  FillGraphicInfo!!\n"));

    lpVIAGraphicInfo->scrnIndex = pScrn->scrnIndex;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].TotalVRAM = pVia->videoRambytes;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].VideoHeapBase = (unsigned long) pVia->FBFreeStart;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].VideoHeapEnd = (unsigned long) (pVia->FBFreeEnd - 1);
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwWidth = pBIOSInfo->CrtcHDisplay;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwHeight = pBIOSInfo->CrtcVDisplay;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwSaveWidth = pBIOSInfo->SaveHDisplay;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwSaveHeight = pBIOSInfo->SaveVDisplay;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwCurWidth = pBIOSInfo->HDisplay;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwCurHeight = pBIOSInfo->VDisplay;    
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwBPP = pScrn->bitsPerPixel;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwPitch = (((pScrn->virtualX) + 15) & ~15) * (pScrn->bitsPerPixel) / 8;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwRefreshRate = (unsigned long)pBIOSInfo->FoundRefresh;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwDVIOn = pBIOSInfo->DVIAttach;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwLCDOn = pBIOSInfo->LCDAttach;
    /*if set viatual screen ,virtual width is virtualX ,height is virtualY ,enable  panning mode */
    if (pBIOSInfo->SaveHDisplay > pBIOSInfo->CrtcHDisplay ||pScrn->virtualX> 0 || pScrn->virtualY >0)
    {
        lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwPanning = 1;
    }
    else
    {
        lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwPanning = 0;
    }
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwExpand = pBIOSInfo->scaleY;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwPanelWidth = pBIOSInfo->LVDSSettingInfo.PanelSizeX;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwPanelHeight = pBIOSInfo->LVDSSettingInfo.PanelSizeY;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwActiveDevice = pBIOSInfo->ActiveDevice;

    lpVIAGraphicInfo->dwRevisionID = pVia->ChipRev;
    lpVIAGraphicInfo->dwDeviceID = pVia->ChipId;
    /* Simplify Device ID + Revision ID combination */
    lpVIAGraphicInfo->dwProjectIDRevision = pBIOSInfo->dwProjectIDRevision;
    lpVIAGraphicInfo->SAMM = pBIOSInfo->SAMM;
    lpVIAGraphicInfo->dwActiveDevice = (pBIOSInfo->SAMM ? pVia->ActiveDevice : pBIOSInfo->ActiveDevice);
    lpVIAGraphicInfo->ScreenPhysAddr = pVia->FrameBufferBase;
    lpVIAGraphicInfo->VideoUseSecondDisplay = pBIOSInfo->UseIGA2;
    lpVIAGraphicInfo->dwXServerEnabled = TRUE;
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "lpVIAGraphicInfo->dwXServerEnabled is %lu\n", lpVIAGraphicInfo->dwXServerEnabled);         
    lpVIAGraphicInfo->TMDSChipID = pBIOSInfo->TMDSSettingInfo.ChipID;
    
    pVidData->Vhwinfo.pjVideo       = pVia->MapBase; /* save register MMIO pointer to VidData */

    if (pVia->IsSecondary)
    {
        FBBase = pVia0->FBBase;
    }
    else
    {
        FBBase = pVia->FBBase;
    }

    pVidData->Vhwinfo.pjScreen       = FBBase; /* save linear screen address to VidData */
 
    pVidData->Vhwinfo.xpjVideo= pVia->MapBase; 
    pVidData->Vhwinfo.xpjScreen       = FBBase; 

    lpVIAGraphicInfo->IGAPanningStatus=(unsigned long)(((pBIOSInfo->IGA1SettingInfo.IsPanning)&0x01) | (((pBIOSInfo->IGA2SettingInfo.IsPanning)&0x01)<<1));
    if(pScrn->scrnIndex==SCREEN_INDEX_0){
        for(i=0;i<3;i++) {
            lpVIAGraphicInfo->Screen_IGA_Map[i] = -1;
            lpVIAGraphicInfo->IGA_Video_Map[i] = -1;
        }
    }
    if(lpVIAGraphicInfo->SAMM) {
        lpVIAGraphicInfo->duoview=0;
        if(pScrn->scrnIndex==SCREEN_INDEX_1) {
            if(pBIOSInfo->IGA1SettingInfo.IsActive) {
                lpVIAGraphicInfo->Screen_IGA_Map[SCREEN_INDEX_1] = IGA1;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwWidth=pBIOSInfo->IGA1SettingInfo.HActive;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwHeight=pBIOSInfo->IGA1SettingInfo.VActive;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwPanning=pBIOSInfo->IGA1SettingInfo.IsPanning;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].IsDownScaling = pBIOSInfo->IGA1SettingInfo.IsDownScaling;			
            }
            if(pBIOSInfo->IGA2SettingInfo.IsActive) {
                lpVIAGraphicInfo->Screen_IGA_Map[SCREEN_INDEX_1] = IGA2;   
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwWidth=pBIOSInfo->IGA2SettingInfo.HActive;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwHeight=pBIOSInfo->IGA2SettingInfo.VActive;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwPanning=pBIOSInfo->IGA2SettingInfo.IsPanning;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].IsDownScaling = pBIOSInfo->IGA2SettingInfo.IsDownScaling;			
            }
        }
        else if(pScrn->scrnIndex==SCREEN_INDEX_0) {
            if(pBIOSInfo->IGA1SettingInfo.IsActive) {
                lpVIAGraphicInfo->Screen_IGA_Map[SCREEN_INDEX_0] = IGA1;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwWidth=pBIOSInfo->IGA1SettingInfo.HActive;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwHeight=pBIOSInfo->IGA1SettingInfo.VActive;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwPanning=pBIOSInfo->IGA1SettingInfo.IsPanning;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].IsDownScaling = pBIOSInfo->IGA1SettingInfo.IsDownScaling;

            }
            if(pBIOSInfo->IGA2SettingInfo.IsActive) {
                lpVIAGraphicInfo->Screen_IGA_Map[SCREEN_INDEX_0] = IGA2;   
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwWidth=pBIOSInfo->IGA2SettingInfo.HActive;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwHeight=pBIOSInfo->IGA2SettingInfo.VActive;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwPanning=pBIOSInfo->IGA2SettingInfo.IsPanning;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].IsDownScaling = pBIOSInfo->IGA2SettingInfo.IsDownScaling;

            }
        }
    } 
    else if(lpVIAGraphicInfo->duoview&DUOVIEW_ENABLED)
    {
        /* If DuoView, Screen_IGA_Map[] are fixed to {IGA1,IGA2} */
        lpVIAGraphicInfo->Screen_IGA_Map[ SCREEN_INDEX_0 ]=IGA1; 
        lpVIAGraphicInfo->Screen_IGA_Map[ SCREEN_INDEX_1 ]=IGA2; 
        /* Screen's size on IGA1,2 may different. Both Screen[0:1] are used */
        lpVIAGraphicInfo->Screen[SCREEN_INDEX_1]=lpVIAGraphicInfo->Screen[SCREEN_INDEX_0]; 
        lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwWidth=pBIOSInfo->IGA1SettingInfo.HActive;
        lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwHeight=pBIOSInfo->IGA1SettingInfo.VActive;
        lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwPanning=pBIOSInfo->IGA1SettingInfo.IsPanning;
        lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].IsDownScaling = pBIOSInfo->IGA1SettingInfo.IsDownScaling;
        lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwWidth=pBIOSInfo->IGA2SettingInfo.HActive;
        lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwHeight=pBIOSInfo->IGA2SettingInfo.VActive;
        lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwPanning=pBIOSInfo->IGA2SettingInfo.IsPanning;
        lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].IsDownScaling = pBIOSInfo->IGA2SettingInfo.IsDownScaling;
    }
    else 
    { /* single or simultanious */
        if(pBIOSInfo->IGA1SettingInfo.IsActive) 
        {
            lpVIAGraphicInfo->Screen_IGA_Map[ SCREEN_INDEX_0 ]=IGA1;
            lpVIAGraphicInfo->Screen_IGA_Map[ SCREEN_INDEX_1 ]=-1;
            lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwWidth=pBIOSInfo->IGA1SettingInfo.HActive;
            lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwHeight=pBIOSInfo->IGA1SettingInfo.VActive;
            lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwPanning=pBIOSInfo->IGA1SettingInfo.IsPanning;
            lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].IsDownScaling = pBIOSInfo->IGA1SettingInfo.IsDownScaling;
        } 
        else if(pBIOSInfo->IGA2SettingInfo.IsActive)
        {
            lpVIAGraphicInfo->Screen_IGA_Map[ SCREEN_INDEX_0 ]=IGA2;
            lpVIAGraphicInfo->Screen_IGA_Map[ SCREEN_INDEX_1 ]=-1;
            lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwWidth=pBIOSInfo->IGA2SettingInfo.HActive;
            lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwHeight=pBIOSInfo->IGA2SettingInfo.VActive;
            lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwPanning=pBIOSInfo->IGA2SettingInfo.IsPanning;
            lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].IsDownScaling = pBIOSInfo->IGA2SettingInfo.IsDownScaling;
        }  
    }

    if((pFilePointer =fopen("/etc/X11/XF86Config-4", "r")) == NULL)
    {
        if((pFilePointer =fopen("/etc/X11/XF86Config", "r")) == NULL)
        {
            if((pFilePointer =fopen("/etc/X11/xorg.conf", "r")) == NULL)
            {
                DBG_DD(("Error: Can NOT open X config file !!!!!!!!!!!!\n"));
            }
        }
    }

    if( pFilePointer != NULL )
    {
        while( !feof(pFilePointer) )
        {
            fgets(szBuffer, sizeof(szBuffer), pFilePointer);
            pBuffer = szBuffer;
            for ( icount=0 ; icount<256 ; icount++ )
            {
                if ( strncmp(pBuffer, "RightOf", 7) == 0 )
                {
                    DBG_DD(("Screen 1 is Right side !!!!!!!!!!!!\n"));
                    break;
                }
                if ( strncmp(pBuffer, "LeftOf", 6) == 0 )
                {
                    lpVIAGraphicInfo->Screen1IsLeft = 1;
                    DBG_DD(("Screen 1 is Left side !!!!!!!!!!!!\n"));
                    break;
                }
                if ( strncmp(pBuffer, "Below", 5) == 0 )
                {
                    DBG_DD(("Screen 1 is Below !!!!!!!!!!!!\n"));
                    break;
                }
                if ( strncmp(pBuffer, "Above", 5) == 0 )
                {
                    lpVIAGraphicInfo->Screen1IsAbove = 1;
                    DBG_DD(("Screen 1 is Above !!!!!!!!!!!!\n"));
                    break;
                }
                pBuffer++;
            }
        }
        fclose( pFilePointer );
    }

    /* Added to pass DRM info to V4L */
#ifdef XF86DRI
    lpVIAGraphicInfo->DRMEnabled = pVia->directRenderingEnabled;
#else
    lpVIAGraphicInfo->DRMEnabled = 0;
#endif

}

static pointer VIASetup(pointer module, pointer opts, int *errmaj, int *errmin)
{
    static Bool setupDone = FALSE;

    if (!setupDone) {
        setupDone = TRUE;
        xf86AddDriver(&VIA, module, 0);
        LoaderRefSymLists(vgaHWSymbols,
#ifdef USE_FB
                          fbSymbols,
#else
                          cfbSymbols,
#endif
                          ramdacSymbols,
                          xaaSymbols,
                          shadowSymbols,
                          vbeSymbols,
                          i2cSymbols,
                          ddcSymbols,
                          /*
                          mpegSymbols,
                          */
#ifdef XF86DRI
                          drmSymbols,
                          driSymbols,
#endif
                          NULL);

        return (pointer) 1;
    }
    else {
        if (errmaj)
            *errmaj = LDR_ONCEONLY;

        return NULL;
    }
} /* VIASetup */

#endif /* XFree86LOADER */


static int
WaitIdleCLE266(VIAPtr pVia)
{
    int loop = 0;

    mem_barrier();

    while (!(VIAGETREG(VIA_REG_STATUS) & VIA_VR_QUEUE_BUSY) && (loop++ < MAXLOOP))
        ;

    while ((VIAGETREG(VIA_REG_STATUS) &
          (VIA_CMD_RGTR_BUSY | VIA_2D_ENG_BUSY | VIA_3D_ENG_BUSY)) &&
          (loop++ < MAXLOOP))
        ;

    return loop >= MAXLOOP;
}

static int
WaitIdleK8M890(VIAPtr pVia)
{
    int loop = 0;

    mem_barrier();

    while ((VIAGETREG(VIA_REG_STATUS) &
          (VIA_CMD_RGTR_BUSY | VIA_2D_ENG_BUSY|VIA_3D_ENG_BUSY)) &&
          (loop++ < MAXLOOP))
        ;

    return loop >= MAXLOOP;
}

static int
WaitIdleVX800(VIAPtr pVia)
{
    int loop = 0;

    mem_barrier();

    while ((VIAGETREG(VIA_REG_STATUS) & 
           (VIA_CMD_RGTR_BUSY_H5 | VIA_2D_ENG_BUSY_H5 | VIA_3D_ENG_BUSY_H5)) &&
           (loop++ < MAXLOOP))
        ;

    return loop >= MAXLOOP;
}

static Bool VIAGetRec(ScrnInfoPtr pScrn)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAGetRec\n"));
    if (pScrn->driverPrivate)
        return TRUE;

    pScrn->driverPrivate = xnfcalloc(sizeof(VIARec), 1);
    ((VIARec *)(pScrn->driverPrivate))->pBIOSInfo =
        xnfcalloc(sizeof(VIABIOSInfoRec), 1);
    ((VIARec *)(pScrn->driverPrivate))->pBIOSInfo->pModeTable =
        xnfcalloc(sizeof(VIAModeTableRec), 1);
    ((VIARec *)(pScrn->driverPrivate))->pBIOSVer =
        xnfcalloc(sizeof(VGABIOSVER),1);

    if(pScrn->scrnIndex == 0)
        ((VIARec *)(pScrn->driverPrivate))->pScreensPublicInfo = xnfcalloc(sizeof(VIAScreensPublicInfo), 1);
    else
        ((VIARec *)(pScrn->driverPrivate))->pScreensPublicInfo = ((VIARec *)(xf86Screens[0]->driverPrivate))->pScreensPublicInfo;

    /* initial value in VIARec */
    ((VIARec *)(pScrn->driverPrivate))->SavedReg.mode = 0xFF;
    ((VIARec *)(pScrn->driverPrivate))->ModeReg.mode = 0xFF;

    ((VIARec *)(pScrn->driverPrivate))->pBIOSInfo->FirstInit = TRUE;
    ((VIARec *)(pScrn->driverPrivate))->HwCursorImage = NULL;
    ((VIARec *)(pScrn->driverPrivate))->HwIconImage = NULL;
    ((VIARec *)(pScrn->driverPrivate))->HIScalingPrimImage = NULL;
    ((VIARec *)(pScrn->driverPrivate))->HIScalingSecImage = NULL;

    return TRUE;

} /* VIAGetRec */


static void VIAFreeRec(ScrnInfoPtr pScrn)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAFreeRec\n"));
    if (!pScrn->driverPrivate)
        return;

    if(pScrn->scrnIndex == 0)
    {
        xfree(((VIARec *)(pScrn->driverPrivate))->pScreensPublicInfo);
        ((VIARec *)(pScrn->driverPrivate))->pScreensPublicInfo = NULL;
    }
    else
        ((VIARec *)(pScrn->driverPrivate))->pScreensPublicInfo = NULL;

    xfree(((VIARec *)(pScrn->driverPrivate))->pBIOSVer);
    xfree(((VIARec *)(pScrn->driverPrivate))->pBIOSInfo->pModeTable);
    xfree(((VIARec *)(pScrn->driverPrivate))->pBIOSInfo);    
    xfree(pScrn->driverPrivate);
    pScrn->driverPrivate = NULL;

} /* VIAFreeRec */


static const OptionInfoRec * VIAAvailableOptions(int chipid, int busid)
{

    return VIAOptions;

} /* VIAAvailableOptions */


static void VIAIdentify(int flags)
{
    xf86PrintChipsets(DRIVER_NAME,
                      "driver for VIA chipsets",
                      VIAChipsets);
} /* VIAIdentify */


static Bool VIAProbe(DriverPtr drv, int flags)
{
    GDevPtr *devSections;
    int     *usedChips;
    int     numDevSections;
    int     numUsed;
    Bool    foundScreen = FALSE;
    int     i;

    /* sanity checks */
    if ((numDevSections = xf86MatchDevice(DRIVER_NAME, &devSections)) <= 0)
        return FALSE;

    if (xf86GetPciVideoInfo() == NULL)
        return FALSE;

    numUsed = xf86MatchPciInstances(DRIVER_NAME,
                                    PCI_VIA_VENDOR_ID,
                                    VIAChipsets,
                                    VIAPciChipsets,
                                    devSections,
                                    numDevSections,
                                    drv,
                                    &usedChips);
    xfree(devSections);

    if (numUsed <= 0)
        return FALSE;

    if (flags & PROBE_DETECT) {
        foundScreen = TRUE;
    }
    else {
        for (i = 0; i < numUsed; i++) {
            ScrnInfoPtr pScrn = xf86AllocateScreen(drv, 0);
            EntityInfoPtr pEnt;
            static int instance = 0;
            DevUnion* pPriv;

            if ((pScrn = xf86ConfigPciEntity(pScrn, 0, usedChips[i],
                 VIAPciChipsets, 0, 0, 0, 0, 0)))
            {
                pScrn->driverVersion = VIA_VERSION;
                pScrn->driverName = DRIVER_NAME;
                pScrn->name = "VIA";
                pScrn->Probe = VIAProbe;
                pScrn->PreInit = VIAPreInit;
                pScrn->ScreenInit = VIAScreenInit;
                pScrn->SwitchMode = VIASwitchMode;
                pScrn->AdjustFrame = VIAAdjustFrame;
                pScrn->EnterVT = VIAEnterVT;
                pScrn->LeaveVT = VIALeaveVT;
                pScrn->FreeScreen = VIAFreeScreen;
                pScrn->ValidMode = VIAValidMode;
                pScrn->PMEvent=VIAPMEvent;
                foundScreen = TRUE;
            }

            pEnt = xf86GetEntityInfo(usedChips[i]);

                xf86SetEntitySharable(usedChips[i]);
                xf86SetEntityInstanceForScreen(pScrn,
                    pScrn->entityList[0], instance);

                if(gVIAEntityIndex < 0)
                {
                    gVIAEntityIndex = xf86AllocateEntityPrivateIndex();
                    pPriv = xf86GetEntityPrivate(pScrn->entityList[0],
                            gVIAEntityIndex);

                    if (!pPriv->ptr)
                    {
                        VIAEntPtr pVIAEnt;
                        pPriv->ptr = xnfcalloc(sizeof(VIAEntRec), 1);
                        pVIAEnt = pPriv->ptr;
                        pVIAEnt->IsDRIEnabled = FALSE;
                        pVIAEnt->BypassSecondary = FALSE;
                        pVIAEnt->HasSecondary = FALSE;
                        pVIAEnt->IsSecondaryRestored = FALSE;
                    }
                }
                instance++;

            xfree(pEnt);
        }
    }

    xfree(usedChips);

    return foundScreen;

} /* VIAProbe */


static int LookupChipID(PciChipsets* pset, int ChipID)
{
    while (pset->numChipset >= 0)
    {
        if (pset->PCIid == ChipID)
            return pset->numChipset;

        pset++;
    }

    return -1;

} /* LookupChipID */


static unsigned int
VIAddc1Read(ScrnInfoPtr pScrn)
{
    register vgaHWPtr hwp = VGAHWPTR(pScrn);
    VIAPtr pVia = VIAPTR(pScrn);
    register CARD8 tmp;

    while (hwp->readST01(hwp)&0x8) {};
    while (!(hwp->readST01(hwp)&0x8)) {};

    VGAOUT8(0x3c4, 0x26);
    tmp = VGAIN8(0x3c5);
    return ((unsigned int) ((tmp & 0x08) >> 3));
}



#if XORG_VERSION_CURRENT >= XF86_VERSION_NUMERIC(6,9,0,0,0)

void
VIAvgaHWddc1SetSpeed(ScrnInfoPtr pScrn, xf86ddcSpeed speed)
{
    vgaHWPtr hwp = VGAHWPTR(pScrn);
    unsigned char tmp;
    struct _vgaDdcSave* save;
    switch (speed) {

    case DDC_FAST:
        if (hwp->ddc != NULL) 
            break;
        hwp->ddc = xnfcalloc(sizeof(struct _vgaDdcSave),1);
        save = (struct _vgaDdcSave *)hwp->ddc;
        /* Lightpen register disable - allow access to cr10 & 11; just in case */
        save->cr03 = hwp->readCrtc(hwp, 0x03); 
        hwp->writeCrtc(hwp,0x03,(save->cr03 |0x80));
        save->cr12 = hwp->readCrtc(hwp, 0x12); 
        hwp->writeCrtc(hwp,0x12,DISPLAY_END);
        save->cr15 = hwp->readCrtc(hwp, 0x15); 
        hwp->writeCrtc(hwp,0x15,BLANK_START);
        save->cr10 = hwp->readCrtc(hwp, 0x10); 
        hwp->writeCrtc(hwp,0x10,SYNC_START);
        save->cr11 = hwp->readCrtc(hwp, 0x11); 
        /* unprotect group 1 registers; just in case ...*/
        hwp->writeCrtc(hwp,0x11,((save->cr11 & 0x70) | SYNC_END));
        save->cr16 = hwp->readCrtc(hwp, 0x16); 
        hwp->writeCrtc(hwp,0x16,BLANK_END);
        save->cr06 = hwp->readCrtc(hwp, 0x06); 
        hwp->writeCrtc(hwp,0x06,V_TOTAL);
        /* all values have less than 8 bit - mask out 9th and 10th bits */
        save->cr09 = hwp->readCrtc(hwp, 0x09); 
        hwp->writeCrtc(hwp,0x09,(save->cr09 &0xDF));
        save->cr07 = hwp->readCrtc(hwp, 0x07); 
        hwp->writeCrtc(hwp,0x07,(save->cr07 &0x10));
        /* vsync polarity negativ & ensure a 25MHz clock */
        save->msr = hwp->readMiscOut(hwp); 
        hwp->writeMiscOut(hwp,((save->msr & 0xF3) | 0x80));
        break;
    case DDC_SLOW:
        if (hwp->ddc == NULL) 
            break;
        save = (struct _vgaDdcSave *)hwp->ddc;
        hwp->writeMiscOut(hwp,save->msr);
        hwp->writeCrtc(hwp,0x07,save->cr07);
        tmp = hwp->readCrtc(hwp, 0x09);
        hwp->writeCrtc(hwp,0x09,((save->cr09 & 0x20) | (tmp & 0xDF)));
        hwp->writeCrtc(hwp,0x06,save->cr06);
        hwp->writeCrtc(hwp,0x16,save->cr16);
        hwp->writeCrtc(hwp,0x11,save->cr11);
        hwp->writeCrtc(hwp,0x10,save->cr10);
        hwp->writeCrtc(hwp,0x15,save->cr15);
        hwp->writeCrtc(hwp,0x12,save->cr12);
        hwp->writeCrtc(hwp,0x03,save->cr03);
        xfree(save);
        hwp->ddc = NULL;
        break;
    default:
        break;
    }
}
#else
    #define VIAvgaHWddc1SetSpeed vgaHWddc1SetSpeed
#endif

#else 
    #define VIAvgaHWddc1SetSpeed vgaHWddc1SetSpeed
#endif



static Bool
VIAddc1(int scrnIndex)
{
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
    VIAPtr pVia = VIAPTR(pScrn);
    xf86MonPtr pMon;
    CARD8 tmp;
    Bool success = FALSE;

    /* initialize chipset */
    VGAOUT8(0x3c4, 0x26);
    tmp = VGAIN8(0x3c5);
    VGAOUT8(0x3c4, 0x26);
    VGAOUT8(0x3c5, (tmp | 0x11));

    if ((pMon = xf86PrintEDID(
        xf86DoEDID_DDC1(scrnIndex,VIAvgaHWddc1SetSpeed,VIAddc1Read))) != NULL)
        success = TRUE;
    xf86SetDDCproperties(pScrn,pMon);

    /* undo initialization */
    VGAOUT8(0x3c4, 0x26);
    VGAOUT8(0x3c5, tmp);
    return success;
}

static Bool
VIAddc2(int scrnIndex)
{
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
    VIAPtr pVia = VIAPTR(pScrn);
    xf86MonPtr pMon;
    CARD8 tmp;
    Bool success = FALSE;

    VGAOUT8(0x3c4, 0x26);
    tmp = VGAIN8(0x3c5);
    pMon = xf86DoEDID_DDC2(pScrn->scrnIndex, pVia->I2C_Port1);
    if (pMon)
        success = TRUE;
    pVia->DDC1 =  pMon;
    xf86PrintEDID(pMon);
    xf86SetDDCproperties(pScrn, pMon);
    VGAOUT8(0x3c4, 0x26);
    VGAOUT8(0x3c5, tmp);

    return success;
}

static void
VIAProbeDDC(ScrnInfoPtr pScrn, int index)
{
    vbeInfoPtr pVbe;

    if (xf86LoadSubModule(pScrn, "vbe"))
    {
        if (via_xf86_version_current < XF86_VERSION_NUMERIC(4,2,99,0,0))
            pVbe = VBEInit(NULL,index);
        else
            pVbe = VBEExtendedInit(NULL,index,0);

        ConfiguredMonitor = vbeDoEDID(pVbe, NULL);
    }
}

void VIACheckIfUseDuoView(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVia->pVidData->VIAGraphicInfo;
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIACheckIfUseDuoView\n"));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "ActiveDev: %d\n", pBIOSInfo->ActiveDevice));

    pBIOSInfo->DuoView = TRUE;

    if (xf86ReturnOptValBool(VIAOptions, OPTION_SIMULTANEOUS, FALSE))
    {
        pBIOSInfo->DuoView = FALSE;
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: Simultaneous support!!\n"));
    }
    
    /* Check device number, we only support duo view when enable 2 device. */
    /* LCD + LCD2 case only work on IGA2, so disable "Douview" flag */
    if ((pBIOSInfo->ActiveDevice == VIA_DEVICE_CRT1) ||
        (pBIOSInfo->ActiveDevice == VIA_DEVICE_LCD)  ||
        (pBIOSInfo->ActiveDevice == (VIA_DEVICE_LCD+VIA_DEVICE_LCD2)) ||
        (pBIOSInfo->ActiveDevice == VIA_DEVICE_DFP)  ||
        (pBIOSInfo->ActiveDevice == VIA_DEVICE_CRT2))
    {
        pBIOSInfo->DuoView = FALSE;
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Not support DuoView because only enable one device.\n"));
    }

    if (pVia->HasSecondary)
    {
        pBIOSInfo->DuoView = FALSE;
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Not support DuoView because SAMM is on.\n"));
    }

    if (pBIOSInfo->DuoView)
    {
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "DuoView support!!\n"));
        lpVIAGraphicInfo->duoview |= DUOVIEW_ENABLED;
    }
    else
    {
        lpVIAGraphicInfo->duoview &= ~DUOVIEW_ENABLED;
    }        
}

void VIACheckCursorTypeToUse(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVia->pVidData->VIAGraphicInfo;

    if ((pVia->shadowFB)        ||
        (pVia->ForceSWCursor)   ||
        (pBIOSInfo->TwoLCD)     ||
        (pBIOSInfo->MergedFB)   ||
        (pVia->IsHWRotateEnabled) ||
        (pVia->DynamicRotation))
    {
        /* Use SW Cursor */
        pVia->pScreensPublicInfo->UseHwCursor = FALSE;
    }
    else
    {
        if(pBIOSInfo->DuoView)
        {
            switch(pBIOSInfo->Chipset )
            {
                case VIA_CX700:
                case VIA_CN750:
                case VIA_P4M890:
                case VIA_P4M900:
                case VIA_VX800:
                    pVia->pScreensPublicInfo->UseHwCursor = TRUE;
                    break;
                default:
                    pVia->pScreensPublicInfo->UseHwCursor = FALSE;
            }
        }
        else
        {
            if((pBIOSInfo->ActiveDevice&VIA_DEVICE_LCD) &&
                (pBIOSInfo->Chipset == VIA_CLE266)) 
            {
                pVia->pScreensPublicInfo->UseHwCursor = FALSE;
            }
            else
            {
                /* Use HW Cursor */
                pVia->pScreensPublicInfo->UseHwCursor = TRUE;
            }
        }
    }

#ifdef RANDRV12_TEST_ENABLE
    pVia->pScreensPublicInfo->UseHwCursor = FALSE;
#endif

    lpVIAGraphicInfo->IsHWCursorEnabled = pVia->pScreensPublicInfo->UseHwCursor;
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Using %s cursor\n", pVia->pScreensPublicInfo->UseHwCursor ? "HW" : "SW"));
}



static int DeviceCounter = 0;

/* For Hotkey Timer Use. */
CARD32 VIACheckHotkeyStatus(ScrnInfoPtr pScrn)
{            
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;           
    CARD16  deviceCount = 0;        
    CARD16  i;
    CARD16  deviceSeq[4];
    
    /* Check the change of register CR3C[0]. */
    if (read_reg(VIACR, CR3C) & BIT0)
    {
        /* Clear */                
        write_reg_mask(CR3C, VIACR, 0x00, BIT0);                         

        /* To count how many prefered and valid devices can be actived.*/
        for (i=0; i<4; i++)
        {
            if (pVia->deviceSequence[i])
            {
                if ((pVia->deviceSequence[i] == VIA_DEVICE_CRT1) &&
                    (VIASenseCRT(pScrn)))
                {
                    deviceSeq[deviceCount] = pVia->deviceSequence[i];
                    deviceCount++;                                        
                }
                
                if ((pVia->deviceSequence[i] == VIA_DEVICE_DFP) &&
                    (VIASenseDVI(pBIOSInfo)))
                {
                    deviceSeq[deviceCount] = pVia->deviceSequence[i];
                    deviceCount++;                      
                }
                
                if (pVia->deviceSequence[i] == VIA_DEVICE_LCD)
                {
                    deviceSeq[deviceCount] = pVia->deviceSequence[i];
                    deviceCount++;                      
                }
            }
        }                
        
        /* Only one device was conneted. */                                
        if (deviceCount == 1)
        {
            DeviceCounter = 0;
        }
        else if (deviceCount == 2) /* 2 devices were input. ex: CRT, LCD */
        {
            if (pBIOSInfo->ActiveDevice == deviceSeq[0])
            {
                DeviceCounter = 0;
            }
            else if (pBIOSInfo->ActiveDevice == deviceSeq[1])
            {
                DeviceCounter = 1;
            }
            else if (pBIOSInfo->ActiveDevice == (deviceSeq[0] | deviceSeq[1]))
            {
                DeviceCounter = 2;            
            }
            else
            {
                DeviceCounter = 0;
            }
        }
        else if (deviceCount == 3) /* 3 devices were input. ex: CRT, LCD, DVI */
        {
            /* Configure current state */        
            if (pBIOSInfo->ActiveDevice == deviceSeq[0])
            {
                DeviceCounter = 0;
            }
            else if (pBIOSInfo->ActiveDevice == deviceSeq[1])
            {
                DeviceCounter = 1;
            }
            else if (pBIOSInfo->ActiveDevice == deviceSeq[2])
            {
                DeviceCounter = 2;
            }
            else if (pBIOSInfo->ActiveDevice == (deviceSeq[0] | deviceSeq[1]))
            {
                DeviceCounter = 3;            
            }
            else if (pBIOSInfo->ActiveDevice == (deviceSeq[0] | deviceSeq[2]))
            {
                DeviceCounter = 4;            
            }
            else if (pBIOSInfo->ActiveDevice == (deviceSeq[1] | deviceSeq[2]))
            {
                DeviceCounter = 5;
            }
            else
            {
                DeviceCounter = 0;
            }
        }
        else if (deviceCount == 4) /* 4 devices were input. ex: CRT, LCD, DVI */
        {
            /* Configure current state */        
            if (pBIOSInfo->ActiveDevice == deviceSeq[0])
            {
                DeviceCounter = 0;
            }
            else if (pBIOSInfo->ActiveDevice == deviceSeq[1])
            {
                DeviceCounter = 1;
            }
            else if (pBIOSInfo->ActiveDevice == deviceSeq[2])
            {
                DeviceCounter = 2;
            }
            else if (pBIOSInfo->ActiveDevice == deviceSeq[3])
            {
                DeviceCounter = 3;
            }
            else if (pBIOSInfo->ActiveDevice == (deviceSeq[0] | deviceSeq[1]))
            {
                DeviceCounter = 4;            
            }
            else if (pBIOSInfo->ActiveDevice == (deviceSeq[0] | deviceSeq[2]))
            {
                DeviceCounter = 5;            
            }
            else if (pBIOSInfo->ActiveDevice == (deviceSeq[0] | deviceSeq[3]))
            {
                DeviceCounter = 6;
            }
            else if (pBIOSInfo->ActiveDevice == (deviceSeq[1] | deviceSeq[2]))
            {
                DeviceCounter = 7;
            }
            else if (pBIOSInfo->ActiveDevice == (deviceSeq[1] | deviceSeq[3]))
            {
                DeviceCounter = 8;
            }
            else if (pBIOSInfo->ActiveDevice == (deviceSeq[2] | deviceSeq[3]))
            {
                DeviceCounter = 9;
            }            
            else
            {
                DeviceCounter = 0;
            }
        }               
                                                  
        
        /* Loop to switch device */
        DeviceCounter++;        

        if (deviceCount == 1)
        {
            DeviceCounter = 0;   
            pBIOSInfo->ActiveDevice = deviceSeq[0];
        }
        else if (deviceCount == 2)
        {
            DeviceCounter = DeviceCounter%3;   
            switch(DeviceCounter)
            {
                case 0:                
                    pBIOSInfo->ActiveDevice = deviceSeq[0];
                    break;                    
                case 1:                
                    pBIOSInfo->ActiveDevice = deviceSeq[1];
                    break;                    
                case 2:                
                    pBIOSInfo->ActiveDevice = deviceSeq[0] | deviceSeq[1];
                    break;                       
            } 
        }
        else if (deviceCount == 3)
        {
            DeviceCounter = DeviceCounter%6; 
            switch(DeviceCounter)
            {
                case 0:                
                    pBIOSInfo->ActiveDevice = deviceSeq[0];
                    break;
                    
                case 1:                
                    pBIOSInfo->ActiveDevice = deviceSeq[1];
                    break;
                    
                case 2:                
                    pBIOSInfo->ActiveDevice = deviceSeq[2];
                    break;    

                case 3:                
                    pBIOSInfo->ActiveDevice = deviceSeq[0] | deviceSeq[1];
                    break;    

                case 4:                
                    pBIOSInfo->ActiveDevice = deviceSeq[0] | deviceSeq[2];
                    break;    

                case 5:                
                    pBIOSInfo->ActiveDevice = deviceSeq[1] | deviceSeq[2];
                    break;                
            } 
        }    
        else if (deviceCount == 4)
        {
            DeviceCounter = DeviceCounter%10;    
            switch(DeviceCounter)
            {
                case 0:                
                    pBIOSInfo->ActiveDevice = deviceSeq[0];
                    break;                    
                case 1:                
                    pBIOSInfo->ActiveDevice = deviceSeq[1];
                    break;                    
                case 2:                
                    pBIOSInfo->ActiveDevice = deviceSeq[2];
                    break;   
                case 3:                
                    pBIOSInfo->ActiveDevice = deviceSeq[3];
                    break;
                case 4:                
                    pBIOSInfo->ActiveDevice = deviceSeq[0] | deviceSeq[1];
                    break;    
                case 5:                
                    pBIOSInfo->ActiveDevice = deviceSeq[0] | deviceSeq[2];
                    break;    
                case 6:                
                    pBIOSInfo->ActiveDevice = deviceSeq[0] | deviceSeq[3];
                    break;   
                case 7:                
                    pBIOSInfo->ActiveDevice = deviceSeq[1] | deviceSeq[2];
                    break;  
                case 8:                
                    pBIOSInfo->ActiveDevice = deviceSeq[1] | deviceSeq[3];
                    break;  
                case 9:                
                    pBIOSInfo->ActiveDevice = deviceSeq[2] | deviceSeq[3];
                    break;  
            } 
        } 
        else
        {
            pBIOSInfo->ActiveDevice = VIA_DEVICE_LCD;
        }
                
        /* If there is no any device connected. Use the first device as default */
        if (!pBIOSInfo->ActiveDevice)
        {
            pBIOSInfo->ActiveDevice = deviceSeq[0];
        }
        VIASwitchMode(pScrn->scrnIndex, pScrn->currentMode, 0);
               
    }        

    return 1000;
}


CARD32 VIASyncTimer(OsTimerPtr timer, CARD32 now, pointer arg)
{        
    ScrnInfoPtr pScrn = (ScrnInfoPtr) arg;
    VIAPtr pVia = VIAPTR(pScrn);

    if (pVia->IsHotkeyTimerEnabled)
    {
        VIACheckHotkeyStatus(pScrn);
    }

    return 30;
}


static void VIAGetOptions_MergedFB(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    char        *strptr;
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
	LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVia->pVidData->VIAGraphicInfo;
   
    /* collect MergedFB options */
    pBIOSInfo->MergedFB = FALSE;
    pBIOSInfo->UseVIAXinerama = TRUE;
    pBIOSInfo->Scrn2Position = viaRightOf;
    
    if (xf86ReturnOptValBool(VIAOptions, OPTION_MERGEDFB, FALSE))
    {  
        pBIOSInfo->MergedFB = TRUE;
        pBIOSInfo->DuoView = TRUE;
        lpVIAGraphicInfo->duoview = 1;
        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "MergedFB mode forced on.\n");
    }
    else
    {
        pBIOSInfo->MergedFB = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "MergedFB mode forced off.\n");
    }
       
    /* Do some MergedFB mode initialization */
   
    if(pBIOSInfo->MergedFB)
    {
        strptr = (char *)xf86GetOptValString(VIAOptions, OPTION_SCRN2POS);
        if(strptr)
        {
            if((!strcmp(strptr,"LeftOf")) || (!strcmp(strptr,"leftof")))
            {
                pBIOSInfo->Scrn2Position = viaLeftOf;
            }
            else if((!strcmp(strptr,"RightOf")) || (!strcmp(strptr,"rightof")))
            {
                pBIOSInfo->Scrn2Position = viaRightOf;
            }
            else if((!strcmp(strptr,"Above")) || (!strcmp(strptr,"above")))
            {
                pBIOSInfo->Scrn2Position = viaAbove;
            }
            else if((!strcmp(strptr,"Below")) || (!strcmp(strptr,"below")))
            {
                pBIOSInfo->Scrn2Position = viaBelow;
            }
            else
            {
	            xf86DrvMsg(pScrn->scrnIndex, X_WARNING,
	                "\"%s\" is not a valid parameter for Option \"Scrn2Position\"\n", strptr);
	    	    xf86DrvMsg(pScrn->scrnIndex, X_INFO,
	                "Valid parameters are \"RightOf\", \"LeftOf\", \"Above\", or\"Below\"\n");
            }
        }

        /*don't use pseudo xinerama,but virtual screen*/
        if (xf86ReturnOptValBool(VIAOptions, OPTION_NOVIAXINERAMA, FALSE))
        {  
            pBIOSInfo->UseVIAXinerama = FALSE;
        }
    }
}

static void VIAGetOptions_Cursor(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    /* The default if neither is specified is HW. */
    pVia->ForceSWCursor = FALSE;
    
    if (xf86ReturnOptValBool(VIAOptions, OPTION_SWCURSOR, FALSE))
    {
        pVia->ForceSWCursor = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option SWCursor enabled.\n");
    }
 
    /* cle266 only use SW cursor */
    if (pBIOSInfo->Chipset==VIA_CLE266)
    {
         pVia->ForceSWCursor = TRUE;
    }
}

static void VIAGetOptions_ActiveDevice(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    char    *pStr = NULL;
    
    pBIOSInfo->TwoLCD = FALSE;

    /*=* For Advantech Project, support two LCD MHS *=*/
    if (xf86ReturnOptValBool(VIAOptions, OPTION_TWO_LCD, FALSE))
    {
        pBIOSInfo->TwoLCD = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: Two LCD support!!\n");
    }    
    
    if(pBIOSInfo->TwoLCD)
    {
        pBIOSInfo->ActiveDevice = VIA_DEVICE_LCD2 | VIA_DEVICE_LCD;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Active Device is LCD1 and LCD2.\n");
    }    
    else
    {
        /* ActiveDevice Option for device selection */
        pBIOSInfo->ActiveDevice = 0x00;
        
        if ((pStr = xf86GetOptValString(VIAOptions, OPTION_ACTIVEDEVICE)))
        {
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Active Device is %s.\n", pStr);
            
            if(!xf86NameCmp(pStr, "CRT,LCD"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1 | VIA_DEVICE_LCD;
                pBIOSInfo->PrimaryDevice = VIA_DEVICE_CRT1;
            }
            else if(!xf86NameCmp(pStr, "LCD,CRT"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1 | VIA_DEVICE_LCD;
                pBIOSInfo->PrimaryDevice = VIA_DEVICE_LCD;
            }
            else if(!xf86NameCmp(pStr, "LCD,LCD2"))
            {
                if (pVia->Chipset == VIA_CX700)
                {
                    pBIOSInfo->ActiveDevice = VIA_DEVICE_LCD | VIA_DEVICE_LCD2;
                    pBIOSInfo->PrimaryDevice = VIA_DEVICE_LCD;
                }
                else
                {
                    pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1 | VIA_DEVICE_LCD;
                    pBIOSInfo->PrimaryDevice = VIA_DEVICE_CRT1;
                    pBIOSInfo->TwoLCD = TRUE;
                }
            }
            else if(!xf86NameCmp(pStr, "LCD2,LCD"))
            {
                if (pVia->Chipset == VIA_CX700)
                {
                    pBIOSInfo->ActiveDevice = VIA_DEVICE_LCD | VIA_DEVICE_LCD2;
                    pBIOSInfo->PrimaryDevice = VIA_DEVICE_LCD2;
                }
                else
                {
                    pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1 | VIA_DEVICE_LCD;
                    pBIOSInfo->PrimaryDevice = VIA_DEVICE_LCD;
                    pBIOSInfo->TwoLCD = TRUE;
                    pBIOSInfo->UseIGA2 = TRUE;
                }
            }
            else if(!xf86NameCmp(pStr, "CRT,DFP") ||!xf86NameCmp(pStr, "CRT,DVI"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1 | VIA_DEVICE_DFP;
                pBIOSInfo->PrimaryDevice = VIA_DEVICE_CRT1;
            }
            else if(!xf86NameCmp(pStr, "DFP,CRT") || !xf86NameCmp(pStr, "DVI,CRT"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1 | VIA_DEVICE_DFP;
                pBIOSInfo->PrimaryDevice = VIA_DEVICE_DFP;
            }
            else if(!xf86NameCmp(pStr, "DFP,LCD") || !xf86NameCmp(pStr, "DVI,LCD"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_DFP | VIA_DEVICE_LCD;
                pBIOSInfo->PrimaryDevice = VIA_DEVICE_DFP;
            }
            else if(!xf86NameCmp(pStr, "LCD,DFP") || !xf86NameCmp(pStr, "LCD,DVI"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_DFP | VIA_DEVICE_LCD;
                pBIOSInfo->PrimaryDevice = VIA_DEVICE_LCD;
            }
            else if(!xf86NameCmp(pStr, "CRT") || !xf86NameCmp(pStr, "CRT ONLY"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1;
            }
            else if(!xf86NameCmp(pStr, "LCD") || !xf86NameCmp(pStr, "LCD ONLY"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_LCD;
            }
            else if(!xf86NameCmp(pStr, "DFP") || !xf86NameCmp(pStr, "DFP ONLY") ||
                    !xf86NameCmp(pStr, "DVI") || !xf86NameCmp(pStr, "DVI ONLY"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_DFP;
            }            
            else 
            {
                xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                    "Option \"%s\" can't recognize!, Active Device by default.\n", pStr);
            }
        }
    }

    if(pBIOSInfo->TwoLCD)
    {
        pBIOSInfo->OptRefresh = 60;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Refresh is 60.\n");
    }
    else
    {
        pBIOSInfo->OptRefresh = 0;
        if (xf86GetOptValInteger(VIAOptions, OPTION_REFRESH,
                                 &(pBIOSInfo->OptRefresh)))
        {
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Refresh is %d\n",
                       pBIOSInfo->OptRefresh);
        }
    }

    /* Initialize CRT2DIport value even if not support two crt. */
    pBIOSInfo->CRTDIPort = VIA_DI_NONE;
}

static void VIAGetOptions_DisplayHardwareLayout(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    char    *pStr = NULL;
    
    /* Because we can't get any information about hardware layout */
    /* from BIOS, so add option DisplayHardwareLayout to determine it. */
    pBIOSInfo->DisplayHardwareLayout = HW_LAYOUT_LCD_DVI;
    
    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_DISPLAY_HARDWARE_LAYOUT)))
    {
        if (!xf86NameCmp(pStr, "LCD") || !xf86NameCmp(pStr, "LCD ONLY"))
        {
            pBIOSInfo->DisplayHardwareLayout = HW_LAYOUT_LCD_ONLY;
        }
        else if (!xf86NameCmp(pStr, "DVI") || !xf86NameCmp(pStr, "DVI ONLY"))
        {
            pBIOSInfo->DisplayHardwareLayout = HW_LAYOUT_DVI_ONLY;
        }
        else if (!xf86NameCmp(pStr, "LCD+LCD2") || !xf86NameCmp(pStr, "LCD2+LCD"))
        {
            pBIOSInfo->DisplayHardwareLayout = HW_LAYOUT_LCD_LCD2;
        }
        else if (!xf86NameCmp(pStr, "DVI+LCD") || !xf86NameCmp(pStr, "LCD+DVI"))
        {
            pBIOSInfo->DisplayHardwareLayout = HW_LAYOUT_LCD_DVI;
        }
        else if (!xf86NameCmp(pStr, "LCD+ExternalLCD2") || !xf86NameCmp(pStr, "ExternalLCD2+LCD"))
        {
            pBIOSInfo->DisplayHardwareLayout = HW_LAYOUT_LCD_EXTERNAL_LCD2;
        }
        else if (!xf86NameCmp(pStr, "TTLLCD"))
        {
            pBIOSInfo->DisplayHardwareLayout = HW_LAYOUT_TTL_LCD;
        }
        
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Display Hardware Layout is %s\n", pStr);
    }
}


int TranslateDIPortNameToID(char *pStr)
{
    if (!xf86NameCmp(pStr, "DFP_HIGH"))
    {
        return VIA_DI_DFPHIGH;
    }
    else if (!xf86NameCmp(pStr, "DFP_LOW"))
    {
        return VIA_DI_DFPLOW;
    }
    else if (!xf86NameCmp(pStr, "DFP_HIGHLOW"))
    {
        return VIA_DI_DFPHIGHLOW;
    }
    else if (!xf86NameCmp(pStr, "DVP0"))
    {
        return VIA_DI_DVP0;
    }
    else if (!xf86NameCmp(pStr, "DVP1"))
    {
        return VIA_DI_DVP1;
    }
    else
    {
        /* Invalid port. */
        return VIA_DI_NONE;
    }
}


static void VIAGetOptions_LCDInfo(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;    
    int option_panel_id;
    char    *pStr = NULL;
    
    /* Initialize LCD panel size: */
    pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL_INVALID;
    
    if (xf86ReturnOptValBool(VIAOptions, OPTION_FORCE_LCD, FALSE))
    {
        pBIOSInfo->MobileFamily = TRUE;        
        pBIOSInfo->LVDSSettingInfo.ChipID = VIA_Hardwired;
        
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "(OPTION) Force enable LCD. \n");
    }    

    /* LCD MSB/LSB Option */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_LCD_MSB_ENABLE, FALSE))
    {
        pBIOSInfo->LVDSSettingInfo.MsbEnable=TRUE;
    }
    else
    {
        pBIOSInfo->LVDSSettingInfo.MsbEnable=FALSE;
    }
    
    /* LCD Center/Expand Option */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_CENTER, FALSE))
    {
        pBIOSInfo->LVDSSettingInfo.Center = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "(OPTION) LCD Center is On\n");
    }
    else
    {
        pBIOSInfo->LVDSSettingInfo.Center = FALSE;
    }

    /* Set LCD Panel Info. */
    if (xf86GetOptValInteger(VIAOptions, OPTION_LCDPANELID, &option_panel_id))
    {
        VIASetLCDPanelInfo(pBIOSInfo, option_panel_id);
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "(OPTION) LCD Panel ID is %d.\n", option_panel_id);
    }

    /* Bus Width Option */
    pBIOSInfo->LVDSSettingInfo.BusWidth = VIA_DI_12BIT;

    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_LCD_BUS_WIDTH)))
    {
        if (!xf86NameCmp(pStr, "12BIT"))
        {
            pBIOSInfo->LVDSSettingInfo.BusWidth = VIA_DI_12BIT;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "LCD Bus Width is 12BIT\n");
        }
        else if (!xf86NameCmp(pStr, "24BIT"))
        {
            pBIOSInfo->LVDSSettingInfo.BusWidth = VIA_DI_24BIT;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "LCD Bus Width is 24BIT\n");
        }
    }

    /* Support LCD port selection */
    pBIOSInfo->LVDSSettingInfo.DIPort = VIA_DI_NONE;
    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_LCD_PORT)))
    {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "LCD port set to %s.\n", pStr);
        pBIOSInfo->LVDSSettingInfo.DIPort = TranslateDIPortNameToID(pStr);        
    }
}


static void VIAGetOptions_DVIInfo(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    char    *pStr = NULL;

    /* Initialize DVI panel size: */
    pBIOSInfo->TMDSSettingInfo.DFPSize = M_INVALID;

    /* DVI on EPIA: */
    pBIOSInfo->EPIA_DVI = FALSE;
    if (xf86ReturnOptValBool(VIAOptions, OPTION_EPIA_DVI, FALSE))
    {
         pBIOSInfo->EPIA_DVI = TRUE;
         xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: support DVI on EPIA!!\n");
    }

    /* Support DVI port selection */
    pBIOSInfo->IsDemoBoard = TRUE;
    pBIOSInfo->DFPDIPort = VIA_DI_NONE;
    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_DVI_PORT)))
    {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "DVI port set to %s.\n", pStr);
        
        if (!xf86NameCmp(pStr, "DFP_HIGH"))
        {
            pBIOSInfo->IsDemoBoard = FALSE;
            pBIOSInfo->DFPDIPort = VIA_DI_DFPHIGH;
        }
        else if (!xf86NameCmp(pStr, "DFP_LOW"))
        {
            pBIOSInfo->DFPDIPort = VIA_DI_DFPLOW;
        }
        else if (!xf86NameCmp(pStr, "DVP0"))
        {
            pBIOSInfo->DFPDIPort = VIA_DI_DVP0;
        }
        else if (!xf86NameCmp(pStr, "DVP1"))
        {
            pBIOSInfo->DFPDIPort = VIA_DI_DVP1;
        }
    }

    /* DVI Dual Edge: */
    pBIOSInfo->TMDSSettingInfo.IsDualEdge = FALSE;

    if (xf86ReturnOptValBool(VIAOptions, OPTION_DVI_DUAL_EDGE, FALSE))
    {
        pBIOSInfo->TMDSSettingInfo.IsDualEdge = TRUE;
    }

    /* Bus Width Option */
    pBIOSInfo->TMDSSettingInfo.BusWidth = VIA_DI_12BIT;

    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_DVI_BUS_WIDTH)))
    {
        if (!xf86NameCmp(pStr, "12BIT"))
        {
            pBIOSInfo->TMDSSettingInfo.BusWidth = VIA_DI_12BIT;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "DVI Bus Width is 12BIT\n");
        }
        else if (!xf86NameCmp(pStr, "24BIT"))
        {
            pBIOSInfo->TMDSSettingInfo.BusWidth = VIA_DI_24BIT;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "DVI Bus Width is 24BIT\n");
        }
    }
}


static void VIAParseDPAValues(char *pStr, int *pReturnedBuffer, int BufSize)
{
    char *pToken;
    int i = 0;

    pToken = strtok(pStr, ",");
    
    if (pToken)
    {
        *pReturnedBuffer = atoi(pToken);
        pReturnedBuffer++;
        i++;
        
    }

    for (; i < BufSize; i++)
    {
        pToken = strtok('\0', ",");

        if (pToken)
        {
            *pReturnedBuffer = atoi(pToken);
            pReturnedBuffer++;
        }
    }
}

static void VIAGetOptions_DPASettingInfo(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    char    *pStr = NULL;
    int     TmpBuffer[3];

    pBIOSInfo->UserDPASettingInfo.HasUserSetting_Gfx = FALSE;

    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_DPA_SETTING_DVP0)))
    {
        /* The data we get should be "v1,v2,v3", they are for */
        /* "Clock Polarity/Adjust,Data Driving,Clock Driving". */
        VIAParseDPAValues(pStr, TmpBuffer, 3);
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "DVP0 DPA=%d, DataDriving=%d, ClockDriving=%d.\n", TmpBuffer[0], TmpBuffer[1], TmpBuffer[2]);
        
        pBIOSInfo->UserDPASettingInfo.HasUserSetting_Gfx = TRUE;
        pBIOSInfo->UserDPASettingInfo.GfxDPA.DVP0 = TmpBuffer[0] & 0x0F;
        pBIOSInfo->UserDPASettingInfo.GfxDPA.DVP0DataDri_S = (TmpBuffer[1] & BIT0) << 1;     /* for SR1B[1] */
        pBIOSInfo->UserDPASettingInfo.GfxDPA.DVP0DataDri_S1 = (TmpBuffer[1] & BIT1) << 4;    /* for SR2A[5] */
        pBIOSInfo->UserDPASettingInfo.GfxDPA.DVP0ClockDri_S = (TmpBuffer[2] & BIT0) << 2;    /* for SR1E[2] */
        pBIOSInfo->UserDPASettingInfo.GfxDPA.DVP0ClockDri_S1 = (TmpBuffer[2] & BIT1) << 3;   /* for SR2A[4] */
    }

    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_DPA_SETTING_DVP1)))
    {
        /* The data we get should be "v1,v2,v3", they are for */
        /* "Clock Polarity/Adjust,Data Driving,Clock Driving". */
        VIAParseDPAValues(pStr, TmpBuffer, 3);
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "DVP1 DPA=%d, DataDriving=%d, ClockDriving=%d.\n", TmpBuffer[0], TmpBuffer[1], TmpBuffer[2]);

        pBIOSInfo->UserDPASettingInfo.HasUserSetting_Gfx = TRUE;
        pBIOSInfo->UserDPASettingInfo.GfxDPA.DVP1 = TmpBuffer[0] & 0x0F;
        pBIOSInfo->UserDPASettingInfo.GfxDPA.DVP1Driving = ((TmpBuffer[2] & 0x03) << 2) | (TmpBuffer[1] & 0x03);
    }

    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_DPA_SETTING_DFPHIGH)))
    {
        /* The data we get is for "Clock Polarity/Adjust" */
        VIAParseDPAValues(pStr, TmpBuffer, 1);

        pBIOSInfo->UserDPASettingInfo.HasUserSetting_Gfx = TRUE;
        pBIOSInfo->UserDPASettingInfo.GfxDPA.DFPHigh = TmpBuffer[0] & 0x0F;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "DFPHIGH DPA = %d.\n", pBIOSInfo->UserDPASettingInfo.GfxDPA.DFPHigh);
    }

    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_DPA_SETTING_DFPLOW)))
    {
        /* The data we get is for "Clock Polarity/Adjust" */
        VIAParseDPAValues(pStr, TmpBuffer, 1);

        pBIOSInfo->UserDPASettingInfo.HasUserSetting_Gfx = TRUE;
        pBIOSInfo->UserDPASettingInfo.GfxDPA.DFPLow = TmpBuffer[0] & 0x0F;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "DFPLOW DPA = %d.\n", pBIOSInfo->UserDPASettingInfo.GfxDPA.DFPLow);
    }

    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_DPA_SETTING_VT1636)))
    {
        /* The data we get should be "v1,v2", they are for "CLK_SEL_ST1, CLK_SEL_ST2" */
        VIAParseDPAValues(pStr, TmpBuffer, 2);

        pBIOSInfo->UserDPASettingInfo.HasUserSetting_VT1636 = TRUE;        
        pBIOSInfo->UserDPASettingInfo.VT1636DPA.CLK_SEL_ST1 = TmpBuffer[0] & 0x1F;
        pBIOSInfo->UserDPASettingInfo.VT1636DPA.CLK_SEL_ST2 = TmpBuffer[1] & 0x0F;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "VT1636 DPA: CLK_SEL_ST1=%d, CLK_SEL_ST2=%d.\n", 
        pBIOSInfo->UserDPASettingInfo.VT1636DPA.CLK_SEL_ST1,
        pBIOSInfo->UserDPASettingInfo.VT1636DPA.CLK_SEL_ST2);
    }
    else
    {
        pBIOSInfo->UserDPASettingInfo.HasUserSetting_VT1636 = FALSE;
    }

}


static void VIAGetOptions_RotateInfo(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    LPVIAGRAPHICINFO lpVIAGraphicInfo;        
    CARD32 RotateDegree = VIA_ROTATE_DEGREE_0;
    CARD32 RotateType = VIA_ROTATE_TYPE_SW;
    char* s = NULL;
    
    lpVIAGraphicInfo = &pVia->pVidData->VIAGraphicInfo;
    lpVIAGraphicInfo->duoview = lpVIAGraphicInfo->rotate = 0; /*initilize Xserver rotation status */
    lpVIAGraphicInfo->IsRotationEnabled = FALSE;
    
    /* Initilize */    
    pVia->SWRotate = VIA_ROTATE_DEGREE_0;
    pBIOSInfo->HWRotate = VIA_ROTATE_DEGREE_0;
    pVia->RandRRotation = FALSE;
    pVia->IsHWRotateEnabled = FALSE;
    
    if ((s = xf86GetOptValString(VIAOptions, OPTION_ROTATE)))
    {                    
        if (!xf86NameCmp(s, "CW"))
        {                        
            RotateDegree = VIA_ROTATE_DEGREE_90;                        
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Rotating screen clockwise.\n");
        }
        else if(!xf86NameCmp(s, "CCW"))
        {            
            RotateDegree = VIA_ROTATE_DEGREE_270;                        
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,  "Rotating screen counter-clockwise. \n");
        }
        else if(!xf86NameCmp(s, "UD"))
        {            
            RotateDegree = VIA_ROTATE_DEGREE_180;                                    
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,  "Rotating screen upside-down. \n");
        }        
        else
        {
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "\"%s\" is not a valid"
                       "value for Option \"Rotate\"\n", s);
            xf86DrvMsg(pScrn->scrnIndex, X_INFO,
                       "Valid options are \"CW\" or \"CCW\" or \"UD\" \n");
        }        
    }    
        
        
    if ((s = xf86GetOptValString(VIAOptions, OPTION_ROTATE_TYPE)))
    {        
        if (!xf86NameCmp(s, "SW"))
        {
            RotateType = VIA_ROTATE_TYPE_SW;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Use Software Rotation - Acceleration disabled\n");
        }
        else if(!xf86NameCmp(s, "HW"))
        {            
            RotateType = VIA_ROTATE_TYPE_HW;          
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Use Hardware Rotation.\n");
        }
        else if(!xf86NameCmp(s, "RandR"))
        {
            RotateType = VIA_ROTATE_TYPE_RANDR;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,  "Rotating screen"
                       "Enable RandR - acceleration disabled\n");
        }
        else if(!xf86NameCmp(s, "Dynamic"))
        {
            RotateType = VIA_ROTATE_TYPE_DYNAMIC;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,  "Rotating screen"
                       "Enable Dynamic Rotation\n");
        }
        else
        {
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "\"%s\" is not a valid"
                       "value for Option \"RotateType\"\n", s);
            xf86DrvMsg(pScrn->scrnIndex, X_INFO,
                       "Valid options are \"SW\" or \"HW\"  \n");
        }
    }


    if ((RotateDegree != VIA_ROTATE_DEGREE_0)
     || (RotateType == VIA_ROTATE_TYPE_RANDR)
     || (RotateType == VIA_ROTATE_TYPE_DYNAMIC))   
    {
        switch (RotateType)
        {
            /* accel is disabled below for shadowFB */
            case VIA_ROTATE_TYPE_SW:            
                pVia->shadowFB = TRUE;
                pVia->NoXVideo = FALSE;
                pVia->SWRotate = RotateDegree; 
                lpVIAGraphicInfo->rotate = RotateDegree;            
                break;
                
            case VIA_ROTATE_TYPE_HW:
                pVia->IsHWRotateEnabled = TRUE;
                pBIOSInfo->HWRotate = RotateDegree;
                lpVIAGraphicInfo->rotate = RotateDegree;
                break;
                
            case VIA_ROTATE_TYPE_RANDR:
                pVia->shadowFB = TRUE;
                pVia->NoXVideo = TRUE;
                pVia->NoAccel = TRUE;
                pVia->pScreensPublicInfo->UseHwCursor = FALSE;
                pVia->RandRRotation = TRUE;
                break;

            case VIA_ROTATE_TYPE_DYNAMIC:    
#if  (defined(XORG_VERSION_CURRENT))&&(XORG_VERSION_CURRENT >= XF86_VERSION_NUMERIC(7,2,0,0,0)) 
                pVia->DynamicRotation = TRUE;                             
#else
                pVia->DynamicRotation = FALSE;
#endif                
                break; 
        }

        lpVIAGraphicInfo->IsRotationEnabled = TRUE;
    }

    /* for video rotation */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_INVERSE, FALSE))
    {
        lpVIAGraphicInfo->rotate = VIA_ROTATE_DEGREE_180;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: Inverse video\n");
    }
}

static void VIAGetOptions_Video(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVia->pVidData->VIAGraphicInfo;
    int VideoColorKey;
    
    if (xf86GetOptValInteger(VIAOptions, OPTION_VIDEO_COLOR_KEY, &VideoColorKey))
    {
        lpVIAGraphicInfo->VideoColorKey = VideoColorKey;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "video color key set to 0x%lx\n", lpVIAGraphicInfo->VideoColorKey);
    } 
    else
    {
        lpVIAGraphicInfo->VideoColorKey = (1 << pScrn->offset.red) |
                                          (1 << pScrn->offset.green) |
                                          (((pScrn->mask.blue >> pScrn->offset.blue) - 1) << pScrn->offset.blue);
    }
}

static void VIAGetOptions_Capture(ScrnInfoPtr pScrn)
{   
    VIAPtr          pVia;
    VIABIOSInfoPtr  pBIOSInfo;
     LPVIAGRAPHICINFO lpVIAGraphicInfo;
    char *s = NULL;
    int iValue = 0;

    pVia = VIAPTR(pScrn);
    pBIOSInfo = pVia->pBIOSInfo;
    lpVIAGraphicInfo = &pVia->pVidData->VIAGraphicInfo;
    
    /* Set HQV filter mode */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_HQVFILTER_MANUALSELECT, FALSE)) {
        lpVIAGraphicInfo->HQVFilter_ManualSelect = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: HQV Filter Manual Select Enabled\n");
    }
    else {
        lpVIAGraphicInfo->HQVFilter_ManualSelect = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: HQV Filter Manual Select Disabled\n");
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_MPEG_HQV_VFILTER_MODE, &(lpVIAGraphicInfo->MpegHQV_VFilterMode))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "MPEG HQV vertical filter mode: %d\n", lpVIAGraphicInfo->MpegHQV_VFilterMode);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_MPEG_HQV_HFILTER_MODE, &(lpVIAGraphicInfo->MpegHQV_HFilterMode))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "MPEG HQV horizontal filter mode: %d\n", lpVIAGraphicInfo->MpegHQV_HFilterMode);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_CAP_HQV_VFILTER_MODE, &(lpVIAGraphicInfo->CapHQV_VFilterMode))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Capture HQV vertical filter mode: %d\n", lpVIAGraphicInfo->CapHQV_VFilterMode);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_CAP_HQV_HFILTER_MODE, &(lpVIAGraphicInfo->CapHQV_HFilterMode))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Capture HQV horizontal filter mode: %d\n", lpVIAGraphicInfo->CapHQV_HFilterMode);
    }

    /* Mpeg decode frame buffer number */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_SETMPEG_FBNUMBER, FALSE)) {
        lpVIAGraphicInfo->SetMpeg_FBNumber = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Set MPEG decode frame buffer number Enable\n");
    }
    else {
        lpVIAGraphicInfo->SetMpeg_FBNumber = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Set MPEG decode frame buffer number Disable\n");
    }

    if (xf86ReturnOptValBool(VIAOptions, OPTION_NOMPEGAddOneLineOnBottom, FALSE)) {
        lpVIAGraphicInfo->NoMPEGAddOneLineOnBottom = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: No mpeg add one line on bottom = %lx\n", lpVIAGraphicInfo->NoMPEGAddOneLineOnBottom);
    }
    else {
        lpVIAGraphicInfo->NoMPEGAddOneLineOnBottom = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: No mpeg add one line on bottom = %lx\n", lpVIAGraphicInfo->NoMPEGAddOneLineOnBottom);
    }

    /* for DeBlocking */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_DeBlockingEnable, FALSE))
    {
        lpVIAGraphicInfo->DeBlockingEnable = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,"Option: DeBlocking Enable!!\n");
    }
    else
    {
        lpVIAGraphicInfo->DeBlockingEnable = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,"Option: DeBlocking Disable!!\n");
    }

    if ((s = xf86GetOptValString(VIAOptions, OPTION_VideoOnDevice)))
    {/*For Option VideoOnDevice in duo view mode*/
        if(!xf86NameCmp(s, "DVI")) {
            lpVIAGraphicInfo->duoview |= DUOVIEW_VIDEOOPTION;
            lpVIAGraphicInfo->duoview |= DUOVIEW_VIDEOONDVI;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Video has been set to DVI\n");
        }
        else if(!xf86NameCmp(s, "LCD")) {
            lpVIAGraphicInfo->duoview |= DUOVIEW_VIDEOOPTION;
            lpVIAGraphicInfo->duoview |= DUOVIEW_VIDEOONLCD;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Video has been set to LCD\n");
        }
        else if(!xf86NameCmp(s, "CRT")) {
            lpVIAGraphicInfo->duoview |= DUOVIEW_VIDEOOPTION;
            lpVIAGraphicInfo->duoview |= DUOVIEW_VIDEOONCRT;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Video has been set to CRT\n");
        }
    }
    
    return ;
}

static void VIAGetOptions_general(ScrnInfoPtr pScrn)
{
    VIAPtr          pVia;
    VIABIOSInfoPtr  pBIOSInfo;
    MessageType     from = X_DEFAULT;

    pVia = VIAPTR(pScrn);
    pBIOSInfo = pVia->pBIOSInfo;    
    
    #ifdef XF86DRI
    pVia->drixinerama = FALSE;
    if (xf86IsOptionSet(VIAOptions, OPTION_DRIXINERAMA))
        pVia->drixinerama = TRUE;
#else
    if (xf86IsOptionSet(VIAOptions, OPTION_DRIXINERAMA))
    	xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
    		"Option: drixinerama ignored, no DRI support compiled into driver.\n");
#endif

    if (xf86ReturnOptValBool(VIAOptions, OPTION_PCI_BURST, FALSE)) {
        pVia->pci_burst = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: pci_burst - PCI burst read enabled\n");
    }
    else {
        pVia->pci_burst = FALSE;
    }

    pVia->NoPCIRetry = 1;       /* default */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_PCI_RETRY, FALSE)) {
        if (xf86ReturnOptValBool(VIAOptions, OPTION_PCI_BURST, FALSE)) {
            pVia->NoPCIRetry = 0;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: pci_retry\n");
        }
        else {
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                       "\"pci_retry\" option requires \"pci_burst\"\n");
        }
    }

    if (xf86IsOptionSet(VIAOptions, OPTION_SHADOW_FB))
    {
        pVia->shadowFB = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: ShadowFB %s.\n",
                   pVia->shadowFB ? "enabled" : "disabled");
    }
    else
    {
        pVia->shadowFB = FALSE;
    }

    from = X_DEFAULT;
    if (xf86ReturnOptValBool(VIAOptions, OPTION_USEBIOS, FALSE)) {
        from = X_CONFIG;
        pBIOSInfo->UseBIOS = TRUE;
    }
    else {
        pBIOSInfo->UseBIOS = FALSE;
    }
    xf86DrvMsg(pScrn->scrnIndex, from, "%ssing video BIOS to set modes\n",
               pBIOSInfo->UseBIOS ? "U" : "Not u" );

    pScrn->videoRam = 0;
    if(xf86GetOptValInteger(VIAOptions, OPTION_VIDEORAM, &pScrn->videoRam)) {
        xf86DrvMsg( pScrn->scrnIndex, X_CONFIG,
                    "Option: VideoRAM %dkB\n", pScrn->videoRam );
    }

    if (xf86ReturnOptValBool(VIAOptions, OPTION_DISABLEVQ, FALSE)) {
        pVia->VQEnable = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: DisableVQ -VQ Disabled\n");
    }
    else {
        pVia->VQEnable = TRUE;
    }
    /* Specified a number to replace the original driver version patch level. */
    pBIOSInfo->DriverVersionPatchLevel = 0;
    if(xf86GetOptValInteger(VIAOptions, OPTION_DRIVER_VERSION, &pBIOSInfo->DriverVersionPatchLevel))
    {
        xf86DrvMsg( pScrn->scrnIndex, X_CONFIG, "Option: Driver Version = %d\n", pBIOSInfo->DriverVersionPatchLevel);
    }

    
    /* Get rotate option. */
    VIAGetOptions_RotateInfo(pScrn);   

    if (xf86ReturnOptValBool(VIAOptions, OPTION_NOACCEL, FALSE))
    {
        pVia->NoAccel = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: NoAccel -Acceleration Disabled\n");
    }
    else
    {
        pVia->NoAccel = FALSE;
    }

    if (pVia->shadowFB && !pVia->NoAccel)
    {
        xf86DrvMsg(pScrn->scrnIndex, X_WARNING,
                   "HW acceleration not supported with \"shadowFB\".\n");
        pVia->NoAccel = TRUE;
    }    

    /* NoDDCValue Option */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_NODDCVALUE, FALSE))
    {        
        pBIOSInfo->NoDDCValue = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                    "Option: Not using DDC probed value to set HorizSync & VertRefresh\n");
    }
    else
    {       
        pBIOSInfo->NoDDCValue = FALSE;
    }

    /* Interlace Mode Option */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_INTERLACE_MODE, FALSE))
    {        
        pBIOSInfo->IsEnableInterlaceMode = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: Enable Interlace Mode. \n");
    }
    else
    {       
        pBIOSInfo->IsEnableInterlaceMode = FALSE;
    }
    
    return ;
}


static void VIAGetOptions_Hotkey(ScrnInfoPtr pScrn)
{
    VIAPtr          pVia;
    VIABIOSInfoPtr  pBIOSInfo;    
    pVia = VIAPTR(pScrn);
    pBIOSInfo = pVia->pBIOSInfo;
    char    *pStr = NULL;
    char    *pTmpStr = NULL;       
    CARD32  i = 0;

    /* Device Switch Hotkey Option */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_DEVICE_SWITCH_HOTKEY, FALSE))
    {
        pBIOSInfo->Hotkey = TRUE;
        pScrn->zoomLocked = TRUE;           /* Locked Zoom Hotkey */
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Using Device Switch Hotkey\n");
    }
    else
    {
        pBIOSInfo->Hotkey = FALSE;
    }

    /* Hotkey triggered by a timer. */
    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_FN_HOTKEY)))
    {
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: Enable a timer to active hotkey.\n"));
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Device Sequence = %s\n", pStr));
 
        pTmpStr = strtok(pStr, ",");
        while (pTmpStr != NULL)
        {
            if (!xf86NameCmp(pTmpStr, "CRT"))
            {
                pVia->deviceSequence[i] = VIA_DEVICE_CRT1;
            }
            else if (!xf86NameCmp(pTmpStr, "LCD"))
            {
                pVia->deviceSequence[i] = VIA_DEVICE_LCD;
            }
            else if (!xf86NameCmp(pTmpStr, "DVI"))
            {
                pVia->deviceSequence[i] = VIA_DEVICE_DFP;
            }
            else
            {
                ErrorF("Unknow Option %s for hotkey used. \n", pTmpStr);
            }
                        
            pTmpStr = strtok(NULL, ",");
            i++;
        }

        /* It should has more than one prefered device at least. */
        if (i>1)
        {
            pVia->IsHotkeyTimerEnabled = TRUE;
        }
        else
        {
            pVia->IsHotkeyTimerEnabled = FALSE;
        }
    }

}


static void VIAProcessCfgOptions(ScrnInfoPtr pScrn)
{
    VIAPtr          pVia;
    VIABIOSInfoPtr  pBIOSInfo;

    pVia = VIAPTR(pScrn);
    pBIOSInfo = pVia->pBIOSInfo;
    
    xf86CollectOptions(pScrn, NULL);
    xf86ProcessOptions(pScrn->scrnIndex, pScrn->options, VIAOptions);  

    VIAGetOptions_general(pScrn);

    VIAGetOptions_Hotkey(pScrn);
    
    /*get mergedfb options*/
    if (!pVia->IsSecondary)
        VIAGetOptions_MergedFB(pScrn);    
    
     /* Probe the capabilities about rotation. */
    ProbeRotateCaps(pScrn);   

    VIAGetOptions_ActiveDevice(pScrn);
    VIAGetOptions_DisplayHardwareLayout(pScrn);
    VIAGetOptions_LCDInfo(pScrn);
    VIAGetOptions_DVIInfo(pScrn);
    VIAGetOptions_DPASettingInfo(pScrn);

    /*add this function because  chipset id is identified just above*/ 
    VIAGetOptions_Cursor(pScrn);
    VIAGetOptions_Video(pScrn);
    VIAGetOptions_Capture(pScrn);

    return ;    
}



static void VIAProbeChipset(ScrnInfoPtr pScrn, EntityInfoPtr pEnt)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAProbeChipSet!\n"));
    
    VIAPtr pVia;
    VIABIOSInfoPtr pBIOSInfo;
    MessageType     from = X_DEFAULT;

    pVia = VIAPTR(pScrn);
    pBIOSInfo = pVia->pBIOSInfo;
    
    pVia->PciInfo = xf86GetPciInfoForEntity(pEnt->index);
    xf86RegisterResources(pEnt->index, NULL, ResNone);

    if (pEnt->device->chipset && *pEnt->device->chipset)
    {
        pScrn->chipset = pEnt->device->chipset;
        pVia->ChipId = pEnt->device->chipID;
        pVia->Chipset = xf86StringToToken(VIAChipsets, pScrn->chipset);
        from = X_CONFIG;
    }
    else if (pEnt->device->chipID >= 0)
    {
        pVia->ChipId = pEnt->device->chipID;
        pVia->Chipset = LookupChipID(VIAPciChipsets, pVia->ChipId);
        pScrn->chipset = (char *)xf86TokenToString(VIAChipsets,
                                                   pVia->Chipset);
        from = X_CONFIG;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "ChipID override: 0x%04X\n",
                   pEnt->device->chipID);
    }
    else
    {
        from = X_PROBED;
        pVia->ChipId = pVia->PciInfo->chipType;
        pVia->Chipset = LookupChipID(VIAPciChipsets, pVia->ChipId);
        pScrn->chipset = (char *)xf86TokenToString(VIAChipsets,
                                                   pVia->Chipset);
    }

    if (pEnt->device->chipRev >= 0)
    {
        pVia->ChipRev = pEnt->device->chipRev;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "ChipRev override: %d\n",
                   pVia->ChipRev);
    }
    else
    {
        /* Read PCI bus 0, dev 0, function 0, index 0xF6 to get chip rev. */
        pVia->ChipRev = pciReadByte(pciTag(0, 0, 0), 0xF6);
    }

    pBIOSInfo->Chipset = pVia->Chipset;
    pBIOSInfo->ChipRev = pVia->ChipRev;

    
    xf86DrvMsg(pScrn->scrnIndex, from, "Chipset: \"%s\"\n", pScrn->chipset);

    pVia->PciTag = pciTag(pVia->PciInfo->bus, pVia->PciInfo->device,
                          pVia->PciInfo->func);
    xf86DrvMsg(pScrn->scrnIndex, from, "Chipset Rev.: %d\n", pVia->ChipRev);

     CARD8  BondingOption;;
    switch (pVia->Chipset)
    {
        case VIA_CLE266:
            if (pVia->ChipRev < 15)
            {
                if (pVia->ChipRev == 2)
                {
                    pBIOSInfo->CLE266A2 = TRUE;
                }
                pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3123AX;
            }
            else
            {
                pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3123CX;
            }
            break;
        case VIA_KM400:
            pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3205;
            break;
        case VIA_K8M800:
            pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3204;
            break;
        case VIA_P4M800PRO:
            pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3314;
            break;
        case VIA_PM800:
            if (pciReadByte(pciTag(0, 0, 2), 0x57) & 0x20)
            {
                pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3259;
            }
            else
            {
                DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_PROBED, "PCI:0,0,2 : 0x57 = 0x%2X !\n", pciReadByte(pciTag(0, 0, 2), 0x57)));
                DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_PROBED, "VT3259 Lite Version Found!!\n"));
                pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3259Lite;
            }
            break;
        case VIA_CX700:
            outb(VIASR, SR43);
            BondingOption = inb(VIASR + 1);
            if(BondingOption & 0x02)
            {
                pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3324M2;
            }
            else if(BondingOption & 0x40)
            {
                pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3324M;
            }
            else
            {
                pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3324;
            }    
            break;
        case VIA_K8M890:
            pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3336;
            break;
        case VIA_P4M890:
            pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3327;            
            break;
        case VIA_CN750:
            pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3293;
            break;
        case VIA_P4M900:
            pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3364;
            break;
        case VIA_VX800:
            pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3353;
            break;
        default:
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "VIABIOSInit: Unknown Device ID\n");
            break;
    }
}


/*For RandR Resize. */
static Bool VIAAddRotationMode(ScrnInfoPtr pScrn)
{
    DisplayModePtr newMode;
    VIAPtr         pVia;
    pVia = VIAPTR(pScrn);
    char    pTmpStr[10]; 
    char    pTmpStr2[10]; 
    char    *pHorStr = NULL; 
    char    *pVerStr = NULL; 
        
    /* Allocate a memory block and copy the current mode's content as its content */
    newMode = xnfcalloc(1, sizeof(DisplayModeRec));      
    memcpy(newMode, pScrn->modes, sizeof(DisplayModeRec));

    /* Add this mode into original mode list. */    
    newMode->next = xnfcalloc(1, sizeof(DisplayModeRec));
    newMode->prev = xnfcalloc(1, sizeof(DisplayModeRec));
    
    newMode->next = pScrn->modes->next;    
    newMode->prev = pScrn->modes;    
    pScrn->modes->next = newMode;    
    pScrn->modes->next->prev = newMode;
    
    /* Modify mode name */
    newMode->name = xnfalloc(10);
    strcpy(pTmpStr, pScrn->modes->name);
    pHorStr = strtok(pTmpStr, "x");       
    pVerStr = strtok(NULL, "\0");    
    sprintf(pTmpStr2, "%sx%s", pVerStr, pHorStr);    
    strcpy(newMode->name, pTmpStr2);

    /* Change size, otherwise xrandr can't show this mode. */
    newMode->HDisplay = pScrn->modes->VDisplay;
    newMode->VDisplay = pScrn->modes->HDisplay;

    return TRUE;
}

static Bool VIAFilterMode(ScrnInfoPtr pScrn)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAFilterMode!\n"));
    
    ClockRangePtr   clockRanges;
    VIAPtr          pVia;
    int result;
    pVia = VIAPTR(pScrn);
    
    /* Use this function to replace the modeline in the config file. */
    ViaModesAttachHelper(pScrn, pScrn->monitor, ViaSupportModes);
    
     /*Setup the ClockRanges, which describe what clock ranges are available,
       and what sort of modes they can be used for. */

    clockRanges = xnfcalloc(sizeof(*clockRanges), 1);
    clockRanges->next = NULL;
    clockRanges->minClock = 20000;
    clockRanges->maxClock = 270000;

    clockRanges->clockIndex = -1;
    clockRanges->interlaceAllowed = TRUE;
    clockRanges->doubleScanAllowed = FALSE;
    

    pScrn->xInc = 2;
    
    /* Select valid modes from those available */
    result = xf86ValidateModes(pScrn,
                          pScrn->monitor->Modes,    /* List of modes available for the monitor */
                          pScrn->display->modes,    /* List of mode names that the screen is requesting. */
                          clockRanges,              /* list of clock ranges */
                          NULL,                     /* list of line pitches */
                          256,                      /* mini line pitch */
                          3840,                     /* max line pitch edit by cliff for virtual screen*/
                          16 * 8,                  /* pitch increment (in bits), we just wanna 16 bytes alignment.  */
                          128,                      /* min height */
                          2880,                     /* max height edit by cliff for virtual screen*/
                          pScrn->display->virtualX, /* virtual width */
                          pScrn->display->virtualY, /* virutal height */
                          pVia->videoRambytes,      /* size of video memory */
                          LOOKUP_BEST_REFRESH);     /* lookup mode flags */    

    if (result == -1)
    {
        xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "xf86ValidateModes failure\n");
        return FALSE;
    }
    
    /* This function deletes modes in the modes field of the ScrnInfoRec that have been marked as invalid. */
    xf86PruneDriverModes(pScrn);

    if (result == 0 || pScrn->modes == NULL)
    {
        xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "No valid modes found\n");
        return FALSE;
    }

    /* This function fills in the Crtc fields for all the modes in the modes field of the ScrnInfoRec. */
    xf86SetCrtcForModes(pScrn, INTERLACE_HALVE_V);    

    /* Set the current mode to the first in the list */
    pScrn->currentMode = pScrn->modes;
    
    /* Print the list of modes being used */    
    xf86PrintModes(pScrn);

    /* Set display resolution */
    xf86SetDpi(pScrn, 0, 0);
           
    return TRUE;
}


void VIAFindMaxResFromEstablishedTiming(xf86MonPtr MonitorConf, int* MaxH, int* MaxV)
{               
    if (MonitorConf->timings1.t1 & 0xD0)
    {
        *MaxH = 720;
        *MaxV = 400;
    }
    
    if (MonitorConf->timings1.t1 & 0x3D)
    {
        *MaxH = 640;
        *MaxV = 480;
    }

    if (MonitorConf->timings1.t1 & 0x03)
    {
        *MaxH = 800;
        *MaxV = 600;
    }

    if (MonitorConf->timings1.t2 & 0xD0)
    {
        *MaxH = 800;
        *MaxV = 600;
    }

    if (MonitorConf->timings1.t2 & 0x20)
    {
        *MaxH = 832;
        *MaxV = 624;
    }

    if (MonitorConf->timings1.t2 & 0x1E)
    {
        *MaxH = 1024;
        *MaxV = 768;
    }

    if (MonitorConf->timings1.t2 & 0x01)
    {
        *MaxH = 1280;
        *MaxV = 1024;
    }        
}

static Bool VIAProbeMonitorLimitation(ScrnInfoPtr pScrn)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAProbeMonitorLimitation!\n"));
    VIAPtr          pVia;
    VIABIOSInfoPtr pBIOSInfo;
    int max_h = 0, max_v = 0;
    int i;
    int h = 0, v = 0;    
    
    pVia = VIAPTR(pScrn);
    pBIOSInfo = pVia->pBIOSInfo;

    /*  If we set Option "NoDDCValue", then CRT's  MonitorSizeH/MonitorSizeV
        which is gotten from CRT's EDID/DDC is meaningless and should not be used to judge CRT panning
        or not. set these two value bigger than max resolution that we support is safe. now we max support
        1920x1440.*/
    if (pBIOSInfo->NoDDCValue)
    {               
        pBIOSInfo->CRTSettingInfo.MonitorSizeH = 4096;
        pBIOSInfo->CRTSettingInfo.MonitorSizeV = 2048;

        /* Assign a default value to indicate the limitation of monitor.*/
        pScrn->monitor->nHsync = 1;         /* number of hsync. */
        pScrn->monitor->hsync[0].hi = 113;  /* upper bound */
        pScrn->monitor->hsync[0].lo = 30;   /* lower bound */
        
        return TRUE;
    }
    
    if (!xf86LoadSubModule(pScrn, "ddc"))
    {
        return FALSE;
    }
    else
    {
        xf86MonPtr pMon = NULL;

        xf86LoaderReqSymLists(ddcSymbols, NULL);
        if ((pVia->pVbe) &&
            ((pMon = xf86PrintEDID(vbeDoEDID(pVia->pVbe, NULL))) != NULL))
        {
            pVia->DDC1 = pMon;
            xf86SetDDCproperties(pScrn,pMon);
        }
        else if (!VIAddc2(pScrn->scrnIndex))
        {
            VIAddc1(pScrn->scrnIndex);
        }

        /* Try to get monitor's max size. */
        if (pVia->DDC1)
        {
            /*Find max resolution from Established timing*/
            VIAFindMaxResFromEstablishedTiming(pVia->DDC1, &max_h, &max_v);

            /*Find max resolution from Standard timing*/
            for (i = 0; i < 8; i++)
            {
                if (pVia->DDC1->timings2[i].hsize > max_h)
                {
                    max_h = pVia->DDC1->timings2[i].hsize;
                    max_v = pVia->DDC1->timings2[i].vsize;
                }                
            }

            /*Find max resolution from Detail timing*/
            for (i = 0; i < 4; i++)
            {
                if ((pVia->DDC1->det_mon[i].type == DT) && 
                    (pVia->DDC1->det_mon[i].section.d_timings.h_active > max_h))
                {
                    max_h = pVia->DDC1->det_mon[i].section.d_timings.h_active;
                    max_v = pVia->DDC1->det_mon[i].section.d_timings.v_active;
                }                
            }
            
            /* Correct monitor size to avoid invalid mode. */
            VIACorrectMonitorSize(&max_h, &max_v);            

            /* To prevent to get a wrong EDID data. We should assign a default value.*/
            if ((max_h == 0) || (max_v == 0))
            {
                max_h = 1024;
                max_v = 768;
            }
            
            pBIOSInfo->CRTSettingInfo.MonitorSizeH = max_h;
            pBIOSInfo->CRTSettingInfo.MonitorSizeV = max_v;
        }
        else
        {
            /* If DDC fail, set a default value as monitor size. */
            pBIOSInfo->CRTSettingInfo.MonitorSizeH = 1024;
            pBIOSInfo->CRTSettingInfo.MonitorSizeV = 768;
        }

        /* If the monitor is analog but it doesn't support GTF,
           we consider that it should be a flat monitor with D-SUB. */
        pBIOSInfo->CRTSettingInfo.IsFlatMonitor = FALSE;
        if(pMon != NULL)
        {
            if (!DIGITAL((&pMon->features)->input_type))        
            {            
                if (!GFT_SUPPORTED((&pMon->features)->msc))            
                {                
                    pBIOSInfo->CRTSettingInfo.IsFlatMonitor = TRUE;

                    /* If the monitor is a flat monitor, we can get the actual size from h_active & v_active. */
                    for (i = 0; i < 4; i++)
                    {
                        if (pMon->det_mon[i].type == DT)
                        {                    
                            if (pMon->det_mon[i].section.d_timings.h_active > max_h)
                            {                        
                                pBIOSInfo->CRTSettingInfo.MonitorSizeH = pMon->det_mon[i].section.d_timings.h_active;
                                pBIOSInfo->CRTSettingInfo.MonitorSizeV = pMon->det_mon[i].section.d_timings.v_active;                            
                            }
                        }
                    }
                }
            }
        }

        xf86DrvMsg(pScrn->scrnIndex, X_PROBED, "Max Monitor H Size =  %d\n",
               pBIOSInfo->CRTSettingInfo.MonitorSizeH);
        xf86DrvMsg(pScrn->scrnIndex, X_PROBED, "Max Monitor V Size =  %d\n",
               pBIOSInfo->CRTSettingInfo.MonitorSizeV);
    }     

        
    /* Reset HorizSync & VertRefresh Rang Using DDC Value */
    if (pVia->DDC1 && !pBIOSInfo->NoDDCValue)
    {
        for (i = 0; i < DET_TIMINGS; i++)
        {
            if (pVia->DDC1->det_mon[i].type == DS_RANGES)
            {
                pScrn->monitor->hsync[h].lo
                    = pVia->DDC1->det_mon[i].section.ranges.min_h;
                pScrn->monitor->hsync[h++].hi
                    = pVia->DDC1->det_mon[i].section.ranges.max_h;
                pScrn->monitor->vrefresh[v].lo
                    = pVia->DDC1->det_mon[i].section.ranges.min_v;
                pScrn->monitor->vrefresh[v++].hi
                    = pVia->DDC1->det_mon[i].section.ranges.max_v;

                if (pVia->DDC1->det_mon[i].section.ranges.max_clock != 0)
                {                    
                    pBIOSInfo->CRTSettingInfo.MaxPixelClock = pVia->DDC1->det_mon[i].section.ranges.max_clock;
                }                
                break;
            }
        }

        if(h > 0 )
        {
           pScrn->monitor->hsync[h-1].hi=113;
           if(pScrn->monitor->hsync[h-1].lo >= pScrn->monitor->hsync[h-1].hi)
               pScrn->monitor->hsync[h-1].lo = 30;             
        }
        
        pScrn->monitor->nHsync = h;
        pScrn->monitor->nVrefresh = v;
    }
    if(h == 0)
    {
        /* assign a default value to indicate the limitation of monitor. */
        pScrn->monitor->nHsync = 1;         /* number of hsync. */
        pScrn->monitor->hsync[0].hi = 113;  /* upper bound */
        pScrn->monitor->hsync[0].lo = 30;   /* lower bound */
        pScrn->monitor->nVrefresh = 1;
        pScrn->monitor->vrefresh[0].hi = 100;  /* upper bound */
        pScrn->monitor->vrefresh[0].lo = 50;   /* lower bound */
      
    }
    
    return TRUE;
}


static Bool VIAProbeBIOSInfo(ScrnInfoPtr pScrn)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAProbeBIOSInfo!\n"));
    VIAPtr          pVia;
    VIABIOSInfoPtr pBIOSInfo;
    
    pVia = VIAPTR(pScrn);
    pBIOSInfo = pVia->pBIOSInfo;
    int tmp;

    if (!pVia->IsMAMMEnable)
    {
        /* Get BIOS ver. From BIOS ROM or Function Call */
        if (VIAGetBIOSInfoFromROM(pVia, VIAGETBIOSVER))
        {
            pBIOSInfo->BIOSMajorVersion = (int)pVia->pBIOSVer->MajorVer;
            pBIOSInfo->BIOSMinorVersion = (int)pVia->pBIOSVer->MinorVer;
            
            /* If we don't assign FORCELCD option, check BIOS type. */
            if(!pBIOSInfo->MobileFamily)
            {
                if (pVia->pBIOSVer->CoreBase & 0x08)        
                {
                    pBIOSInfo->MobileFamily = FALSE;
                    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Desktop BIOS Found!!\n"));
                }
                else
                {
                    pBIOSInfo->MobileFamily = TRUE;
                    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Mobile BIOS Found!!\n"));
                }        
            }
            
            if (pBIOSInfo->Chipset == VIA_VX800)
            {
                DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VX800 Force Mobile Flag On!!\n"));
                pBIOSInfo->MobileFamily = TRUE;

                /* Get BIOS version through function call. */
                tmp = BIOS_GetBIOSVersion(pScrn);
                pBIOSInfo->BIOSMajorVersion = tmp >> 8;
                pBIOSInfo->BIOSMinorVersion = tmp & 0xFF;
            }
        }
        else
        {
            if (pBIOSInfo->Chipset == VIA_VX800)
            {
                DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VX800 Force Mobile Flag On!!\n"));
                pBIOSInfo->MobileFamily = TRUE;                
            }
            
            /* Get BIOS version through function call. */
            tmp = BIOS_GetBIOSVersion(pScrn);
            pBIOSInfo->BIOSMajorVersion = tmp >> 8;
            pBIOSInfo->BIOSMinorVersion = tmp & 0xFF;
        }
        
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "BIOS Version is = %x.%x\n",
                        pBIOSInfo->BIOSMajorVersion, pBIOSInfo->BIOSMinorVersion));

        /* Get BIOS date through function call. */
        BIOS_GetBIOSDate(pScrn);

        if (pBIOSInfo->Hotkey)
        {
            VIABuildModeList(pScrn);
        }
    }
    
    if (!VIAGetBIOSTable(pBIOSInfo))
    {
        return FALSE;
    }
    return TRUE;
    
}


/* If use VIA's ROM, return true. */
static Bool VIACheckROMName()
{    
    unsigned char   *pBIOS = NULL, *pRom;
    int             romSize, sum, i;
    
    /* Allocate shadow memory */
    if (!(pBIOS=xcalloc(1, 0x10000)))
    {
        ErrorF("Allocate memory fail !!\n");
        return FALSE;
    }
    /* Read BIOS ROM and check! */
    if (xf86ReadBIOS(0xC0000, 0, pBIOS, 0x10000) != 0x10000)
    {
        xfree(pBIOS);
        ErrorF("Read VGA BIOS image fail !!\n");
    }
    else
    {
        if (*((CARD16 *) pBIOS) != 0xAA55)
        {
            xfree(pBIOS);
            ErrorF("VGA BIOS image is wrong!!\n");
            return FALSE;
        }
        else
        {
            romSize = *((CARD8 *) (pBIOS + VIA_BIOS_SIZE_POS)) * 512;
            pRom = pBIOS;
            sum = 0;

            for (i = 0; i < romSize; i++)
            {
                sum += *pRom++;
            }

            if (((CARD8) sum) != 0)
            {
                xfree(pBIOS);
                ErrorF("VGA BIOS image is wrong!! CheckSum = %x\n", sum);
                return FALSE;
            }
        }
    }
    
    pBIOS += 0x61;  

    if(!memcmp(pBIOS, "VIA", 3))
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

#ifdef RANDRV12_TEST_ENABLE
void wait_debug()
{
    FILE *debugFile;

    while(1)
    {
        debugFile = fopen("/etc/X11/file_debug","r");
        if (debugFile)
        {
            fclose(debugFile);
            sleep(1);
        }
        else
        {
            break;
        }
    }
}

#endif    
static Bool VIAPreInit(ScrnInfoPtr pScrn, int flags)
{
    EntityInfoPtr   pEnt;
    VIAPtr          pVia;
    VIABIOSInfoPtr  pBIOSInfo;
#ifndef USE_FB
    char            *mod = NULL;
    const char      *reqSym = NULL;
#endif
    LPVIAGRAPHICINFO lpVIAGraphicInfo;

#if (defined(PMLINK))&&(LINUX_VERSION_CODE>132116)
    /*variables for socket*/
    char buf[BUFFER_SIZE];
    struct cn_msg *data;
    int len=-1; 
#endif /* PMLINK **/       

#ifdef RANDRV12_TEST_ENABLE
    xf86CrtcConfigPtr   xf86_config;
    int                 num_pipe = 0;
    int                 height, width;
#endif

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAPreInit\n"));
    DEBUG(ErrorF("XORG_VERSION_CURRENT = %d", XORG_VERSION_CURRENT));
    via_xf86_version_current = xf86GetVersion();    
    
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "XF86_VER=%d\n", via_xf86_version_current));    

    if (pScrn->numEntities > 1)
        return FALSE;

    if (flags & PROBE_DETECT)
        return FALSE;

#ifdef  VIA_ACPI
    if (xf86LoadSubModule(pScrn, "via_acpi"))
    {  
        /* try to load the video kernel module now */
        xf86LoadKernelModule("video");

        if (xf86LoaderCheckSymbol("ViaACPIOpen"))
        {
            void (*acpiOpen)(void) = NULL;

            acpiOpen = LoaderSymbol("ViaACPIOpen");

            if (acpiOpen) 
            {
                (*acpiOpen)();
            }
        }
    }
#endif
   
    if (!xf86LoadSubModule(pScrn, "vgahw"))
        return FALSE;

    xf86LoaderReqSymLists(vgaHWSymbols, NULL);
    if (!vgaHWGetHWRec(pScrn))
        return FALSE;


    if (!VIAGetRec(pScrn)) {
        return FALSE;
    }

    pVia = VIAPTR(pScrn);
    pBIOSInfo = pVia->pBIOSInfo;
    pBIOSInfo->s3utility=FALSE;
    pVia->IsSuspending = FALSE;
    pVia->IsVIAModeInitRunOnceDone = FALSE;
    pVia->eventTimer=NULL;
    /* Add VidData struct allocation & initial codes */
    if (!IsVidDataAllocated)
    {
#if 1
        /* Allocate shared memory for VidData */
        pVia->shmid = shmget((key_t)0x1234, sizeof(VIDDATA), 0666 | IPC_CREAT);

        if (pVia->shmid == -1)
        {
            /*There may be undeleted share memory, we should delete it first*/
            /*Or if our share memory is bigger , allocation will fail*/
            pVia->shmid = shmget((key_t)0x1234, 4, 0666 | IPC_CREAT);
            shmctl(pVia->shmid, IPC_RMID, 0);
            pVia->shmid = shmget((key_t)0x1234, sizeof(VIDDATA), 0666 | IPC_CREAT);
            if (pVia->shmid == -1)
            {
                DBG_DD(("shmget failed\n"));
            }
        }

        pVia->pVidData = (PVIDDATA) shmat(pVia->shmid, NULL, 0);

        if (pVia->pVidData == NULL)
        {
            DBG_DD(("shmat failed\n"));
        }

        DBG_DD(("VidData memory attached at %p\n", pVia->pVidData));

        memset(pVia->pVidData, 0, sizeof(VIDDATA));
        IsVidDataAllocated = TRUE;
#endif
    }
    else
    {
        /* for SAMM case - Second display */
        if ( pScrn->scrnIndex == 1 ) 
        {
            ScrnInfoPtr pScrn0 = xf86Screens[0];
            VIAPtr pVia0 = VIAPTR(pScrn0);
            pVia->pVidData = pVia0->pVidData;
        }
    }

    lpVIAGraphicInfo = &pVia->pVidData->VIAGraphicInfo;
    lpVIAGraphicInfo->duoview = lpVIAGraphicInfo->rotate = 0;	/*initilize Xserver rotation status */
    lpVIAGraphicInfo->IsSupportedColorSpaceChange = FALSE;
    pVia->IsSecondary = FALSE;
    pEnt = xf86GetEntityInfo(pScrn->entityList[0]);
    if (pEnt->resources) {
        xfree(pEnt);
        VIAFreeRec(pScrn);
        return FALSE;
    }

    pVia->EntityIndex = pEnt->index;

    if(xf86IsEntityShared(pScrn->entityList[0]))
    {
        if(xf86IsPrimInitDone(pScrn->entityList[0]))
        {
            DevUnion* pPriv;
            VIAEntPtr pVIAEnt;
            VIAPtr pVia1;

            pVia->IsSecondary = TRUE;
            pVia->HasSecondary = TRUE;
            pPriv = xf86GetEntityPrivate(pScrn->entityList[0],
                    gVIAEntityIndex);
            pVIAEnt = pPriv->ptr;
            if (pVIAEnt->BypassSecondary)
            {
                xfree(pEnt);
                VIAFreeRec(pScrn);
                return FALSE;
            }
            pVIAEnt->pSecondaryScrn = pScrn;
            pVIAEnt->HasSecondary = TRUE;
            pVia1 = VIAPTR(pVIAEnt->pPrimaryScrn);
            pVia1->HasSecondary = TRUE;
            pVia1->pBIOSInfo->DuoView=FALSE;
            pVIAEnt->pSecondaryScrn->zoomLocked = TRUE;
            pVIAEnt->pPrimaryScrn->zoomLocked = TRUE;
        }
        else
        {
            DevUnion* pPriv;
            VIAEntPtr pVIAEnt;

            xf86SetPrimInitDone(pScrn->entityList[0]);
            pPriv = xf86GetEntityPrivate(pScrn->entityList[0],
                    gVIAEntityIndex);
            pVIAEnt = pPriv->ptr;
            pVIAEnt->pPrimaryScrn = pScrn;
            pVIAEnt->IsDRIEnabled = FALSE;
            pVIAEnt->BypassSecondary = FALSE;
            pVIAEnt->HasSecondary = FALSE;
            pVIAEnt->RestorePrimary = FALSE;
            pVIAEnt->IsSecondaryRestored = FALSE;
        }
    }

#ifndef RANDRV12_TEST_ENABLE
    if (flags & PROBE_DETECT) {
        VIAProbeDDC(pScrn, pVia->EntityIndex);
        return TRUE;
    }
#endif

    pScrn->monitor = pScrn->confScreen->monitor;

    /*
     * We support depths of 8, 16 and 24.
     * We support bpp of 8, 16, and 32.
     */

    if (!xf86SetDepthBpp(pScrn, 0, 0, 0, Support32bppFb))
    {
        xfree(pEnt);
        VIAFreeRec(pScrn);
        return FALSE;
    }
    else
    {
        switch (pScrn->depth)
        {
        case 8:
        case 16:
        case 24:
        case 32:
            /* OK */
            break;
        default:
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                       "Given depth (%d) is not supported by this driver\n",
                       pScrn->depth);
            xfree(pEnt);
            VIAFreeRec(pScrn);
            return FALSE;
        }
    }

    xf86PrintDepthBpp(pScrn);

    if (pScrn->depth == 32) {
        pScrn->depth = 24;
    }

    if (pScrn->depth > 8)
    {
        rgb zeros = {0, 0, 0};

        if (!xf86SetWeight(pScrn, zeros, zeros))
        {
            xfree(pEnt);
            VIAFreeRec(pScrn);
            return FALSE;
        } 
        else
        {
            /* TODO check weight returned is supported */
            ;
        }
    }

    if (!xf86SetDefaultVisual(pScrn, -1)) {
        return FALSE;
    }
    else
    {
        /* We don't currently support DirectColor at > 8bpp */
        if (pScrn->depth > 8 && pScrn->defaultVisual != TrueColor)
        {
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Given default visual"
                       " (%s) is not supported at depth %d\n",
                       xf86GetVisualName(pScrn->defaultVisual), pScrn->depth);
            xfree(pEnt);
            VIAFreeRec(pScrn);
            return FALSE;
        }
    }

    /* Set the bits per RGB for 8bpp mode */
    if (pScrn->depth == 8)
        pScrn->rgbBits = 8;

    /* We use a programmable clock */
    pScrn->progClock = TRUE;

    /* Check if VIA's ROM. If not, it imply that we are under MAMM. */
    if(VIACheckROMName())
    {        
        pVia->IsMAMMEnable = FALSE;
        pBIOSInfo->IsMAMMEnable = FALSE;
    }
    else
    {        
        pVia->IsMAMMEnable = TRUE;   
        pBIOSInfo->IsMAMMEnable = TRUE;
    }
    
    if (!pVia->IsMAMMEnable)
    {
        if (xf86LoadSubModule(pScrn, "int10"))
        {
            xf86LoaderReqSymLists(int10Symbols, NULL);
            pVia->pInt10 = xf86InitInt10(pEnt->index);
        }
    }    
     if (!pVia->IsMAMMEnable) /* For MAMM */
    {
        if (pVia->pInt10 && xf86LoadSubModule(pScrn, "vbe"))
        {
            
            if (via_xf86_version_current < XF86_VERSION_NUMERIC(4,2,99,0,0))
            {
                xf86LoaderReqSymLists(vbeSymbols, NULL);
                pVia->pVbe = VBEInit(pVia->pInt10, pVia->EntityIndex);
            }    
            else
            {
                xf86LoaderReqSymLists(vbeSymbols429900, NULL);
                pVia->pVbe = VBEExtendedInit(pVia->pInt10, pVia->EntityIndex,0);
            }   
        }
    }

    if (!pVia->IsSecondary)
    {
        pVia->PciInfo = xf86GetPciInfoForEntity(pEnt->index);	
        pVia->MmioBase = pVia->PciInfo->memBase[1];	 
        pVia->MapBase = xf86MapPciMem(pScrn->scrnIndex, VIDMEM_MMIO, pVia->PciTag,
                                  pVia->MmioBase, VIA_MMIO_REGSIZE);                                  
        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "MapBase = %p, MmioBase=%lx\n",
            pVia->MapBase, pVia->MmioBase);	
	 
        VIARegPtr       save = &pVia->SavedReg; 
        /* Unlock Extended Regs */
        VGAOUT8(0x3c4, 0x10);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x01);
        /* Unlock CRTC register protect */
        VGAOUT8(0x3d4, 0x47);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) & ~0x01);
        VGAOUT8(0x3d4, CR08);
        save->CRTCRegs[0x08] = VGAIN8(0x3d5);
        VGAOUT8(0x3d4, CR09);
        save->CRTCRegs[0x09] = VGAIN8(0x3d5);
        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "RegCR08 = %x, RegCR09=%x\n",
            save->CRTCRegs[0x08], save->CRTCRegs[0x09]);

        VGAOUT8(0x3C4, SR1A);
        save->SRRegs[0x1A] = VGAIN8(0x3C5);
        
        xf86UnMapVidMem(pScrn->scrnIndex, (pointer)pVia->MapBase,
                        VIA_MMIO_REGSIZE);	
    }

     /* Detect which chipset we used. */
    VIAProbeChipset(pScrn, pEnt);

     /* Set status word positions based on chip type. */
    switch (pVia->Chipset)
    {
        case VIA_P4M890:    
        case VIA_K8M890:
        case VIA_CN750:
        case VIA_P4M900:        
            pVia->myWaitIdle = WaitIdleK8M890;
            break;
        case VIA_VX800:
            pVia->myWaitIdle = WaitIdleVX800;
            break;
        default:
            pVia->myWaitIdle = WaitIdleCLE266;
            break;
    }

    /* We process all config file's options in one function */
    VIAProcessCfgOptions(pScrn);
    
    if (pEnt->device->videoRam != 0)
    {
        if (!pScrn->videoRam)
        {
            pScrn->videoRam = pEnt->device->videoRam;
        }
        else
        {
            xf86DrvMsg(pScrn->scrnIndex, X_WARNING,
                      "Video Memory Size in Option is %d KB, Detect is %d KB!",
            pScrn->videoRam, pEnt->device->videoRam);
        }
    }

    xfree(pEnt);    

    if (!VIAMapMMIO(pScrn))
    {
        if (pVia->pVbe!=NULL)
            vbeFree(pVia->pVbe);
            return FALSE;
    }

    /* Get some useful information(BIOS version, date, mobile flag)
       through BIOS.(ROM or Function call) */
    if (!VIAProbeBIOSInfo(pScrn))
    {
        VIAFreeRec(pScrn);
        return FALSE;
    }

    /* Initialize the colormap and this step must be done before calling xf86HandleColormaps. */
    Gamma zeros = {0.0, 0.0, 0.0};
    if (!xf86SetGamma(pScrn, zeros))
    {
        VIAFreeRec(pScrn);
        vbeFree(pVia->pVbe);
        return FALSE;
    }

    /* Read PCI configuration space to get frame buffer size. */
    pScrn->videoRam = VIAGetFBSize(pScrn);

    /* Split FB for SAMM */
    if (pVia->IsSecondary)
    {
        DevUnion* pPriv;
        VIAEntPtr pVIAEnt;
        VIAPtr    pVia1;
        int	videoram1=0;

        pPriv = xf86GetEntityPrivate(pScrn->entityList[0],gVIAEntityIndex);
        pVIAEnt = pPriv->ptr;

        if(xf86GetOptValInteger(VIAOptions, OPTION_VIDEORAM1, &videoram1))
        {
            xf86DrvMsg( pScrn->scrnIndex, X_CONFIG,
                        "Option: VideoRAM1 %dkB\n", videoram1 );
        }
        
        if(videoram1 > 0)
            videoram1 = (videoram1 >> 10) << 10;
  
        if(videoram1 >= pScrn->videoRam - 16384)
            videoram1 = pScrn->videoRam - 16384;

        if((videoram1 <= 16384) && (videoram1 > 0))
            videoram1 = 16384;

        if(videoram1)
            pScrn->videoRam = videoram1;
        else
            pScrn->videoRam = 11 * 1024; /* Reserve 11M for second scrn, because 1920x1440x4/2^20 = 10.5M  */
        
        pVIAEnt->pPrimaryScrn->videoRam = pVIAEnt->pPrimaryScrn->videoRam - pScrn->videoRam;
        pVia1 = VIAPTR(pVIAEnt->pPrimaryScrn);
        pVia1->videoRambytes = pVIAEnt->pPrimaryScrn->videoRam << 10;
        pVia->FrameBufferBase += (pVIAEnt->pPrimaryScrn->videoRam << 10);
                   
    }

    pVia->videoRambytes = pScrn->videoRam << 10;
    xf86DrvMsg(pScrn->scrnIndex, X_PROBED, "videoram =  %dk\n",
               pScrn->videoRam);

    /* here load any modules that we need */    
    /* Before using DDC, we must initialize the I2C. */
    if (!xf86LoadSubModule(pScrn, "i2c"))
    {
        VIAFreeRec(pScrn);
        return FALSE;
    }
    else
    {
        xf86LoaderReqSymLists(i2cSymbols,NULL);
        VIAI2CInit(pScrn);
    }
    
#ifdef USE_FB
    if (xf86LoadSubModule(pScrn, "fb") == NULL)
    {
        xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Failed to load fb module");
        VIAFreeRec(pScrn);
        return FALSE;
    }

    xf86LoaderReqSymLists(fbSymbols, NULL);

#else
    /* load bpp-specific modules */
    switch (pScrn->bitsPerPixel)
    {
        case 8:
            mod = "cfb";
            reqSym = "cfbScreenInit";
            break;
        case 16:
            mod = "cfb16";
            reqSym = "cfb16ScreenInit";
            break;
        case 32:
            mod = "cfb32";
            reqSym = "cfb32ScreenInit";
            break;
    }

    if (mod && xf86LoadSubModule(pScrn, mod) == NULL)
    {
        VIAFreeRec(pScrn);
        return FALSE;
    }

    xf86LoaderReqSymbols(reqSym, NULL);
#endif

    if (!pVia->NoAccel)
    {
        if(!xf86LoadSubModule(pScrn, "xaa"))
        {
            VIAFreeRec(pScrn);
            return FALSE;
        }
        xf86LoaderReqSymLists(xaaSymbols, NULL);
    }

    if (!pVia->ForceSWCursor)
    {
        if (!xf86LoadSubModule(pScrn, "ramdac"))
        {
            VIAFreeRec(pScrn);
            return FALSE;
        }
        xf86LoaderReqSymLists(ramdacSymbols, NULL);
    }

    if (pVia->shadowFB)
    {
        if (!xf86LoadSubModule(pScrn, "shadowfb"))
        {
            VIAFreeRec(pScrn);
            return FALSE;
        }
        xf86LoaderReqSymLists(shadowSymbols, NULL);
    }
    
#ifndef RANDRV12_TEST_ENABLE
    /* Use DDC to detect the capability limitation of CRT monitor. */
    if (!VIAProbeMonitorLimitation(pScrn))
    {
        VIAFreeRec(pScrn);
        return FALSE;
    }
                    
    if (!VIAFilterMode(pScrn))
    {
        VIAFreeRec(pScrn);
        return FALSE;
    }    
#endif    

    /* Before we detect LVDS and TMDS chip, we have to assign I2C port and DDC.*/
    pBIOSInfo->I2C_Port1 = pVia->I2C_Port1;
    pBIOSInfo->I2C_Port2 = pVia->I2C_Port2;
    pBIOSInfo->DDC1 = pVia->DDC1;
    pBIOSInfo->DDC2 = pVia->DDC2;

    VIAPreInit_LVDS(pBIOSInfo);

#ifdef RANDRV12_TEST_ENABLE
    /*============= For RandR v1.2 =======================*/    
    int             i;
    /* Load DDC module for EDID. */
    xf86LoadSubModule(pScrn, "ddc");           
    
    xf86SetDpi(pScrn, 0, 0);   
    
    xf86CrtcConfigInit (pScrn, &via_xf86crtc_config_funcs);
    xf86CrtcSetSizeRange (pScrn, 320, 200, 8192, 8192);     
    
    via_crtc_init(pScrn, 2);    
    via_output_init(pScrn);        
    
    if (!xf86InitialConfiguration(pScrn, FALSE))
    {        
        xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
            "No valid initial configuration found\n");   
        return FALSE;
    }     

    pScrn->currentMode = pScrn->modes;
    pScrn->displayWidth = pScrn->virtualX;        

    xf86CrtcConfigPtr   via_crtc_config = XF86_CRTC_CONFIG_PTR (pScrn);
    
    /* If active device doesn't assigned. */
    if (!pBIOSInfo->ActiveDevice)
    {
        for(i = 0; i < via_crtc_config->num_output; i++)
        {
            if (via_crtc_config->output[i]->crtc)
            {
                if (!strcmp(via_crtc_config->output[i]->name, "CRT"))
                {
                    DEBUG(ErrorF("Output CRT is Enable\n"));
                    pBIOSInfo->ActiveDevice |= VIA_DEVICE_CRT1;
                }

                if (!strcmp(via_crtc_config->output[i]->name, "DVI"))
                {
                    DEBUG(ErrorF("Output DVI is Enable\n"));
                    pBIOSInfo->ActiveDevice |= VIA_DEVICE_DFP;
                }   

                if (!strcmp(via_crtc_config->output[i]->name, "LCD"))
                {
                    DEBUG(ErrorF("Output LCD is Enable\n"));
                    pBIOSInfo->ActiveDevice |= VIA_DEVICE_LCD;
                }   

            }
        }
    }
        
    /*============= End RandR v1.2 =======================*/
#endif

    /* After calling xf86ValidateModes(), pScrn->virtualX(Y) will get the final value of virtual size. */
    /* At RandR v1.2, pScrn->virtualX,Y will get value after xf86InitialConfiguration */
    pBIOSInfo->VirtualX = pScrn->virtualX;
    pBIOSInfo->VirtualY = pScrn->virtualY;

    /* Set up screen parameters. */
    pVia->Bpp = pScrn->bitsPerPixel >> 3;
    pVia->Bpl = pScrn->displayWidth * pVia->Bpp;

    if (pBIOSInfo->MergedFB)
    {
        /* set virtul X,Y value while mergedfb*/
        if((pBIOSInfo->Scrn2Position==viaRightOf)||(pBIOSInfo->Scrn2Position==viaLeftOf))
        {
            pScrn->virtualX += pScrn->virtualX;
        }

        if((pBIOSInfo->Scrn2Position==viaAbove)||(pBIOSInfo->Scrn2Position==viaBelow))
        {
            pScrn->virtualY += pScrn->virtualY;

        }
        pScrn->displayWidth = pScrn->virtualX; 
    }

    VGAOUT8(0x3c4, SR1E);
    if (VGAIN8(0x3c5) & BIT3)
    {        
        pBIOSInfo->IsSpreadSpectrumEnabled = TRUE;   

        /* Disable spread spectrum temporarily, we will turn it on when set mode. */
        VGAOUT8(0x3c4, SR3D);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xFE);

        VGAOUT8(0x3c4, SR1E);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xF7);     
    }
    else
    {
        pBIOSInfo->IsSpreadSpectrumEnabled = FALSE;
    }    
    
#ifndef RANDRV12_TEST_ENABLE

    /* Detect TMDS Transmiter */
    if (VIATMDSIdentify(pBIOSInfo))
    {
        if (pBIOSInfo->NoDDCValue)
        {            
            pBIOSInfo->TMDSSettingInfo.DFPSize = VIAGetModeIndex(pScrn->virtualX, pScrn->virtualY);
            VIAGetModeSizeByIndex(pBIOSInfo->TMDSSettingInfo.DFPSize, &pBIOSInfo->TMDSSettingInfo.PanelSizeH, &pBIOSInfo->TMDSSettingInfo.PanelSizeV);                        
        }
        else
        {
            VIAGetPanelInfo(pBIOSInfo);
        }
    }
#endif
    
    /* Detect LVDS Transmiter */
    if (pVia->Chipset == VIA_CLE266)
    {
        /* For CLE266 device work on AMR slot */
        VIAPostLCDNotDVO(pBIOSInfo);
    }
    else
    {
        /* For KM400 or later device work on AGP slot */
        VIAPostLCDDVO(pBIOSInfo);
    }    

    pBIOSInfo->ConnectedDevice = VIAGetDeviceDetect(pBIOSInfo);
    
    VIADeviceSelection(pScrn);

    VIASetDisplayPath(pScrn);
    
    /* Check what type of cursor (SW/HW) to use. */
    VIACheckCursorTypeToUse(pScrn);            

    if(!VIALoadVideoOption(pVia, lpVIAGraphicInfo)) 
        VIALoadVideoOptionDefault(pVia, lpVIAGraphicInfo);
    
    if (pBIOSInfo->DuoView)
    {
        VIADeviceDispatchDuoView(pScrn);
    }    

    VIAUnmapMem(pScrn);

  /* 1.register via_pmlink to receive power event
   * 2. set eventTimer */
#if (defined(PMLINK))&&(LINUX_VERSION_CODE>132116)
    if (sock_fd==-1)
    {
        sock_fd=socket(PF_NETLINK,SOCK_DGRAM,NETLINK_CONNECTOR);
        if (sock_fd==-1)
           goto out;
    }

    local.nl_family=AF_NETLINK;
    local.nl_groups=VIA_LINK_GROUP;
    local.nl_pid=0;
    if (bind(sock_fd,(struct sockaddr *)&local,\
                sizeof(struct sockaddr_nl))==-1)
        goto out;

    /*at this stage send a register message to via_pmlink*/
    do{
        memset(buf,0,sizeof(buf));
        data=(struct cn_msg *)buf;
        data->id.idx=VIA_LINK_GROUP;
        data->id.val=VIA_LINK_VAL;
        data->seq=getpid();
        data->ack=0;
        data->len=0;
        data->flags=LINK_REGISTER;
        len= VIAPMLinkSendMSG(sock_fd,data);
    }while(len==-1);

    if(!pVia->eventTimer)
    {
        pVia->eventTimer=TimerSet(NULL,0,VIA_SCHEDULE_SLICE,VIAPMHandler,pScrn);
    return TRUE;
    } 
out:  
#endif /* PMLINK */
    return TRUE;
}

static Bool VIAEnterVT(int scrnIndex, int flags)
{
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
    ScrnInfoPtr pScrn0 = xf86Screens[0];
    VIAPtr      pVia = VIAPTR(pScrn);
    VIAPtr      pVia0 = VIAPTR(pScrn0);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    vgaHWPtr hwp = VGAHWPTR(pScrn);
    vgaRegPtr   vgaSavePtr = &hwp->SavedReg;
    VIARegPtr   viaSavePtr = &pVia->SavedReg;
    Bool        ret;
    CARD8       cr36 = 0;
    CARD32      TmpSwapValue;

    xf86DrvMsg(scrnIndex, X_INFO, "VIAEnterVT\n");

    if(pBIOSInfo->Chipset == VIA_CX700)
    {
        if(1 != VGAIN8(0x3c3)) VGAOUT8(0x3c3,1);
    }

    DEBUG(xf86DrvMsg(scrnIndex, X_INFO, "VIAEnterVT\n"));

    vgaHWUnlock(hwp);

    if ((pVia->SWRotate == VIA_ROTATE_DEGREE_90) ||
        (pVia->SWRotate == VIA_ROTATE_DEGREE_270)||
        (pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_90)||
        (pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_270))
    {      

        TmpSwapValue = pScrn->currentMode->HDisplay;
        pScrn->currentMode->HDisplay = pScrn->currentMode->VDisplay;
        pScrn->currentMode->VDisplay = TmpSwapValue;          
        
        TmpSwapValue = pScrn->virtualX;
        pScrn->virtualX = pScrn->virtualY;
        pScrn->virtualY = TmpSwapValue;                       
    }

    if (!pVia->IsSecondary)
    {
        /* Restore first or some of BIOS settings will lose. */
        VIARestore(pScrn, vgaSavePtr, viaSavePtr);
        /*Clear FB with black color to avoid garbage caused by VIARestore*/
        memset(pVia->FBBase, 0x00, ((pScrn->displayWidth * pScrn->bitsPerPixel>>3) * pScrn->virtualY));
    }
    else
    {
        /*Clear FB with black color to avoid garbage caused by VIARestore*/
        memset(pVia->FBBase+pScrn->fbOffset, 0x00, ((pScrn->displayWidth * pScrn->bitsPerPixel>>3) * pScrn->virtualY));
    }
    
    ret = VIAModeInit(pScrn, pScrn->currentMode);

    /* For dynamic rotation: graphic -> console mode -> graphic */
    if(pVia->DynamicRotation)
        EnableDynamicRotate(pScrn);
    
    if (pVia->pScreensPublicInfo->UseHwCursor)
    {
        DEBUG(xf86DrvMsg(scrnIndex, X_INFO, "Restore Cursor Image!\n"));
        if(!pVia->IsSecondary)
        {
            if(pVia->HwCursorImage)
                memcpy(pVia->FBBase + pVia->pScreensPublicInfo->CursorInfo.HwCursorBufStart, pVia->HwCursorImage, VIA_CURSOR_SIZE);
            if(pVia->HwIconImage)
                memcpy(pVia->FBBase + pVia->pScreensPublicInfo->CursorInfo.HIBufStart, pVia->HwIconImage, VIA_CURSOR_SIZE);
        }
        else
        {
            if(pVia->HwCursorImage)
                memcpy(pVia0->FBBase + pVia->pScreensPublicInfo->CursorInfo.HwCursorBufStart, pVia->HwCursorImage, VIA_CURSOR_SIZE);
            if(pVia->HwIconImage)
                memcpy(pVia0->FBBase + pVia->pScreensPublicInfo->CursorInfo.HIBufStart, pVia->HwIconImage, VIA_CURSOR_SIZE);
        }        
        VIASETREG(VIA_REG_CURSOR_FG, pVia->pScreensPublicInfo->CursorInfo.CursorFG);
        VIASETREG(VIA_REG_CURSOR_BG, pVia->pScreensPublicInfo->CursorInfo.CursorBG);
        VIASETREG(VIA_REG_CURSOR_MODE, pVia->pScreensPublicInfo->CursorInfo.CursorMC);
        if(pVia->HwCursorImage)
        {
            xfree(pVia->HwCursorImage);
            pVia->HwCursorImage = NULL;
        }
        if(pVia->HwIconImage)
        {
            xfree(pVia->HwIconImage);
            pVia->HwIconImage = NULL;
        }
        if(pVia->HIScalingPrimImage)
        {
            xfree(pVia->HIScalingPrimImage);
            pVia->HIScalingPrimImage = NULL;
        }
        if(pVia->HIScalingSecImage)
        {
            xfree(pVia->HIScalingSecImage);
            pVia->HIScalingSecImage = NULL;
        }
    }

    VIAARGBCursorInit(pScrn);
    
    /* retore video status */
    if (!pVia->IsSecondary)
        viaRestoreVideo(pScrn);

    VIAAdjustFrame(scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);

    /* Insert a timer */
    if  ((pVia->IsHotkeyTimerEnabled) && (!pBIOSInfo->SAMM))
    {
        if(!pVia->devicesTimer)
        {
            pVia->devicesTimer = TimerSet(NULL, 0, 30, VIASyncTimer, pScrn);
        }
    }
    else
    {
        if(pVia->devicesTimer)
        {
            TimerCancel(pVia->devicesTimer);
            pVia->devicesTimer=NULL;
        }
    }

    
    pVia->IsSuspending = FALSE;    /* System has resumed. */

    /* Revert to the rotated size.(e.g. 1024x768 -> 768x1024)*/
    if ((pVia->SWRotate == VIA_ROTATE_DEGREE_90) ||
        (pVia->SWRotate == VIA_ROTATE_DEGREE_270)||
        (pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_90)||
        (pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_270))
    {       
        TmpSwapValue = pScrn->currentMode->HDisplay;
        pScrn->currentMode->HDisplay = pScrn->currentMode->VDisplay;
        pScrn->currentMode->VDisplay = TmpSwapValue;         
        
        TmpSwapValue = pScrn->virtualX;
        pScrn->virtualX = pScrn->virtualY;
        pScrn->virtualY = TmpSwapValue;   
    }
    
    return ret;
}


static void VIALeaveVT(int scrnIndex, int flags)
{
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
    ScrnInfoPtr pScrn0 = xf86Screens[0];
    vgaHWPtr    hwp = VGAHWPTR(pScrn);
    VIAPtr      pVia = VIAPTR(pScrn);
    VIAPtr      pVia0 = VIAPTR(pScrn0);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    vgaRegPtr   vgaSavePtr = &hwp->SavedReg;
    VIARegPtr   viaSavePtr = &pVia->SavedReg;
    CARD8       cr36 = 0;
    xf86DrvMsg(scrnIndex, X_INFO, "VIALeaveVT\n");

    /* Unload Timer */
    if (pVia->devicesTimer)
    {
        TimerCancel(pVia->devicesTimer);
    }
    pVia->devicesTimer = NULL; 
    
    
    /* Wait Hardware Engine idle to exit graphicd mode */
    WaitIdle();

    if (pVia->VQEnable)
    {
        /* if we use VQ, disable it before we exit */
        switch( pVia->Chipset )	
        { 
        case VIA_P4M890: 
        case VIA_K8M890:
            VIASETREG(0x41c, 0x00100000);
            VIASETREG(0x420, 0x74301000);
            break;
        default:	 
            VIASETREG(0x43c, 0x00fe0000);
            VIASETREG(0x440, 0x00000004);
            VIASETREG(0x440, 0x40008c0f);
            VIASETREG(0x440, 0x44000000);
            VIASETREG(0x440, 0x45080c04);
            VIASETREG(0x440, 0x46800408);
            break;
        }  
    }

    /* Save video status and turn off all video activities */
    if (!pVia->IsSecondary)
        viaSaveVideo(pScrn);

    if (pVia->pScreensPublicInfo->UseHwCursor)
    {
        pVia->HwCursorImage = (unsigned char *)xcalloc(1, VIA_CURSOR_SIZE);
        pVia->HwIconImage = (unsigned char *)xcalloc(1, VIA_CURSOR_SIZE);
        pVia->HIScalingPrimImage = (unsigned char *)xcalloc(1, VIA_CURSOR_SIZE);
        pVia->HIScalingSecImage = (unsigned char *)xcalloc(1, VIA_CURSOR_SIZE);
        if(!pVia->IsSecondary)
        {
            if(pVia->HwCursorImage)
                memcpy(pVia->HwCursorImage, pVia->FBBase + pVia->pScreensPublicInfo->CursorInfo.HwCursorBufStart, VIA_CURSOR_SIZE);
            if(pVia->HwIconImage)
                memcpy(pVia->HwIconImage, pVia->FBBase + pVia->pScreensPublicInfo->CursorInfo.HIBufStart, VIA_CURSOR_SIZE);
        }
        else
        {
            if(pVia->HwCursorImage)
                memcpy(pVia->HwCursorImage, pVia0->FBBase + pVia->pScreensPublicInfo->CursorInfo.HwCursorBufStart, VIA_CURSOR_SIZE);
            if(pVia->HwIconImage)
                memcpy(pVia->HwIconImage, pVia0->FBBase + pVia->pScreensPublicInfo->CursorInfo.HIBufStart, VIA_CURSOR_SIZE);
        }
        pVia->pScreensPublicInfo->CursorInfo.CursorFG= (CARD32)VIAGETREG(VIA_REG_CURSOR_FG);
        pVia->pScreensPublicInfo->CursorInfo.CursorBG = (CARD32)VIAGETREG(VIA_REG_CURSOR_BG);
        pVia->pScreensPublicInfo->CursorInfo.CursorMC = (CARD32)VIAGETREG(VIA_REG_CURSOR_MODE);
    }

    VIARestore(pScrn, vgaSavePtr, viaSavePtr);

    pVia->IsSuspending = TRUE;     /* System will suspend. */
    pVia->IsVIAModeInitRunOnceDone = FALSE;
    
    VGAOUT8(0x3c4, 0x32);
    VGAOUT8(0x3c5, 0x00);

    vgaHWLock(hwp);
}


static void VIASave(ScrnInfoPtr pScrn)
{
    vgaHWPtr        hwp = VGAHWPTR(pScrn);
    vgaRegPtr       vgaSavePtr = &hwp->SavedReg;
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    VIARegPtr       save = &pVia->SavedReg;
    int             vgaCRIndex, vgaCRReg, vgaIOBase;
    int             i;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASave\n"));

    if(pVia->IsSecondary)
    {
        DevUnion* pPriv;
        VIAEntPtr pVIAEnt;
        VIAPtr   pVia1;
        vgaHWPtr hwp1;
        pPriv = xf86GetEntityPrivate(pScrn->entityList[0],
              gVIAEntityIndex);
        pVIAEnt = pPriv->ptr;
        hwp1 = VGAHWPTR(pVIAEnt->pPrimaryScrn);
        pVia1 = VIAPTR(pVIAEnt->pPrimaryScrn);
        hwp->SavedReg = hwp1->SavedReg;
        pVia->SavedReg = pVia1->SavedReg;
    }
    else
    {
        vgaHWProtect(pScrn, TRUE);

        if (xf86IsPrimaryPci(pVia->PciInfo))
        {
            vgaHWSave(pScrn, vgaSavePtr, VGA_SR_ALL);
            DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO,
                             "Primary Adapter! saving VGA_SR_ALL !!\n"));
        }
        else
        {
            vgaHWSave(pScrn, vgaSavePtr, VGA_SR_MODE);
            DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO,
                             "Non-Primary Adapter! saving VGA_SR_MODE only !!\n"));
        }

        if (pBIOSInfo->Hotkey)
        {
    	    (void)VBEGetVBEMode(pVia->pVbe, &pVia->stateMode);
    	}
    	else
    	{
            vgaIOBase = hwp->IOBase;
            vgaCRReg = vgaIOBase + 5;
            vgaCRIndex = vgaIOBase + 4;

            /* Unlock Extended Regs */
            VGAOUT8(0x3c4, 0x10);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x01);
            /* Unlock CRTC register protect */
            VGAOUT8(0x3d4, 0x47);
            VGAOUT8(0x3d5, VGAIN8(0x3d5) & ~0x01);

            /* Save Sequencer regs: from SR00 to SRFF */
            i = 0;
            
            while (1)
            {
                /* We have saved SR1A in VIAPreInit(). */
                if (i != 0x1A)
                {
                    VGAOUT8(0x3C4, i);
                    save->SRRegs[i] = VGAIN8(0x3C5);
                }

                if (i == 0xFF)
                {
                    break;
                }
                else
                {
                    i++;
                }
            }
            
            
            /* Save CRTC regs: from CR00 to CRFF */
            i = 0;
            
            while (1)
            {
                /* We have saved CR08 and CR09 in VIAPreInit(). */
                if ((i != 0x08) && (i != 0x09))
                {
                    VGAOUT8(vgaCRIndex, i);
                    save->CRTCRegs[i] = VGAIN8(vgaCRReg);
                }

                if (i == 0xFF)
                {
                    break;
                }
                else
                {
                    i++;
                }
            }

            /* Save for VT1636 LVDS: */
            if (pBIOSInfo->LVDSSettingInfo.ChipID == VIA_VT1636)
            {
                VIASaveLVDS_VT1636(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo), save);
            }
            else if(pBIOSInfo->LVDSSettingInfo2.ChipID == VIA_VT1636)
            {
                VIASaveLVDS_VT1636(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo2), save);
            }

            if (!pVia->ModeStructInit)
            {
                vgaHWCopyReg(&hwp->ModeReg, vgaSavePtr);
                memcpy(&pVia->ModeReg, save, sizeof(VIARegRec));
                pVia->ModeStructInit = TRUE;
            }
        } /* non-using device switch hotkey */
        
        /* Saving HW Cursor state */
        save->dwCursorMode = VIAGETREG(VIA_REG_CURSOR_MODE);
        save->dwCursorPOS = VIAGETREG(VIA_REG_CURSOR_POS);
        save->dwCursorORG = VIAGETREG(VIA_REG_CURSOR_ORG);
        save->dwCursorBG = VIAGETREG(VIA_REG_CURSOR_BG);
        save->dwCursorFG = VIAGETREG(VIA_REG_CURSOR_FG);
        
        vgaHWProtect(pScrn, FALSE);
    }
    return;
}

static void VIARestore(ScrnInfoPtr pScrn, vgaRegPtr vgaSavePtr,
                            VIARegPtr restore)
{
    vgaHWPtr        hwp = VGAHWPTR(pScrn);
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    int             vgaCRIndex, vgaCRReg, vgaIOBase;
    int             i, mode;
       
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIARestore\n"));
    vgaIOBase = hwp->IOBase;
    vgaCRIndex = vgaIOBase + 4;
    vgaCRReg = vgaIOBase + 5;

    /* Disable output device avoid set mode garbage */
    if (!pVia->IsSecondary && !pBIOSInfo->Hotkey)
    {
        VIADisableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));

        if (pBIOSInfo->LVDSSettingInfo2.ChipID)
        {
            VIADisableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo2));
        }
        
        VIADisableDFP(pBIOSInfo);
    }
    
    vgaHWProtect(pScrn, TRUE);

    /* Unlock Extended Regs */
    VGAOUT8(0x3c4, 0x10);
    VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x01);
    /* Unlock CRTC register protect */
    VGAOUT8(0x3d4, 0x47);
    VGAOUT8(0x3d5, VGAIN8(0x3d5) & ~0x01);    
      
    VGAOUT8(0x3d4, 0x36);
    VGAOUT8(0x3d5, VGAIN8(0x3d5) | 0x30);

    VGAOUT8(0x3d4, 0x6A);
    VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0xBF);   /* Set CR6A[6] to 0. */
    VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0x7F);   /* Set CR6A[7] to 0. */
    VGAOUT8(0x3d5, VGAIN8(0x3d5) | 0x40);   /* Set CR6A[6] to 1. */
    VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0xC0);   /* Set CR6A[5:0] to 0. */

    VGAOUT8(0x3d4, 0x6B);
    VGAOUT8(0x3d5, 0);
    VGAOUT8(0x3d4, 0x6C);
    VGAOUT8(0x3d5, 0);

    /* Gamma must disable before restore pallette */
    VGAOUT8(0x3d4, 0x33);
    VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0x7F);
   
   
    /* restore the standard vga regs */
    if (xf86IsPrimaryPci(pVia->PciInfo))
        vgaHWRestore(pScrn, vgaSavePtr, VGA_SR_ALL);
    else
        vgaHWRestore(pScrn, vgaSavePtr, VGA_SR_MODE);

    VIADisabledExtendedFIFO(pBIOSInfo);    

    /* Disable Down Scaling Function. */
    VIADisableDownScaling(pScrn);
       
    if (pBIOSInfo->Hotkey)
    {
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)
        {
            /* set LCD expansion by default */
            BIOS_SetFlatPanelState(pScrn, TRUE);
        }
        mode = pVia->stateMode | (1 << 15);
        VBESetVBEMode(pVia->pVbe, mode, NULL);
        BIOS_SetActiveDevice(pScrn);
    }
    else
    {
        /* restore Sequencer regs: From SR14 to SR3F  */         
        for (i=0x14; i<0x40; i++)
        {
            VGAOUT8(0x3C4, i);
            VGAOUT8(0x3C5, restore->SRRegs[i]);             
        } 

        VGAOUT8(0x3C4, SR42);
        VGAOUT8(0x3C5, restore->SRRegs[0x42]);
        VGAOUT8(0x3C4, SR4F);
        VGAOUT8(0x3C5, restore->SRRegs[0x4F]);
        VGAOUT8(0x3C4, SR50);
        VGAOUT8(0x3C5, restore->SRRegs[0x50]);
        VGAOUT8(0x3C4, SR57);
        VGAOUT8(0x3C5, restore->SRRegs[0x57]);
        VGAOUT8(0x3C4, SR5D);
        VGAOUT8(0x3C5, restore->SRRegs[0x5D]);

        /* DVP1 Clock and Data Pads Driving: */
        VGAOUT8(0x3C4, SR65);
        VGAOUT8(0x3C5, restore->SRRegs[0x65]);
           
        /*=* restore VCK, LCDCK and ECK *=*/
        if (pVia->Chipset >= VIA_K8M800)
        {
            /* VCK: */
            VGAOUT8(0x3c4, SR44);
            VGAOUT8(0x3c5, restore->SRRegs[0x44]);
            VGAOUT8(0x3c4, SR45);
            VGAOUT8(0x3c5, restore->SRRegs[0x45]);
            VGAOUT8(0x3c4, SR46);
            VGAOUT8(0x3c5, restore->SRRegs[0x46]);
            /* Reset VCK PLL */
            VGAOUT8(0x3c4, SR40);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x02);    /* Set SR40[1] to 1 */
            VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xFD);    /* Set SR40[1] to 0 */

            /* LCK: */
            VGAOUT8(0x3c4, SR4A);
            VGAOUT8(0x3c5, restore->SRRegs[0x4A]);
            VGAOUT8(0x3c4, SR4B);
            VGAOUT8(0x3c5, restore->SRRegs[0x4B]);
            VGAOUT8(0x3c4, SR4C);
            VGAOUT8(0x3c5, restore->SRRegs[0x4C]);
            /* Reset LCK PLL */
            VGAOUT8(0x3c4, SR40);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x04);    /* Set SR40[2] to 1 */
            VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xFB);    /* Set SR40[2] to 0 */

            /* ECK: */
            VGAOUT8(0x3c4, SR47);
            VGAOUT8(0x3c5, restore->SRRegs[0x47]);
            VGAOUT8(0x3c4, SR48);
            VGAOUT8(0x3c5, restore->SRRegs[0x48]);
            VGAOUT8(0x3c4, SR49);
            VGAOUT8(0x3c5, restore->SRRegs[0x49]);
            /* Reset ECK PLL */
            VGAOUT8(0x3c4, SR40);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x01);    /* Set SR40[0] to 1 */
            VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xFE);    /* Set SR40[0] to 0 */
        }
        else
        {
            /* LCK: */
            VGAOUT8(0x3c4, SR44);
            VGAOUT8(0x3c5, restore->SRRegs[0x44]);
            VGAOUT8(0x3c4, SR45);
            VGAOUT8(0x3c5, restore->SRRegs[0x45]);
            /* Reset LCK PLL */
            VGAOUT8(0x3c4, SR40);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x01);    /* Set SR40[0] to 1 */
            VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xFE);    /* Set SR40[0] to 0 */

            /* VCK: */
            VGAOUT8(0x3c4, SR46);
            VGAOUT8(0x3c5, restore->SRRegs[0x46]);
            VGAOUT8(0x3c4, SR47);
            VGAOUT8(0x3c5, restore->SRRegs[0x47]);
            /* Reset VCK PLL */
            VGAOUT8(0x3c4, SR40);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x02);    /* Set SR40[1] to 1 */
            VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xFD);    /* Set SR40[1] to 0 */

            /* ECK: */
            VGAOUT8(0x3c4, SR48);
            VGAOUT8(0x3c5, restore->SRRegs[0x48]);
            VGAOUT8(0x3c4, SR49);
            VGAOUT8(0x3c5, restore->SRRegs[0x49]);
            /* Reset ECK PLL */
            VGAOUT8(0x3c4, SR40);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x04);    /* Set SR40[2] to 1 */
            VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xFB);    /* Set SR40[2] to 0 */
        }
   
        /* Restore CRTC controller extended regs: */
        VGAOUT8(vgaCRIndex, CR08);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x08]);
        VGAOUT8(vgaCRIndex, CR09);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x09]);

        VGAOUT8(vgaCRIndex, CR13);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x13]);
        VGAOUT8(vgaCRIndex, CR30);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x30]);
        VGAOUT8(vgaCRIndex, CR31);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x31]);
        VGAOUT8(vgaCRIndex, CR32);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x32]);
        VGAOUT8(vgaCRIndex, CR33);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x33]);

        /* From CR35 to CR47 */
        i = 0x35;

        while (1)
        {
            VGAOUT8(vgaCRIndex, i);
            VGAOUT8(vgaCRReg, restore->CRTCRegs[i]);
   
            if (i == 0x47)
            {
               break;
            }
            else
            {
               i++;
            }
        }
   
        /* Starting Address: */
        VGAOUT8(vgaCRIndex, CR0C);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x0C]);
        VGAOUT8(vgaCRIndex, CR0D);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x0D]);
        VGAOUT8(vgaCRIndex, CR48);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x48]);
        VGAOUT8(vgaCRIndex, CR34);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x34]);
   
        /* From CR49 to CRFF */
        for (i=0x49; i<0x100; i++)
        {
            if (i == 0x91)
            {
                /*  LCD Panel could use software power sequence, so we should resotore it 
                individually to avoid incorrect enable LCD. */
            }
            else
            {
                VGAOUT8(vgaCRIndex, i);
                VGAOUT8(vgaCRReg, restore->CRTCRegs[i]);
            }
        }
           
        /* Restore for VT1636 LVDS: */
        if (pBIOSInfo->LVDSSettingInfo.ChipID == VIA_VT1636)
        {
            VIARestoreLVDS_VT1636(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo), restore);
        }
        else if(pBIOSInfo->LVDSSettingInfo2.ChipID == VIA_VT1636)
        {
            VIARestoreLVDS_VT1636(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo2), restore);
        }
   
        if (pBIOSInfo->DefaultActiveDevice & VIA_DEVICE_LCD)
        {
            if (pBIOSInfo->LVDSSettingInfo.ChipID != VIA_INTEGRATED_LVDS)
            {
                VIAEnableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
            }

            if (pBIOSInfo->IsSpreadSpectrumEnabled)
            {
                if ((pBIOSInfo->Chipset == VIA_CX700) ||
                    (pBIOSInfo->Chipset == VIA_VX800))
                {
                    VGAOUT8(0x3c4, SR3D);
                    VGAOUT8(0x3c5, VGAIN8(0x3c5) | BIT0);
                    VGAOUT8(0x3c4, SR1E);
                    VGAOUT8(0x3c5, VGAIN8(0x3c5) | BIT3);
                }
                else
                {
                    VGAOUT8(0x3c4, SR2C);
                    VGAOUT8(0x3c5, VGAIN8(0x3c5) | BIT0);
                    VGAOUT8(0x3c4, SR1E);
                    VGAOUT8(0x3c5, VGAIN8(0x3c5) | BIT3);
                }        
            }
        }

        /* Use Software power-sequence to turn on TTL-LCD. */
        if ((pBIOSInfo->Chipset == VIA_VX800) &&
        (pBIOSInfo->LVDSSettingInfo.DIPort == VIA_DI_LVDS0DVP1) &&
        ((restore->CRTCRegs[0x91] & 0x13) == 0x13))
        {
            VIAEnableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
        }
        else
        {
            VGAOUT8(vgaCRIndex, CR91);
            VGAOUT8(vgaCRReg, restore->CRTCRegs[CR91]);
        }

        /* Reset clock */
        VGAOUT8(0x3c2, VGAIN8(0x3cc));
    } /* non-using device switch hotkey */
   
    /* Restore HW Cursor state */
    VIASETREG(VIA_REG_CURSOR_MODE, restore->dwCursorMode);
    VIASETREG(VIA_REG_CURSOR_POS, restore->dwCursorPOS);
    VIASETREG(VIA_REG_CURSOR_ORG, restore->dwCursorORG);
    VIASETREG(VIA_REG_CURSOR_BG, restore->dwCursorBG);
    VIASETREG(VIA_REG_CURSOR_FG, restore->dwCursorFG);
       
    if (pBIOSInfo->DefaultActiveDevice & VIA_DEVICE_CRT1)
    {
        VGAOUT8(0x3d4, 0x36);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0xCF);
    }

    vgaHWProtect(pScrn, FALSE);
   
    return;
}


static void VIAWriteMode(ScrnInfoPtr pScrn, vgaRegPtr vgaSavePtr,
                         VIARegPtr restore)
{
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAWriteMode\n"));

    /* Set up the mode.  Don't clear video RAM. */
    if (pVia->IsSecondary)
    {
        VIASetModeForMHS(pBIOSInfo);
    }
    else if (pBIOSInfo->Hotkey)
    {
        VIAPrimarySetMode(pScrn);
    }
    else
    {
        VIASetMode(pBIOSInfo);
    }

    if((pBIOSInfo->s3utility==TRUE)&&(pBIOSInfo->SAMM))
    {
        if(pVia->ActiveDevice & VIA_DEVICE_CRT1)
            VIACRTEnable();
                    
        if(pVia->ActiveDevice & VIA_DEVICE_DFP)
            VIAEnableDFP(pBIOSInfo);

        if(pVia->ActiveDevice & VIA_DEVICE_LCD)
            VIAEnableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
    }

    /* Enable down scaling function. */
    if (pBIOSInfo->IsDownScaleEnable)
    {
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)
        {
            VIASetDownScalingEngineReg(pScrn, pBIOSInfo->LVDSSettingInfo.IGAPath);
        }
        else if (pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP)
        {                
            VIASetDownScalingEngineReg(pScrn, pBIOSInfo->TMDSSettingInfo.IGAPath);
        }            
    } 
    else
    {
        /* When we switch device from DVI only to CRT only, the down-scaling function have to be disabled. */
        VIADisableDownScaling(pScrn);
    }
        
    pBIOSInfo->s3utility=FALSE;
    /* Restore the DAC.*/
    if (pBIOSInfo->FirstInit ) {
        vgaHWRestore(pScrn, vgaSavePtr, VGA_SR_CMAP);
    }

    pBIOSInfo->FirstInit = FALSE;
    /* Enable the graphics engine. */
    if (!pVia->NoAccel)
        VIAInitialize2DEngine(pScrn);

#ifdef XF86DRI
    VIAInitialize3DEngine(pScrn);
#endif        
    return;
}


static Bool VIAMapMMIO(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    vgaHWPtr hwp;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAMapMMIO\n"));

    pVia->FrameBufferBase = pVia->PciInfo->memBase[0];
    pVia->MmioBase = pVia->PciInfo->memBase[1];

    xf86DrvMsg(pScrn->scrnIndex, X_PROBED,
               "mapping MMIO @ 0x%lx with size 0x%x\n",
               pVia->MmioBase, VIA_MMIO_REGSIZE);

    pVia->MapBase = xf86MapPciMem(pScrn->scrnIndex, VIDMEM_MMIO, pVia->PciTag,
                                  pVia->MmioBase,
                                  VIA_MMIO_REGSIZE);
    pBIOSInfo->MapBase = pVia->MapBase;
    MMIOMapBase = pVia->MapBase + 0x8000;

    xf86DrvMsg(pScrn->scrnIndex, X_PROBED,
               "mapping BitBlt MMIO @ 0x%lx with size 0x%x\n",
               pVia->MmioBase + VIA_MMIO_BLTBASE, VIA_MMIO_BLTSIZE);

    pVia->BltBase = xf86MapPciMem(pScrn->scrnIndex, VIDMEM_MMIO, pVia->PciTag,
                                  pVia->MmioBase + VIA_MMIO_BLTBASE,
                                  VIA_MMIO_BLTSIZE);

    if (!pVia->MapBase || !pVia->BltBase) {
        xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                   "Internal error: cound not map registers\n");
        return FALSE;
    }

    /* Memory mapped IO for Video Engine */
    pVia->VidMapBase = pVia->MapBase + 0x200;

    VIAEnableMMIO(pScrn);
    hwp = VGAHWPTR(pScrn);
    vgaHWGetIOBase(hwp);
	if (!vgaHWMapMem(pScrn))
	    return FALSE;

    return TRUE;
}


static Bool VIAMapFB(ScrnInfoPtr pScrn)
{
    DevUnion* pPriv;
    VIAEntPtr pVIAEnt;
    VIAPtr    pVia;
    pPriv =xf86GetEntityPrivate(pScrn->entityList[0],gVIAEntityIndex);
    pVIAEnt = pPriv->ptr;
    pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAMapFB\n"));
    xf86DrvMsg(pScrn->scrnIndex, X_PROBED,
               "mapping framebuffer @ 0x%lx with size 0x%lx\n",
               pVia->FrameBufferBase, pVia->videoRambytes);

    if (pVia->videoRambytes) {
        pVia->FBBase = xf86MapPciMem(pScrn->scrnIndex, VIDMEM_FRAMEBUFFER,
                                     pVia->PciTag, pVia->FrameBufferBase,
                                     pVia->videoRambytes);
        pBIOSInfo->FBBase = pVia->FBBase;
        pBIOSInfo->videoRambytes = pVia->videoRambytes;

        if (!pVia->FBBase) {
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                       "Internal error: could not map framebuffer\n");
            return FALSE;
        }

        pVia->FBStart = pVia->FBBase;

        if (pVia->DynamicRotation)
        {
            /* When dynamic rotation is enabled, the framebuffer size will be VirtualX x VirtualX. */
            pVia->FBFreeStart = (pScrn->displayWidth * pScrn->bitsPerPixel >> 3) * pScrn->virtualX;
        }
        else
        {            
            pVia->FBFreeStart = (pScrn->displayWidth * pScrn->bitsPerPixel >> 3) * pScrn->virtualY;
        }
        
        pVia->FBFreeEnd = pVia->videoRambytes;

        xf86DrvMsg(pScrn->scrnIndex, X_PROBED,
                   "Frame buffer start: %p, free start: 0x%x end: 0x%x\n",
                   pVia->FBStart, pVia->FBFreeStart, pVia->FBFreeEnd);
    }

    pScrn->memPhysBase = pVia->PciInfo->memBase[0];
    pScrn->fbOffset = 0;
    if(pVia->IsSecondary) pScrn->fbOffset = pVIAEnt->pPrimaryScrn->videoRam << 10;

    return TRUE;
}


static void VIAUnmapMem(ScrnInfoPtr pScrn)
{
    VIAPtr pVia;

    pVia = VIAPTR(pScrn);

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAUnmapMem\n"));

    if (pVia->MapBase) {
        xf86UnMapVidMem(pScrn->scrnIndex, (pointer)pVia->MapBase,
                        VIA_MMIO_REGSIZE);
    }

    if (pVia->BltBase) {
        xf86UnMapVidMem(pScrn->scrnIndex, (pointer)pVia->BltBase,
                        VIA_MMIO_BLTSIZE);
    }

    if (pVia->FBBase) {
        xf86UnMapVidMem(pScrn->scrnIndex, (pointer)pVia->FBBase,
                        pVia->videoRambytes);
    }
   
    return;
}
 

static Bool VIAScreenInit(int scrnIndex, ScreenPtr pScreen,
                          int argc, char **argv)
{
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    vgaHWPtr hwp = VGAHWPTR(pScrn);
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    PVIDDATA pVidData = pVia->pVidData;
    int ret;

    unsigned char minorNum;
    unsigned int TmpSwapValue;

    int ScrnIndex = pScrn->scrnIndex;
    LPVIAGRAPHICINFO lpVIAGraphicInfo =  &(pVidData->VIAGraphicInfo);
    
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAScreenInit\n"));    

    /* For MAMM*/
    if (!pVia->IsMAMMEnable)
    { 
        vgaHWUnlock(hwp);
    }
    
    if (!VIAMapFB(pScrn))
        return FALSE;

    if (!VIAMapMMIO(pScrn))
        return FALSE;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- Mem Mapped\n"));

    /* read minor number of video device from BIOS register 3C5.32 */
    /* this is for AP use dynamical method to open device */
    VGAOUT8(0x3c4, 0x32);
    minorNum = VGAIN8(0x3c5);	
    if(minorNum < 2)
        minorNum = 2;

#ifdef XF86DRI
        pVia->directRenderingEnabled = VIADRIScreenInit(pScreen);
#endif

    VIASave(pScrn);

    if (pBIOSInfo->IsSpreadSpectrumEnabled)
    {                 
        /* Disable spread spectrum temporarily, we will turn it on when set mode. */
        VGAOUT8(0x3c4, SR3D);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xFE);

        VGAOUT8(0x3c4, SR1E);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xF7);        
    }

    vgaHWBlankScreen(pScrn, FALSE);   

    if (!VIAModeInit(pScrn, pScrn->currentMode))
        return FALSE;

#ifdef RANDRV12_TEST_ENABLE

    if (!xf86SetDesiredModes(pScrn))
        return FALSE;            
    
#endif

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- State saved\n"));

    /* Darken the screen for aesthetic reasons and set the viewport */
    VIASaveScreen(pScreen, SCREEN_SAVER_ON);
    pScrn->AdjustFrame(scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- Blanked\n"));

    miClearVisualTypes();

    if (pScrn->bitsPerPixel > 8 && !pVia->IsSecondary)
    {
        if (!miSetVisualTypes(pScrn->depth, TrueColorMask,
                              pScrn->rgbBits, pScrn->defaultVisual))
            return FALSE;
        if (!miSetPixmapDepths())
            return FALSE;
    }
    else
    {
        if (!miSetVisualTypes(pScrn->depth, miGetDefaultVisualMask(pScrn->depth),
                              pScrn->rgbBits, pScrn->defaultVisual))
            return FALSE;
        if (!miSetPixmapDepths())
            return FALSE;
    }

#ifdef RANDRV12_TEST_ENABLE
    /* Reserverd a big enough space for rotation use. */    
    pVia->RotateShadowOffset[0] = pVia->FBFreeStart;
    pVia->RotateShadowOffset[1] = pVia->FBFreeStart + ((pScrn->virtualX) * (pScrn->virtualY)* (pScrn->bitsPerPixel/8));    
    pVia->NumOfShadowInUse = 0x0;
    pVia->FBFreeStart += (pScrn->virtualX * pScrn->virtualY * (pScrn->bitsPerPixel/8))*2;    
#endif


    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- Visuals set up\n"));

    ret = VIAInternalScreenInit(scrnIndex, pScreen);

    if (!ret)
        return FALSE;

    xf86SetBlackWhitePixels(pScreen);
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- B & W\n"));

    if (pScrn->bitsPerPixel > 8) {
        VisualPtr visual;

        visual = pScreen->visuals + pScreen->numVisuals;
        while (--visual >= pScreen->visuals) {
            if ((visual->class | DynamicClass) == DirectColor) {
                visual->offsetRed = pScrn->offset.red;
                visual->offsetGreen = pScrn->offset.green;
                visual->offsetBlue = pScrn->offset.blue;
                visual->redMask = pScrn->mask.red;
                visual->greenMask = pScrn->mask.green;
                visual->blueMask = pScrn->mask.blue;
            }
        }
    }

#ifdef USE_FB
    /* must be after RGB ordering fixed */
    fbPictureInit(pScreen, 0, 0);
#endif
    /* for 2D accel fail */
    if (!pVia->NoAccel)
    {
        VIAInitAccel(pScreen);
    }
#ifdef X_USE_LINEARFB
    else
    {
        BoxRec AvailFBArea;

        AvailFBArea.x1 = 0;
        AvailFBArea.y1 = 0;
        AvailFBArea.x2 = pScrn->displayWidth;
        AvailFBArea.y2 = pScrn->virtualY + 1;
	/*
	 * Update FBFreeStart also for other memory managers, since
	 * we steal one line to make xf86InitFBManager work.
	 */
    	pVia->FBFreeStart = (AvailFBArea.y2 + 1) * pVia->Bpl;
    	xf86InitFBManager(pScreen, &AvailFBArea);
    }
#endif /* X_USE_LINEARFB */
#ifdef XF86DRI
    /* for VIDEO play memory overlap PIXEL_MAP_CACHE */
    if (pVia->directRenderingEnabled) 
    {
    	if (!(VIADRIFBInit(pScreen, pVia))) {
    	    VIADRICloseScreen(pScreen);
    	    xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "[dri] frame buffer initialize fial .\n" );
    	    pVia->directRenderingEnabled = FALSE;
    	}
    }
#endif

    miInitializeBackingStore(pScreen);
    xf86SetBackingStore(pScreen);
    xf86SetSilkenMouse(pScreen);
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- Backing store set up\n"));

    if(!pVia->shadowFB)         /* hardware cursor needs to wrap this layer */
        VIADGAInit(pScreen);

    miDCInitialize(pScreen, xf86GetPointerScreenFuncs());
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- SW cursor set up\n"));

#ifndef RANDRV12_TEST_ENABLE
    if (!pVia->ForceSWCursor)
    {
        if (!VIAHWCursorInit(pScreen))
        {
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                       "Hardware cursor initialization failed\n");
        }
    }
#else
    if (!pVia->ForceSWCursor)
    {
        if(pVia->pScreensPublicInfo->UseHwCursor)
            VIASETREG(VIA_REG_CURSOR_MODE, pVia->pScreensPublicInfo->CursorInfo.HwCursorBufStart & 0x3FFFF00);                        

        pVia->pScreensPublicInfo->CursorInfo.CursorBG = 0xffffff;
        pVia->pScreensPublicInfo->CursorInfo.CursorFG = 0x0;

        VIAARGBCursorInit(pScrn);
        
        if(!xf86_cursors_init(pScreen, 64, 64,
            HARDWARE_CURSOR_TRUECOLOR_AT_8BPP |            
            HARDWARE_CURSOR_SOURCE_MASK_INTERLEAVE_64 |
            HARDWARE_CURSOR_AND_SOURCE_WITH_MASK |                                 
            HARDWARE_CURSOR_INVERT_MASK |
            HARDWARE_CURSOR_BIT_ORDER_MSBFIRST |
            HARDWARE_CURSOR_ARGB))
        {
            DEBUG(ErrorF("Fail to initial xf86_cursors_init()\n"));
        }        
    }
#endif


    if (pVia->shadowFB)
    {
        RefreshAreaFuncPtr refreshArea = VIARefreshArea;
        
        if(pVia->SWRotate || pVia->RandRRotation)
        {            
            switch(pScrn->bitsPerPixel)
            {
                case 8:
                    refreshArea = VIARefreshArea8;
                    break;
                case 16:
                    refreshArea = VIARefreshArea16;
                    break;
                case 32:                
                    refreshArea = VIARefreshArea32;
                    break;
            }           
        }
        ShadowFBInit(pScreen, refreshArea);
    }    
    
    if (pVia->RandRRotation)
    {
        #if XORG_VERSION_CURRENT >= XF86_VERSION_NUMERIC(6,8,0,0,0)
            #if XORG_VERSION_CURRENT >= XF86_VERSION_NUMERIC(6,9,0,0,0)            
            pScrn->DriverFunc = VIADriverFunc;
            #else
            pScrn->RRFunc = VIADriverFunc;
            #endif
        #endif
    }        

#ifdef RANDRV12_TEST_ENABLE
    /* For RandR v1.2 */
    if (!xf86CrtcScreenInit(pScreen)) {
       DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "xf86CrtcScreenInit Fail\n"));
       return FALSE;
    }    


#endif

    if (!miCreateDefColormap(pScreen))
        return FALSE;
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- Def Color map set up\n"));

    /* only 3123Ax IGA2 use 6-Bit LUT, others chipset IGA2 use 8-Bit LUT */
    /* So, except 3123Ax, we use 8-Bit LUT as default. */
    if ((pBIOSInfo->Chipset == VIA_CLE266) && (pBIOSInfo->ChipRev < 15))
    {
        pScrn->rgbBits = 6;
        VGAOUT8(0x3C4, SR15);
        VGAOUT8(0x3C5, VGAIN8(0x3C5) & 0xEF);
        if (!xf86HandleColormaps(pScreen, 256, 6, VIALoadPalette, NULL,
                                 CMAP_RELOAD_ON_MODE_SWITCH | CMAP_PALETTED_TRUECOLOR))
        {
            return FALSE;
        }
    }
    else
    {
        pScrn->rgbBits = 8;
        VGAOUT8(0x3C4, SR15);
        VGAOUT8(0x3C5, VGAIN8(0x3C5) | 0x80);
        if (!xf86HandleColormaps(pScreen, 256, 8, VIALoadPalette, NULL,
                                 CMAP_RELOAD_ON_MODE_SWITCH | CMAP_PALETTED_TRUECOLOR))
        {
            return FALSE;
        }
    }

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- Palette loaded\n"));

    if ((pBIOSInfo->Chipset == VIA_VX800) &&
        (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) &&
        (pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_TTL_LCD))
    {
        /* IGA2 Screen Off */
        VGAOUT8(0x3D4, 0x6B);
        VGAOUT8(0x3D5, VGAIN8(0x3D5) | 0x04); 
    }
    
    if ((pBIOSInfo->Chipset == VIA_VX800) &&
        (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) &&
        (pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_TTL_LCD))
    {
        /* IGA2 Screen ON */
        VGAOUT8(0x3D4, 0x6B);
        VGAOUT8(0x3D5, VGAIN8(0x3D5) & 0x0B);
    }
    
    vgaHWBlankScreen(pScrn, TRUE);

    pVia->CloseScreen = pScreen->CloseScreen;
    pScreen->SaveScreen = VIASaveScreen;    
    pScreen->CloseScreen = VIACloseScreen;

    
    xf86DPMSInit(pScreen, VIADPMS, 0);
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- DPMS set up\n"));

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- Color maps etc. set up\n"));

#ifdef XF86DRI

    if (pVia->directRenderingEnabled) 
    {
        pVia->directRenderingEnabled = VIADRIFinishScreenInit(pScreen);
    }
    if (pVia->directRenderingEnabled) 
    {
        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "direct rendering enabled\n");
    }
    else 
    {
        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "direct rendering disabled\n");
    }
#endif

    /* Initial VIA Video Driver */
    FillGraphicInfo(pScrn);

    /* If there is bandwidth issue, block the H/W overlay */
    pVidData->dwRejectOverlay = FALSE;

    /* For capture 1 use S/W flip */
    pVidData->Address0_Switch = TRUE;
    pVidData->Address1_Switch = TRUE;
    pVidData->V3Address0_Switch = TRUE;
    pVidData->V3Address1_Switch = TRUE;

    pVidData->AUDIO_MUTE=TRUE;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "libddmpeg: Heap Begin 0x%lx,End 0x%lx\n", 
        lpVIAGraphicInfo->Screen[ScrnIndex].VideoHeapBase, lpVIAGraphicInfo->Screen[ScrnIndex].VideoHeapEnd));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "libddmpeg: dwXserverEnabled=%lu\n", 
        lpVIAGraphicInfo->dwXServerEnabled));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "dwWidth: 0x%lx\n", 
        lpVIAGraphicInfo->Screen[ScrnIndex].dwWidth));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "dwHeight: 0x%lx\n", 
        lpVIAGraphicInfo->Screen[ScrnIndex].dwHeight));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "dwSaveWidth: 0x%lx\n", 
        lpVIAGraphicInfo->Screen[ScrnIndex].dwSaveWidth));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "dwBPP: 0x%lx\n", 
        lpVIAGraphicInfo->Screen[ScrnIndex].dwBPP));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "dwRefreshRate: 0x%lx\n", 
        lpVIAGraphicInfo->Screen[ScrnIndex].dwRefreshRate));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "DRMEnabled: 0x%x\n", 
        lpVIAGraphicInfo->DRMEnabled));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "dwExpand: 0x%lx\n", 
        lpVIAGraphicInfo->Screen[ScrnIndex].dwExpand));

    /*  Init video structure, video flags & HW difference flags */		   
    VGAOUT8(0x3C4, 0x16); 
    pVidData->Save_3C4_16 = VGAIN8(0x3C5);
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "store 3c4.16 : %08x \n", inb(0x3C5)));
    VGAOUT8(0x3C4, 0x17); pVidData->Save_3C4_17 = VGAIN8(0x3C5);
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "store 3c4.17 : %08x \n", inb(0x3C5)));
    VGAOUT8(0x3C4, 0x18); pVidData->Save_3C4_18 = VGAIN8(0x3C5);
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "store 3c4.18 : %08x \n", inb(0x3C5)));
   
    /*  Init video structure, video flags & HW difference flags */
    vfInitVideoInfo(pVidData );
    
    viaInitVideo(pScreen);

    if(pBIOSInfo->MergedFB) {
        /* Psuedo xinerama */
        if(pBIOSInfo->UseVIAXinerama) {
            VIAnoPanoramiXExtension = FALSE;
            VIAXineramaExtensionInit(pScrn);
        }
    } 

    /* For Ubuntu 7.10 Rotation. 
       When graphic mode login, OS will create desktop size by referencing mode->HDisplay, VDisplay
       When console mode login, OS will create desktop size by referencing pScrn->virtualX,Y
    */
    if ((pVia->SWRotate == VIA_ROTATE_DEGREE_90) ||
        (pVia->SWRotate == VIA_ROTATE_DEGREE_270)||
        (pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_90)||
        (pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_270))
    {           
        TmpSwapValue = pScrn->currentMode->HDisplay;
        pScrn->currentMode->HDisplay = pScrn->currentMode->VDisplay;
        pScrn->currentMode->VDisplay = TmpSwapValue;
    }

    /* Add a rotation mode */
    if (pVia->DynamicRotation)
    {
        VIAAddRotationMode(pScrn);    
    }    

    if (serverGeneration == 1)
        xf86ShowUnusedOptions(pScrn->scrnIndex, pScrn->options);

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- Done\n"));
    return TRUE;
}


static int VIAInternalScreenInit(int scrnIndex, ScreenPtr pScreen)
{
    ScrnInfoPtr     pScrn;
    VIAPtr          pVia;
    VIABIOSInfoPtr  pBIOSInfo;
    int             width, height, displayWidth, shadowHeight;
    unsigned char   *FBStart;
    int             ret = TRUE;
        
    xf86DrvMsg(scrnIndex, X_INFO, "VIAInternalScreenInit\n");

    pScrn = xf86Screens[pScreen->myNum];
    pVia = VIAPTR(pScrn);
    pBIOSInfo = pVia->pBIOSInfo;

    displayWidth = pScrn->displayWidth;

    if ((pVia->SWRotate==VIA_ROTATE_DEGREE_90)||
        (pVia->SWRotate==VIA_ROTATE_DEGREE_270))
    {
        height = pScrn->virtualX;
        width = pScrn->virtualY;
        /* For FC7 */
        pScrn->virtualX = width;
        pScrn->virtualY = height;
    }
    else
    {
        width = pScrn->virtualX;
        height = pScrn->virtualY;        
    }

    if(pVia->RandRRotation)
        shadowHeight = max(width, height);
    else
        shadowHeight = height;
    
    if (pVia->shadowFB) {
        pVia->ShadowPitch = BitmapBytePad(pScrn->bitsPerPixel * width);
        pVia->ShadowPtr = xalloc(pVia->ShadowPitch * shadowHeight);
        displayWidth = pVia->ShadowPitch / (pScrn->bitsPerPixel >> 3);
        FBStart = pVia->ShadowPtr;
    }
    else {
        pVia->ShadowPtr = NULL;
        FBStart = pVia->FBStart;
    }    
    
    /* 
    *   Use Direct Access Window Rotate. 
    *   When screen will be rotated, we must report a rotated size to OS
    *   to generate a rotated screen data. 
    */
    switch (pBIOSInfo->HWRotate)
    {
        case VIA_ROTATE_DEGREE_90:
        case VIA_ROTATE_DEGREE_270:
            height = pScrn->virtualX;
            width = pScrn->virtualY;
            displayWidth = width;
            /* For FC7 */
            pScrn->virtualX = width;
            pScrn->virtualY = height;
            break;
    }
    
#ifdef USE_FB   

    /* For Dynamic Rotation */
    if(pVia->DynamicRotation)
    {
        height = width;
    }     
    
    ret = fbScreenInit(pScreen, FBStart, width, height,
                       pScrn->xDpi, pScrn->yDpi, displayWidth,
                       pScrn->bitsPerPixel);
#else    
    switch (pScrn->bitsPerPixel) {
    case 8:
        ret = cfbScreenInit(pScreen, FBStart, width, height, pScrn->xDpi,
                            pScrn->yDpi, displayWidth);
        break;

    case 16:
        ret = cfb16ScreenInit(pScreen, FBStart, width, height, pScrn->xDpi,
                              pScrn->yDpi, displayWidth);
        break;

    case 32:
        ret = cfb32ScreenInit(pScreen, FBStart, width, height, pScrn->xDpi,
                              pScrn->yDpi, displayWidth);
        break;

    default:
        xf86DrvMsg(scrnIndex, X_ERROR,
                   "Internal error: invalid bpp (%d) in SavageScreenInit\n",
                   pScrn->bitsPerPixel);
        ret = FALSE;
        break;
    }
#endif

    return ret;
}


static ModeStatus VIAValidMode(int scrnIndex, DisplayModePtr mode,
                               Bool verbose, int flags)
{
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
    VIAPtr      pVia = VIAPTR(pScrn);

    DEBUG(xf86DrvMsg(scrnIndex, X_INFO, "VIAValidMode\n"));

    /* check modes */
    switch (mode->CrtcHDisplay)
    {
        case 480:
            if (mode->CrtcVDisplay == 640)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
            
        case 640:
            if (mode->CrtcVDisplay == 480)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 720:
            if (mode->CrtcVDisplay == 480 || mode->CrtcVDisplay == 576)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 800:
            if (mode->CrtcVDisplay == 600 || mode->CrtcVDisplay == 480)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 848:
            if (mode->CrtcVDisplay == 480)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 856:
            if (mode->CrtcVDisplay == 480)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 960:
            if (mode->CrtcVDisplay == 600)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1000:
            if (mode->CrtcVDisplay == 600)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1088:
            if (mode->CrtcVDisplay == 612)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1024:
            if (mode->CrtcVDisplay == 512 || mode->CrtcVDisplay == 576 ||
                mode->CrtcVDisplay == 768  || mode->CrtcVDisplay == 600)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1152:
            if ((mode->CrtcVDisplay == 864) || (mode->CrtcVDisplay == 720))
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1200:
            if (mode->CrtcVDisplay == 720)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1280:
            if (mode->CrtcVDisplay == 720 || mode->CrtcVDisplay == 768 || mode->CrtcVDisplay == 800 ||
                mode->CrtcVDisplay == 960 || mode->CrtcVDisplay == 1024 || mode->CrtcVDisplay == 600)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1366:
            if (mode->CrtcVDisplay == 768)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1360:
            if (mode->CrtcVDisplay == 768)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1400:
            if (mode->CrtcVDisplay == 1050)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1440:
            if ((mode->CrtcVDisplay == 1050) || (mode->CrtcVDisplay == 900))
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1600:
            if ((mode->CrtcVDisplay == 1200) || (mode->CrtcVDisplay == 900) || 
                (mode->CrtcVDisplay == 1024))
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1680:
            if ((mode->CrtcVDisplay == 1050) || (mode->CrtcVDisplay == 600))
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1792:
            if (mode->CrtcVDisplay == 1344)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1856:
            if (mode->CrtcVDisplay == 1392)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1920:
            if ((pVia->Chipset != VIA_CLE266) &&
                ((mode->CrtcVDisplay == 1440) || 
                 (mode->CrtcVDisplay == 1080) ||
                 (mode->CrtcVDisplay == 1200)))
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 2048:
            if (mode->CrtcVDisplay == 1536)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
    }

    return MODE_BAD_HVALUE;
}


static void VIABIOSInit(VIAPtr pVia, ScrnInfoPtr pScrn, DisplayModePtr pMode)
{
    int             i;
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIABIOSInit begin\n"));

    pBIOSInfo->bitsPerPixel = pScrn->bitsPerPixel;
    pBIOSInfo->displayWidth = pScrn->displayWidth;
    pBIOSInfo->frameX1 = pScrn->frameX1;
    pBIOSInfo->frameY1 = pScrn->frameY1;
    pBIOSInfo->VirtualX = pScrn->virtualX;
    pBIOSInfo->VirtualY = pScrn->virtualY;
    pBIOSInfo->scrnIndex = pScrn->scrnIndex;
    pBIOSInfo->IGA1SettingInfo.IsActive = FALSE;
    pBIOSInfo->IGA2SettingInfo.IsActive = FALSE;

    pBIOSInfo->Clock = pMode->Clock;
    pBIOSInfo->HTotal = pMode->HTotal;
    pBIOSInfo->VTotal = pMode->VTotal;
    pBIOSInfo->HDisplay = pMode->HDisplay;
    pBIOSInfo->VDisplay = pMode->VDisplay;
    pBIOSInfo->CrtcHDisplay = pMode->CrtcHDisplay;
    pBIOSInfo->CrtcVDisplay = pMode->CrtcVDisplay;

    if (pBIOSInfo->FirstInit)
    {
        pBIOSInfo->SaveframeX1 = pScrn->frameX1;
        pBIOSInfo->SaveframeY1 = pScrn->frameY1;
        pBIOSInfo->SaveHDisplay = pMode->HDisplay;
        pBIOSInfo->SaveVDisplay = pMode->VDisplay;
        pBIOSInfo->SaveCrtcHDisplay = pMode->CrtcHDisplay;
        pBIOSInfo->SaveCrtcVDisplay = pMode->CrtcVDisplay;
        pBIOSInfo->SaveVirtualX = pScrn->virtualX;
        pBIOSInfo->SaveVirtualY = pScrn->virtualY;
    }

    pBIOSInfo->IsSecondary = pVia->IsSecondary;
    pBIOSInfo->HasSecondary = pVia->HasSecondary;

    if (pBIOSInfo->Hotkey)
    {
        pBIOSInfo->VIAModeList = pVia->VIAModeList;
    }

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "pBIOSInfo:bitsPerPixel=%d,displayWidth=%d,frameX1=%d,frameY1=%d\n",pBIOSInfo->bitsPerPixel,pBIOSInfo->displayWidth,pBIOSInfo->frameX1,pBIOSInfo->frameY1));     
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "pBIOSInfo:,VirtualX=%d,VirtualY=%d\n",pBIOSInfo->VirtualX,pBIOSInfo->VirtualY)); 
    
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "pMode:HTotal=%d,VTotal=%d,HDisplay=%d,VDisplay=%d\n",pMode->HTotal,pMode->VTotal,pMode->HDisplay,pMode->VDisplay));     
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "pMode:,CrtcHDisplay=%d,CrtcVDisplay=%d\n",pMode->CrtcHDisplay,pMode->CrtcVDisplay)); 
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIABIOSInit exit\n"));    
}


static void VIAPostFindMode(VIAPtr pVia, ScrnInfoPtr pScrn, DisplayModePtr pMode)
{
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAPostFindMode begin\n"));

    pVia->ModeReg.mode = pBIOSInfo->mode;
    pVia->ModeReg.resMode = pBIOSInfo->resMode;
    pVia->ModeReg.refresh = pBIOSInfo->Refresh;
    pVia->ModeReg.offsetWidthByQWord = pBIOSInfo->offsetWidthByQWord;
    pVia->ModeReg.countWidthByQWord = pBIOSInfo->countWidthByQWord;

    pScrn->frameX0 = 0;
    pScrn->frameY0 = 0;
    pScrn->frameX1 = pBIOSInfo->frameX1;
    pScrn->frameY1 = pBIOSInfo->frameY1;
    /* for 2D SW rotation 
        if use SW rotation, exchange frameX1 and frameY1*/
    if ((pVia->SWRotate == VIA_ROTATE_DEGREE_90) ||
        (pVia->SWRotate == VIA_ROTATE_DEGREE_270) ||
        pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_90 ||
        pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_270)        
    {
        pScrn->frameX1 = pBIOSInfo->frameY1;
        pScrn->frameY1 = pBIOSInfo->frameX1;
    }   

    if (pBIOSInfo->ActualDesktopSizeX > pBIOSInfo->HDisplay)
    {
        pMode->HDisplay = pBIOSInfo->HDisplay;
    }
    else
    {
        pMode->HDisplay = pBIOSInfo->ActualDesktopSizeX;
    }

    if (pBIOSInfo->ActualDesktopSizeY > pBIOSInfo->VDisplay)
    {
        pMode->VDisplay = pBIOSInfo->VDisplay;
    }
    else
    {
        pMode->VDisplay = pBIOSInfo->ActualDesktopSizeY;
    }
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "pBIOSInfo:mode=%d,resMode=%d,offsetWidthByQWord=%d,countWidthByQWord=%d\n",pBIOSInfo->mode,pBIOSInfo->resMode,pBIOSInfo->offsetWidthByQWord,pBIOSInfo->countWidthByQWord));     
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "pBIOSInfo:,frameX1=%d,frameY1=%d,HDisplay=%d,VDisplay=%d\n",pBIOSInfo->frameX1,pBIOSInfo->frameY1,pBIOSInfo->HDisplay,pBIOSInfo->VDisplay)); 
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "pBIOSInfo:,ActualDesktopSizeX=%d,ActualDesktopSizeY=%d\n",pBIOSInfo->ActualDesktopSizeX,pBIOSInfo->ActualDesktopSizeY)); 
    
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAPostFindMode exit\n"));    
}


void VIACorrectModeList(ScrnInfoPtr pScrn)
{
    DisplayModePtr mode;

    for (mode = pScrn->modes; ; mode = mode->next)
    {
        VIAGetModeSizeByName(mode->name, &mode->HDisplay, &mode->VDisplay);

        if (mode->next == pScrn->modes)
        {
            break;  /* The end of mode list. */
        }
    }
}


void VIAModeInitRunOnce(ScrnInfoPtr pScrn)
{
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;

    /* All the things can be added here if they just need to initialize once */
    /* until the system suspends or logout. */
    VIATMDSRegInit(pBIOSInfo);
}


static Bool VIAModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
{
    vgaHWPtr        hwp = VGAHWPTR(pScrn);
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    VIARegPtr       new = &pVia->ModeReg;
    vgaRegPtr       vganew = &hwp->ModeReg;
    PVIDDATA        pVidData = pVia->pVidData;
    Bool            bRet;
    
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAModeInit\n"));
    
    if (!(pVia->IsHWRotateEnabled && pVia->DynamicRotation))
    {
        VIACorrectModeList(pScrn);
    }        
    
    if (!vgaHWInit(pScrn, mode))
    {
        vgaHWBlankScreen(pScrn, TRUE);
        return FALSE;
    }

    pScrn->vtSema = TRUE;

    if (!pVia->IsVIAModeInitRunOnceDone)
    {
        VIAModeInitRunOnce(pScrn);

        /* Just run once. */
        pVia->IsVIAModeInitRunOnceDone = TRUE;
    }

    VIABIOSInit(pVia, pScrn, mode);
    
    if(!pBIOSInfo->SAMM)
    {        
        VIASetDisplayPath(pScrn);

        /* when the system resumes from suspends, just keep the user settings. */
        if (pBIOSInfo->DuoView && (!pVia->IsSuspending))
        {
            VIADeviceDispatchDuoView(pScrn);
        }

        /* Check what type of cursor (SW/HW) to use. */
        VIACheckCursorTypeToUse(pScrn);
    }

    if (pBIOSInfo->Hotkey)
    {
        if (!VIAFindModeUseModeList(pBIOSInfo))
        {
            vgaHWBlankScreen(pScrn, TRUE);
            return FALSE;
        }
    }
    else
    {
        if (!VIAFindModeUseBIOSTable(pBIOSInfo))
        {
            vgaHWBlankScreen(pScrn, TRUE);
            return FALSE;
        }
    }

    VIAPostFindMode(pVia, pScrn, mode);

    /* Enable Direct Access Window Rotate. */
    if (pVia->IsHWRotateEnabled) {
        EnableDirectAccessWindow(pScrn);
    }

    VIAWriteMode(pScrn, vganew, new);

    /* pass graphic info to via_v4l kernel module */
    /* Coz mode changes, some member in VIAGraphicInfo need to modify */
    FillGraphicInfo(pScrn);

    DBG_DD(("SWOV:  VIAVidSet2DInfo\n"));

    /* Save MCLK value */
    VGAOUT8(0x3C4, 0x16); pVidData->Save_3C4_16 = VGAIN8(0x3C5);
    DBG_DD(("        3c4.16 : %08x \n",VGAIN8(0x3C5)));
    VGAOUT8(0x3C4, 0x17); pVidData->Save_3C4_17 = VGAIN8(0x3C5);
    DBG_DD(("        3c4.17 : %08x \n",VGAIN8(0x3C5)));
    VGAOUT8(0x3C4, 0x18); pVidData->Save_3C4_18 = VGAIN8(0x3C5);
    DBG_DD(("        3c4.18 : %08x \n",VGAIN8(0x3C5)));

    VIAAdjustFrame(pScrn->scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);
    /* Insert a timer */
    if  ((pVia->IsHotkeyTimerEnabled) && (!pBIOSInfo->SAMM))
    {
        if(!pVia->devicesTimer)
        {
            pVia->devicesTimer = TimerSet(NULL, 0, 30, VIASyncTimer, pScrn);
        }
    }
    else
    {
        if(pVia->devicesTimer)
        {
            TimerCancel(pVia->devicesTimer);
            pVia->devicesTimer=NULL;
        }
    }        
    
    return TRUE;
}


static
Bool VIACloseScreen(int scrnIndex, ScreenPtr pScreen)
{
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
    vgaHWPtr    hwp = VGAHWPTR(pScrn);
    VIAPtr      pVia = VIAPTR(pScrn);
    vgaRegPtr   vgaSavePtr = &hwp->SavedReg;
    VIARegPtr   viaSavePtr = &pVia->SavedReg;
    CARD32      dwCursorMode;
    PVIDDATA pVidData = pVia->pVidData;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;

#if (defined(PMLINK))&&(LINUX_VERSION_CODE>132116)
    char buf[BUFFER_SIZE];
    struct cn_msg *data;
    int len=-1;
#endif  /* PMLINK **/

    if (pVia->devicesTimer)
    {
        TimerCancel(pVia->devicesTimer);
    }
    pVia->devicesTimer = NULL;
#ifdef  VIA_ACPI

    if (xf86LoaderCheckSymbol("ViaCloseACPI"))
    {
        void (*acpiClose)(void) = NULL;

        acpiClose = LoaderSymbol("ViaCloseACPI");

        if (acpiClose ) 
        {
            (*acpiClose )();
        }
    }
  
#endif
    
#if (defined(PMLINK))&&(LINUX_VERSION_CODE>132116)
    if (pVia->eventTimer)
        TimerCancel(pVia->eventTimer);

    pVia->eventTimer=NULL;
    do{    
        memset(buf,0,sizeof(buf));
        data=(struct cn_msg *)buf;
        data->id.idx=VIA_LINK_GROUP;
        data->id.val=VIA_LINK_VAL;
        data->seq=getpid();
        data->ack=0;  
        data->flags=LINK_UNREGISTER;
        len= VIAPMLinkSendMSG(sock_fd,data);
    }while (len==-1);
    close(sock_fd); 
#endif /* PMLINK **/
    /*when log out X, if DRI is enabled, the DRI resource should be cleaned up*/
    if(pVia->directRenderingEnabled)
    {
        VIADRICloseScreen(pScreen);
    }

    DEBUG(xf86DrvMsg(scrnIndex, X_INFO, "VIACloseScreen\n"));
    xf86DrvMsg(scrnIndex, X_INFO, "VIACloseScreen\n");

    /* Clear direct access window. */
    DisableDirectAccessWindow(pScrn);
    
    /*Turn off color cursor here*/
    VIASETREG(PRIM_HI_CTRL, 0);
    VIASETREG(HI_CONTROL, 0);

    if (pScrn->vtSema)
    {
        viaExitVideo(pScrn);
        if(!(H5_UMA_CHIPID||H6_UMA_CHIPID))/*H5 needn't reset PCI*/
        {
            VGAOUT8(0x3c4, SR1A);
            VGAOUT8(0x3c5, viaSavePtr->SRRegs[0x1A] | 0x40);
        }
    }
 
    if (!pVia->IsSecondary)
    {
        /* clean viddata structure*/
        int i;

        /* Reset panning mode parameter */
        /* for SAMM*/
        for (i=0;i<2;i++)
        {
            pVidData->panning_x[i] = 0;
            pVidData->panning_y[i] = 0;
            pVidData->panning_old_x[i] = 0;
            pVidData->panning_old_y[i] = 0;
        }

        lpVIAGraphicInfo->dwXServerEnabled = FALSE;
        xf86DrvMsg(scrnIndex, X_INFO, "VIACloseScreen, lpVIAGraphicInfo->dwXServerEnabled = FALSE\n");            
        lpVIAGraphicInfo->DRMEnabled = FALSE;
    

        /* Diable Hardware Cursor */
        dwCursorMode = VIAGETREG(VIA_REG_CURSOR_MODE);
        VIASETREG(VIA_REG_CURSOR_MODE, dwCursorMode & 0xFFFFFFFE);
    }

    /* Wait Hardware Engine idle to exit graphic mode */
    WaitIdle();

    if ((pVia->VQEnable)&&(pScrn->vtSema))
    {
        /* if we use VQ, disable it before we exit */
        xf86DrvMsg(scrnIndex, X_INFO, "VIACloseScreen, VQ, disable \n");            
        
        switch( pVia->Chipset )   
        { 
        case VIA_P4M900: 
        case VIA_K8M890:
        case VIA_CN750:
        case VIA_VX800:
            break;
        default:  
            VIASETREG(0x43c, 0x00fe0000);
            VIASETREG(0x440, 0x00000004);
            VIASETREG(0x440, 0x40008c0f);
            VIASETREG(0x440, 0x44000000);
            VIASETREG(0x440, 0x45080c04);
            VIASETREG(0x440, 0x46800408);
            break;
        } 
    }

    if(pVia->HwCursorImage)
    {
        xfree(pVia->HwCursorImage);
        pVia->HwCursorImage = NULL;
    }
    if(pVia->HwIconImage)
    {
        xfree(pVia->HwIconImage);
        pVia->HwIconImage = NULL;
    }
    if(pVia->HIScalingPrimImage)
    {
        xfree(pVia->HIScalingPrimImage);
        pVia->HIScalingPrimImage = NULL;
    }
    if(pVia->HIScalingSecImage)
    {
        xfree(pVia->HIScalingSecImage);
        pVia->HIScalingSecImage = NULL;
    }
    if (pVia->AccelInfoRec)
    {
        XAADestroyInfoRec(pVia->AccelInfoRec);
        pVia->AccelInfoRec = NULL;
    }
    if (pVia->CursorInfoRec)
    {
        xf86DestroyCursorInfoRec(pVia->CursorInfoRec);
        pVia->CursorInfoRec = NULL;
    }
    if (pVia->ShadowPtr)
    {
        xfree(pVia->ShadowPtr);
        pVia->ShadowPtr = NULL;
    }
    if (pVia->DGAModes)
    {
        xfree(pVia->DGAModes);
        pVia->DGAModes = NULL;
    }
    if (pScrn->vtSema)
    {    
        xf86DrvMsg(scrnIndex, X_INFO, "VIACloseScreen, pScrn->vtSema is true \n");            
        /* Clear Memory to avoid garbage */
        memset(pVia->pBIOSInfo->FBBase, 0x00, pVia->pBIOSInfo->videoRambytes - VIA_FB_CURSOR_SIZE);                
        VIARestore(pScrn, vgaSavePtr, viaSavePtr);
        vgaHWLock(hwp);
        VIAUnmapMem(pScrn);
        vgaHWUnmapMem(pScrn);
    }

    /* For MAMM*/
    if (!pVia->IsMAMMEnable)
    {
        if(pVia->pInt10)
        {
            xf86FreeInt10(pVia->pInt10);
            pVia->pInt10 = NULL;
        }
    }
    pVia->IsVIAModeInitRunOnceDone = FALSE;
    pScrn->vtSema = FALSE;
    pScreen->CloseScreen = pVia->CloseScreen;

    return (*pScreen->CloseScreen)(scrnIndex, pScreen);
}

/*
 * This only gets called when a screen is being deleted.  It does not
 * get called routinely at the end of a server generation.
 */
static void VIAFreeScreen(int scrnIndex, int flags)
{
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];    
    VIAPtr      pVia = VIAPTR(pScrn);
    
    DEBUG(xf86DrvMsg(scrnIndex, X_INFO, "VIAFreeScreen\n"));
    xf86DrvMsg(scrnIndex, X_INFO, "VIAFreeScreen\n");
    /* delete the shared memory of VidData */
    if (shmdt((void *)pVia->pVidData) == -1)
    {
        DBG_DD(("shmdt failed\n"));
    }

    if (shmctl(pVia->shmid, IPC_RMID, 0) == -1)
    {
        DBG_DD(("shmctl(IPC_RMID) failed\n"));
    }
    
    if (xf86LoaderCheckSymbol("vgaHWFreeHWRec"))
        vgaHWFreeHWRec(xf86Screens[scrnIndex]);
    VIAUnmapMem(xf86Screens[scrnIndex]);
    VIAFreeRec(xf86Screens[scrnIndex]);
}

static Bool VIASaveScreen(ScreenPtr pScreen, int mode)
{
    return vgaHWSaveScreen(pScreen, mode);
}

int VIAGetIGAInUse(VIABIOSInfoPtr pBIOSInfo)
{
    switch (pBIOSInfo->ActiveDevice)
    {
        case VIA_DEVICE_CRT1:
            return pBIOSInfo->CRTSettingInfo.IGAPath;

        case VIA_DEVICE_LCD:
            return pBIOSInfo->LVDSSettingInfo.IGAPath;

        case VIA_DEVICE_DFP:
            return pBIOSInfo->TMDSSettingInfo.IGAPath;

        case VIA_DEVICE_CRT2:
            return pBIOSInfo->CRTSettingInfo2.IGAPath;

        default:
            /* Wrong Device ID! */
            return 0;
    }
}

/* This function for tuning XY position in xorg to position in on_screen.
   Only rotation need to care about this.*/
static void TuneXYToOnScreenXY(ScrnInfoPtr pScrn, int *x, int *y)
{
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    int tmpx = *x, tmpy = *y;

    if(!pVia->SWRotate && !pBIOSInfo->HWRotate)
    {
        return ;
    }

    /* rotation exist. to calculate the x y position according to the position in xorg.*/
    if(pVia->SWRotate == VIA_ROTATE_DEGREE_90 || pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_90)
    {
        *y = *x;
        *x = pBIOSInfo->VirtualX - tmpy - pBIOSInfo->HDisplay;        
    }
    else if(pVia->SWRotate == VIA_ROTATE_DEGREE_270 || pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_270)
    {
        *x = *y;
        *y = pBIOSInfo->VirtualY - tmpx - pBIOSInfo->VDisplay;
    }
    else if(pVia->SWRotate == VIA_ROTATE_DEGREE_180 || pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_180)
    {
        *x = pBIOSInfo->VirtualX - tmpx - pBIOSInfo->HDisplay;
        *y = pBIOSInfo->VirtualY - tmpy - pBIOSInfo->VDisplay;  
    }

    return ;    
}

void VIASetStartingAddress(ScrnInfoPtr pScrn, int x, int y)
{
    vgaHWPtr        hwp = VGAHWPTR(pScrn);
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    int             vgaCRIndex, vgaCRReg, vgaIOBase;
    CARD32          tmp, Base, tmpBase;
    Bool            bAdjustPrimaryStartAddr, bAdjustSecondaryStartAddr;
    unsigned long HalfScreen;
    int HalfLine;
    vgaIOBase = hwp->IOBase;
    vgaCRIndex = vgaIOBase + 4;
    vgaCRReg = vgaIOBase + 5;

    TuneXYToOnScreenXY(pScrn, &x, &y);

    Base = (y * pScrn->displayWidth + x) * (pScrn->bitsPerPixel / 8);
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASetStartingAddress begin\n"));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASetStartingAddress:pScrn->fbOffset=%lu,Base=%lu\n",pScrn->fbOffset,Base));

    /* For Dynamic Rotation. */
    if (pVia->DynamicRotation)
        return;

    if(pBIOSInfo->UseIGA2 && pBIOSInfo->TwoLCD)
    {
        if (!pVia->IsSecondary)
        {
            Base = Base >> 1;
            VGAOUT8(vgaCRIndex, CR62);
            tmp = VGAIN8(vgaCRReg) & 0x01;
            tmp |= (Base & 0x7F) << 1;
            VGAOUT8(vgaCRReg, tmp);
            VGAOUT8(vgaCRIndex, CR63);
            VGAOUT8(vgaCRReg, ((Base & 0x7F80) >> 7));
            VGAOUT8(vgaCRIndex, CR64);
            VGAOUT8(vgaCRReg, ((Base & 0x7F8000) >> 15));
            VGAOUT8(vgaCRIndex, CRA3);
            VGAOUT8(vgaCRReg, ((Base & 0x03800000) >> 23));
        }
        else
        {
            Base = (Base + pScrn->fbOffset) >> 1;
            VGAOUT16(vgaCRIndex, (Base & 0x00ff00) | CR0C);
            VGAOUT16(vgaCRIndex, ((Base & 0x00ff) << 8) | CR0D);
            VGAOUT16(vgaCRIndex, ((Base & 0xff0000) >> 8) | CR34);
            VGAOUT16(vgaCRIndex, ((Base & 0xff000000) >> 16) | CR48);
        }
    } 
    else
    {
        /* Check which starting should be updated: */
        if (pBIOSInfo->UseIGA2)
        {
            bAdjustPrimaryStartAddr = FALSE;
            bAdjustSecondaryStartAddr = TRUE;
        }
        else if (!pBIOSInfo->DuoView || pBIOSInfo->SAMM)
        {
            /* For single and SAMM case: */
            if (VIAGetIGAInUse(pBIOSInfo) == IGA1)
            {                
                bAdjustPrimaryStartAddr = TRUE;
                bAdjustSecondaryStartAddr = FALSE;
            }
            else
            {                
                bAdjustPrimaryStartAddr = FALSE;
                bAdjustSecondaryStartAddr = TRUE;
            }
        }
        else
        {
            /* We will set starting address when set mode or panning. */
            if (pBIOSInfo->IGA1SettingInfo.SetStartingAddrFirstTime)
            {
                bAdjustPrimaryStartAddr = TRUE;
                pBIOSInfo->IGA1SettingInfo.SetStartingAddrFirstTime = FALSE;
            }
            else
            {
                bAdjustPrimaryStartAddr = pBIOSInfo->IGA1SettingInfo.IsPanning;
            }

            if (pBIOSInfo->IGA2SettingInfo.SetStartingAddrFirstTime)
            {
                bAdjustSecondaryStartAddr = TRUE;
                pBIOSInfo->IGA2SettingInfo.SetStartingAddrFirstTime = FALSE;
            }
            else
            {
                bAdjustSecondaryStartAddr = pBIOSInfo->IGA2SettingInfo.IsPanning;
            }
        }

        /* Decide which image (primary or secondary) to show: */
        /* We will show primary image except the secondary device in SAMM case.  */
        if (pBIOSInfo->SAMM && (pBIOSInfo->ActiveDevice != pBIOSInfo->PrimaryDevice))
        {
            Base = Base + pScrn->fbOffset;
        }

        if (bAdjustPrimaryStartAddr)
        {
            tmpBase = Base >> 1;
            
            if(pBIOSInfo->MergedFB)
            {
                /* set IGA1 start address while mergedfb*/
                if((pBIOSInfo->Scrn2Position == viaRightOf)||(pBIOSInfo->Scrn2Position == viaBelow))
                {
          
                    VGAOUT16(vgaCRIndex,  0 | CR0C);
                    VGAOUT16(vgaCRIndex,  0 | CR0D);
                    VGAOUT16(vgaCRIndex,  0 | CR34);
                }
                else if(pBIOSInfo->Scrn2Position == viaLeftOf)
                {
                    HalfLine = pBIOSInfo->HalfLine*2;  
                    VGAOUT16(vgaCRIndex, (HalfLine & 0x00FF00) | CR0C);
                    VGAOUT16(vgaCRIndex, ((HalfLine& 0x0000FF) << 8) | CR0D);
                    VGAOUT16(vgaCRIndex, ((HalfLine & 0xFF0000) >> 8) | CR34);
                }
                else if(pBIOSInfo->Scrn2Position == viaAbove)
                {
                    HalfScreen = pBIOSInfo->HalfScreen*2;  
                    VGAOUT16(vgaCRIndex, (HalfScreen & 0x00FF00) | CR0C);
                    VGAOUT16(vgaCRIndex, ((HalfScreen& 0x0000FF) << 8) | CR0D);
                    VGAOUT16(vgaCRIndex, ((HalfScreen & 0xFF0000) >> 8) | CR34);
                    VGAOUT16(vgaCRIndex, ((HalfScreen & 0x03000000) >> 16) | CR48);
                }
            }
            else
            {
                VGAOUT16(vgaCRIndex, (tmpBase & 0x00ff00) | CR0C);
                VGAOUT16(vgaCRIndex, ((tmpBase & 0x00ff) << 8) | CR0D);
                VGAOUT16(vgaCRIndex, ((tmpBase & 0xff0000) >> 8) | CR34);
                VGAOUT16(vgaCRIndex, ((tmpBase & 0x1f000000) >> 16) | CR48);
            }
        }
        
        if (bAdjustSecondaryStartAddr)
        {
            tmpBase = Base >> 3;
            
            VGAOUT8(vgaCRIndex, CR62);

            if((pBIOSInfo->Chipset == VIA_PM800)||(pBIOSInfo->Chipset == VIA_P4M800PRO)||(pBIOSInfo->Chipset == VIA_K8M890)||(pBIOSInfo->Chipset == VIA_P4M890))
            {
                tmp = VGAIN8(vgaCRReg) & 0x01;
                tmp |= (tmpBase & 0x7E) << 1;
            }
            else
            {
                tmp = VGAIN8(vgaCRReg) & 0x01;
                tmp |= (tmpBase & 0x7F) << 1;
            }
            
            if(pBIOSInfo->MergedFB)
            {
                /* set IGA2 start address while mergedfb*/
                if(pBIOSInfo->Scrn2Position == viaRightOf)
                {
                    HalfLine =  pBIOSInfo->HalfLine;      
                    VGAOUT8(vgaCRReg, HalfLine & 0xFF);
                    VGAOUT8(vgaCRIndex, CR63);
                    VGAOUT8(vgaCRReg, ((HalfLine & 0xFF00) >>8));
                    VGAOUT8(vgaCRIndex, CR64);
                    VGAOUT8(vgaCRReg, ((HalfLine & 0xFF0000) >> 16));
                }
                else if(pBIOSInfo->Scrn2Position == viaBelow)
                {        
                    HalfScreen =  pBIOSInfo->HalfScreen;  
                    VGAOUT8(vgaCRReg, HalfScreen & 0xFF);
                    VGAOUT8(vgaCRIndex, CR63);
                    VGAOUT8(vgaCRReg, ((HalfScreen & 0xFF00) >>8));
                    VGAOUT8(vgaCRIndex, CR64);
                    VGAOUT8(vgaCRReg, ((HalfScreen & 0xFF0000) >> 16));
                    VGAOUT8(vgaCRIndex, CRA3);
                    VGAOUT8(vgaCRReg, ((HalfScreen & 0xFF000000) >> 24));
                }
                else if((pBIOSInfo->Scrn2Position == viaAbove)||(pBIOSInfo->Scrn2Position == viaLeftOf))
                {
                    VGAOUT8(vgaCRReg, 0);
                    VGAOUT8(vgaCRIndex, CR63);
                    VGAOUT8(vgaCRReg, 0);
                    VGAOUT8(vgaCRIndex, CR64);
                    VGAOUT8(vgaCRReg, 0);
                }
            }
            else
            {
                VGAOUT8(vgaCRReg, tmp);
                VGAOUT8(vgaCRIndex, CR63);
                VGAOUT8(vgaCRReg, ((tmpBase & 0x7F80) >> 7));
                VGAOUT8(vgaCRIndex, CR64);
                VGAOUT8(vgaCRReg, ((tmpBase & 0x7F8000) >> 15));
                VGAOUT8(vgaCRIndex, CRA3);
                VGAOUT8(vgaCRReg, ((tmpBase & 0x03800000) >> 23));
            }
        }        
    } 

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASetStartingAddress:pScrn->fbOffset=%lu,Base=%lu\n",pScrn->fbOffset,Base));
}

void VIAAdjustFrame(int scrnIndex, int x, int y, int flags)
{
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
    VIAPtr      pVia = VIAPTR(pScrn);
    int         fd0 = -1, fd1 = -1, fd2 = -1;
    DDUPDATEOVERLAY UpdateOverlayRec;
    ADJUSTFRAME AdjustFrame;
    PVIDDATA pVidData = pVia->pVidData;
    unsigned long handle = pVidData->VMI_Info.handle;
    LPVIDEOSTATUS lpVidStatus = &(pVidData->VidStatus);
    struct video_window      ov_win_cap1;
    struct video_window      ov_win_cap0;
    LPOVERLAYRECORD lpOverlayRecordMPEG = NULL;
    LPOVERLAYRECORD lpOverlayRecordSW = &pVidData->OverlayRecordSet.OverlayRecordSW[handle];
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo; 
    unsigned int isCapture1Enabled = 0; 

    DEBUG(xf86DrvMsg(scrnIndex, X_INFO, "VIAAdjustFrame\n"));

    /* now program the start address registers*/
    VIASetStartingAddress(pScrn, x, y);

    /* for vt3324,3293 capture1*/
    if(( lpVIAGraphicInfo->dwProjectIDRevision == UNICHROME_VT3324) ||(lpVIAGraphicInfo->dwProjectIDRevision == UNICHROME_VT3324M)  ||
       (lpVIAGraphicInfo->dwProjectIDRevision == UNICHROME_VT3324M2)  || ( lpVIAGraphicInfo->dwProjectIDRevision == UNICHROME_VT3293))
    {
        isCapture1Enabled = VIAGETREG(CAPTURE0_CONTROL|REG_Capture1_INDEX) & 0x1;
    }
    else
    {
        isCapture1Enabled = VIAGETREG(CAPTURE1_CONTROL) & 0x1;
    }
    
    /* Pass Panning (x, y) info to V4L */
    AdjustFrame.x = x;
    AdjustFrame.y = y;

    /* for SAMM*/
    pVidData->panning_x[pScrn->scrnIndex] = AdjustFrame.x;
    pVidData->panning_y[pScrn->scrnIndex] = AdjustFrame.y;

    if(pVidData->VidStatus.dwVideoFlagMPEG[handle]) 
    {
        UpdateOverlayRec = pVidData->UpdateOverlayMPEGBackup;
        lpOverlayRecordMPEG = &pVidData->OverlayRecordSet.OverlayRecordMPEG[handle];
        UpdateOverlayRec.dwColorSpaceLowValue = lpOverlayRecordMPEG->dwKeyLow;
        UpdateOverlayRec.rSrc.left = 0;
        UpdateOverlayRec.rSrc.top = 0;
        UpdateOverlayRec.rSrc.right = lpOverlayRecordMPEG->dwOriSrcWidth;
        UpdateOverlayRec.rSrc.bottom = lpOverlayRecordMPEG->dwOriSrcHeight;
        UpdateOverlayRec.rDest = lpOverlayRecordMPEG->dwOriDestRect;
        UpdateOverlayRec.dwFlags |= (DDOVER_SHOW | DDOVER_KEYDEST);
        pVidData->UpdateOverlayRec = UpdateOverlayRec;
        UpdateOverlay(pVidData, &pVidData->UpdateOverlayRec);
    } /* MPEG */
    if(pVidData->VidStatus.dwVideoFlagSW[handle]) 
    {
        UpdateOverlayRec = pVidData->UpdateOverlaySWBackup;
        lpOverlayRecordSW = &pVidData->OverlayRecordSet.OverlayRecordSW[handle];
        UpdateOverlayRec.dwColorSpaceLowValue = lpOverlayRecordSW->dwKeyLow;
        UpdateOverlayRec.rSrc.left = 0;
        UpdateOverlayRec.rSrc.top = 0;
        UpdateOverlayRec.rSrc.right = lpOverlayRecordSW->dwOriSrcWidth;
        UpdateOverlayRec.rSrc.bottom = lpOverlayRecordSW->dwOriSrcHeight;
        UpdateOverlayRec.rDest = lpOverlayRecordSW->dwOriDestRect;
        UpdateOverlayRec.dwFourcc = FOURCC_YUY2;
        UpdateOverlayRec.dwFlags |= (DDOVER_SHOW | DDOVER_KEYDEST);
        pVidData->UpdateOverlayRec = UpdateOverlayRec;
        UpdateOverlay(pVidData, &pVidData->UpdateOverlayRec);
    } /* SW */

    return;
}

Bool VIASwitchMode(int scrnIndex, DisplayModePtr mode, int flags)
{   
#ifdef RANDRV12_TEST_ENABLE
    DEBUG(xf86DrvMsg(scrnIndex, X_INFO, "VIASwitchMode\n"));
    DEBUG(xf86DrvMsg(scrnIndex, X_INFO, "xf86SetSingleMode\n"));
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
    return xf86SetSingleMode(pScrn, mode, RR_Rotate_0);

#else
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
    VIAPtr      pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    CARD32      TmpSwapValue;
    Bool        result;
    
    DEBUG(xf86DrvMsg(scrnIndex, X_INFO, "VIASwitchMode\n"));

    if ((pVia->DynamicRotation) && (pVia->IsHWRotateEnabled))
        return FALSE;

    if ((pVia->SWRotate == VIA_ROTATE_DEGREE_90) ||
        (pVia->SWRotate == VIA_ROTATE_DEGREE_270)||
        (pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_90)||
        (pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_270))
    {        
        TmpSwapValue = mode->HDisplay;
        mode->HDisplay = mode->VDisplay;
        mode->VDisplay = TmpSwapValue;               

        TmpSwapValue = pScrn->virtualX;
        pScrn->virtualX = pScrn->virtualY;
        pScrn->virtualY = TmpSwapValue;               
    }

    /* Wait Hardware Engine idle to switch graphicd mode */
    WaitIdle();

    if (pVia->VQEnable) 
    { 	
        /* if we use VQ, disable it before we exit */
        switch (pVia->Chipset)
        {
        case VIA_P4M890:             
        case VIA_K8M890:
            VIASETREG(0x41c, 0x00100000);
            VIASETREG(0x420, 0x74301000);
            break;
        default:  
            VIASETREG(0x43c, 0x00fe0000);
            VIASETREG(0x440, 0x00000004);
            VIASETREG(0x440, 0x40008c0f);
            VIASETREG(0x440, 0x44000000);
            VIASETREG(0x440, 0x45080c04);
            VIASETREG(0x440, 0x46800408);
            break;
        }
    }

    result = VIAModeInit(xf86Screens[scrnIndex], mode);

#ifdef USEHwIconSimulateHwCursor
    if(pVia->pScreensPublicInfo->UseHwCursor)
    {
        VIAHideCursor(pScrn);
        VIALoadIconImage(pScrn, (CARD32 *)(pVia->FBBase + pVia->pScreensPublicInfo->CursorInfo.HIBufStart));
        VIAShowCursor(pScrn);
    }
#endif

    /* Revert to the rotated size.(e.g. 1024x768 -> 768x1024)*/
    if ((pVia->SWRotate == VIA_ROTATE_DEGREE_90) ||
        (pVia->SWRotate == VIA_ROTATE_DEGREE_270)||
        (pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_90)||
        (pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_270))
    {                   
        TmpSwapValue = mode->HDisplay;
        mode->HDisplay = mode->VDisplay;
        mode->VDisplay = TmpSwapValue;       

        TmpSwapValue = pScrn->virtualX;
        pScrn->virtualX = pScrn->virtualY;
        pScrn->virtualY = TmpSwapValue;          
    }
    return result;
#endif    
}


void VIAEnableMMIO(ScrnInfoPtr pScrn)
{
    vgaHWPtr hwp = VGAHWPTR(pScrn);
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    unsigned char val;

    if (xf86IsPrimaryPci(pVia->PciInfo))
    {
        vgaHWSetStdFuncs(hwp);
    }
    else
    {
        vgaHWSetMmioFuncs(hwp, pVia->MapBase, 0x8000);
    }

    val = VGAIN8(0x3c3);
    VGAOUT8(0x3c3, val | 0x01);
    val = VGAIN8(VGA_MISC_OUT_R);
    VGAOUT8(VGA_MISC_OUT_W, val | 0x01);

    /* Unlock Extended IO Space */
    VGAOUT8(0x3c4, 0x10);
    VGAOUT8(0x3c5, 0x01);
    /* Unlock CRTC register protect */
    VGAOUT8(0x3d4, 0x47);
    VGAOUT8(0x3d5, VGAIN8(0x3d5) & ~0x01);

    /* Enable MMIO */
    if (pVia->IsSecondary || pBIOSInfo->UseIGA2)
    {
        VGAOUT8(0x3c4, 0x1a);
        val = VGAIN8(0x3c5);
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "secondary val = %x\n", val));
        VGAOUT8(0x3c5, val | 0x38);
    }
    else
    {
        VGAOUT8(0x3c4, 0x1a);
        val = VGAIN8(0x3c5);
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "primary val = %x\n", val));
        VGAOUT8(0x3c5, val | 0x68);
    }

    return;
}

void VIALoadPalette(ScrnInfoPtr pScrn, int numColors, int *indicies,
                    LOCO *colors, VisualPtr pVisual)
{
    VIAPtr pVia = VIAPTR(pScrn);
    int i, index;
    int sr1a, sr1b, cr67, cr6a;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIALoadPalette\n"));    
    
    if (pScrn->bitsPerPixel != 8)
        return;
    
    VGAOUT8(0x3C4, SR1A);
    sr1a = VGAIN8(0x3C5);
    VGAOUT8(0x3C4, SR1B);
    sr1b = VGAIN8(0x3C5);
    VGAOUT8(0x3D4, CR67);
    cr67 = VGAIN8(0x3D5);
    VGAOUT8(0x3D4, CR6A);
    cr6a = VGAIN8(0x3D5);

    /* Prepare for IGA2's LUT initialization: */
    VGAOUT8(0x3C4, SR1A);
    VGAOUT8(0x3C5, sr1a | 0x01);    /* Change Shadow to Secondary Display's LUT. */
    
    VGAOUT8(0x3C4, SR1B);    
    VGAOUT8(0x3C5, sr1b | 0x80);    /* Enable Secondary Display Engine. */
    
    VGAOUT8(0x3D4, CR67);
    VGAOUT8(0x3D5, cr67 & 0x3F);    /* Second Display Color Depth should be 8bpp. */

    /* If secondary display had been enable, we don't need to enable it again.*/
    VGAOUT8(0x3D4, CR6A);           
    if(!(VGAIN8(0x3D5) & 0x80))
    {
        /* Enable Second Display Channel. Please keep the setting sequence. */
        VGAOUT8(0x3D5, VGAIN8(0x3D5) & 0xBF);   /* Set CR6A[6] to 0. */
        VGAOUT8(0x3D5, VGAIN8(0x3D5) | 0x80);   /* Set CR6A[7] to 1. */
        VGAOUT8(0x3D5, VGAIN8(0x3D5) | 0x40);   /* Set CR6A[6] to 1. */
    }

    /* Fill in IGA2's LUT. */
    for (i = 0; i < numColors; i++)
    {
        index = indicies[i];
        VGAOUT8(0x3c8, index);
        VGAOUT8(0x3c9, colors[index].red);
        VGAOUT8(0x3c9, colors[index].green);
        VGAOUT8(0x3c9, colors[index].blue);
    }
    
    /* Prepare for initialize IGA1's LUT: */
    VGAOUT8(0x3C4, SR1A);
    VGAOUT8(0x3C5, sr1a); /* Change to Primary Display's LUT. */
    VGAOUT8(0x3C4, SR1B);
    VGAOUT8(0x3C5, sr1b);
    VGAOUT8(0x3D4, CR67);
    VGAOUT8(0x3D5, cr67);
    VGAOUT8(0x3D4, CR6A);
    VGAOUT8(0x3D5, cr6a);

    /* Fill in IGA1's LUT. */
    for (i = 0; i < numColors; i++)
    {
        index = indicies[i];
        VGAOUT8(0x3c8, index);
        VGAOUT8(0x3c9, colors[index].red);
        VGAOUT8(0x3c9, colors[index].green);
        VGAOUT8(0x3c9, colors[index].blue);
    }
}


static void VIADPMS(ScrnInfoPtr pScrn, int mode, int flags)
{
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    CARD8           cr36;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIADPMS\n"));    

    /* Clear DPMS setting */
    VGAOUT8(0x3d4, 0x36);
    cr36 = VGAIN8(0x3d5) & ~0x30;

    switch (mode)
    {
    case DPMSModeOn:
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT1)
        {
            VGAOUT(0x3d4, 0x36);
            VGAOUT(0x3d5, cr36);
        }
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)
        {
            /* Enable LCD */
            VIAEnableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
        }
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP)
        {
            /* Enable DFP */
            VIAEnableDFP(pBIOSInfo);
        }
        break;
    case DPMSModeStandby:
    case DPMSModeSuspend:
    case DPMSModeOff:
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT1)
        {
            VGAOUT(0x3d4, 0x36);
            VGAOUT(0x3d5, cr36 | 0x30);
        }
        if (pBIOSInfo->ActiveDevice & (VIA_DEVICE_DFP | VIA_DEVICE_LCD))
        {
            VIADisableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
        }
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP)
        {
            VIADisableDFP(pBIOSInfo);
        }
        break;
    default:
        xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                   "Invalid DPMS mode %d\n", mode);
        break;
    }

    return;
}

/* query NB to get frame buffer size */
int VIAGetFBSize(ScrnInfoPtr pScrn)
{
    unsigned long   configid,deviceid,FBSize = 0;
    int   VideoMemSize;
    int   DeviceFound = FALSE;
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    
    for ( configid = 0x80000000; configid < 0x80010800; configid += 0x100 )
    {
        outl((unsigned long)0xCF8,configid);
        deviceid = (inl((unsigned long)0xCFC)>>16) & 0xffff;

        switch (deviceid)
        {
            case CLE266:
            case KM400:
                outl((unsigned long)0xCF8,configid+0xE0);
                FBSize = inl((unsigned long)0xCFC);
                DeviceFound = TRUE; /* Found device id */
                break;
                
            case CN400_FUNCTION3:
            case CN700_FUNCTION3:
            case CX700_FUNCTION3:
            case KM800_FUNCTION3:
            case KM890_FUNCTION3:
            case P4M890_FUNCTION3:
            case P4M900_FUNCTION3:
            case VX800_FUNCTION3:
                outl((unsigned long)0xCF8,configid+0xA0);
                FBSize = inl((unsigned long)0xCFC);
                DeviceFound = TRUE; /* Found device id */
                break;

            default:    
                break;
         }

         if (DeviceFound)
         {
            break; /* Found device id, so exit for loop */
         }
    }

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Device ID = %lx\n", deviceid));
    
    FBSize = FBSize & 0x00007000;
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "FB Size = %lx\n", FBSize));
    
    if (pBIOSInfo->Chipset < VIA_CX700)
    {
        switch(FBSize)
        {
            case 0x00004000:
                VideoMemSize = (16 << 10);  /*16M*/              
                break;

            case 0x00005000:
                VideoMemSize = (32 << 10);  /*32M*/                
                break;

            case 0x00006000:
                VideoMemSize = (64 << 10);  /*64M*/                
                break;

            default:
                VideoMemSize = (32 << 10);  /*32M*/
                break;     
        }
    }
    else
    {
        switch(FBSize)
        {
            case 0x00001000:
                VideoMemSize = (8 << 10);   /*8M*/                
                break;

            case 0x00002000:
                VideoMemSize = (16 << 10);  /*16M*/               
                break;

            case 0x00003000:
                VideoMemSize = (32 << 10);  /*32M*/                
                break;

            case 0x00004000:
                VideoMemSize = (64 << 10);  /*64M*/               
                break;

            case 0x00005000:
                VideoMemSize = (128 << 10); /*128M*/
                break;

            case 0x00006000:
                VideoMemSize = (256 << 10); /*256M*/
                break;

            default:
                VideoMemSize = (32 << 10);  /*32M*/                
                break;
        }
    }

    return VideoMemSize;
}




/* Active Device according connected status */
Bool VIADeviceSelection(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    pBIOSInfo->SAMM = FALSE;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIADeviceSelection\n"));
    
    /* For MAMM*/
    if (!pVia->IsMAMMEnable)
    {    
        pBIOSInfo->DefaultActiveDevice = BIOS_GetActiveDevice(pScrn);
    }
    else
    {
        pBIOSInfo->DefaultActiveDevice= VIA_DEVICE_CRT1;
    }
    
    /* if XF86Config Option "ActiveDevice" hasn't set, active device according CMOS */
    if (!pBIOSInfo->ActiveDevice) {
        pBIOSInfo->ActiveDevice = pBIOSInfo->DefaultActiveDevice;
    }
    /* if XF86Config-4 set SAMM, and only active one device, active device according CMOS */
    else if (pVia->IsSecondary &&
            (pBIOSInfo->ActiveDevice == VIA_DEVICE_CRT1 ||
             pBIOSInfo->ActiveDevice == VIA_DEVICE_LCD  ||
             pBIOSInfo->ActiveDevice == VIA_DEVICE_DFP  ||
             pBIOSInfo->ActiveDevice == VIA_DEVICE_CRT2))
    {
        pBIOSInfo->ActiveDevice = BIOS_GetActiveDevice(pScrn);
    }
    if (((pBIOSInfo->ActiveDevice & pBIOSInfo->ConnectedDevice) == pBIOSInfo->ActiveDevice)
         && (pVia->IsSecondary)) {
        pBIOSInfo->SAMM = TRUE;
    }
    if((pBIOSInfo->ActiveDevice & (VIA_DEVICE_CRT1 | VIA_DEVICE_CRT2)) && pVia->IsSecondary)
    {
        pBIOSInfo->SAMM = TRUE;
        pBIOSInfo->ConnectedDevice|=VIA_DEVICE_CRT2;
    }    
    pBIOSInfo->ActiveDevice &= pBIOSInfo->ConnectedDevice;
    pVia->ActiveDevice = pBIOSInfo->ActiveDevice;
    
    /* To keep the actual active device no matter it is in SAMM or Duoview. */
    pBIOSInfo->ActualActiveDevice = pVia->ActiveDevice;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Actual Connected Device is %d\n", pBIOSInfo->ConnectedDevice));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Actual Active Device is %d\n", pBIOSInfo->ActiveDevice));

    return TRUE;
}

/* SAMM device dispatch */
Bool VIASetDisplayPath_SAMM(ScrnInfoPtr pScrn)
{
    DevUnion* pPriv = xf86GetEntityPrivate(pScrn->entityList[0], gVIAEntityIndex);
    VIAEntPtr pVIAEnt = pPriv->ptr;
    VIAPtr pVia1 = VIAPTR(pScrn);
    VIAPtr pVia0 = VIAPTR(pVIAEnt->pPrimaryScrn);
    VIABIOSInfoPtr pBIOSInfo1 = pVia1->pBIOSInfo;
    VIABIOSInfoPtr pBIOSInfo0 = pVia0->pBIOSInfo;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASetDisplayPath_SAMM\n"));
    pBIOSInfo0->SAMM = TRUE;
    pBIOSInfo1->PrimaryDevice = pBIOSInfo0->PrimaryDevice;    
    
    switch (pBIOSInfo1->ActiveDevice)
    {
        case (VIA_DEVICE_CRT1 | VIA_DEVICE_LCD):
            if (pBIOSInfo0->PrimaryDevice == VIA_DEVICE_LCD)
            {
                /* LCD+CRT */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_LCD;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_CRT1;                
                pBIOSInfo0->LVDSSettingInfo.IGAPath = IGA2;
                pBIOSInfo1->CRTSettingInfo.IGAPath = IGA1;
            }
            else
            {
                /* CRT+LCD */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_CRT1;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_LCD;
                pBIOSInfo0->CRTSettingInfo.IGAPath = IGA1;
                pBIOSInfo1->LVDSSettingInfo.IGAPath = IGA2;
            }
            
            break;
            
        case (VIA_DEVICE_CRT1 | VIA_DEVICE_DFP):
            if (pBIOSInfo0->PrimaryDevice == VIA_DEVICE_DFP)
            {
                /* DVI+CRT */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_DFP;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_CRT1;
                pBIOSInfo0->TMDSSettingInfo.IGAPath = IGA1;
                pBIOSInfo1->CRTSettingInfo.IGAPath = IGA2;
            }
            else
            {
                /* CRT+DVI */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_CRT1;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_DFP;
                pBIOSInfo0->CRTSettingInfo.IGAPath = IGA1;
                pBIOSInfo1->TMDSSettingInfo.IGAPath = IGA2;
            }

            break;
            
        case (VIA_DEVICE_LCD | VIA_DEVICE_DFP):
            if (pBIOSInfo0->PrimaryDevice == VIA_DEVICE_DFP)
            {
                /* DVI+LCD */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_DFP;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_LCD;
                pBIOSInfo0->TMDSSettingInfo.IGAPath = IGA1;
                pBIOSInfo1->LVDSSettingInfo.IGAPath = IGA2;
            }
            else
            {
                /* LCD+DVI */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_LCD;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_DFP;
                pBIOSInfo0->LVDSSettingInfo.IGAPath = IGA2;
                pBIOSInfo1->TMDSSettingInfo.IGAPath = IGA1;
            }
            
            break;

        case (VIA_DEVICE_LCD | VIA_DEVICE_LCD2):
            if (pBIOSInfo0->PrimaryDevice == VIA_DEVICE_LCD)
            {
                /* LCD+LCD2 */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_LCD;
                pBIOSInfo0->LVDSSettingInfo.IGAPath = IGA1;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_LCD2;
                pBIOSInfo1->LVDSSettingInfo2.IGAPath = IGA2;
            }
            else
            {
                /* LCD2+LCD */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_LCD2;                
                pBIOSInfo0->LVDSSettingInfo2.IGAPath = IGA1;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_LCD;
                pBIOSInfo1->LVDSSettingInfo.IGAPath = IGA2;
            }
            
            break;
            
        case (VIA_DEVICE_CRT1 | VIA_DEVICE_CRT2):
            if (pBIOSInfo0->PrimaryDevice == VIA_DEVICE_CRT2)
            {
                /* CRT2+CRT1 */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_CRT2;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_CRT1;
                pBIOSInfo0->CRTSettingInfo2.IGAPath = IGA1;
                pBIOSInfo1->CRTSettingInfo.IGAPath = IGA2;
            }
            else
            {
                /* CRT1+CRT2 */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_CRT1;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_CRT2;
                pBIOSInfo0->CRTSettingInfo.IGAPath = IGA1;
                pBIOSInfo1->CRTSettingInfo2.IGAPath = IGA2;
            }
            
            break;
            
        default:
            return FALSE;            
    }

    DEBUG(xf86DrvMsg(pVIAEnt->pPrimaryScrn->scrnIndex, X_INFO, "Scrn0: Active Device is 0x%x\n", pBIOSInfo0->ActiveDevice));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Scrn1: Active Device is 0x%x\n", pBIOSInfo1->ActiveDevice));
    DEBUG(xf86DrvMsg(pVIAEnt->pPrimaryScrn->scrnIndex,  X_INFO, "Primary: CRT=%d, DFP=%d, LCD=%d\n", pBIOSInfo0->CRTSettingInfo.IGAPath, pBIOSInfo0->TMDSSettingInfo.IGAPath, pBIOSInfo0->LVDSSettingInfo.IGAPath));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Secondary: CRT=%d, DFP=%d, LCD=%d\n", pBIOSInfo1->CRTSettingInfo.IGAPath, pBIOSInfo1->TMDSSettingInfo.IGAPath, pBIOSInfo1->LVDSSettingInfo.IGAPath));

    return TRUE;
}

void VIASetDisplayPath(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;

    VIACheckIfUseDuoView(pScrn);    

    if (pVia->IsSecondary)
    {
        /* SAMM: */
        if (pBIOSInfo->SAMM)
        {
            xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASetDisplayPath is SAMM\n");        
            VIASetDisplayPath_SAMM(pScrn);
        }
    }
    else if (pBIOSInfo->DuoView)
    {
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASetDisplayPath is DuoView\n"); 
        /* DuoView: */
        switch (pBIOSInfo->ActiveDevice)
        {
            case (VIA_DEVICE_CRT1 | VIA_DEVICE_LCD):
                pBIOSInfo->CRTSettingInfo.IGAPath = IGA1;
                pBIOSInfo->LVDSSettingInfo.IGAPath = IGA2;
                break;
                
            case (VIA_DEVICE_CRT1 | VIA_DEVICE_DFP):
                pBIOSInfo->CRTSettingInfo.IGAPath = IGA1;
                pBIOSInfo->TMDSSettingInfo.IGAPath = IGA2;
                break;
                
            case (VIA_DEVICE_LCD | VIA_DEVICE_DFP):
                pBIOSInfo->TMDSSettingInfo.IGAPath = IGA1;
                pBIOSInfo->LVDSSettingInfo.IGAPath = IGA2;
                break;

            case (VIA_DEVICE_LCD | VIA_DEVICE_LCD2):
                pBIOSInfo->LVDSSettingInfo.IGAPath = IGA2;
                pBIOSInfo->LVDSSettingInfo2.IGAPath = IGA2;
                break;
                
            case (VIA_DEVICE_CRT1 | VIA_DEVICE_CRT2):
                pBIOSInfo->CRTSettingInfo.IGAPath = IGA1;
                pBIOSInfo->CRTSettingInfo2.IGAPath = IGA2;
                break;
                
            default :
                pBIOSInfo->CRTSettingInfo.IGAPath = IGA1;
                break;
        }
    }    
    else if (pBIOSInfo->ActualActiveDevice == (VIA_DEVICE_LCD & VIA_DEVICE_LCD2))
    {
        pBIOSInfo->LVDSSettingInfo.IGAPath = IGA2;
        pBIOSInfo->LVDSSettingInfo2.IGAPath = IGA2;
    }
    else
    {
        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASetDisplayPath is Single\n"); 
        /* Simultaneous: */
        pBIOSInfo->CRTSettingInfo.IGAPath = IGA1;
        pBIOSInfo->TMDSSettingInfo.IGAPath = IGA1;
        pBIOSInfo->LVDSSettingInfo.IGAPath = IGA2;        
    }
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Active Device is 0x%x\n", pBIOSInfo->ActiveDevice);
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "IGA: CRT=%d, DFP=%d, LCD=%d\n", 
                                                pBIOSInfo->CRTSettingInfo.IGAPath,
                                                pBIOSInfo->TMDSSettingInfo.IGAPath,
                                                pBIOSInfo->LVDSSettingInfo.IGAPath);
                                                
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Active Device is 0x%x\n", pBIOSInfo->ActiveDevice));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "IGA: CRT=%d, DFP=%d, LCD=%d\n", 
                                                pBIOSInfo->CRTSettingInfo.IGAPath,
                                                pBIOSInfo->TMDSSettingInfo.IGAPath,
                                                pBIOSInfo->LVDSSettingInfo.IGAPath));
}

/* DuoVoiew device dispatch */
Bool VIADeviceDispatchDuoView(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    LPVIAGRAPHICINFO lpVIAGinfo = &pVia->pVidData->VIAGraphicInfo;

    /* add for duoview device switch */
    int only_one_video_engine = 0;
    switch (pVia->Chipset)
    {
        case VIA_KM400:
        case VIA_K8M800:
        case VIA_P4M800PRO:
        case VIA_K8M890:
        case VIA_P4M890:
        case VIA_P4M900:            
		  	only_one_video_engine = 1;
			break;
        default:
            break;	  	
    }
	
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIADeviceDispatchDuoView !!\n"));
    switch (pBIOSInfo->ActiveDevice) {
        case (VIA_DEVICE_CRT1 | VIA_DEVICE_LCD) :       /* If CRT + LCD, CRT is IGA1 */
            if((only_one_video_engine)&& (pBIOSInfo->s3utility==FALSE))
            {
                if(lpVIAGinfo->duoview & DUOVIEW_VIDEOOPTION)
                {
                    if(lpVIAGinfo->duoview & DUOVIEW_VIDEOONLCD)  
                        lpVIAGinfo->duoview |= DUOVIEW_V3_ON_IGA2;
                    else
                        lpVIAGinfo->duoview &= ~DUOVIEW_V3_ON_IGA2;
                }
            }
            break;
        case (VIA_DEVICE_CRT1 | VIA_DEVICE_DFP) :       /* If CRT + DFP, CRT is IGA1 */
            if((only_one_video_engine)&& (pBIOSInfo->s3utility==FALSE))
            {			
                if(lpVIAGinfo->duoview & DUOVIEW_VIDEOOPTION)
                {
                    if(lpVIAGinfo->duoview & DUOVIEW_VIDEOONDVI)  
                        lpVIAGinfo->duoview |= DUOVIEW_V3_ON_IGA2;
                    else
                        lpVIAGinfo->duoview &= ~DUOVIEW_V3_ON_IGA2;
                }
                else
                {
                    lpVIAGinfo->duoview |= DUOVIEW_V3_ON_IGA2;
                }
            }
            break;
        case (VIA_DEVICE_LCD | VIA_DEVICE_DFP) :        /* If LCD + DFP, DFP is IGA1 */
            if((only_one_video_engine)&& (pBIOSInfo->s3utility==FALSE))
            {			
                if(lpVIAGinfo->duoview & DUOVIEW_VIDEOOPTION)
                {
                    if(lpVIAGinfo->duoview & DUOVIEW_VIDEOONLCD)  
                        lpVIAGinfo->duoview |= DUOVIEW_V3_ON_IGA2;
                    else
                        lpVIAGinfo->duoview &= ~DUOVIEW_V3_ON_IGA2;
                }
            }
            break;
        default :
            break;
    }
    
    return TRUE;
}

/* Build a list of the VESA modes the BIOS reports as valid */
static void
VIABuildModeList(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
	VIAModeInfoPtr m;
	int serno1 = 0, ModeNo = 0, Xres = 0, Yres = 0, bpp = 0;
	int serno2 = 0, refIndex = 0;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIABuildModeList\n"));
    while ((serno1 = BIOS_QueryModeList(pScrn, serno1, &ModeNo)) != 0)
    {
        BIOS_GetVESAModeInfo(pScrn, ModeNo, &Xres, &Yres, &bpp);
    	m = xnfcalloc(sizeof(VIAModeInfoRec),1);
    	m->width = Xres;
    	m->height = Yres;
    	m->bpp = bpp;
    	m->modeno = ModeNo;

        /* Get Support Refresh Rate by Mode No. */
        while ((serno2 = BIOS_QuerySupportRefreshRate(pScrn, ModeNo, serno2, &refIndex)) != 0)
        {
            switch (refIndex)
            {
                case 0:
                    m->SupRefMask |= VIA_REF_60HZ;
                    break;
                case 1:
                    m->SupRefMask |= VIA_REF_56HZ;
                    break;
                case 2:
                    m->SupRefMask |= VIA_REF_65HZ;
                    break;
                case 3:
                    m->SupRefMask |= VIA_REF_70HZ;
                    break;
                case 4:
                    m->SupRefMask |= VIA_REF_72HZ;
                    break;
                case 5:
                    m->SupRefMask |= VIA_REF_75HZ;
                    break;
                case 6:
                    m->SupRefMask |= VIA_REF_80HZ;
                    break;
                case 7:
                    m->SupRefMask |= VIA_REF_85HZ;
                    break;
                case 8:
                    m->SupRefMask |= VIA_REF_90HZ;
                    break;
                case 9:
                    m->SupRefMask |= VIA_REF_100HZ;
                    break;
                case 10:
                    m->SupRefMask |= VIA_REF_120HZ;
                    break;
                case 11:
                    m->SupRefMask |= VIA_REF_86HZ;
                    break;
                case 12:
                    m->SupRefMask |= VIA_REF_94HZ;
                    break;
            }
        }
    	m->next = pVia->VIAModeList;

    	xf86DrvMsg(pScrn->scrnIndex, X_PROBED,
    	      "Support VESA mode 0x%x: X:%d Y:%d bpp:%d Refresh:0x%x\n",
    	       m->modeno, m->width, m->height, m->bpp, m->SupRefMask);

    	pVia->VIAModeList = m;
    }
}

Bool VIACheckLCDExpand(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    int countWidthByQWord, offsetWidthByQWord;
    unsigned char   cr13, cr35, sr1c, sr1d;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIACheckLCDExpand\n"));
    VGAOUT8(0x3c4, 0x33);
    /* Hotkey event occur */
    if (VGAIN8(0x3c5) & 0x01)
    {
        xf86DrvMsg(pScrn->scrnIndex, X_INFO,
                   "Device Switch Hotkey has been press!!\n");
        pBIOSInfo->ActiveDevice = BIOS_GetActiveDevice(pScrn);
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)
        {
            VGAOUT8(0x3d4, 0x79);
            if (VGAIN8(0x3d5) & 0x1)
            {
                pBIOSInfo->scaleY = TRUE;
            }
            else
            {
                pBIOSInfo->scaleY = FALSE;
            }
        }
        else
        {
            pBIOSInfo->scaleY = FALSE;
            /* if CRT Only, no panning */
            if (pBIOSInfo->ActiveDevice == VIA_DEVICE_CRT1)
            {
                pScrn->frameX1 = pBIOSInfo->SaveframeX1;
                pScrn->frameY1 = pBIOSInfo->SaveframeY1;
                pScrn->currentMode->HDisplay = pBIOSInfo->SaveHDisplay;
                pScrn->currentMode->VDisplay = pBIOSInfo->SaveVDisplay;
                pScrn->currentMode->CrtcHDisplay = pBIOSInfo->SaveCrtcHDisplay;
                pScrn->currentMode->CrtcVDisplay = pBIOSInfo->SaveCrtcVDisplay;
                VIAAdjustFrame(pScrn->scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);
            }
        }

        /* clean hotkey event */
        VGAOUT8(0x3c4, 0x33);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) & ~0x01);

        /* Set Quadword offset and counter */
        countWidthByQWord = pBIOSInfo->countWidthByQWord;
        offsetWidthByQWord = pBIOSInfo->offsetWidthByQWord;

        VGAOUT8(0x3d4, 0x35);
        cr35 = VGAIN8(0x3d5) & 0x1f;
        cr35 |= (offsetWidthByQWord & 0x700) >> 3;        /* bit 7:5: offset 10:8 */
        cr13 = offsetWidthByQWord & 0xFF;
 
        /* Enable Refresh to avoid data lose when enter screen saver */
        /* Refresh disable & 128-bit alignment */
        VGAOUT8(0x3c4, 0x1d);
        sr1d = VGAIN8(0x3c5);     
        sr1d = (sr1d & 0xfc) | (countWidthByQWord >> 9);
        sr1c = ((countWidthByQWord >> 1) + 1) & 0xff;

        VGAOUT8(0x3d4, 0x13);
        VGAOUT8(0x3d5, cr13);
        VGAOUT8(0x3d4, 0x35);
        VGAOUT8(0x3d5, cr35);
        VGAOUT8(0x3c4, 0x1c);
        VGAOUT8(0x3c5, sr1c);
        VGAOUT8(0x3c4, 0x1d);
        VGAOUT8(0x3c5, sr1d);               
    }

    return pBIOSInfo->scaleY;
}

void VIADebugBreak()
{
    /* use int3 to set breakpoint */
    __asm __volatile("int $3");
}


void
VIAInitialize3DEngine(ScrnInfoPtr pScrn)
{
}



#if XORG_VERSION_CURRENT >= XF86_VERSION_NUMERIC(6,8,0,0,0)

static Bool
VIARandRGetInfo(ScrnInfoPtr pScrn, Rotation *rotations)
{
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIARandRGetInfo!\n");
    
    VIAPtr pVia = VIAPTR(pScrn);

    /* to report what ability we can support. */
    if(pVia->RandRRotation)
       *rotations = RR_Rotate_0 | RR_Rotate_90 |RR_Rotate_180 | RR_Rotate_270;
    else
       *rotations = RR_Rotate_0;

    return TRUE;
}

static Bool
VIARandRSetConfig(ScrnInfoPtr pScrn, xorgRRConfig *config)
{
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIARandRSetConfig!\n");
    VIAPtr pVia = VIAPTR(pScrn);
    PVIDDATA pVidData = pVia->pVidData;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVia->pVidData->VIAGraphicInfo;
    int fd;
    
    switch(config->rotation) {
        case RR_Rotate_0:            
            pVia->SWRotate = VIA_ROTATE_DEGREE_0;
            break;

        case RR_Rotate_90:            
            pVia->SWRotate = VIA_ROTATE_DEGREE_270;            
            break;
            
        case RR_Rotate_180:            
            pVia->SWRotate = VIA_ROTATE_DEGREE_180;            
            break;

        case RR_Rotate_270:            
            pVia->SWRotate = VIA_ROTATE_DEGREE_90;
            break;

        default:
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                    "Unexpected rotation in VIARandRSetConfig!\n");
            pVia->SWRotate = VIA_ROTATE_DEGREE_0;
            return FALSE;
    }

    lpVIAGraphicInfo->rotate = pVia->SWRotate;

    return TRUE;
}

#if XORG_VERSION_CURRENT >= XF86_VERSION_NUMERIC(6,9,0,0,0)
static Bool
VIADriverFunc(ScrnInfoPtr pScrn, xorgDriverFuncOp op, pointer data)
#else
static Bool
VIADriverFunc(ScrnInfoPtr pScrn, xorgRRFuncFlags op, pointer data)
#endif
{
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIADriverFunc!\n");
    
    switch(op) {
    case RR_GET_INFO:          
        return VIARandRGetInfo(pScrn, (Rotation*)data);
    case RR_SET_CONFIG:          
        return VIARandRSetConfig(pScrn, (xorgRRConfig*)data);
    default:
        return FALSE;
    }

    return FALSE;
}

#endif

#if (defined(PMLINK))&&(LINUX_VERSION_CODE>132116)
/*function: we listen on the power evnet message from the socket
 * selected and then dispatch work to suspend module and resume module
 */
CARD32 VIAPMHandler(OsTimerPtr timer,CARD32 now,pointer arg)
{
    ScrnInfoPtr pScrn=(ScrnInfoPtr)arg;
    VIAPtr pVia=VIAPTR(pScrn);
    struct pollfd pfd;
    struct nlmsghdr *msg_in;
    struct cn_msg *data;
    int ret;
    int len;
    char buf[BUFFER_SIZE];
    
    /*At this stage we will listen to the kernel evnet notification
     * for a while,if there is something,enter the processing
     * routine
     */
    pfd.fd = sock_fd;
    pfd.events = POLLIN;
    pfd.revents = 0;
    ret=poll(&pfd, 1, VIA_POLL_TIMEOUT);  /*do not wait*/
    if (ret)    /*received something*/
    {      
        memset(buf, 0, sizeof(buf));
        len = recv(sock_fd, buf, sizeof(buf), 0);
        if (len == -1)
        {
            return VIA_SCHEDULE_SLICE ;  /*schedule next time*/
        }
        msg_in = (struct nlmsghdr *)buf;
        switch (msg_in->nlmsg_type)
        {
        case NLMSG_ERROR:
            xf86DrvMsg(pScrn->scrnIndex,X_INFO,"Error message received.\n");
            break;
        case NLMSG_DONE:/*the right message received*/
            data = (struct cn_msg *)NLMSG_DATA(msg_in);   /*strip off the header*/

            if ((flag_lastrecv!=data->flags)&&(data->seq!=getpid()))
            {
                xf86DrvMsg(pScrn->scrnIndex,X_INFO,\
                            " rev[%x.%x] [pid:%x.flags:%x]. ack:%d\n",\
                data->id.idx, data->id.val, data->seq, data->flags,data->ack);
                flag_lastrecv=data->flags;
                switch (data->flags){
                case NOEVENT_NOTIFIER: /*never happens...*/
                    break;
                case SUSPEND_NOTIFIER:
                    VIASuspendRoutine(pScrn,data);
                    break;
                case RESUME_NOTIFIER:
                    VIAResumeRoutine(pScrn,data);
                    break;
                case STANDBY_NOTIFIER:
                    VIAStandbyRoutine(pScrn,data);
                    break;
                default:
                    break;
                }
            }    
            break;
        default:
            break;
        }
    }  
    return VIA_SCHEDULE_SLICE;   /*return for next schedule*/   
}

/*send link message to master*/
static int VIAPMLinkSendMSG(int sock_fd,struct cn_msg *msg)
{
	struct nlmsghdr *nlh;
	unsigned int size;
	int byte_sent;
	char buf[BUFFER_SIZE];
	struct cn_msg *msg_out;  

	size = NLMSG_SPACE(sizeof(struct cn_msg) + msg->len);
    memset(buf,0,sizeof(buf));
  
	nlh = (struct nlmsghdr *)buf;
	nlh->nlmsg_seq = msg->seq;
	nlh->nlmsg_pid = getpid();
	nlh->nlmsg_type = NLMSG_DONE;
	nlh->nlmsg_len = NLMSG_LENGTH(size - sizeof(*nlh));
	nlh->nlmsg_flags = 0;

	msg_out = NLMSG_DATA(nlh);

	printf("sends out: [%x.%x],[pid:%d,flags:%x], ack:%d\n",\
	msg->id.idx, msg->id.val,msg->seq,msg->flags, msg->ack);
    fflush(stdout);     

	memcpy(msg_out, msg, sizeof(*msg_out) + msg->len);

	byte_sent = send(sock_fd, nlh, size, 0);
	if (byte_sent == -1)
		printf("Failed to send: %s [%d].\n",strerror(errno), errno);
        fflush(stdout);
	return byte_sent;

}

/*if anything that must be done before suspend, add to here*/
static void VIASuspendRoutine(ScrnInfoPtr pScrn,struct cn_msg *msg)
{
    char buf[BUFFER_SIZE];
    struct cn_msg *data;
    int len=-1;
    b3DRegsInitialized=0;
	xf86DrvMsg(pScrn->scrnIndex,X_INFO,"VIASuspendRoutine\n");
    xf86DrvMsg(pScrn->scrnIndex,X_INFO,"b3DRegsInitialized:%d\n",b3DRegsInitialized);
    /*add suspend routine processing here*/
    b3DRegsInitialized=0;
    /*....*/
    /*confirm the pm link master we are ready to go on*/
    do{
        memset(buf,0,sizeof(buf));
        data=(struct cn_msg *)buf;
        data->id.idx=VIA_LINK_GROUP;
        data->id.val=VIA_LINK_VAL;
        data->seq=getpid();
        data->ack=1;  /*confirm*/
        data->flags=msg->flags|CONFIRM_NOTIFIER;
        len= VIAPMLinkSendMSG(sock_fd,data);
    }while (len==-1);
	return;
}

/*if anything that must be done before entering standby mode,
 * add to here 
 */
static void VIAStandbyRoutine(ScrnInfoPtr pScrn,struct cn_msg *msg)
{
    char buf[BUFFER_SIZE];
    struct cn_msg *data;
    int len=-1;
    xf86DrvMsg(pScrn->scrnIndex,X_INFO,"VIAStandbyRoutine\n");
    do{    
        memset(buf,0,sizeof(buf));
        data=(struct cn_msg *)buf;
        data->id.idx=VIA_LINK_GROUP;
        data->id.val=VIA_LINK_VAL;
        data->seq=getpid();
        data->ack=1;  /*confirm*/
        data->flags=msg->flags|CONFIRM_NOTIFIER;
        len= VIAPMLinkSendMSG(sock_fd,data);
    }while(len==-1);
	return;
}

/*if anything that must be done after resume back,add to here */
static void VIAResumeRoutine(ScrnInfoPtr pScrn,struct cn_msg *msg)
{
    char buf[BUFFER_SIZE];
    struct cn_msg *data;
    int len=-1;
    xf86DrvMsg(pScrn->scrnIndex,X_INFO,"VIAResumeRoutine\n");
    /*add suspend routine processing here*/
    /*....*/
    /*confirm the pm link master we are ready to go on*/
    do{    
        memset(buf,0,sizeof(buf));
        data=(struct cn_msg *)buf;
        data->id.idx=VIA_LINK_GROUP;
        data->id.val=VIA_LINK_VAL;
        data->seq=getpid();
        data->ack=1;  /*confirm*/
        data->flags=msg->flags|CONFIRM_NOTIFIER;
        len= VIAPMLinkSendMSG(sock_fd,data);
    }while(len==-1);
    return;
}
#endif /* PMLINK **/

/*
 * This function is only required if we need to do anything related to power managemant
 * and via private event handling.
 */
static Bool VIAPMEvent(int scrnIndex, pmEvent event, Bool undo)
{
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
    VIAPtr pVia = VIAPTR(pScrn);
     
    xf86DrvMsg(X_INFO,pScrn->scrnIndex,"enter pm event\n");
    ErrorF("VIAPMEvent: received APM event %d\n", event);
    switch(event)
    {
        case XF86_APM_SYS_SUSPEND:
        case XF86_APM_CRITICAL_SUSPEND: /*do we want to delay a critical suspend?*/
        case XF86_APM_USER_SUSPEND:
        case XF86_APM_SYS_STANDBY:
        case XF86_APM_USER_STANDBY:
            if (!undo && !pVia->IsSuspending)
            {
                pScrn->LeaveVT(scrnIndex, 0);
                pVia->IsSuspending = TRUE;
                sleep(50);
                pScrn->EnterVT(scrnIndex, 0);
            } 
            else if (undo && pVia->IsSuspending)
            {
                sleep(1);
                pScrn->EnterVT(scrnIndex, 0);
                pVia->IsSuspending = FALSE;
            }
            break;
        case XF86_APM_STANDBY_RESUME:
        case XF86_APM_NORMAL_RESUME:
        case XF86_APM_CRITICAL_RESUME:
            if (pVia->IsSuspending)
            {
                sleep(1);
                pScrn->EnterVT(scrnIndex, 0);
                pVia->IsSuspending = FALSE;
            }
            break;
        case XF86_APM_CAPABILITY_CHANGED:
            if (pVia->IsSecondary)
                return TRUE;
        default:
            ErrorF("VIAPMEvent: received APM event %d\n", event);
            return TRUE;
    }
    return TRUE;
}
